/* -*- linux-c -*- (Autoset linux indent style when using emacs) */

/*
 * Copyright 1999 Digi International (www.digi.com)
 *    Jeff Randall <Jeff_Randall@digi.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the 
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


/*
 * This is a driver for the Digi International boards that speak the
 * DXB2 protocol.  This currently includes the DataFireRAS 24, the
 * DataFireRAS 8, the DataFireRAS 4 and the AccelePort Xp families
 * of products.
 *
 */
char *dgdm_version = "$Id: dgdm_driver.c,v 1.51 2002/05/22 16:44:11 dickl Exp $";


/* all of the includes, defines, typedefs, and structures are here */
#include "dgdm_driver.h"

MODULE_AUTHOR("Digi International, http://www.digi.com");
MODULE_DESCRIPTION("Driver for dxb2 based product line");
MODULE_SUPPORTED_DEVICE("dgdm");

/*
 * insmod command line overrideable parameters
 *
 * There are either 5 or 10 consequtive majors assigned:
 *      major   terminal device (low 4 boards)  (e.g. /dev/dg/dgdm/ttyG0_0)
 *      major+1 callout device                  (e.g. /dev/dg/dgdm/cug0_0)
 *      major+2 transparent print device        (e.g. /dev/dg/dgdm/lpG0_0)
 *      major+3 ISDN test device
 *      major+4 control device                  (e.g. /dev/dg/dgdm/ram0, reg0)
 *      major+5 terminal device (High 4 boards) (e.g. /dev/dg/dgdm/ttyG4_0)
 *      major+6 callout device                  (e.g. /dev/dg/dgdm/cug4_0)
 *      major+7 transparent print device        (e.g. /dev/dg/dgdm/lpG4_0)
 *      major+8 ISDN test device
 *      major+9 control device                  (e.g. /dev/dg/dgdm/ram0, reg0)
 *
 * All pathnames to devices can be absolute, but it is not recommended.
 * The embedded tabs in the defaults are not required, they are there to make
 * the default output table look pretty to human eyes.
 *
 * The last 5 majors are only used if there are more than 4 board present
 * in the system.  This also requires that the define MAXBOARDS is > 4.
 *
 * NOTE: we use a set of macros to create the variables, which allows
 * us to specify the variable type, name, initial value, and description.
 * This makes things work swell for both 2.0 and 2.1+ kernels.
 */
PARM_STR(indevice,	"ttyG%b_%p",	"");
PARM_STR(outdevice,	"cug%b_%p",	"");
PARM_STR(printdevice,	"lpG%b_%p",	"");
PARM_STR(specdevice,	"spec%b_%p",	"");
PARM_STR(ramdevice,	"ram%b\t",	"");
PARM_STR(dspramdevice,	"dspram%b_%p",	"");
PARM_STR(flashdevice,	"flash%b_%p",	"");

PARM_STR(vpdalldevice,	"vpd%b_%p",	"");
PARM_STR(iodevice,	"io%b_%p\t",	"");
PARM_STR(regdevice,	"reg%b\t",	"");
PARM_STR(gdbdevice,	"gdb%b\t",	"");
PARM_STR(dmabufdevice,	"dmabuf%b_%p",	"");
PARM_STR(rawdxbdevice,	"dxbStrm%b_%p",	"");

PARM_INT(major,		70,	"Default major number");
PARM_INT(tty,		1,	"Run as tty (Serial and Callout) driver");
PARM_INT(print,		1,	"Enable Transparent Print");
PARM_INT(control,	1,	"Register Control devices");
PARM_INT(isdn,		1,	"Run as an ISDN driver");
PARM_INT(debug,		0x0,	"Driver debugging level");
PARM_INT(rawreadok,	1,	"Bypass flip buffers on input");
PARM_INT(cook_size,	128,	"Size of each channels LD cook buffer");
PARM_INT(loopback,	0,	"All tty channels are loopbacks");
PARM_INT(intr_period,	10,	"Period for interrupts");
PARM_INT(polling,	0,	"Poll boards instead of using interrupts");
PARM_INT(download,	1,	"If not set, we do not download the FEP");
PARM_INT(gdb,		0,	"If set, we are trying to GDB very early");
PARM_INT(dxb_align,	1,	"If set, force alignment of DXB struct");
PARM_INT(print_waits,	0,	"If set, FEP will wait when print buf full");
PARM_INT(erase_nvm,	0,	"If set, FEP will erase NV parms at start");
PARM_INT(rxbufsize,	4096,	"Drv rx dxb (not flip) buf size. Power of 2.");
PARM_INT(trcbuf_size,	0x100000, "Debugging trace buffer size. 1M default.");

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
PARM_INT(latency,	0,	"Set the LD low latency flag,");
#endif

#if ALLOW_ALT_FAIL_OPEN
PARM_INT(alt_fail_open,	ALT_FAIL_OPEN_DEFAULT, "Enable alternate failed open behavior");
#endif


#define	NOFILL	0xdadadada
PARM_ULONG(fillmem,	NOFILL,	"Fill memory with value before loading FEP");

PARM_STR(dxbttyname,	"SERIAL",	"Name of DXB serial device");
PARM_STR(usrlibdir,	 USRLIBSTR,	"Directory where firmware is located");

#if defined(MODULE_LICENSE)
    MODULE_LICENSE("GPL");
#endif

/*
 * A generic list of Product names, PCI Vendor ID, and PCI Device ID.
 *
 * NOTE: AccelePort Xp products are specifically named during the
 *       interpretation of the POST and the name listed here is just a
 *       placeholder.
 */
typedef struct {
	char *name;
	uint16 vendor;
	uint16 device;
	uint32 ramlen;
} ID;

STATIC ID Ids[] =
{
	{DIGI_PCIHOUSTON_FAMILY_NAME,  DIGI_VID, DataFireRAS_P_DID,	8*MEG},
	{DIGI_CPCIHOUSTON_FAMILY_NAME, DIGI_VID, DataFireRAS_Pcpci_DID,16*MEG},
	{DIGI_LONDON_FAMILY_NAME,      DIGI_VID, DataFireRAS_B4_DID,	4*MEG},
	{DIGI_CHASKA_FAMILY_NAME,      DIGI_VID, AccelePortRAS_A_DID,	4*MEG},
	{DIGI_AP2K_FAMILY_NAME,	       DIGI_VID, DIGI_AP2K_DID,		2*MEG},
	{"Houston/Bluegreen",	0x1014,		0x00c1,			8*MEG},
	{"London/Bluegreen",	0x1014,		0x00bf,			4*MEG},
	{"Chaska/Bluegreen",	0x1014,		0x00c3,			4*MEG}
};
#define	NIDS	(sizeof(Ids)/sizeof(ID))




/**************************************************************************
 *
 * protos for this file
 *
 */

/* Driver load/unload functions */
GLOBAL int		init_module(void);
GLOBAL void		cleanup_module(void);
STATIC void		dgdm_cleanup_board(BOARD_T *brd);

STATIC void		dgdm_init_globals(void);
STATIC int		dgdm_scan(void);
STATIC int		dgdm_found_board(struct pci_dev *pdev, int id);
STATIC int		dgdm_finalize_board_init(BOARD_T *brd);

STATIC int		dgdm_triboot(void);
STATIC int		dgdm_triboot_step(BOARD_T *brd);
STATIC int		dgdm_tribl_map_addrs(BOARD_T *brd);
STATIC int		dgdm_tribl_dxbinit(BOARD_T *brd);
STATIC int		dgdm_tribl_fwload(BOARD_T *brd);
STATIC void		dgdm_triboot_cb(uint p0, uint *p1, uint *p2,
                                        uint *p3, uint *p4);

STATIC void		dgdm_decode_board(BOARD_T *brd, BOARD *triboard);
STATIC int		dgdm_decode_post(BOARD_T *brd);

/* interrupt handler */
STATIC void		dgdm_intr(int irq, void *voidbrd,struct pt_regs *regs);

/* Control device open/close/read/write/ioctl/mmap */
STATIC int		dgdm_boardopen(struct inode *inode, struct file *file);
STATIC CLOSE_RETURN_T	dgdm_boardclose(struct inode *inode,struct file *file);
STATIC READ_RETURN_T	dgdm_boardread(READ_PROTO);
STATIC int		dgdm_debugread(BOARD_T *brd, READ_PROTO);
STATIC int		dgdm_printread(BOARD_T *brd, READ_PROTO);
STATIC int		dgdm_dspread(BOARD_T* brd, int pmdm, READ_PROTO);
STATIC int		dgdm_dspcache(BOARD_T *brd, uint pos, int pmdm);

STATIC WRITE_RETURN_T	dgdm_boardwrite(WRITE_PROTO);
STATIC int		dgdm_debugwrite(BOARD_T *brd, WRITE_PROTO);
STATIC int		dgdm_printwrite(BOARD_T *brd, WRITE_PROTO);
STATIC int		dgdm_flashwrite16(BOARD_T* brd, int subtype,
                                          WRITE_PROTO);
STATIC int		dgdm_flashwrite8(BOARD_T *brd, uchar *vbase,
                                         FLASHAREA *flasharea, WRITE_PROTO);
STATIC int		dgdm_dspwrite(BOARD_T *brd, int pmdm, WRITE_PROTO);

STATIC int		dgdm_boardioctl(struct inode *inode, struct file *file,
                                        unsigned int cmd, unsigned long arg);
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,1,0)
STATIC int		dgdm_boardmmap(struct inode *inode, struct file *file,
                                       struct vm_area_struct *vma);
#else
STATIC int		dgdm_boardmmap(struct file *file,
                                       struct vm_area_struct *vma);
#endif

/* Utility functions */
static inline uint32	swap32(uint32 val);
static inline uint16	swap16(uint16 val);
STATIC void		*dgdm_driver_kzmalloc(size_t size, int priority);


#if LINUX_VERSION_CODE < KERNEL_VERSION(2,1,0)
/* shim functions to provide a 2.2 feel to 2.0 kernels */
STATIC struct pci_dev	*pci_find_device(int vendor, int device,
                                         struct pci_dev *pdev);

STATIC int		pci_read_config_byte(struct pci_dev *pdev,
                                             u8 where, u8 *val);
STATIC int		pci_read_config_word(struct pci_dev *pdev,
                                             u8 where, u16 *val);
STATIC int		pci_read_config_dword(struct pci_dev *pdev,
                                              u8 where, u32 *val);
# if defined(DGDM_ALL_OLD_PCI_CODE)
/* not used in this driver, but included for completeness */
STATIC int		pci_write_config_byte(struct pci_dev *pdev,
                                              u8 where, u8 val);
STATIC int		pci_write_config_word(struct pci_dev *pdev,
                                              u8 where, u16 val);
STATIC int		pci_write_config_dword(struct pci_dev *pdev,
                                               u8 where, u32 val);
STATIC void		pci_set_master(struct pci_dev *pdev);
# endif
#endif


#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
/*
 * File operations permitted on Control major
 */
STATIC struct file_operations BoardFops =
{
	owner:		THIS_MODULE,
	read:		dgdm_boardread,
	write:		dgdm_boardwrite,
	ioctl:		dgdm_boardioctl,
	mmap:		dgdm_boardmmap,
	open:		dgdm_boardopen,
	release:	dgdm_boardclose
};
#else
/*
 * File operations permitted on Control major
 */
STATIC struct file_operations BoardFops =
{
	NULL,			/* lseek */
	dgdm_boardread,		/* read */
	dgdm_boardwrite,	/* write */
	NULL,			/* readdir */
	NULL,			/* select/poll */
	dgdm_boardioctl,	/* ioctl */
	dgdm_boardmmap,		/* mmap */
	dgdm_boardopen,		/* open */
# if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
	NULL,			/* flush */
# endif
	dgdm_boardclose,	/* close/release */
	NULL,			/* fsync */
# if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
	NULL,			/* fasync */
	NULL,			/* check_media_change */
	NULL,			/* revalidate */
	NULL,			/* lock */
# endif
};
#endif



/*
 * Statics/Globals
 */
GLOBAL BOARD_T		 *dgdm_Board[MAXBOARDS];
STATIC PADDRS_T		 pci_addrs[MAXBOARDS];
GLOBAL uchar		 dgdm_NumBoards;
GLOBAL uchar		 dgdm_Major_Serial_Registered		= FALSE;
GLOBAL uchar		 dgdm_Major_Callout_Registered		= FALSE;
GLOBAL uchar		 dgdm_Major_TransparentPrint_Registered = FALSE;
GLOBAL uchar		 dgdm_Major_ISDN_Registered		= FALSE;
GLOBAL uchar		 dgdm_Major_Control_Registered		= FALSE;
GLOBAL uchar		 dgdm_Major_Serial2_Registered		= FALSE;
GLOBAL uchar		 dgdm_Major_Callout2_Registered		= FALSE;
GLOBAL uchar		 dgdm_Major_TransparentPrint2_Registered = FALSE;
GLOBAL uchar		 dgdm_Major_ISDN2_Registered		= FALSE;
GLOBAL uchar		 dgdm_Major_Control2_Registered		= FALSE;


#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
GLOBAL spinlock_t	 dgdm_global_lock = SPIN_LOCK_UNLOCKED;
#endif


/* external variables */
GLOBAL struct file_operations dgdm_BoardIsdnFops;



/*
 * Real H/L/C board.  Note that 48KB after BOOT is currently unused
 *
 * Each UNIT is 8KB.  512KB total flash.
 */
static FLASHAREA        Flash10[FLASH_AREAS] =
{
/* Subtype 0: Entire Flash */
        { "1011100010000000100000001000000010000000100000001000000010000000" },
/* Subtype 1: Program Area */
        { "eeeeeeee10000000100000001000000010000000eeeeeeeeeeeeeeeeeeeeeeee" },
/* Subtype 2: VPD Area on chaska/london */
        { "eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee10000000" },
/* Subtype 3: Parameter Area (last 64k not used on chaska/london) */
        { "eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee100000001000000010000000" },
/* Subtype 4: Boot Area */
        { "10eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee" },
/* Subtype 5: Devious Purposes (area partly unused) */
        { "ee111000eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee" },
/* Subtype 6: Configuration Data Area */
        { "ee1eeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeeee" },
};


/*
 * Houston has a separate VPD flash
 */
static FLASHAREA        FlashVPD[VPD_AREAS] =
{
/* Subtype 0: Entire VPD Flash */
        { "1010101010101010" },
/* Subtype 1: The VPD proper */
        { "10eeeeeeeeeeeeee" },
/* Subtype 2: The rest of the VPD flash for devious purposes */
        { "ee10101010101010" },
};




/************************************************************************
 *
 * Driver load/unload functions
 *
 ************************************************************************/

/*
 * init_module()
 *
 * Module load.  This is where it all starts.
 */
GLOBAL int init_module(void)
{
    int rc;

        /* make sure that the globals are init'd before we do anything else */
        dgdm_init_globals();

	dgdm_NumBoards = 0;
	dgdm_Major_Serial_Registered = FALSE;
	dgdm_Major_Callout_Registered = FALSE;
	dgdm_Major_TransparentPrint_Registered = FALSE;
	dgdm_Major_Control_Registered = FALSE;
	dgdm_Major_ISDN_Registered = FALSE;

	APR(("%s, Digi International Part Number %s\n", DG_NAME, DG_PART));

	/*
	 * Find and configure all the cards
	 */
	rc = dgdm_scan();
	if (!rc) {
		/*
		 * At least one card was found, register the devices
		 * with the kernel.
		 */
		dgdm_proc_register_basic();

		if (dgdm_tty) {
			rc = dgdm_tty_init();
			if (rc < 0) {
				dgdm_tty_uninit();
				APR(("Can't init tty devices (%d)\n", rc));
				return (rc);
			}
			/*
			 * Register FEP related /proc files
			 */
			dgdm_proc_register_fep();
		}
		if (!dgdm_Major_Control_Registered) {
			/*
			 * Register memory/control/debug/mangement devices
			 */
			rc = register_chrdev(major + DGDM_CONTROL_MAJOR,
					     "dxb", &BoardFops);
			if (rc < 0) {
				APR(("Can't register ram device (%d)\n", rc));
				return (rc);
			}
			dgdm_Major_Control_Registered = TRUE;
                }
		if (dgdm_isdn && !dgdm_Major_ISDN_Registered) {
			/*
			 * Register ISDN testing device
			 */
			rc = register_chrdev(major + DGDM_ISDN_MAJOR,
					     "dxb", &dgdm_BoardIsdnFops);
			if (rc < 0) {
				APR(("Can't register ISDN device (%d)\n", rc));
				return (rc);
			}
			dgdm_Major_ISDN_Registered = TRUE;
                }

                if(dgdm_NumBoards > 4) {
                        if (!dgdm_Major_Control2_Registered) {
                                rc = register_chrdev(major+DGDM_CONTROL2_MAJOR,
                                                     "dxb", &BoardFops);
                                if (rc < 0) {
                                        APR(("Can't register high ram "
                                             "device (%d)\n", rc));
                                        return (rc);
                                }
                                dgdm_Major_Control2_Registered = TRUE;
                        }
                        if (dgdm_isdn && !dgdm_Major_ISDN2_Registered) {
                                rc = register_chrdev(major + DGDM_ISDN2_MAJOR,
                                                     "dxb", &dgdm_BoardIsdnFops);
                                if (rc < 0) {
                                        APR(("Can't register high ISDN "
                                             "devices (%d)\n", rc));
                                        return (rc);
                                }
                                dgdm_Major_ISDN2_Registered = TRUE;
                        }
		}
	} else {
		cleanup_module();
	}

	return (rc);
}



/*
 * cleanup_module()
 *
 * Module unload.  This is where it all ends.
 */
GLOBAL void cleanup_module(void)
{
	int i;

	/*
	 * Attempting to free resources in backwards
	 * order of allocation, in case that helps
	 * memory pool fragmentation.
	 */
	dgdm_proc_unregister_all();

	if (dgdm_tty)
		dgdm_tty_uninit();

        #if MAXMAJOR > 4
	if (dgdm_Major_ISDN2_Registered)
		unregister_chrdev(major + DGDM_ISDN2_MAJOR, "dxb");

	if (dgdm_Major_Control2_Registered)
		unregister_chrdev(major + DGDM_CONTROL2_MAJOR, "dxb");
        #endif

	if (dgdm_Major_ISDN_Registered)
		unregister_chrdev(major + DGDM_ISDN_MAJOR, "dxb");

	if (dgdm_Major_Control_Registered)
		unregister_chrdev(major + DGDM_CONTROL_MAJOR, "dxb");

	for (i = 0; i < dgdm_NumBoards; ++i) {
                dgdm_cleanup_board(dgdm_Board[i]);
	}

#if defined(DGDM_TRACER)
        /* last thing, make sure we release the tracebuffer */
        dgdm_tracer_free();
#endif
}



/*
 * dgdm_cleanup_board()
 *
 * Free all the memory associated with a board
 */
STATIC void dgdm_cleanup_board(BOARD_T *brd) {

        TRIBL_BOARD *tribl;
        int i;

        if(!brd)
                return;

        tribl = &brd->tribl;

        if (brd->dxbisdn_chans) {
                for (i = 0; (i < NISDN); i++) {
                        if (brd->dxbisdn_chans[i].tx_buffer)
                                kfree(brd->dxbisdn_chans[i].tx_buffer);
                        if (brd->dxbisdn_chans[i].rx_buffer)
                                kfree(brd->dxbisdn_chans[i].rx_buffer);
                }

                kfree(brd->dxbisdn_chans);
                brd->dxbisdn_chans = NULL;
        }
        if (dgdm_tty) {
                dgdm_tty_uninit();
                dgdm_tty_brduninit(brd);
        }
        for (i = 0; i < 2; ++i)
                if (brd->dma_buf[i])
                        free_page(brd->dma_buf[i]);

        /*
         * Potentially disable receivers, shut down interrupts,
         * place hardware in a reset or anything else required 
         * to deactivate things before we put it to bed.
         * This is our last chance to touch hardware!
         */
        if(tribl) {
                tribl->state = TRIBL_DRIVER_UNLOAD;
                tri_boot(tribl);
        }

        if (brd->vreg) {
                IOUNMAP(brd->vreg);
        }
        if (brd->vram) {
                IOUNMAP(brd->vram);
        }
        if (brd->vdspgpp1)
                IOUNMAP(brd->vdspgpp1);

        if (brd->irq)
                free_irq(brd->irq, brd);

        if(brd->msgbuf_head) {
                unsigned long flags;

                DGDM_LOCK(dgdm_global_lock, flags);
                brd->msgbuf = NULL;
                printk(brd->msgbuf_head);
                vfree(brd->msgbuf_head);
                brd->msgbuf_head = NULL;
                DGDM_UNLOCK(dgdm_global_lock, flags);
        }

        dgdm_Board[brd->boardnum] = NULL;
        vfree(brd);

        DPR(("dgdm_cleanup_board() - done\n"));
}


/*
 * dgdm_init_globals()
 *
 * This is where we initialize the globals from the static insmod
 * configuration variables.  These are declared near the head of
 * this file.
 */
STATIC void dgdm_init_globals(void)
{
        int i;

        /*
         * If we're not loading the FEP, we won't be able to service
         * the TTY devices.  We also don't want any interrupts from the
         * boards.
         */
        if(!download) {
                tty = 0;
                polling = 1;
        }

	dgdm_indevice	  = indevice;
	dgdm_outdevice	  = outdevice;
	dgdm_printdevice  = printdevice;
	dgdm_specdevice	  = specdevice;
	dgdm_ramdevice	  = ramdevice;
	dgdm_dspramdevice = dspramdevice;
	dgdm_flashdevice  = flashdevice;
	dgdm_vpdalldevice = vpdalldevice;
	dgdm_iodevice	  = iodevice;
	dgdm_regdevice	  = regdevice;
	dgdm_gdbdevice	  = gdbdevice;
	dgdm_dmabufdevice = dmabufdevice;
	dgdm_rawdxbdevice = rawdxbdevice;

	dgdm_major	  = major;
	dgdm_tty	  = tty;
	dgdm_print	  = print;
	dgdm_isdn	  = isdn;
	dgdm_control	  = control;
	dgdm_rawreadok	  = rawreadok;
	dgdm_debug	  = debug;
	dgdm_cook_size	  = cook_size;
	dgdm_loopback	  = loopback;
	dgdm_intr_period  = intr_period;
	dgdm_polling	  = polling;
	dgdm_download	  = download;
	dgdm_gdb	  = gdb;
	dgdm_dxb_align	  = dxb_align;
	dgdm_print_waits  = print_waits;
	dgdm_erase_nvm	  = erase_nvm;
	dgdm_rxbufsize	  = rxbufsize;
        dgdm_trcbuf_size  = trcbuf_size;

	dgdm_fillmem	  = fillmem;

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
	dgdm_latency	  = latency;
#endif

	dgdm_dxbttyname	  = dxbttyname;
	dgdm_usrlibdir	  = usrlibdir;

        for(i=0; i < MAXBOARDS; i++) {
                dgdm_Board[i] = NULL;
        }

#if ALLOW_ALT_FAIL_OPEN    
    dgdm_alt_fail_open = alt_fail_open;
#endif
}



/*
 * dgdm_scan()
 *
 * Scan for all boards
 *
 * Because of the way the PCI device list is scanned, we will always detect
 * boards in the order that they are listed in the Ids table defined near the
 * top of this file.
 *
 */
STATIC int dgdm_scan(void)
{
	int id;
	int rc;
        int i;

	if (!PCI_PRESENT())
		return -ENODEV;

	for (id = 0; id < NIDS; id++) {
		struct pci_dev *pdev;

		pdev = NULL;
		while ((pdev = pci_find_device(Ids[id].vendor,
					       Ids[id].device, pdev))) {

			rc = dgdm_found_board(pdev, id);
			if (rc == 0) {
                                dgdm_NumBoards++;
                        }
		}
	}

        APR(("found %d boards.\n", dgdm_NumBoards));

	/*
	 * Do the triboot process for the boards
	 */
	dgdm_triboot();

        for (i=0; i < dgdm_NumBoards ; i++) {
                unsigned long flags;

                if(!dgdm_Board[i])
                        continue;

                rc = dgdm_finalize_board_init(dgdm_Board[i]);
                DPR(("dgdm_scan(%d) - printing out the msgbuf\n", i));
                DGDM_LOCK(dgdm_global_lock, flags);
                dgdm_Board[i]->msgbuf = NULL;
                printk(dgdm_Board[i]->msgbuf_head);
                vfree(dgdm_Board[i]->msgbuf_head);
                dgdm_Board[i]->msgbuf_head = NULL;
                DGDM_UNLOCK(dgdm_global_lock, flags);
                if (rc) {
                        dgdm_cleanup_board(dgdm_Board[i]);
                        APR(("dgdm_finalize_board_init failed - %d\n", rc));
                }

        }


	if (!dgdm_NumBoards) {
                DPR(("dgdm_scan() - returning ENODEV\n"));
		return(-ENODEV);
	} else {
                DPR(("dgdm_scan() - done\n"));
		return(0);
        }

}



/*
 * dgdm_found_board()
 *
 * A board has been found, init it.
 */
STATIC int dgdm_found_board(struct pci_dev *pdev, int id)
{
        BOARD_T *brd;
	DGDM_PCI_IRQ_TYPE pci_irq;
	ulong addrs[NPCIADDRS];
	int i;
        unsigned long flags;

        /* get the board structure and prep it */
        DGDM_LOCK(dgdm_global_lock, flags);
        brd = dgdm_Board[dgdm_NumBoards] =
                (BOARD_T *) vmalloc(sizeof(BOARD_T));
        if(!brd) {
                DGDM_UNLOCK(dgdm_global_lock, flags);
                return(-ENOMEM);
        }
        memset(brd, 0, sizeof(BOARD_T));

        /* make a temporary message buffer for the boot messages */
        brd->msgbuf = brd->msgbuf_head =
                (char *) vmalloc(sizeof(char) * 8192);
        if(!brd->msgbuf) {
                vfree(brd);
                DGDM_UNLOCK(dgdm_global_lock, flags);
                return(-ENOMEM);
        }
        memset(brd->msgbuf_head, 0, sizeof(sizeof(char) * 8192));

        

	/* store the info for the board we've found */
	brd->boardnum = dgdm_NumBoards;
	brd->firstminor = dgdm_NumBoards * 64;
	brd->vendor = Ids[id].vendor;
	brd->device = Ids[id].device;
	brd->name = Ids[id].name;

	/* store which card & revision we have */
	pci_read_config_word(pdev, PCI_SUBSYSTEM_VENDOR_ID, &brd->subvendor);
	pci_read_config_word(pdev, PCI_SUBSYSTEM_ID, &brd->subdevice);
	pci_read_config_byte(pdev, PCI_REVISION_ID, &brd->rev);


#if LINUX_VERSION_CODE > KERNEL_VERSION(2,1,92)
	/* we can access this information directly after 2.1.92 */
	pci_irq = pdev->irq;
	brd->irq = pci_irq;

	/* get the PCI Base Address Registers */
	for (i = 0; i < NPCIADDRS; ++i) {
		addrs[i] = pci_resource_start(pdev, i);
		if (addrs[i] & 1)
			addrs[i] &= ~3;
		else
			addrs[i] &= ~15;
	}
#else
	/* we have to do things the hard way before 2.1.92 */
	pci_read_config_byte(pdev, PCI_INTERRUPT_LINE, &pci_irq);
	brd->irq = pci_irq;

	/* get the PCI Base Address Registers */
	for (i = 0; i < NPCIADDRS; ++i) {
		pci_read_config_dword(pdev, (PCI_BASE_ADDRESS_0 + (i * 4)),
				      (uint *) & addrs[i]);
		if (addrs[i] & 1)
			addrs[i] &= ~3;
		else
			addrs[i] &= ~15;
	}
#endif

	for (i = 0; i < NPCIADDRS; i++) {
		pci_addrs[dgdm_NumBoards].addrs[i].addr = addrs[i];
                pci_addrs[dgdm_NumBoards].addrs[i].type = TRIBL_RAWB;
                pci_addrs[dgdm_NumBoards].addrs[i].size = 0;
                pci_addrs[dgdm_NumBoards].addrs[i].mtag = 0;
	}


	/* Get the proper name for the Acceleport family since we can. */
	if ((brd->vendor == DIGI_VID) &&
	    (brd->device == DIGI_AP2K_DID)) {
		switch (brd->subdevice) {
		case DIGI_AP2K2_SSID:
			brd->name = DIGI_AP2K_NAME;
			break;
		case DIGI_AP2K4_SSID:
			brd->name = DIGI_AP4K_NAME;
			break;
		case DIGI_AP2K8_SSID:
			brd->name = DIGI_AP8K_NAME;
			break;
		case DIGI_AP2K16_SSID:
			brd->name = DIGI_AP16K_NAME;
			break;
#if defined(DIGI_AP2K4_MPI_SSID)	/* temp -- headers */
                case DIGI_AP2K32_SSID:
                        brd->name = DIGI_AP32K_NAME;
                        break;
                case DIGI_AP2K4_MPI_SSID:
                        brd->name = DIGI_AP4K_MPI_NAME;
                        break;
                case DIGI_AP2K8_MPI_SSID:
                        brd->name = DIGI_AP8K_MPI_NAME;
                        break;
                case DIGI_AP2K16_MPI_SSID:
                        brd->name = DIGI_AP16K_MPI_NAME;
                        break;
#endif          
        default:
            break;
		}
	}
 
        DGDM_UNLOCK(dgdm_global_lock, flags);

        /* Log the information about the board */
        dgdm_mbuf(brd, DRVSTR": board %d: %s (rev %d), irq %d, addrs:",
                  dgdm_NumBoards, brd->name, brd->rev, brd->irq);
        for (i = 0; i < NPCIADDRS; ++i) {
                if (!addrs[i])
                        continue;
                dgdm_mbuf(brd, " 0x%lx", addrs[i]);
        }
        dgdm_mbuf(brd, "\n");

        return(0);
}



STATIC int dgdm_finalize_board_init(BOARD_T *brd) {

        int i;
        int rc;

        DPR_TRIBL(("dgdm_finalize_board_init() - start\n"));

        if(!brd)
                return(-ENODEV);

        DPR_TRIBL(("dgdm_finalize_board_init() - start #2\n"));

	/*
	 * Snag the IRQ
	 */
	if (brd->irq) {
		rc = request_irq(brd->irq, dgdm_intr, SA_SHIRQ,
				 "Digi DBX2", brd);
		if (rc) {
			dgdm_mbuf(brd, DRVSTR": Failed to hook IRQ %d\n",
                                  brd->irq);
			brd->irq = 0;
			intr_period = 0;
			brd->intr_used = 0;
                        goto error;
		}
		if (brd->triboard && brd->triboard->unmask_board_int) {
			DPR_TRIBL(("dgdm_found_board(%d) - calling "
				   "unmask_board_int\n", brd->boardnum));
			(*brd->triboard->unmask_board_int) (brd->triboard);
		}
		if (polling)	/* do we do any interrupts at all? */
			brd->intr_used = 0;
		else
			brd->intr_used = 1;

		/* But, don't use interrupts on HLC for now */
		if (brd->device != DIGI_AP2K_DID)
			brd->intr_used = 0;

		DPR_TRIBL(("dgdm_found_board(%d) - %s\n", brd->boardnum,
                           brd->intr_used ? "using interrupts" : "polling"));

	} else {
		brd->irq = 0;
/*
		intr_period = 0;
*/
	}

	/*
	 * If gdb is on, turn off tty support, since the FEP
	 * will be stuck at a db_pause() and won't have completed
	 * tty initialization.
	 */
	if (dgdm_gdb)
		dgdm_tty = 0;

	/*
	 * Do tty device initialization (but not if gdb is set)
	 */
	if (dgdm_tty) {
		rc = dgdm_tty_brdinit(brd);
		if (rc)
			goto error;
	}
	/*
	 * Initialize the dxbisdn_chans[] to an initial state.
	 *
	 * DEPENDENCY:
	 *
	 * dgdm_tty_brdinit() does the DxB interface initialization.
	 */
	if (dgdm_tty && dgdm_isdn) {
		brd->dxbisdn_chans = dgdm_driver_kzmalloc(NISDN *
                                                          sizeof(DXBSTRM),
                                                          GFP_KERNEL);
		if (brd->dxbisdn_chans == NULL) {
			rc = -ENOMEM;
			goto error;
		}
	}


#if defined(DGDM_TRACER)
	/*
	 * Record the various vital board statistics in the tracebuf.
	 */
	DPR_BASIC(("Board %d: firstminor %x, vendor %x, device %x\n",
	      brd->boardnum, brd->firstminor, brd->vendor, brd->device));
	DPR_BASIC(("          subvendor %x, subdevice %x, rev %x\n",
		   brd->subvendor, brd->subdevice, brd->rev));
	DPR_BASIC(("          ram %x, ramlen %x, vram %x\n",
		   (uint)brd->ram, brd->ramlen, (uint)brd->vram));
	DPR_BASIC(("          reg %x, reglen %x, vreg %x\n",
		   (uint)brd->reg, (uint)brd->reglen, (uint)brd->vreg));
	DPR_BASIC(("          flash %x, flashlen %x, vflash %x\n",
		   (uint)brd->flash, (uint)brd->flashlen, (uint)brd->vflash));
	DPR_BASIC(("          flash_refcnt %x, flasharea %x, flash_ided %x\n",
		   brd->flash_refcnt, (uint)brd->flasharea, brd->flash_ided));
	DPR_BASIC(("          vpd %x, vpdlen %x, vvpd %x\n",
		   (uint)brd->vpd, (uint)brd->vpdlen, (uint)brd->vvpd));
	DPR_BASIC(("          vpd_refcnt %x, vpdarea %x\n",
		   brd->vpd_refcnt, (uint)brd->vpdarea));
	DPR_BASIC(("          dspgpp1 %x, vdspgpp1 %x\n",
		   (uint)brd->dspgpp1, (uint)brd->vdspgpp1));
	DPR_BASIC(("          irq %x, intr_count %x, intr_used %x\n",
		   brd->irq, brd->intr_count, brd->intr_used));
	DPR_BASIC(("          gco %x, gcp %x, gdd %x\n",
		   brd->gco, (uint)brd->gcp, (uint)brd->gdd));
	DPR_BASIC(("          dma_buf[0] %x, dma_buf[1] %x\n",
		   (uint)brd->dma_buf[0], (uint)brd->dma_buf[1]));
	DPR_BASIC(("          board_type %x, ndsps %x, nttychans %x\n",
		   brd->board_type, brd->ndsps, brd->nttychans));
	DPR_BASIC(("          nspecchans %x, nchans %x, dc %x\n",
		   (uint)brd->nspecchans, brd->nchans, (uint)brd->dc));
	DPR_BASIC(("          dxb %x, dxb_real %x, dxb_locked %x\n",
		   (uint)brd->dxb, (uint)brd->dxb_real, brd->dxb_locked));
	DPR_BASIC(("          dxb_needpoll %x, deferred_polls %x\n",
		   brd->dxb_needpoll, brd->deferred_polls));
	DPR_BASIC(("          ghd %x, gfd %x\n",
		   (uint)brd->ghd, (uint)brd->gfd));
#endif

	return (0);

      error:
	if (brd->dxbisdn_chans) {
		kfree(brd->dxbisdn_chans);
		brd->dxbisdn_chans = NULL;
	}
	if (dgdm_tty)
		dgdm_tty_brduninit(brd);
	if (brd->vram) {
		IOUNMAP(brd->vram);
		brd->vram = 0;
	}
	if (brd->vreg) {
		IOUNMAP(brd->vreg);
		brd->vreg = 0;
	}
	if (brd->irq)
		free_irq(brd->irq, brd);
	for (i = 0; i < 2; ++i)
		if (brd->dma_buf[i])
			free_page(brd->dma_buf[i]);

        dgdm_mbuf(brd, DRVSTR": dgdm_found_board - returning %d\n", rc);
	return (rc);
}



/*
 * dgdm_triboot()
 *
 * The main triboot interface.  This is responsible for spinning the
 * card up, downloading the boot, the firmware, etc.
 */
STATIC int dgdm_triboot(void)
{
        BOARD_T	*brd;
        int i;
        int done;
	int rc = -EINVAL;
	static int id = 1;
	TRIBL_BOARD *tribl;
	BOARD *triboard;


        for(i=0; i < dgdm_NumBoards; i++) {
                brd = dgdm_Board[i];
                if(!brd) { 
                        DPR_TRIBL(("dgdm_triboot(%d) - continue\n", i));
                        continue;
                }
                
                DPR_TRIBL(("dgdm_triboot(%d) - board at %x (%x)\n",
                           i, (uint)brd, (uint)dgdm_Board[i]));

                /* configure the triboot structure. */
                tribl = &brd->tribl;
                memset(tribl, 0, sizeof(TRIBL_BOARD));
                tribl->id = ++id;	/* must be unique */
                tribl->int_index = -1;	/* disable */
                tribl->pci_vendor = brd->vendor;
                tribl->pci_device = brd->device;
                tribl->tri_cb = (void *) dgdm_triboot_cb;
                tribl->pdata = &pci_addrs[i];
                tribl->count = 30;

                /* Set the state to register, and start things off */
                tribl->state = TRIBL_REGISTER;
                DPR_TRIBL(("dgdm_triboot(%d) - calling triboot library "
                           "to register\n", brd->boardnum));
                tri_boot(tribl);
                DPR_TRIBL(("dgdm_triboot(%d) - done registering\n",
                           brd->boardnum));
        }

        done = 0;
        while(done != ((1 << dgdm_NumBoards) - 1)) {

                for(i=0; i < dgdm_NumBoards; i++) {
                        brd = dgdm_Board[i];
                        if(!brd) {
                                DPR_TRIBL(("dgdm_triboot(%d) - continue #2\n",
                                           i));
                                continue;
                        }

                        tribl = &brd->tribl;
                        
                        if ((tribl->state != TRIBL_DONE) &&
                            (tribl->status == TRIBL_OK)) {
                                rc = dgdm_triboot_step(brd);
                        } else {
                                done |= (1 << i);
                        }
                }
        }


        for(i=0; i < dgdm_NumBoards; i++) {
                brd = dgdm_Board[i];
                if(!brd) continue;
                
                tribl = &brd->tribl;

                if (tribl->state != TRIBL_DONE || tribl->status != TRIBL_OK) {
                        unsigned long flags;

                        APR(("dgdm_triboot(%d) - Adapter download error!  We did not "
                                   "finish! state=0x%x, status=%d\n",
                                   brd->boardnum, tribl->state,
                                   tribl->status));
                        DGDM_LOCK(dgdm_global_lock, flags);
                        brd->msgbuf = NULL;
                        printk(brd->msgbuf_head);
                        vfree(brd->msgbuf_head);
                        brd->msgbuf_head = NULL;
                        DGDM_UNLOCK(dgdm_global_lock, flags);
                        brd->irq = 0;	/* haven't asked for it yet */
                        dgdm_cleanup_board(dgdm_Board[i]);
                        continue;
                }


                DPR_TRIBL(("dgdm_triboot(%d) - calling triboot to get "
                           "BOARD info.\n", brd->boardnum));
                tribl->state = TRIBL_BOARD_STRUCT;
                tri_boot(tribl);
                triboard = (BOARD *) tribl->pdata;
                dgdm_decode_board(brd, triboard);

		/* 
                 * Only get the post information if we're going to 
                 * download the FEP to the board.  The POST isn't
                 * important if we're in debugging mode.
                 */
                if (download) {
                        DPR_TRIBL(("dgdm_triboot(%d) - calling triboot to get "
                                   "POST info.\n", brd->boardnum));
                        tribl->state = TRIBL_POST_DATA;
                        tri_boot(tribl);
                        brd->post = tribl->pdata;
                        rc = dgdm_decode_post(brd);
                }

                DPR_TRIBL(("dgdm_triboot(%d) - triboot complete - returning\n",
                           brd->boardnum));
	}

        DPR(("dgdm_triboot - done\n"));
	return (rc);
}



STATIC int dgdm_triboot_step(BOARD_T *brd) {

        TRIBL_BOARD *tribl;
        int rc;

        if(!brd)
                return(-ENODEV);

        tribl = &brd->tribl;

        DPR_TRIBL(("dgdm_triboot_step(%d)\n", brd->boardnum));

        switch (tribl->state) {
        case TRIBL_MAP_ADDRS:
                rc = dgdm_tribl_map_addrs(brd);
                if (rc) {
                        APR(("dgdm_triboot_step(%d) - TRIBL_MAP_ADDRS"
                                   " failed, returning %d\n",
                                   brd->boardnum, rc));
                        return (rc);
                }
                tribl->state = TRIBL_MADDRS_DONE;
                break;

        case TRIBL_DXBINIT:
                /*
                 * I don't really like doing this because it
                 * seems messy to me to just short circuit the
                 * triboot process but I want this done in one
                 * place with just one method so we do this
                 * place with just one method so we do this
                 * rather than using the previous mapping
                 * functions.
                 */
                if (!download) {
                        tribl->state = TRIBL_DONE;
                        dgdm_mbuf(brd, DRVSTR": Not downloading FEP -- "
                                  "in debugging mode.\n");
                        break;
                }
                rc = dgdm_tribl_dxbinit(brd);
                if (rc) {
                        APR(("dgdm_triboot_step(%d) - TRIBL_DXBINIT "
                                   "failed, returning %d\n",
                                   brd->boardnum, rc));
                        return (rc);
                }
                break;

        case TRIBL_FW_BOOT:
        case TRIBL_FW_POST:
        case TRIBL_FW_FEP:
                rc = dgdm_tribl_fwload(brd);
                if (rc) {
                        APR(("dgdm_triboot_step(%d) - TRIBL_FW "
                                   "failed, returning %d\n",
                                   brd->boardnum, rc));
                        return (rc);
                }
                break;

        case TRIBL_WAIT:
                DPR_TRIBL(("dgdm_triboot_step(%d) - TRIBL_WAIT %u ms\n",
                           brd->boardnum, tribl->count));
                dgdm_sleep(tribl->count / 10);
                tribl->count = 0;
                tribl->state = TRIBL_WAIT_DONE;
                break;

        case TRIBL_UNMAP_ADDRS:
        default:
                /* should never happen in init */
                dgdm_mbuf(brd, DRVSTR": dgdm_triboot_step(%d) - triboot "
                          "library returned an invalid state %d (status %d)\n",
                          brd->boardnum, tribl->state, tribl->status);
                return (1);
        }

        DPR_TRIBL(("dgdm_triboot_step(%d) - calling tri_boot()\n",
                   brd->boardnum));
        tri_boot(tribl);
        return(0);
}






/*
 * dgdm_tribl_map_addrs()
 *
 * The triboot libraray will pass us a pointer to NPCIADDRS number of 
 * PADDRS_T structures.  We will iomap() them, pass the mapped value back
 * in the same structure, and store them for ourselves as appropriate.
 */
STATIC int dgdm_tribl_map_addrs(BOARD_T *brd)
{
	TRIBL_BOARD *tribl = &brd->tribl;
	PADDRS_T *paddr;
	int i;
	int length;
	int address;
	void *vaddress;

	DPR_TRIBL(("dgdm_tribl_map_addrs(%d)\n", brd->boardnum));

	paddr = (PADDRS_T *) tribl->pdata;

	if (paddr == NULL) {
		DPR_TRIBL(("dgdm_tribl_map_addrs(%d) - critical error!\n",
			   brd->boardnum));
		return (1);
	}
	DPR_TRIBL(("dgdm_tribl_map_addrs(%d) - addrs:\n", brd->boardnum));
	for (i = 0; i < NPCIADDRS; i++) {

		DPR_TRIBL(("      in BAR %d: addr %x, size %x, type %x, "
			   "mtag %x\n", i, paddr->addrs[i].addr,
			   paddr->addrs[i].size, paddr->addrs[i].type,
			   paddr->addrs[i].mtag));

		if ((paddr->addrs[i].addr != 0) &&
		    (paddr->addrs[i].size != 0)) {
			paddr->addrs[i].mtag = 1;
			switch (paddr->addrs[i].type) {
			case TRIBL_MEM:
				brd->ram = paddr->addrs[i].addr;
				brd->ramlen = paddr->addrs[i].size;
				brd->vram = IOREMAP(brd->ram, brd->ramlen);
				if (!brd->vram) {
					DPR_TRIBL(("ERROR MAPPING MEMORY!\n"));
					return (-ENXIO);
				}
				paddr->addrs[i].addr = (uint32) brd->vram;
				break;
			case TRIBL_REGS:
				brd->reg = paddr->addrs[i].addr;
				brd->reglen = paddr->addrs[i].size;
				brd->vreg = IOREMAP(brd->reg, brd->reglen);
				if (!brd->vreg) {
					DPR_TRIBL(("ERROR MAPPING REGS!\n"));
					return (-ENXIO);
				}
				paddr->addrs[i].addr = (uint32) brd->vreg;
				break;
			case TRIBL_FLASH:
				brd->flash = paddr->addrs[i].addr;
				brd->flashlen = paddr->addrs[i].size;
				brd->vflash = IOREMAP(brd->flash,
						      brd->flashlen);
				if (!brd->vflash) {
					DPR_TRIBL(("ERROR MAPPING FLASH!\n"));
					return (-ENXIO);
				}
				paddr->addrs[i].addr = (uint32) brd->vflash;
				break;
			default:
				address = paddr->addrs[i].addr;
				length = paddr->addrs[i].size;
				vaddress = IOREMAP(address, length);
				if (!vaddress) {
					DPR_TRIBL(("ERROR MAPPING "
						   "DEFAULT!\n"));
					return (-ENXIO);
				}
				paddr->addrs[i].addr = (uint32) vaddress;
				break;
			}
		} else {
			paddr->addrs[i].mtag = 0;
		}
		DPR_TRIBL(("     out BAR %d: addr %x, size %x, type %x, "
			   "mtag %x\n", i, paddr->addrs[i].addr,
			   paddr->addrs[i].size, paddr->addrs[i].type,
			   paddr->addrs[i].mtag));
	}

	DPR_TRIBL(("dgdm_tribl_map_addrs(%d) - done\n", brd->boardnum));
	return (0);
}



/*
 * dgdm_tribl_dxbinit()
 *
 * Initialize the dxb library and do all of the other pre-initialization
 * needed before we bring the boards up.
 */
STATIC int dgdm_tribl_dxbinit(BOARD_T *brd)
{
	TRIBL_BOARD *tribl = &brd->tribl;

	DPR_TRIBL(("dgdm_tribl_dxbinit(%d)\n", brd->boardnum));

	if (dgdm_tty_preinit(brd)) {
		DPR_TRIBL(("dgdm_tribl_dxbinit(%d) - dxblib_init() failed\n",
			   brd->boardnum));
		return (1);
	}
	if (brd->ghd) {
		tribl->pdata = (void *) dxb_virt2bus(brd->ghd);
		dgdm_sleep(HZ / 2);
	} else {
		DPR_TRIBL(("dgdm_tribl_dxbinit(%d) - null ghd pointer\n",
			   brd->boardnum));
		return (1);
	}
	return (0);
}



/*
 * dgdm_tribl_fwload()
 *
 * Handles the file requests from the triboot library.
 *
 * NOTE: this routine is not (at this time) re-entrant.  At some point, we
 *       may decide to allow multiple boards to be accessing this
 *       routine at one time so that mutliple boards can be running POST
 *       in parallel.
 */
STATIC int dgdm_tribl_fwload(BOARD_T *brd)
{
	static char *buf[MAXBOARDS];
	static int open[MAXBOARDS];
	static struct file file[MAXBOARDS];
	static struct file *fp[MAXBOARDS];
        static int initd = FALSE;
	TRIBL_BOARD *tribl;
	char filename[256];
        int i;

        if(!initd) {
                for(i=0; i < MAXBOARDS; i++) {
                        buf[i] = NULL;
                        open[i] = FALSE;
                        memset(&file[i], 0, sizeof(struct file));
                }
                initd = TRUE;
        }
        

        if(!brd)
                return(-ENODEV);

        tribl = &brd->tribl;
        i = brd->boardnum;

	DPR_TRIBL(("dgdm_tribl_fwload(%d) - file_name=%s, tribl->sequnce=%d\n",
		   brd->boardnum, tribl->file_name, tribl->sequence));

	if (tribl->sequence == 0 && tribl->file_name[0] != '\0') {
		sprintf(filename, "%s/%s", usrlibdir, tribl->file_name);

		DPR_TRIBL(("dgdm_tribl_fwload(%d) - full filename '%s'\n",
			   brd->boardnum, filename));

		fp[i] = dgdm_file_open(&file[i], filename, 0, 0);
		if (!fp[i]) {
			dgdm_mbuf(brd, DRVSTR": unable to open firmware "
                                  "file '%s'\n", filename);
			tribl->pdata = (uint32) NULL;
			tribl->size = 0;
			tribl->sequence++;
			return (1);
		}
		DPR_TRIBL(("dgdm_tribl_fwload(%d) - %s opened successfully\n",
			   brd->boardnum, filename));

		buf[i] = dgdm_driver_kzmalloc(4096, GFP_KERNEL);
		if (!buf) {
			dgdm_mbuf(brd, DRVSTR": unable to allocate memory for "
                                  "file download\n");
			tribl->pdata = (uint32) NULL;
			tribl->size = 0;
			tribl->sequence++;
			return (-ENOMEM);
		}
		DPR_TRIBL(("dgdm_tribl_fwload(%d) - buffer allocated "
                           "at 0x%x\n", brd->boardnum, (uint)buf[i]));

		open[i] = TRUE;
	}
	if (open[i]) {
		tribl->size = dgdm_file_read(&file[i], buf[i], 4096);
		tribl->pdata = buf[i];

		if (tribl->size == 0) {
			/* no data left?  close the file. */
			DPR_TRIBL(("dgdm_tribl_fwload(%d) - no data, "
				   "closing\n", brd->boardnum));
			dgdm_file_close(fp[i]);
			DPR_TRIBL(("dgdm_tribl_fwload(%d) - freeing buf\n",
				   brd->boardnum));
			kfree(buf[i]);
			open[i] = FALSE;
		}
	} else {
		/* be sure to tell the library there's no data left. */
		DPR_TRIBL(("dgdm_tribl_fwload(%d) - file closed\n",
			   brd->boardnum));
		tribl->pdata = (uint32) NULL;
		tribl->size = 0;
	}

	tribl->sequence++;
	return (0);
}



/*
 * dgdm_triboot_cb()
 *
 * This routine is the required callback for the triboot library.  DELAY
 * is the only function used if the memory is all mapped properly.
 *
 */
STATIC void dgdm_triboot_cb(uint p0, uint *p1, uint *p2, uint *p3, uint *p4)
{
	int ticks;
	int rc = 0;

	switch (p0) {
	case DELAY:
		ticks = MIN(*p1, (5 * HZ));	/* never more than 5 seconds */
		DPR_TRIBL(("dgdm_triboot_cb - delaying %u\n", ticks));
		dgdm_sleep(ticks);
		break;
	case MAPP2V:
		DPR_TRIBL(("dgdm_triboot_cb - remapping 0x%x, 0x%x\n",
			   *p1, *p2));
		rc = (uint32) IOREMAP((uint32) * p1, *p2);
		if (!rc) {
			DPR_TRIBL(("dgdm_triboot_cb - remapping failed!\n"));
		} else {
			*p1 = rc;
		}
		break;
	case UNMAP:
		DPR_TRIBL(("dgdm_triboot_cb - unmapping 0x%x\n", *p1));
		IOUNMAP((caddr_t) * p1);
		break;
	default:
		DPR_TRIBL(("dgdm_triboot_cb - unexpected callback %d\n", p0));
		break;
	}

	return;
}



/*
 * dgdm_decode_board()
 *
 * Get the information from the triboot library's BOARD structure
 */
STATIC void dgdm_decode_board(BOARD_T *brd, BOARD *triboard)
{
	brd->triboard = triboard;

	/* this will also set brd->gco */
	brd->gcp = dgdm_get_gc_ptr(brd);

	/*
	 * Get pointer to debug area and allocate two DMA buffers
	 * for testing purposes.
	 */
	brd->gdd = (GLOBAL_DEBUG_DATA *) (brd->vram + GLOBAL_DEBUG_OFFSET);
	brd->gdd->dma_buf_len = 4096;
	brd->gdd->dma_buf[0] = brd->dma_buf[0] = get_free_page(GFP_KERNEL);
	brd->gdd->dma_buf[1] = brd->dma_buf[1] = get_free_page(GFP_KERNEL);

	if (brd->flash) {
                int i;
                
                /* We should only have revision 10 boards or newer now */
                if (brd->rev >= 10) {
                        brd->flasharea = Flash10;
                }

                /* Compute sizes of each main flash segment */
                for (i = 0; i < FLASH_AREAS; ++i) {
                        char    *b, *s, *e;
 
                        s = strchr(b = brd->flasharea[i].secmap, '1');
                        if (!s)
                                s = strchr(brd->flasharea[i].secmap, 0);
                        e = strchr(s, 'e');
                        if (!e)
                                e = strchr(s, 0);
                        if (brd->rev >= 10) {
                                brd->flasharea[i].woffset = (s-b) * 8192 / 2;
                                brd->flasharea[i].wlen = (e-s) * 8192 / 2;
                        }
                        DPR_BASIC(("%d:    %ld %ld\n", i, 
                                   brd->flasharea[i].woffset,
                                   brd->flasharea[i].wlen));
                }

                /*
                 *  Compute sizes of each VPD flash segment. They are byte-wide
                 */
                brd->vpdarea = FlashVPD;
                for (i = 0; i < VPD_AREAS; ++i) {
                        char    *b, *s, *e;
 
                        s = strchr(b = brd->vpdarea[i].secmap, '1');
                        if (!s)
                                s = strchr(brd->vpdarea[i].secmap, 0);
                        e = strchr(s, 'e');
                        if (!e)
                                e = strchr(s, 0);
                        brd->vpdarea[i].woffset = (s-b) * 8192;
                        brd->vpdarea[i].wlen = (e-s) * 8192;
 
                        DPR_BASIC(("%d:    %ld %ld\n", i,
                                   brd->vpdarea[i].woffset,
                                   brd->vpdarea[i].wlen));
                }
                
		brd->dspgpp1 = brd->flash + DSPGPP1_OFFSET;
		brd->vdspgpp1 = IOREMAP(brd->dspgpp1, 0x40000);	/* TODO */
	}
}



/*
 * dgdm_decode_post()
 *
 * Get the information from the POST and interpret it as needed.
 *
 */
STATIC int dgdm_decode_post(BOARD_T *brd)
{
        TRIBL_POST *trpost;
	GLOBAL_POST_DATA *post;
	POST_SUMMARY *summary;
        POST_ERROR_REPORT *ep;
	int error_count, info_count;
	char cfg_str[32];
        int i;
	uint8	geoaddr, *geoaddrp;

	DPR_BASIC(("dgdm_decode_post(%d)\n", brd->boardnum));

	if (!brd || !brd->post) {
		DPR_BASIC(("dgdm_decode_post(%d) - null brd or post\n",
			   brd->boardnum));
		return (1);
	}

        trpost = brd->post;
	post = &trpost->pst;
	summary = &post->summary;
        ep = &trpost->err[0];

	brd->board_type = post->summary.board_type;
	brd->memsize = post->summary.memsize;
	brd->nttychans = post->summary.nchan;

	/* transparent print - if enabled, we have the same number as tty */
	if (print) {
		brd->nxpchans = brd->nttychans;
	} else {
		brd->nxpchans = 0;
	}


	switch (brd->board_type) {
	case POST_TYPE_LONDON_U:
	case POST_TYPE_LONDON_ST:
		brd->ndsps = brd->nttychans + 2; /* D-channel controllers */
		break;
	default:
		brd->ndsps = brd->nttychans;
		break;
	}

	/* The number of special should really come from the FEP.  TODO */
	brd->nspecchans = 4;
	brd->nchans = brd->nttychans + brd->nspecchans + brd->nxpchans;

	/*
	 * build a descriptive message about the board configuration
	 * since we can't just deduce it from the PCI DevID
	 */
	switch (brd->board_type) {
	case POST_TYPE_HOUSTON_T1:
	case POST_TYPE_CAMPINAS_T1:
		sprintf(cfg_str, "T1/%d", brd->nttychans);
		break;
	case POST_TYPE_HOUSTON_E1:
	case POST_TYPE_CAMPINAS_E1:
		sprintf(cfg_str, "E1/%d", brd->nttychans);
		break;
	case POST_TYPE_LONDON_U:
		strcpy(cfg_str, "U-Interface");
		break;
	case POST_TYPE_LONDON_ST:
		strcpy(cfg_str, "ST-Interface");
		break;
	case POST_TYPE_CHASKA:
		/*
		 * on chaska, the DAA type is encoded in the
		 * subsystem device ID, bits 3:1
		 */
		sprintf(cfg_str, "%d port, DAA type %d",
			brd->nttychans, ((brd->subdevice >> 1) & 7));
		break;
#if !defined(DIGI_AP2K4_MPI_SSID)	/* temp -- headers */
	case POST_TYPE_AP2K2_232:
		strcpy(cfg_str, "2 ports");
		break;
	case POST_TYPE_AP2K4_232:
		strcpy(cfg_str, "4 ports");
		break;
	case POST_TYPE_AP2K8_232:
		strcpy(cfg_str, "8 ports");
		break;
	case POST_TYPE_AP2K16_232:
		strcpy(cfg_str, "16 ports");
		break;
#else
	case POST_TYPE_AP2K2_232:
		strcpy(cfg_str, "2 ports");
		break;
	case POST_TYPE_AP2K4_232:
        case POST_TYPE_AP2K4_MPI:
		strcpy(cfg_str, "4 ports");
		break;
	case POST_TYPE_AP2K8_232:
        case POST_TYPE_AP2K8_MPI:
		strcpy(cfg_str, "8 ports");
		break;
	case POST_TYPE_AP2K16_232:
        case POST_TYPE_AP2K16_MPI:
		strcpy(cfg_str, "16 ports");
		break;
        case POST_TYPE_AP2K32_232:
                strcpy(cfg_str, "32 ports");
                break;
#endif
	default:
		sprintf(cfg_str, "unknown (%x)",
			brd->board_type);
		break;
	}
	dgdm_mbuf(brd, DRVSTR":   Configuration: %s, operating %s-endian.\n",
                  cfg_str, brd->bigendian_board ? "big" : "little");

    /*
     * for cPCI Houston (aka Campinas) go get the Geographic Address
     * and stick it in the info struct.  The Right Way to Do It(tm)
     * would be to snarf the INFO record from the POST which contains
     * the Geographic Address...  but, that requires a POST upgrade
     * on some older boards which we can't ensure.  So, we will go
     * directly to the GPP register on the board and read out the
     * the bits.
     */
    switch (brd->board_type) {
    case POST_TYPE_CAMPINAS_T1:
    case POST_TYPE_CAMPINAS_E1:
        geoaddrp = (uint8 *)brd->vdspgpp1;
        geoaddrp += 0x415;
        geoaddr = *geoaddrp;
        geoaddr &= 0x1f;
        brd->geoaddr = geoaddr;
        break;
    default:
        brd->geoaddr = -1;
        break;
    }

	/* count error and info records in the post summary list */
	error_count = 0;
	info_count = 0;


	/*
	 * Decode the post error list and info list.  Print if we have
         * any errors or info to report.
	 */
        DPR(("  post %x err %x\n", (uint)trpost, (uint)ep));
        for (i = 0; i < POST_MAX_ENTRIES; i++, ep++) {
                if (LE16(ep->identifier) == POST_ERROR_EOL)
                        break;
                if (LE16(ep->identifier) >= POST_ERROR_NOP + 256)
                        error_count++;
                else
                        info_count++;
        }

	/*
	 * first display the errors...
	 */
	if (error_count) {
		dgdm_mbuf(brd, DRVSTR":   POST detected %d error%s\n",
                          error_count, (error_count == 1) ? "" : "s");

                /* start at the head of the list */
		ep = &trpost->err[0];
		for (i = 0; i < POST_MAX_ENTRIES; i++, ep++) {
                        if (LE16(ep->identifier) == POST_ERROR_EOL)
				break;
			if (LE16(ep->identifier) >= POST_ERROR_NOP + 256)
				dgdm_mbuf(brd, DRVSTR":     Error %04X: %04X "
                                          "%08X %08X %08X\n",
                                          LE16(ep->identifier),
                                          LE16(ep->ordinal),
                                          LE32(ep->syndrome[0]),
                                          LE32(ep->syndrome[1]),
                                          LE32(ep->syndrome[2]));
		}
	}
	/*
	 * then display the info records...
	 */
	if (info_count) {
		dgdm_mbuf(brd, DRVSTR":   POST issued %d info record%s\n",
                          info_count, (info_count == 1) ? "" : "s");

                /* start at the head of the list */
		ep = &trpost->err[0];
		for (i = 0; i < POST_MAX_ENTRIES; i++, ep++) {
                        if (LE16(ep->identifier) == POST_ERROR_EOL)
				break;
			if (LE16(ep->identifier) < POST_ERROR_NOP + 256)
				dgdm_mbuf(brd, DRVSTR":     Info  %04X: %04X "
                                          "%08X %08X %08X\n",
                                          LE16(ep->identifier),
                                          LE16(ep->ordinal),
                                          LE32(ep->syndrome[0]),
                                          LE32(ep->syndrome[1]),
                                          LE32(ep->syndrome[2]));
		}
	}
	if (error_count)
		return (ENXIO);

	return (0);
}



/*
 * dgdm_get_gc_ptr()
 *
 * Get a pointer to the current global common structure.
 * If pointer is not yet valid, return NULL.
 */
GLOBAL GLOBAL_COMMON *dgdm_get_gc_ptr(BOARD_T *brd)
{
	uint32 gco;
	GLOBAL_COMMON *gcp;

	brd->gco = gco = LE32(*(uint32 *) (brd->vram + GLOBAL_DATA_OFFSET));
	if (gco == 0 || gco >= brd->ramlen)
		return (NULL);	/* Offset is not yet valid */

	gcp = (GLOBAL_COMMON *) (brd->vram + gco);
	DPR_BASIC(("Board Vram=0x%x, Global Common Pointer=0x%x\n",
		   (uint)brd->vram, (uint)gcp));
	return (gcp);
}





/************************************************************************
 *
 * interrupt handler
 *
 ************************************************************************/

/*
 * dgdm_intr()
 *
 * Driver interrupt handler.
 */
STATIC void dgdm_intr(int irq, void *voidbrd, struct pt_regs *regs)
{
	BOARD_T *brd = voidbrd;

        if(!brd) {
                APR(("Received interrupt with null board associated\n"));
                return;
        }

	++brd->intr_count;

	if (brd->triboard && brd->triboard->clear_board_int) {
                if(((*brd->triboard->clear_board_int) (brd->triboard)) &&
                   brd->intr_used) {
                        if(dxb_lock_poller(brd)) {
                                /* poll only receive here */
                                dgdm_tty_poll_board(brd, FALSE, TRUE);
                                dxb_unlock_poller(brd);
                        }
                }
	} else {
		APR(("Board received an interrupt but there is no way to "
                     "to clear it.\nDisabling interrupts for that board\n"));
		free_irq(brd->irq, brd);
	}
}





/************************************************************************
 *
 * open/close/read/write/ioctl for ram/control/debug devices
 *
 ************************************************************************/

/*
 * dgdm_boardopen()
 *
 * Open the ram/control/debug device
 */
STATIC int dgdm_boardopen(struct inode *inode, struct file *file)
{
	unsigned int minor = MINOR(inode->i_rdev);
	int board = minor2board(minor);
	int type = minor2type(minor);
	int subtype = minor2subtype(minor);
	BOARD_T *brd;
	int rc;

	DPR(("dgdm_boardopen()\n"));

        /* if we're using the 2nd major, it's one of the high 4 boards */
        if (MAJOR(inode->i_rdev) == major + DGDM_CONTROL2_MAJOR)
                board += 4;

        brd = dgdm_Board[board];

        if(!brd)
                return(-ENODEV);

	if (board >= dgdm_NumBoards)
		return -ENXIO;

	switch (type) {
	case TYPE_RAM:
		if (subtype >= SUBTYPE_NRAM)
			return -ENXIO;	/* Not implemented */
		break;
	case TYPE_FLASH:
		if (subtype >= SUBTYPE_NFLASH)
			return -ENXIO;	/* Illegal */
		if (brd->flash_refcnt == 0) {
			brd->vflash = IOREMAP(brd->flash, brd->flashlen);
			if (!brd->vflash) {
				APR(("dgdm_boardopen(%d) - cannot map "
                                     "flash memory\n", brd->boardnum));
				return (-ENXIO);
			}
			brd->flash_refcnt++;
		}
		break;
	case TYPE_VPD:
		if (subtype >= SUBTYPE_NVPD)
			return -ENXIO;	/* Illegal */
		if (brd->vpd_refcnt == 0) {
			brd->vvpd = IOREMAP(brd->vpd, brd->vpdlen);
			if (!brd->vvpd) {
				APR(("dgdm_boardopen(%d) - cannot map "
                                     "vpd memory\n", brd->boardnum));
				return (-ENXIO);
			}
			brd->vpd_refcnt++;
		}
		break;
	case TYPE_REG:
		if (subtype != SUBTYPE_NONE)
			return -ENXIO;	/* Illegal */
		break;
	case TYPE_IO:
		switch (subtype) {
		case SUBTYPE_PRINT:
			brd->gdd->print_waits = 1;
			break;
		case SUBTYPE_A2H:
		case SUBTYPE_H2A:
			break;
		case SUBTYPE_AUDIO:
			rc = dgdm_audiotap_open(brd);
			if (rc)
				return (rc);
			break;
		case SUBTYPE_MGMT:
			rc = dgdm_mgmt_open(brd);
			if (rc)
				return (rc);
			break;
		case SUBTYPE_EQM:
			rc = dgdm_eqm_open(brd);
			if (rc)
				return (rc);
			break;
		default:
			return -ENXIO;	/* Illegal */
		}
		break;
	case TYPE_DMABUF:
		if (subtype > 1)
			return -EINVAL;
		break;
	default:
		return -ENXIO;	/* Not implemented */
		break;
	}

	MOD_INC_USE_COUNT;

	return (0);
}



/*
 * dgdm_boardclose()
 *
 * Close the ram/control/debug device
 */
STATIC CLOSE_RETURN_T dgdm_boardclose(struct inode *inode, struct file *file)
{
	unsigned int minor = MINOR(inode->i_rdev);
	int board = minor2board(minor);
	int type = minor2type(minor);
	int subtype = minor2subtype(minor);
	BOARD_T *brd = dgdm_Board[board];

	DPR(("dgdm_boardclose()\n"));

	switch (type) {
	case TYPE_FLASH:
		if (--brd->flash_refcnt == 0) {
			IOUNMAP(brd->vflash);
			brd->vflash = NULL;
		}
		break;
	case TYPE_VPD:
		if (--brd->vpd_refcnt == 0) {
			IOUNMAP(brd->vvpd);
			brd->vvpd = NULL;
		}
		break;
	case TYPE_IO:
		switch (subtype) {
		case SUBTYPE_PRINT:
			brd->gdd->print_waits = print_waits;
			break;
		case SUBTYPE_AUDIO:
			dgdm_audiotap_close(brd);
			break;
		case SUBTYPE_MGMT:
			dgdm_mgmt_close(brd);
			break;
		case SUBTYPE_EQM:
			dgdm_eqm_close(brd);
			break;
		}
		break;
	}

	MOD_DEC_USE_COUNT;

	CLOSE_RETURN(0);
}




/*
 * dgdm_boardread()
 *
 * Read from the ram/control/debug device
 */
STATIC READ_RETURN_T dgdm_boardread(READ_PROTO)
{
	unsigned int minor = MINOR(PARM_I_RDEV);
	int board = minor2board(minor);
	int type = minor2type(minor);
	int subtype = minor2subtype(minor);
	BOARD_T *brd = dgdm_Board[board];
	unsigned long pos = GET_POS();
	void *virt;
	ulong len;

	if (count < 0)
		return (-EINVAL);

	switch (type) {
	case TYPE_RAM:
		switch (subtype) {
		case SUBTYPE_GDB:
			return (dgdm_debugread(brd, READ_ARGS));
		case SUBTYPE_DSPDM:
			return (dgdm_dspread(brd, RW_DM, READ_ARGS));
		case SUBTYPE_DSPPM:
			return (dgdm_dspread(brd, RW_PM, READ_ARGS));
		default:
			virt = brd->vram;
			len = brd->ramlen;
			break;
		}
		break;
	case TYPE_REG:
		virt = brd->vreg;
		len = brd->reglen;
		break;
	case TYPE_FLASH:
		virt = ((ushort *) brd->vflash)
		    + brd->flasharea[subtype].woffset;
		len = 2 * brd->flasharea[subtype].wlen;
		break;
	case TYPE_VPD:
		virt = brd->vvpd + brd->vpdarea[subtype].woffset;
		len = brd->vpdarea[subtype].wlen;
		break;
	case TYPE_DMABUF:
		if (subtype > 1)
			return -EINVAL;
		virt = (uchar *) brd->dma_buf[subtype];
		len = 4096;
		break;

	case TYPE_IO:
		switch (subtype) {
		case SUBTYPE_PRINT:
			return (dgdm_printread(brd, READ_ARGS));
		case SUBTYPE_H2A:
			return (dgdm_dxbread(&brd->dxb->h2a, READ_ARGS));
		case SUBTYPE_A2H:
			return (dgdm_dxbread(&brd->dxb->a2h, READ_ARGS));
		case SUBTYPE_AUDIO:
			return (dgdm_audiotap_read(brd, READ_ARGS));
		case SUBTYPE_MGMT:
			return (dgdm_mgmt_read(brd, READ_ARGS));
		case SUBTYPE_EQM:
			return (dgdm_eqm_read(brd, READ_ARGS));
		default:
			return (-EINVAL);
		}

	default:
		return (-EINVAL);
	}
	if (count > (len - pos))
		count = len - pos;

	COPY_TO_USER(buf, virt + pos, count);

	ADD_POS(count);

	return (count);
}



/*
 * dgdm_debugread()
 *
 * Read from the debug device.  Called from dgdm_boardread().
 *
 * This routine and dgdm_debugwrite() communicate with the gdb
 * stub on an embedded target  by means of circular queues in
 * board memory.  Currently, the addresses of these queues are
 * hardwired at CQ_RESP and CQ_CMD, which are defined in dgdm_driver.h
 */
STATIC int dgdm_debugread(BOARD_T *brd, READ_PROTO)
{
	cq_t *qp = (cq_t *) ((uchar *) (brd->vram) + CQ_RESP), q;
	int resid = 0, avail;
	unsigned char *p;

	do {
		q.cq_base = RE32(qp->cq_base);
		q.cq_in = RE32(qp->cq_in);
		q.cq_out = RE32(qp->cq_out);
		q.cq_size = RE32(qp->cq_size);

		/*
		 * Sanity check the queue structure.
		 */
		if ((q.cq_base < 0x100) || (q.cq_base >= 0x400000) ||
		    (q.cq_size < 16) || (q.cq_size > 1024) ||
		    (q.cq_in >= q.cq_size) ||
		    (q.cq_out >= q.cq_size)) {
			APR(("Preposterous queue structure.\n"));
			return (-ENXIO);
		}
		/*
		 * Trim count to fit the available data.
		 */
		if ((avail = (q.cq_in - q.cq_out)) < 0)
			avail += q.cq_size;

		if (avail == 0) {
			if (dgdm_sleep(HZ / 10))
				return (-EINTR);
		}
	} while (avail == 0);

	if (count > avail)
		count = avail;

	if (count >= (q.cq_size - q.cq_out)) {
		resid = MIN(q.cq_size - q.cq_out, count);
		p = brd->vram + q.cq_base + q.cq_out;

		COPY_TO_USER(buf, p, resid);

		count -= resid;
		buf += resid;
		q.cq_out = 0;
	}
	if (count > 0) {
		p = brd->vram + q.cq_base + q.cq_out;

		COPY_TO_USER(buf, p, count);

		q.cq_out += count;
		resid += count;
	}
	qp->cq_out = RE32(q.cq_out);

	ADD_POS(resid);

	return (resid);
}



/*
 * dgdm_printread()
 *
 * Read the debugging print circular buffer
 */
STATIC int dgdm_printread(BOARD_T *brd, READ_PROTO)
{
	ulong len;
	ulong i;
	GLOBAL_DEBUG_DATA *gdd = brd->gdd;
	ulong head = gdd->print_head;

	/*
	 * "block" until data is available.
	 */
	for (;;) {
		len = (gdd->print_tail - head) & gdd->print_mask;
		len = MIN(count, len);
		if (len)
			break;
		if (dgdm_sleep(1))
			return (-EINTR);
	}

	/*
	 * Copy the data to the user.  This is an inefficent loop,
	 * but I don't want to pay a lot for this muffler.
	 */
	for (i = 0; i < len; ++i) {
		COPY_TO_USER(buf + i, (char *) gdd->print_buf + head, 1);
		head = ++head & gdd->print_mask;
	}

	gdd->print_head = head;

	ADD_POS(count);

	return (len);
}



/*
 * dgdm_dspread()
 *
 * Read from a DSP
 */
STATIC int dgdm_dspread(BOARD_T *brd, int pmdm, READ_PROTO)
{
	int ndsp = brd->ndsps ? brd->ndsps : 30;
	int pos = GET_POS();
	int i;
	int mask = sizeof(brd->rwcache.data) - 1;

	if (pmdm == RW_DM)
		count = MIN(count, ndsp * 2 * 32768 - pos);
	else
		count = MIN(count, ndsp * 4 * 32768 - pos);

	if (count <= 0)
		return (0);

	/*
	 * Copying data a byte at a time to user space looks
	 * hideously inefficient, and it is.  However, compared
	 * to the bandwidth of the RW interface, it gets lost
	 * in the noise.  Hence we keep this simple.
	 */
	for (i = 0; i < count; ++i) {
		int rc;

		rc = dgdm_dspcache(brd, pos, pmdm);
		if (rc) {
			SET_POS(pos);
			return (rc);
		}
		COPY_TO_USER(buf, brd->rwcache.data + (pos & mask), 1);
		++buf;
		++pos;
	}

	SET_POS(pos);

	return (count);
}



/*
 * dgdm_dspcache()
 *
 * Given a file postion and a DSP memory space indicator,
 * insure that the given DSP location is in the cache
 */
STATIC int dgdm_dspcache(BOARD_T *brd, uint pos, int pmdm)
{
	uint cacheline;
	uint dspnum;
	int linesize = sizeof(brd->rwcache.data);

	if (pmdm == RW_DM) {
		dspnum = pos / (2 * 32 * 1024);
		cacheline = (pos & (2 * 32 * 1024 - 1)) / linesize;
	} else {
		dspnum = pos / (4 * 32 * 1024);
		cacheline = (pos & (4 * 32 * 1024 - 1)) / linesize;
	}

	if ((brd->rwcache.jiffies != jiffies) ||
            (brd->rwcache.device != dspnum) ||
	    (brd->rwcache.pmdm != pmdm) ||
	    (brd->rwcache.cacheline != cacheline)) {
		GLOBAL_COMMON *gcp;
		uint dspaddr;
		int i;

		gcp = dgdm_get_gc_ptr(brd);
		if (!gcp)
			return (-ENXIO);

		/*
		 * Its not in the cache, go get it
		 */
		if (pmdm == RW_DM) {
			dspaddr = (cacheline * linesize) / 2;
			gcp->rw.rw_size = linesize / 2;
		} else {
			dspaddr = (cacheline * linesize) / 4;
			gcp->rw.rw_size = linesize / 4;
		}

		gcp->rw.rw_type = RW_DSP;
		gcp->rw.rw_addr = dspaddr;
		gcp->rw.rw_subtype = pmdm | dspnum;
		gcp->rw.rw_req = RW_READ;

		/*
		 * Spin host CPU until we get the data.
                 *
		 * But if we spin too long, sleep for a tick
		 * so other things can get done and so we can
		 * interrupt this if the board is hosed.
		 */
		for (i = 0; gcp->rw.rw_req; ++i) {
			if (i == 10000) {
				if (dgdm_sleep(1))
					return (-EINTR);
				i = 0;
			}
		}

		brd->rwcache.jiffies = jiffies;
		brd->rwcache.device = dspnum;
		brd->rwcache.pmdm = pmdm;
		brd->rwcache.cacheline = cacheline;
		memcpy(brd->rwcache.data, (uchar *) gcp->rw.rw_data,
		       linesize);

		/*
		 * A further optimization could be to initiate a read-ahead
		 * of the next cacheline if we just read the last word of
		 * a cacheline.  Left as an exercise for the reader.
		 */
	}
	return (0);
}



/*
 * dgdm_boardwrite()
 *
 * Write to the ram/control/debug device
 */
STATIC WRITE_RETURN_T dgdm_boardwrite(WRITE_PROTO)
{
	unsigned int minor = MINOR(PARM_I_RDEV);
	int board = minor2board(minor);
	int type = minor2type(minor);
	int subtype = minor2subtype(minor);
	BOARD_T *brd = dgdm_Board[board];

	unsigned long pos = GET_POS();
	void *virt;
	ulong len;

	if (count < 0)
		return (-EINVAL);
	if (count == 0)
		return (0);

	switch (type) {
	case TYPE_RAM:
		switch (subtype) {
		case SUBTYPE_GDB:
			return (dgdm_debugwrite(brd, WRITE_ARGS));
		case SUBTYPE_DSPDM:
			return (dgdm_dspwrite(brd, RW_DM, WRITE_ARGS));
		case SUBTYPE_DSPPM:
			return (dgdm_dspwrite(brd, RW_PM, WRITE_ARGS));
		default:
			virt = brd->vram;
			len = brd->ramlen;
			break;
		}
		break;
	case TYPE_REG:
		virt = brd->vreg;
		len = brd->reglen;
		break;
	case TYPE_FLASH:
		return dgdm_flashwrite16(brd, subtype, WRITE_ARGS);
		break;
	case TYPE_IO:
		switch (subtype) {
		case SUBTYPE_PRINT:
			return (dgdm_printwrite(brd, WRITE_ARGS));
		case SUBTYPE_AUDIO:
			return (dgdm_audiotap_write(brd, WRITE_ARGS));
		case SUBTYPE_EQM:
			return (dgdm_eqm_write(brd, WRITE_ARGS));
		default:
			return (-EINVAL);
		}
		break;
	case TYPE_VPD:
		return dgdm_flashwrite8(brd, (uchar *) brd->vvpd,
				     &brd->vpdarea[subtype], WRITE_ARGS);
		break;
	case TYPE_DMABUF:
		if (subtype > 1)
			return -EINVAL;
		virt = (uchar *) brd->dma_buf[subtype];
		len = 4096;
		break;
	default:
		return (-EINVAL);
	}

	if (count > (len - pos))
		count = len - pos;

	COPY_FROM_USER(virt + pos, buf, count);

	ADD_POS(count);

	return (count);
}



/*
 * dgdm_debugwrite()
 *
 * Write to the debug device.  Called from dgdm_boardwrite().
 */
STATIC int dgdm_debugwrite(BOARD_T *brd, WRITE_PROTO)
{
	cq_t *qp = (cq_t *) ((uchar *) (brd->vram) + CQ_CMD), q;
	int resid = 0, space;
	unsigned char *p;

	q.cq_base = RE32(qp->cq_base);
	q.cq_in = RE32(qp->cq_in);
	q.cq_out = RE32(qp->cq_out);
	q.cq_size = RE32(qp->cq_size);

	/*
	 * Sanity check the queue structure.
	 */
	if ((q.cq_base < 0x100) || (q.cq_base >= 0x400000) ||
	    (q.cq_size < 16) || (q.cq_size > 1024) ||
	    (q.cq_in >= q.cq_size) ||
	    (q.cq_out >= q.cq_size)) {
		APR(("Preposterous queue structure.\n"));
		return (-ENXIO);
	}
	/*
	 * Trim count to fit the available space.
	 */
	if ((space = (q.cq_out - q.cq_in - 1)) < 0)
		space += q.cq_size;

	if (count > space) {
		if ((count = space) <= 0)
			return (0);
	}
	if (count >= (q.cq_size - q.cq_in)) {
		resid = MIN(q.cq_size - q.cq_in, count);
		p = brd->vram + q.cq_base + q.cq_in;

		COPY_FROM_USER(p, buf, resid);

		count -= resid;
		buf += resid;
		q.cq_in = 0;
	}
	if (count > 0) {
		p = brd->vram + q.cq_base + q.cq_in;

		COPY_FROM_USER(p, buf, count);

		q.cq_in += count;
		resid += count;
	}

	/* Call hardware specific function to get the board's attention */
	if (brd->triboard && brd->triboard->post_board_int)
		(*brd->triboard->post_board_int) (brd->triboard);

	qp->cq_in = RE32(q.cq_in);

	ADD_POS(resid);

	return (resid);
}



/*
 * dgdm_printwrite()
 *
 * Write the debugging print circular buffer
 */
STATIC int dgdm_printwrite(BOARD_T *brd, WRITE_PROTO)
{
	ulong len;
	ulong i;
	GLOBAL_DEBUG_DATA *gdd = brd->gdd;
	uint32 tail = gdd->get_tail;
	int space;

	space = (gdd->get_head - tail - 1) & gdd->get_mask;

	len = MIN(space, count);
	/*
	 * Copy the data to the user.  This is an inefficent loop,
	 * but I don't want to pay a lot for this muffler.
	 */
	for (i = 0; i < len; ++i) {
		COPY_FROM_USER((char *) gdd->get_buf + tail, buf + i, 1);
		tail = ++tail & gdd->get_mask;
	}

	gdd->get_tail = tail;

	ADD_POS(len);

	return (len);
}



/*
 * dgdm_flashwrite16()
 *
 * Write to 16 bit Flash devices like on a Houston board.
 * Called from dgdm_boardwrite().
 */
STATIC int dgdm_flashwrite16(BOARD_T *brd, int subtype, WRITE_PROTO)
{
	unsigned long pos = GET_POS();
	FLASHAREA *flasharea = &brd->flasharea[subtype];
	volatile ushort *virt = (ushort *) brd->vflash;
	ulong len = 2 * flasharea->wlen;

	int i, j;
	ulong flags;
	int flen;
	ushort *fbuf;
	ulong hwordaddr;

	DPR(("dgdm_flashwrite16(%d)\n", (int) count));

	if (count & 1 || pos & 1)
		return (-EINVAL);	/* 16 bit halfword writes only */
	if (pos >= len || count > (len - pos))
		return (-ENOSPC);

	/*
	 * Reset the part.
	 */
	DGDM_LOCK(dgdm_global_lock, flags);
	virt[0x5555] = RE16(0x00aa);
	virt[0x2aaa] = RE16(0x0055);
	virt[0x5555] = RE16(0x00f0);
	DGDM_UNLOCK(dgdm_global_lock, flags);

	/*
	 * Read part ID's
	 */
	if (!brd->flash_ided) {
		ushort id[2];

		virt[0x0000] = RE16(0x00f0);	/* Reset part */

		DGDM_LOCK(dgdm_global_lock, flags);
		virt[0x5555] = RE16(0x00aa);
		virt[0x2aaa] = RE16(0x0055);
		virt[0x5555] = RE16(0x0090);
		id[1] = virt[1];
		id[0] = virt[0];
		virt[0x0000] = RE16(0x00f0);	/* Reset part */
		DGDM_UNLOCK(dgdm_global_lock, flags);

		DPR(("Flash Manuf:Device=0x%02x:0x%02x\n", id[0], id[1]));

		brd->flash_ided = 1;
	}

	/*
	 * Allocate a buffer for the write
	 */
	flen = 512 * sizeof(ushort);
	fbuf = dgdm_driver_kzmalloc(flen, GFP_KERNEL);
	if (!fbuf)
		return (-ENOMEM);

	/*
	 * Loop, processing all data in users request
	 */
	len = count;
	hwordaddr = flasharea->woffset + (pos >> 1);
	while (len) {
		ulong amt;

		/*
		 * Read data from user into the buffer
		 */
		amt = (len > flen) ? flen : len;
		COPY_FROM_USER(fbuf, buf, amt);
		len -= amt;
		ADD_POS(amt);
		buf += amt;

		/*
		 * Loop, writing each word
		 */
		amt /= 2;	/* Now # of 16 bit halfwords */
		for (i = 0; i < amt; ++i) {
			ushort new = fbuf[i];
			ushort val;
			ushort dq7;
			ushort DQ7 = RE16(0x0080);
			ushort DQ5 = RE16(0x0020);

			/*
			 * If we are on a sector boundary, erase the sector.
			 */
			if ((hwordaddr & 0x0FFF) == 0
			  && flasharea->secmap[hwordaddr >> 12] == '1') {
				DPR(("checking sector protect address %lx\n",
				     hwordaddr));

				DGDM_LOCK(dgdm_global_lock, flags);
				virt[0x0000] = RE16(0x00f0); /* Reset part */
				virt[0x5555] = RE16(0x00aa);
				virt[0x2aaa] = RE16(0x0055);
				virt[0x5555] = RE16(0x0090);
				val = virt[hwordaddr | 0x02]; /* Read protect*/
				DGDM_UNLOCK(dgdm_global_lock, flags);

				virt[0x0000] = RE16(0x00f0); /* Reset part */

				if (val == 0x01) {
					kfree(fbuf);
					APR(("sector %lx is write protected\n",
                                             hwordaddr));
					return (-EPERM);
				}
				DPR(("erasing sector halfword address %lx\n",
				     hwordaddr));

				DGDM_LOCK(dgdm_global_lock, flags);
				virt[0x5555] = RE16(0x00aa);
				virt[0x2aaa] = RE16(0x0055);
				virt[0x5555] = RE16(0x0080);
				virt[0x5555] = RE16(0x00aa);
				virt[0x2aaa] = RE16(0x0055);
				virt[hwordaddr] = RE16(0x0030);
				DGDM_UNLOCK(dgdm_global_lock, flags);

				for (j = 0;; ++j) {
                                        /* Sleep for one second */
					dgdm_sleep(HZ);

					if (j >= 15)
						goto eio;

					val = virt[hwordaddr];
					if (val == 0xffff)
						break;
					else {
						/* Check for error */
						val = RE16(val) & 0x00A0;
						if (val == 0x0020) {
							goto eio;
						}
					}
				}
				DPR(("erased status %x, loops %d\n", val, j));
			}
			/*
			 * Write a halfword
			 */
			DGDM_LOCK(dgdm_global_lock, flags);
			virt[0x5555] = RE16(0x00aa);
			virt[0x2aaa] = RE16(0x0055);
			virt[0x5555] = RE16(0x00a0);
			virt[hwordaddr] = new;
			DGDM_UNLOCK(dgdm_global_lock, flags);

			dq7 = new & DQ7;

			for (j = 0;; ++j) {
				if (j >= 1000000) {
					APR(("write time exceeded @ 0x%lx\n",
					     hwordaddr << 1));
					goto eio;
				}
				val = virt[hwordaddr];
				if (val == new)
					break;

				if ((val & DQ7) != dq7 && (val & DQ5)
				    && (virt[hwordaddr] & DQ7) != dq7) {

					DPR(("wrote 0x%x @ 0x%lx, but "
                                             "val=%x %x\n", new,
                                             (hwordaddr << 1), val,
                                             virt[hwordaddr]));
					goto eio;
				}
			}
			++hwordaddr;
		}

		/*
		 * Let other processes run for a clock tick
		 */
		if (len)
			dgdm_sleep(1);
	}

	kfree(fbuf);
	return (count);

      eio:
	kfree(fbuf);
	APR(("Error in dgdm_flashwrite16\n"));
	return (-EIO);
}



/*
 * dgdm_flashwrite8()
 *
 * Write to 8 bit Flash devices like VPD.
 * Called from dgdm_boardwrite().
 */
STATIC int dgdm_flashwrite8(BOARD_T *brd, uchar *vbase,
                            FLASHAREA *flasharea, WRITE_PROTO)
{
	unsigned long pos = GET_POS();
	volatile uchar *virt = (uchar *) vbase;
	ulong len = flasharea->wlen;

	int i, j;
	ulong flags;
	int flen;
	uchar *fbuf;
	ulong byteaddr;

	DPR(("dgdm_flashwrite8(%d)\n", (int) count));

	if (pos >= len || count > (len - pos))
		return (-ENOSPC);

	/*
	 * Reset the part.
	 */
	DGDM_LOCK(dgdm_global_lock, flags);
	virt[0x5555] = 0xaa;
	virt[0x2aaa] = 0x55;
	virt[0x5555] = 0xf0;
	DGDM_UNLOCK(dgdm_global_lock, flags);

	/*
	 * Allocate a buffer for the write
	 */
	flen = 512 * sizeof(uchar);
	fbuf = dgdm_driver_kzmalloc(flen, GFP_KERNEL);
	if (!fbuf)
		return (-ENOMEM);

	/*
	 * Loop, processing all data in users request
	 */
	len = count;
	byteaddr = flasharea->woffset + pos;
	while (len) {
		ulong amt;

		/*
		 * Read data from user into the buffer
		 */
		amt = (len > flen) ? flen : len;
		COPY_FROM_USER(fbuf, buf, amt);
		len -= amt;
		ADD_POS(amt);
		buf += amt;

		/*
		 * Loop, writing each byte
		 */
		for (i = 0; i < amt; ++i) {
			uchar new = fbuf[i];
			uchar val;
			uchar dq7;
			uchar DQ7 = 0x80;
			uchar DQ5 = 0x20;

			/*
			 * If we are on a sector boundary, erase
			 * the sector.
			 */
			if ((byteaddr & 0x1FFF) == 0
			    && flasharea->secmap[byteaddr >> 13] == '1') {
				DPR(("erasing sector byte address %lx\n",
                                     byteaddr));

				DGDM_LOCK(dgdm_global_lock, flags);
				virt[0x5555] = 0xaa;
				virt[0x2aaa] = 0x55;
				virt[0x5555] = 0x80;
				virt[0x5555] = 0xaa;
				virt[0x2aaa] = 0x55;
				virt[byteaddr] = 0x30;
				DGDM_UNLOCK(dgdm_global_lock, flags);

				for (j = 0;; ++j) {
                                        /* Sleep for one second */
                                        dgdm_sleep(HZ);

					if (j >= 15)
						goto eio;

					val = virt[byteaddr];
					if (val == 0xff)
						break;
					else {
						/* Check for error */
						val = val & 0xA0;
						if (val == 0x20)
							goto eio;
					}
				}
				DPR(("erased status %x, loops %d\n", val, j));
			}
			/*
			 * Write a byte
			 */
			DGDM_LOCK(dgdm_global_lock, flags);
			virt[0x5555] = 0xaa;
			virt[0x2aaa] = 0x55;
			virt[0x5555] = 0xa0;
			virt[byteaddr] = new;
			DGDM_UNLOCK(dgdm_global_lock, flags);

			dq7 = new & DQ7;

			for (j = 0;; ++j) {
				if (j >= 1000000) {
					APR(("write time exceeded\n"));
					goto eio;
				}
				val = virt[byteaddr];
				if (val == new)
					break;

				if ((val & DQ7) != dq7 && (val & DQ5)
				    && (virt[byteaddr] & DQ7) != dq7) {

					DPR(("wrote 0x%x @ 0x%lx, but "
                                             "val=%x %x %d\n",
					     new, byteaddr, val,
                                             virt[byteaddr], j));
					goto eio;
				}
			}
			++byteaddr;
		}

		/*
		 * Let other processes run for a clock tick
		 */
		if (len)
			dgdm_sleep(1);
	}

	kfree(fbuf);
	return (count);

      eio:
	kfree(fbuf);
	APR(("Error in dgdm_flashwrite8\n"));
	return (-EIO);
}


/*
 * dgdm_dspwrite()
 *
 * Write to a DSP (TODO)
 */
STATIC int dgdm_dspwrite(BOARD_T *brd, int pmdm, WRITE_PROTO)
{
	int ndsp = brd->ndsps ? brd->ndsps : 30;
	int maxlen;
	int len;
	unsigned long pos = GET_POS();

	if (pmdm == RW_DM) {
		if (pos & 1 || count & 1)
			return (-EINVAL);
		maxlen = ndsp * 32768 * 2;
	} else {
		if (pos & 3 || count & 3)
			return (-EINVAL);
		maxlen = ndsp * 32768 * 4;
	}

	brd->rwcache.device = 0xffffffff;	/* Invalidate RW read cache */

	len = count = MIN(count, maxlen - pos);

	while (len) {
		int amt;
		int dspaddr;
		int dspnum;
		int i;
		GLOBAL_COMMON *gcp;

		gcp = dgdm_get_gc_ptr(brd);
		if (!gcp)
			return (-ENXIO);

		amt = MIN(len, sizeof(gcp->rw.rw_data));

		if (pmdm == RW_DM) {
			dspaddr = (pos / 2) & 32767;
			dspnum = (pos / 2) / 32768;
			gcp->rw.rw_size = amt / 2;
		} else {
			dspaddr = (pos / 4) & 32767;
			dspnum = (pos / 4) / 32768;
			gcp->rw.rw_size = amt / 4;
		}
		if (dspnum >= ndsp)
			return (-ENXIO);

		COPY_FROM_USER((char *) gcp->rw.rw_data, buf, amt);

		gcp->rw.rw_type = RW_DSP;
		gcp->rw.rw_addr = dspaddr;
		gcp->rw.rw_subtype = pmdm | dspnum;
		gcp->rw.rw_req = RW_WRITE;

		/*
		 * Spin host CPU until the data is written.
                 *
		 * But if we spin too long, sleep for a tick
		 * so other things can get done and so we can
		 * interrupt this if the board is hosed.
		 */
		for (i = 0; gcp->rw.rw_req; ++i) {
			if (i == 10000) {
				if (dgdm_sleep(1))
					return (-EINTR);
				i = 0;
			}
		}

		pos += amt;
		buf += amt;
		len -= amt;
	}

	SET_POS(pos);

	return (count);
}



/*
 * dgdm_boardioctl()
 *
 * ioctl() the ram/control/debug device.
 * Only the Management channel should have ioctls.. 
 */
STATIC int dgdm_boardioctl(struct inode *inode, struct file *file,
                           unsigned int cmd, unsigned long arg)
{
	unsigned int minor = MINOR(inode->i_rdev);
	int type = minor2type(minor);
	int subtype = minor2subtype(minor);

	switch (type) {
	case TYPE_IO:
		if (subtype == SUBTYPE_MGMT) {
			return (dgdm_mgmt_ioctl(minor, file, cmd, arg));
		} else {
			return (0);
		}

	case TYPE_RAM:
		/*
		 * Silently ignore ioctls on the gdb port
		 * to keep the signal to noise ratio high.
		 */
		if (subtype == SUBTYPE_GDB) {
			return (0);
		} else {
			return (-EINVAL);
		}

	default:
		return (-EINVAL);
	}
}



/*
 * dgdm_boardmmap()
 *
 * Mmap() the ram/control/debug device
 */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,1,0)
STATIC int dgdm_boardmmap(struct inode *inode, struct file *file,
                          struct vm_area_struct *vma)

#else
STATIC int dgdm_boardmmap(struct file *file, struct vm_area_struct *vma)
#endif
{
	unsigned int minor = MINOR(PARM_I_RDEV);
	int board = minor2board(minor);
	int type = minor2type(minor);
	int subtype = minor2subtype(minor);
	BOARD_T *brd = dgdm_Board[board];

	ulong addr;

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
	if (vma->vm_offset & ~PAGE_MASK)
		return -ENXIO;
#endif

	switch (type) {
	case TYPE_RAM:
		addr = brd->ram;
		break;
	case TYPE_REG:
		addr = brd->reg;
		break;
	case TYPE_FLASH:
		addr = brd->flash + brd->flasharea[subtype].woffset;
		break;
	case TYPE_VPD:
		addr = brd->vpd;
		break;
	case TYPE_DMABUF:
		addr = brd->dma_buf[subtype];
		break;
	default:
		return (-EINVAL);
	}

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
	if (remap_page_range(vma->vm_start, addr, vma->vm_end - vma->vm_start,
                             vma->vm_page_prot))
		return -EAGAIN;
#else
	if (remap_page_range(vma->vm_start, addr + vma->vm_offset,
			 vma->vm_end - vma->vm_start, vma->vm_page_prot))
		return -EAGAIN;
#endif

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,1,0)
	vma->vm_inode = inode;
	inode->i_count++;
#endif

	return (0);
}





/************************************************************************
 *
 * Utility functions
 *
 ************************************************************************/

/*
 * swap32()
 *
 * Swap a 32 bit int
 */
static inline uint32 swap32(uint32 val)
{
	uint32 d = (val << 16) | (val >> 16);
	return ((d >> 8) & 0x00ff00ff) | ((d << 8) & 0xff00ff00);
}


/*
 * swap16()
 *
 * Swap a 16 bit int
 */
static inline uint16 swap16(uint16 val)
{
	return ((val << 8) | (val >> 8));
}



/*
 * dgdm_driver_kzmalloc()
 *
 * Malloc and clear memory,
 */
STATIC void *dgdm_driver_kzmalloc(size_t size, int priority)
{
        void *p;
 
        p = kmalloc(size, priority);
        if(p)
                memset(p, 0, size);
        return(p);
}



/*
 * dgdm_mbuf()
 *
 * Used to print to the message buffer during board init.
 */
GLOBAL void dgdm_mbuf(BOARD_T *brd, const char *fmt, ...) {
        va_list          ap;
        char		 buf[1024];
        int              i;
        unsigned long    flags;

        DGDM_LOCK(dgdm_global_lock, flags);

        /* Format buf using fmt and arguments contained in ap. */
        va_start(ap, fmt);
        i = vsprintf(buf, fmt,  ap);
        va_end(ap);

        DPR_TRIBL((buf));

        if(!brd || !brd->msgbuf) {
                printk(buf);
                DGDM_UNLOCK(dgdm_global_lock, flags);
                return;
        }

        memcpy(brd->msgbuf, buf, strlen(buf));
        brd->msgbuf += strlen(buf);
        *brd->msgbuf = (char) NULL;

        DGDM_UNLOCK(dgdm_global_lock, flags);
}



/*
 * dgdmsleep()
 *
 * Put the driver to sleep for "ticks" clock ticks
 *
 * Returns 0 if timed out, !0 (showing signal) if interrupted by a signal.
 */
GLOBAL int dgdm_sleep(int ticks)
{
	current->state = TASK_INTERRUPTIBLE;
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,1,0)
	current->timeout = jiffies + ticks;
	schedule();
#else
	schedule_timeout(ticks);
#endif
	return (signal_pending(current));
}






/************************************************************************
 *
 * Simulate the Linux 2.2.0 PCI functions to keep dgdm_scan() from
 * turning into an unmaintainable #ifdef spaghetti mess.
 *
 * NOTE: this may or may not work for kernels older than 2.0.36
 *
 * stolen from 2.2.0 linux/drivers/pci/pci.c
 */
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,1,0)
STATIC struct pci_dev *pci_find_device(int vendor, int device,
                                       struct pci_dev *pdev)
{
	static int index;
	static struct pci_dev dev;
	static struct pci_bus bus;
	int rc;
	uchar pci_device_fn;

	if (pdev == NULL) {
		index = 0;
		dev.bus = &bus;
	}
	if (index > 255)
		return NULL;

	rc = pcibios_find_device(vendor, device, index++,
				 &dev.bus->number, &pci_device_fn);

	if (rc != 0) {
		return NULL;
	} else {
		dev.devfn = pci_device_fn;
		return &dev;
	}
}


STATIC int pci_read_config_byte(struct pci_dev *pdev, u8 where, u8 *val)
{
	return pcibios_read_config_byte(pdev->bus->number, pdev->devfn,
					where, val);
}

STATIC int pci_read_config_word(struct pci_dev *pdev, u8 where, u16 *val)
{
	return pcibios_read_config_word(pdev->bus->number, pdev->devfn,
					where, val);
}

STATIC int pci_read_config_dword(struct pci_dev *pdev, u8 where, u32 *val)
{
	return pcibios_read_config_dword(pdev->bus->number, pdev->devfn,
					 where, val);
}

# if defined(DGDM_ALL_OLD_PCI_CODE)
/*
 * this code is not currently used by the driver, but is included here
 * for completeness in case it is needed in the future
 */
STATIC int pci_write_config_byte(struct pci_dev *pdev, u8 where, u8 val)
{
	return pcibios_write_config_byte(pdev->bus->number, pdev->devfn,
					 where, val);
}

STATIC int pci_write_config_word(struct pci_dev *pdev, u8 where, u16 val)
{
	return pcibios_write_config_word(pdev->bus->number, pdev->devfn,
					 where, val);
}

STATIC int pci_write_config_dword(struct pci_dev *pdev, u8 where, u32 val)
{
	return pcibios_write_config_dword(pdev->bus->number, pdev->devfn,
					  where, val);
}

STATIC void pci_set_master(struct pci_dev *pdev)
{
	u16 pci_command;
	u8 pci_latency;

	pci_read_config_word(pdev, PCI_COMMAND, &pci_command);
	if (!(pci_command & PCI_COMMAND_MASTER)) {
		APR((" Setting the PCI MasterBit!\n"));
		pci_command |= PCI_COMMAND_MASTER;
		pci_write_config_word(pdev, PCI_COMMAND, pci_command);
	}
	pci_read_config_byte(pdev, PCI_LATENCY_TIMER, &pci_latency);
	if (latency && pci_latency != latency) {
		APR((" Overriding PCI latency timer: was %d, now is %d.\n",
		     pci_latency, latency));
		pci_write_config_byte(pdev, PCI_LATENCY_TIMER, latency);
	}
	return;
}
# endif
#endif
