/* -*- linux-c -*- (Autoset linux indent style when using emacs) */
 
/*
 * Copyright 1999 Digi International (www.digi.com)
 *    Jeff Randall <Jeff_Randall@digi.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/*************************************************************************
 *
 * Driver includes
 *
 *************************************************************************/
#define RAWDXBDRV

#define TRC_TO_CONSOLE 1

#include <linux/config.h>
#include <linux/version.h>
#include <linux/module.h>

#include <linux/errno.h>
#include <linux/major.h>
#include <linux/kernel.h>
#include <linux/signal.h>

#if LINUX_VERSION_CODE >= 0x20400
    #include <linux/slab.h>
#else
    #include <linux/malloc.h>
#endif

#include <linux/mm.h>

#include <linux/sched.h>
#include <linux/timer.h>
#include <linux/interrupt.h>
#include <linux/string.h>
#include <linux/delay.h>
#include <linux/ioport.h>
#include <linux/stddef.h>
#include <linux/pci.h>
#include <asm/atomic.h>
#include <asm/bitops.h>
#include <asm/io.h>
#include <asm/byteorder.h>
#include <asm/system.h>

#if LINUX_VERSION_CODE < 0x20400
# if LINUX_VERSION_CODE >= 0x20100
# include <asm/spinlock.h>	/* no spinlocks in 2.0 - not needed in 2.4 */
# endif
#endif

#if LINUX_VERSION_CODE >= 0x20200
#include <linux/poll.h>
#endif

#include <linux/types.h>
#include <linux/wrapper.h>	/* abstracts some diffs between 2.0 and 2.2 */
#include <linux/serial.h>	/* for struct async_serial */

#include "dxbtypes.h"		/* Additional types needed by header files */


/* typedef's that are missing from Digi header files. */
typedef struct digiheader	DIGIHDR;

/* typedef's that are missing from Linux header files. */
typedef struct tty_driver	TTYDRIVER;
typedef struct tty_struct	TTY;
typedef struct file		FILE;
typedef struct termios		TERMIOS;
typedef struct semaphore	SEMAPHORE;

/* Driver specific includes */
#include "hou2.h"
#include "hou2mmap.h"
#include "debugcomm.h"
#include "triglbl.h"
#include "dxbglbl.h"
#include "fepglbl.h"

#define DXB_HAS_INLINE	1
#include "dxblib.h"

#include "tribl.h"
#include "mipsheader.hh"
#include "dgdm_isdn.h"
#include "pci_ids.h"
#include "sdxb_prot.h"
#include "rascon.h"





/*************************************************************************
 *
 * Driver defines
 *
 *************************************************************************/

/*
 * Driver identification, error and debugging statments
 *
 * In theory, you can change all occurances of "digi" in the next
 * three lines, and the driver printk's will all automagically change.
 *
 * APR((fmt, args, ...));	Always prints message
 * DPR((fmt, args, ...));	Only prints if DGDM_TRACER is defined at
 *				  compile time and dgdm_debug!=0
 */
#define	PROCSTR		"dgdm"			/* /proc entries	 */
#define	USRLIBSTR	"/usr/lib/dg/dgdm"	/* /usr/lib/ entries	 */
#define	DEVSTR		"/dev/dg/dgdm"		/* /dev entries		 */
#define	DRVSTR		"dgdm"			/* Driver name string 
						 * displayed by APR	 */
#define	APR(args)	do { PRINTF_TO_KMEM(args); printk(DRVSTR": "); printk args; \
			   } while (0)
#define	RAPR(args)	do { PRINTF_TO_KMEM(args); printk args; } while (0)

/*
 * Debugging levels can be set using debug insmod variable
 * They can also be compiled out completely.
 */

#define	DBG_BASIC		(dgdm_debug & 0x01)
#define	DBG_READ		(dgdm_debug & 0x02)
#define	DBG_WRITE		(dgdm_debug & 0x04)
#define	DBG_DXB			(dgdm_debug & 0x08)
#define	DBG_MCTRL		(dgdm_debug & 0x10)
#define	DBG_IOCTL		(dgdm_debug & 0x20)
#define	DBG_PROC		(dgdm_debug & 0x40)
#define	DBG_TRIBL		(dgdm_debug & 0x80)

#if defined(DGDM_TRACER)

# if defined(TRC_TO_KMEM)
/* Choose one: */
#  define TRC_ON_OVERFLOW_WRAP_AROUND
#  undef  TRC_ON_OVERFLOW_SHIFT_BUFFER
# endif //TRC_TO_KMEM

# define TRC_MAXMSG		1024
# define TRC_OVERFLOW		"(OVERFLOW)"
# define TRC_DTRC		"/usr/bin/dtrc"

#if defined TRC_TO_CONSOLE
#define PRINTF_TO_CONSOLE(args) { printk(DRVSTR": "); printk args; }
#else //!defined TRACE_TO_CONSOLE
#define PRINTF_TO_CONSOLE(args)
#endif

#if defined TRC_TO_KMEM
#define PRINTF_TO_KMEM(args) dgdm_tracef args 
#else //!defined TRC_TO_KMEM
#define PRINTF_TO_KMEM(args)
#endif

#define	TRC(args)	{ PRINTF_TO_KMEM(args); PRINTF_TO_CONSOLE(args) }

# define DPR_BASIC(ARGS)	if (DBG_BASIC) TRC(ARGS)
# define DPR_READ(ARGS)		if (DBG_READ)  TRC(ARGS)
# define DPR_WRITE(ARGS)	if (DBG_WRITE) TRC(ARGS)
# define DPR_DXB(ARGS)		if (DBG_DXB)   TRC(ARGS)
# define DPR_MCTRL(ARGS)	if (DBG_MCTRL) TRC(ARGS)
# define DPR_IOCTL(ARGS)	if (DBG_IOCTL) TRC(ARGS)
# define DPR_PROC(ARGS)		if (DBG_PROC)  TRC(ARGS)
# define DPR_TRIBL(ARGS)	if (DBG_TRIBL) TRC(ARGS)

# define STATIC
# define GLOBAL
# define DPR(ARGS)		if (dgdm_debug) TRC(ARGS)
# define P(X)			dgdm_tracef(#X "=%p\n", X)
# define X(X)			dgdm_tracef(#X "=%x\n", X)

#else//!defined DGDM_TRACER

#define PRINTF_TO_KMEM(args)
# define TRC(ARGS)
# define DPR_BASIC(ARGS)
# define DPR_READ(ARGS)
# define DPR_WRITE(ARGS)
# define DPR_DXB(ARGS)
# define DPR_MCTRL(ARGS)
# define DPR_IOCTL(ARGS)
# define DPR_PROC(ARGS)
# define DPR_TRIBL(ARGS)

# define STATIC			static
# define GLOBAL
# define DPR(args)

#endif//DGDM_TRACER

/* Number of boards we support at once. this number should be either 4 or 8 */
#define	MAXBOARDS	4

#define	NISDN		64	/* FIXME should be dynamic like ttys are   */
#define	NPCIADDRS	6	/* Number of PCI addresses in config space */
#define MAXBAUD		1228800 /* Highest legal baudrate for any card     */



/* major offsets from dgdm_major */
#define DGDM_SERIAL_MAJOR	0
#define DGDM_CALLOUT_MAJOR	1
#define DGDM_PRINT_MAJOR	2
#define DGDM_ISDN_MAJOR		3
#define DGDM_CONTROL_MAJOR	4
#define DGDM_SERIAL2_MAJOR	5
#define DGDM_CALLOUT2_MAJOR	6
#define DGDM_PRINT2_MAJOR	7
#define DGDM_ISDN2_MAJOR	8
#define DGDM_CONTROL2_MAJOR	9



/* Size of circular buffer used to collect audio tap samples */
#define AUDIOBUF_SIZE   4096

/* Size of circular buffer used to collect EQM samples */
#define EQMBUF_SIZE     2048



/*
 * tty defines
 */
#define	SERIAL_TYPE_NORMAL	1
#define	SERIAL_TYPE_CALLOUT	2

#define DXB_TIMEOUT		(jiffies + 1)
#define DEFAULT_DTRLOW		(3*HZ)

#define WRITEBUFLEN		4096
#define MYFLIPLEN		N_TTY_BUF_SIZE

#define CIBAUD_SHIFT		16

#define	IXONA			0x2000

/*
 * defines to make calls to sdxbcmd clearer
 */
#define	CMD_HIPRI	1
#define	CMD_NORMAL	0
#define	CMD_WAIT	1
#define	CMD_NOWAIT	0

/*
 * The parameters we use to define the periods of the moving averages.
 */
#define		MA_PERIOD	(HZ/10)
#define		SMA_DUR		(1*HZ)
#define		EMA_DUR		(1*HZ)
#define		SMA_NPERIODS	(SMA_DUR / MA_PERIOD)
#define		EMA_NPERIODS	(EMA_DUR / MA_PERIOD)




/*
 * Define a local default termios struct. All ports will be created
 * with this termios initially.  This is the same structure that is defined
 * as the default in tty_io.c with the same settings overriden as in serial.c
 *
 * In short, this should match the internal serial ports' defaults.
 */
#define	DEFAULT_IFLAGS	(ICRNL | IXON)
#define	DEFAULT_OFLAGS	(OPOST | ONLCR)
#define	DEFAULT_CFLAGS	(B9600 | CS8 | CREAD | HUPCL | CLOCAL)
#define	DEFAULT_LFLAGS	(ISIG | ICANON | ECHO | ECHOE | ECHOK | \
			ECHOCTL | ECHOKE | IEXTEN)









/*************************************************************************
 *
 *	Allocation of minor numbers for ram/control/debug devices:
 *		+-------+-----------+-----------+
 *		| B : B | T : T : T : S : S : S |
 *		+-------+-----------+-----------+
 *		 Board#	  Type	       Subtype
 *
 *	Board#	0 to 3
 *	Type	0	RAM
 *				Subtype 0	4640 RAM
 *				Subtype 1	unused
 *				Subtype 2	GDB memory port
 *				Subtype 3	DSPs data memory
 *				Subtype 4	DSPs program memory
 *		1	Flash
 *				Subtype 0	Whole thing
 *				Subtype 1	Program Area
 *				Subtype 2	VPD on chaska/london
 *				Subtype 3	Parameter Area
 *				Subtype 4	Boot Block
 *				Subtype 5	Devious Purposes
 *				Subtype 6	Configuration Data
 *		2	64011 Registers
 *				No subtypes
 *		3	Misc I/O
 *				Subtype 0	Send FEP command
 *				Subtype 1	Read debug printf's
 *				Subtype 2	Read h2a DXB buffer
 *				Subtype 3	Read a2h DXB buffer
 *				Subtype 4	Audio tap
 *		4	VPD Flash on houston (EPROM for Gal-9)
 *				Subtype 0	Entire VPD or EPROM
 *				Subtype 1	The VPD Proper
 *				Subtype 2	The Rest of VPD flash
 *		5	DMABUF
 *				Subtype is buffer number (0 or 1)
 *
 *************************************************************************/

#define	minor2board(MINOR)	( (MINOR) >> 6 )
#define	board2minor(BOARD)	( (BOARD) << 6 )

#define	minor2type(MINOR)	( ((MINOR) >> 3) & 7 )
#define	type2minor(TYPE)	( (TYPE) << 3 )

#define	minor2subtype(MINOR)	( (MINOR) & 0x07 )



#define	TYPE_RAM	0
#define SUBTYPE_RAM	0
#define SUBTYPE_GDB	2
#define SUBTYPE_DSPDM	3
#define SUBTYPE_DSPPM	4
#define	SUBTYPE_NRAM	5

#define	TYPE_FLASH	1
#define SUBTYPE_ALL	0
#define SUBTYPE_PROG	1
#define SUBTYPE_LCVPD	2	/* London/chaska */
#define SUBTYPE_PARM	3
#define SUBTYPE_BOOT	4
#define	SUBTYPE_DEVIOUS	5	/* Unused :-) */
#define	SUBTYPE_CFGDATA	6
#define	SUBTYPE_NFLASH	7

#define	TYPE_REG	2
#define SUBTYPE_NONE	0
#define SUBTYPE_NREG	1

#define	TYPE_IO		3
#define	SUBTYPE_UNUSED1	0
#define	SUBTYPE_PRINT	1
#define	SUBTYPE_H2A	2
#define	SUBTYPE_A2H	3
#define	SUBTYPE_AUDIO	4
#define	SUBTYPE_MGMT	5
#define	SUBTYPE_EQM	6
#define SUBTYPE_NIO	7

#define	TYPE_VPD	4
#define	SUBTYPE_VPDALL	0
#define	SUBTYPE_VPD	1
#define	SUBTYPE_VPDREST	2
#define	SUBTYPE_NVPD	3

#define	TYPE_DMABUF	5





/*
 * Macros for accessing board data of a specific endianess
 *
 * LE*()	- board data that is always little endian
 * BE*()	- board data that is always big endian
 * RE*()	- board data that depends on whether the board is running
 *		  little endian or big endian (brd->bigendian_board variable)
 */
#ifdef __LITTLE_ENDIAN
	#define	LE16(X)	(X)
	#define	LE32(X)	(X)
	#define	BE16(X)	swap16(X)
	#define	BE32(X)	swap32(X)
	#define	RE16(X)	( brd->bigendian_board ? swap16(X) : (X) )
	#define	RE32(X)	( brd->bigendian_board ? swap32(X) : (X) )
#else
	#define	LE16(X)	swap16(X)
	#define	LE32(X)	swap32(X)
	#define	BE16(X)	(X)
	#define	BE32(X)	(X)
	#define	RE16(X)	( brd->bigendian_board ? (X) : swap16(X) )
	#define	RE32(X)	( brd->bigendian_board ? (X) : swap32(X) )
#endif

/* Convert a board RAM address to a host virtual address */
#define F2H(X)	( brd->vram + ((uint32) (X)) )




/*************************************************************************
 *
 * Utility defines - should be defined elsewhere, but be paranoid
 *
 *************************************************************************/

#if !defined(KERNEL_VERSION) /* defined in version.h in later kernels */
# define KERNEL_VERSION(a,b,c)	(((a) << 16) + ((b) << 8) + (c))
#endif

/* The number of elements in an array */
#if !defined(asizeof)
# define asizeof(X)	(sizeof(X) / sizeof(X[0]))
#endif

#if !defined(MIN)
# define MIN(a,b)	((a) < (b) ? (a) : (b))
#endif

#if !defined(MAX)
# define MAX(a,b)	((a) > (b) ? (a) : (b))
#endif






/*************************************************************************
 *
 * Some API's changed at linux version 2.1.x.  I'm not sure exactly
 * when they changed during the 2.1.x development, but it doesn't
 * matter to us.  Create some macros to preserve compatibility.
 *
 * Here you can see what has changed between 2.0 and 2.1+. If you
 * are reading/writing this file, you should be well aware of these
 * macros or you won't be able to follow whats going on.  The
 * alternative was #ifdef spaghetti.
 */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)

# include <asm/uaccess.h>

# define IOREMAP(ADDR, LEN)		ioremap(ADDR, LEN)
# define IOUNMAP(ADDR)			iounmap(ADDR)
# define COPY_FROM_USER(DST,SRC,LEN)	copy_from_user(DST,SRC,LEN)
# define COPY_TO_USER(DST,SRC,LEN)	copy_to_user(DST,SRC,LEN)
# define GET_USER(A1,A2)		get_user(A1,(unsigned int *)A2)
# define PUT_USER(A1,A2)		put_user(A1,(unsigned long *)A2)

# ifdef __sparc__
/* FIXME!  I don't understand this */
#  undef IOREMAP
#  define IOREMAP(ADDR, LEN)		((void *) ADDR)
# endif

# define READ_PROTO	struct file *file, char *buf, size_t count,loff_t *ppos
# define READ_ARGS	file, buf, count, ppos
# define READ_RETURN_T	ssize_t

# define WRITE_PROTO	struct file *file, const char *buf, size_t count, \
			loff_t *ppos
# define WRITE_ARGS	file, buf, count, ppos
# define WRITE_RETURN_T	ssize_t

# define CLOSE_RETURN_T		int
# define CLOSE_RETURN(X)	return(X)

# define PARM_I_RDEV		file->f_dentry->d_inode->i_rdev
# define GET_POS()		*ppos
# define ADD_POS(AMT)		*ppos += (AMT)
# define SET_POS(AMT)		*ppos = (AMT)

# define DGDM_PCI_IRQ_TYPE	uint
# define PCI_PRESENT()		pci_present()

# define PARM_STR(VAR, INIT, DESC) \
		static char *VAR = INIT; \
		char *dgdm_##VAR; \
		MODULE_PARM(VAR, "s"); \
		MODULE_PARM_DESC(VAR, DESC);

# define PARM_INT(VAR, INIT, DESC) \
		static int VAR = INIT; \
		int dgdm_##VAR; \
		MODULE_PARM(VAR, "i"); \
		MODULE_PARM_DESC(VAR, DESC);

# define PARM_ULONG(VAR, INIT, DESC) \
		static ulong VAR = INIT; \
		ulong dgdm_##VAR; \
		MODULE_PARM(VAR, "l"); \
		MODULE_PARM_DESC(VAR, DESC);

# define DGDM_LOCK(x,y)		spin_lock_irqsave(&(x), y);
# define DGDM_UNLOCK(x,y)	spin_unlock_irqrestore(&(x), y);

# define DGDM_PROCID()		smp_processor_id()

typedef wait_queue_head_t	WAITQ_T;

/*
 *  In some revisions within the 2.4 kernel series, there is a patch
 *  which changes the behavior of a failed open.  In 2.0 and 2.2 and
 *  most 2.4 revisions, a failed open will be followed _always_ by
 *  a call to the driver's close routine.  In some 2.4 revisions,
 *  the close will not be called (and therefore driver use counts
 *  will be too high, and ports may become unusable).
 *
 *  In the case that the ALT_FAIL_OPEN code is allowed to exist,
 *  some new parameters are required, like ALT_FAIL_OPEN_DEFAULT.
 */
# define ALLOW_ALT_FAIL_OPEN 1
# define ALT_FAIL_OPEN_DEFAULT 0

#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2,3,0)

# error "this driver does not support the 2.3 kernel series."

#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
# include <asm/uaccess.h>

# define IOREMAP(ADDR, LEN)		ioremap(ADDR, LEN)
# define IOUNMAP(ADDR)			iounmap(ADDR)
# define COPY_FROM_USER(DST,SRC,LEN)	copy_from_user(DST,SRC,LEN)
# define COPY_TO_USER(DST,SRC,LEN)	copy_to_user(DST,SRC,LEN)
# define GET_USER(A1,A2)		get_user(A1,(unsigned int *)A2)
# define PUT_USER(A1,A2)		put_user(A1,(unsigned long *)A2)
# define pci_resource_start(pdev, reg)	(pdev)->base_address[reg]

# ifdef __sparc__
/* FIXME!  I don't understand this */
#  undef IOREMAP
#  define IOREMAP(ADDR, LEN)		((void *) ADDR)
# endif

# define READ_PROTO	struct file *file, char *buf, size_t count,loff_t *ppos
# define READ_ARGS	file, buf, count, ppos
# define READ_RETURN_T	ssize_t

# define WRITE_PROTO	struct file *file, const char *buf, size_t count, \
			loff_t *ppos
# define WRITE_ARGS	file, buf, count, ppos
# define WRITE_RETURN_T	ssize_t

# define CLOSE_RETURN_T		int
# define CLOSE_RETURN(X)	return(X)

# define PARM_I_RDEV		file->f_dentry->d_inode->i_rdev
# define GET_POS()		*ppos
# define ADD_POS(AMT)		*ppos += (AMT)
# define SET_POS(AMT)		*ppos = (AMT)

# define DGDM_PCI_IRQ_TYPE	uint
# define PCI_PRESENT()		pci_present()

# define PARM_STR(VAR, INIT, DESC) \
		static char *VAR = INIT; \
		char *dgdm_##VAR; \
		MODULE_PARM(VAR, "s"); \
		MODULE_PARM_DESC(VAR, DESC);

# define PARM_INT(VAR, INIT, DESC) \
		static int VAR = INIT; \
		int dgdm_##VAR; \
		MODULE_PARM(VAR, "i"); \
		MODULE_PARM_DESC(VAR, DESC);

# define PARM_ULONG(VAR, INIT, DESC) \
		static ulong VAR = INIT; \
		ulong dgdm_##VAR; \
		MODULE_PARM(VAR, "l"); \
		MODULE_PARM_DESC(VAR, DESC);

# define DGDM_LOCK(x,y)		spin_lock_irqsave(&(x), y);
# define DGDM_UNLOCK(x,y)	spin_unlock_irqrestore(&(x), y);

# define DGDM_PROCID()		smp_processor_id()

#define DECLARE_WAITQUEUE(w,c)	struct wait_queue w = {(c), NULL}
#define wait_queue_head_t	struct wait_queue *
typedef wait_queue_head_t	WAITQ_T;

# define ALLOW_ALT_FAIL_OPEN 0

#else

# include <linux/bios32.h>

# define IOREMAP(ADDR, LEN)		vremap(ADDR, LEN)
# define IOUNMAP(ADDR)			vfree(ADDR)
# define COPY_FROM_USER(DST,SRC,LEN)	memcpy_fromfs(DST,SRC,LEN)
# define COPY_TO_USER(DST,SRC,LEN)	memcpy_tofs(DST,SRC,LEN)
# define GET_USER(A1,A2)		A1 = get_fs_long((unsigned long *) A2)
# define PUT_USER(A1,A2)		put_fs_long(A1, (unsigned long *) A2)
# define pci_resource_start(pdev, reg)	(pdev)->base_register[reg]

# define READ_PROTO	struct inode *inode, struct file *file, char *buf, \
			int count
# define READ_ARGS	inode, file, buf, count
# define READ_RETURN_T	int

# define WRITE_PROTO	struct inode *inode,struct file *file,const char *buf,\
			 int count
# define WRITE_ARGS	inode, file, buf, count
# define WRITE_RETURN_T	int

# define CLOSE_RETURN_T		void
# define CLOSE_RETURN(X)	return

# define PARM_I_RDEV		inode->i_rdev
# define GET_POS()		file->f_pos
# define ADD_POS(AMT)		file->f_pos += (AMT)
# define SET_POS(AMT)		file->f_pos = (AMT)

# define DGDM_PCI_IRQ_TYPE	uchar
# define PCI_PRESENT()		pcibios_present()
# define signal_pending(x)	((x)->signal & ~(x)->blocked)

/*
 * <linux/pci.h> on 2.0 kernels has the PCI_SUBSYSTEM_VENDOR_ID and
 * PCI_SUBSYSTEM_ID values swapped in violation of the spec.
 * 2.2 kernels have them defined correctly (as below).
 */
# undef  PCI_SUBSYSTEM_VENDOR_ID
# define PCI_SUBSYSTEM_VENDOR_ID 0x2c
# undef  PCI_SUBSYSTEM_ID
# define PCI_SUBSYSTEM_ID        0x2e

# if !defined(MODULE_PARM)
#  define MODULE_AUTHOR(NAME)
#  define MODULE_DESCRIPTION(DESC)
#  define MODULE_SUPPORTED_DEVICE(DEV)
#  define MODULE_PARM(VAR,TYPE)
#  define MODULE_PARM_DESC(VAR,DESC)
# endif

# define PARM_STR(VAR,INIT,DESC) \
			static char *VAR = INIT; \
			char *dgdm_##VAR;

# define PARM_INT(VAR,INIT,DESC) \
			static int VAR = INIT; \
			int dgdm_##VAR;

# define PARM_ULONG(VAR,INIT,DESC) \
			static ulong VAR = INIT; \
			ulong dgdm_##VAR;

# define DGDM_LOCK(x,y)		do { save_flags(y); cli(); } while(0)
# define DGDM_UNLOCK(x,y)	do { restore_flags(y); } while(0)

# define DGDM_PROCID()		0 

#define DECLARE_WAITQUEUE(w,c)	struct wait_queue w = {(c), NULL}
#define wait_queue_head_t	struct wait_queue *
typedef wait_queue_head_t	WAITQ_T;

# define ALLOW_ALT_FAIL_OPEN 0

#endif


/*
 * We need to kickstart select on 2.2.15 or later,
 * even if SERIAL_HAVE_POLL_WAIT isn't defined...
 */
#ifndef SERIAL_HAVE_POLL_WAIT
# if LINUX_VERSION_CODE >= KERNEL_VERSION(2,2,15) && \
     LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
#  define SERIAL_HAVE_POLL_WAIT
# endif
#endif




/*************************************************************************
 *
 * Structures and closely related defines.
 *
 *************************************************************************/

/*
 * Flash information structure and flash definitions for
 * rev 10+ (real houston board).
 *
 * In the map, "1" indicates this UNIT is the beginning of a sector
 * boundary that is assigned to this slice.  A "0" indicates this
 * UNIT is more of the sector assigned to this slice.  An "e" means
 * this UNIT is not allocated to this slice.  The size of a UNIT is
 * variable.  On H/L/C, its 8KB.
 */
#define FLASH_AREAS	7
#define VPD_AREAS	3	/*  Houston has a separate VPD flash */
 
typedef struct
{
	char		secmap[64+1];	/* Sector boundaries in "UNITS" */
	ulong		woffset;	/* halfword offset from loc 0 */
	ulong		wlen;		/* length in halfwords */
} FLASHAREA;


/*
 * management message structure
 */
typedef struct _mgmt
{
        int     opencnt;
        int     dxbchan;
 
        int     error_code;
        int     eof;
 
        int     reply_length;
        void    *buf;
        WAITQ_T	openack_wait;
        WAITQ_T	closeack_wait;
        WAITQ_T	ioctlack_wait;
        WAITQ_T	read_wait;
} MGMT;


/*
 * ISDN audiotap structure
 */
typedef struct
{
	int	opencnt;
	int	dxbchan;

        int	error_code;
	int	eof;

        WAITQ_T	openack_wait;
        WAITQ_T	closeack_wait;
        WAITQ_T	read_wait;

} AUDIOTAP;


/*
 * eye quality measurement structure
 */
typedef struct
{
        int     opencnt;
        int     dxbchan;
 
        int     error_code;
        int     eof;
 
        WAITQ_T	openack_wait;
        WAITQ_T	closeack_wait;
        WAITQ_T	read_wait;
 
} EQM;



/*
 * The RW interface is very slow, and kme issues memory
 * reads 1, 2, or 4 bytes at a time.  So we need a cache
 * to fully utilize the full 128 byte rw_data[].  One
 * entry is probably all we need, though.
 */
typedef struct
{
	ulong	device;		/* E.G. DSP number */
	ulong	pmdm;		/* E.G. DSP memory space */
	ulong	cacheline;	/* Address/128 of cacheline */
	ulong	jiffies;	/* Jiffy when data is valid */
	uchar	data[128];	/* The cached data */
} CACHE;


/*
 * Tty parameters sent to FEP.	We cache these so we only
 * write them to the FEP when they change.
 */
typedef struct
{
	ushort	cflag;
	ushort	iflag;
	ushort	oflag;
	uchar	stopc;
	uchar	startc;
	uchar	stopca;
	uchar	startca;
	uchar	hflow;
} PARAM;

/*
 * A structure to hold a statistics counter.  We also
 * compute moving averages for this counter.
 */
typedef struct
{
	uint64		cnt;	/* Total count */
	ulong		accum;	/* Acuumulator per period */
	ulong		sma;	/* Simple moving average */
	ulong		ema;	/* Exponential moving average */
} MACOUNTER;



/*
 * Drivers per-channel information
 */
typedef struct _drvchan
{
	int		channum;	/* Channel number (users view) */
	int		opencnt;	/* Reference count */
	int		blocked_open;	/* Reference count */

	int		dxbchan;	/* DXB channel number */

	int		poll_ok;	/* Poll only after openack */
					/* and only until close */

	long		maxcred;	/* maximum credits seen */
	SEMAPHORE	ch_lock;	/* provide for serialization */

	/*
	 * The usual assortment of flags for serial devices
	 * These are public, see rascon.h
	 */
	int		flags;

	/*
	 * State bits set atomically with set_bit() et al.
	 */
	ulong		state;
			#define STATE_TXBUSY	1
			#define	STATE_LOWWAIT	2
			#define	STATE_RXSTOP	3
			#define	STATE_RXAVAIL	4

	/*
	 * Pointers back to frequently needed related structures
	 */
	struct _board	*brd;		/* BOARD_T pointer */
	TTY		*tty;

	/*
	 * Termio parameters
	 */
	TERMIOS		normal_termios;
	TERMIOS		callout_termios;
	PARAM		old;
	PARAM		new;
	tcflag_t	ld_oflag;	/* what to tell the LD */
	tcflag_t	user_oflag;	/* what the user told us */


	/*
	 * Alternate XON and XOFF characters
	 */
	uchar		xxon;	
	uchar		xxoff;

	/*
	 * The modem control positions can vary depending on FEP
	 * Here are the values for us to use.
	 */
	uchar		m_rts;
	uchar		m_dcd;
	uchar		m_dsr;
	uchar		m_cts;
	uchar		m_ri;
	uchar		m_dtr;

	/*
	 * Transparent Print configuration
	 */
	int		xp_maxcps;	/* Max sustained printing rate */
	int		xp_maxchar;	/* Allowable data in print queue */
	int		xp_bufsize;	/* size of printer buffer */
	int		xp_onlen;	/* length of the txprint on string */
	int		xp_offlen;	/* length of the txprint off string */
	char		xp_onstr[SDXB_TXP_PLEN];	/* txprint on string */
	char		xp_offstr[SDXB_TXP_PLEN];	/* txprint off string*/
	char		xp_term[SDXB_TXP_TLEN];		/* terminal type */

	/*
	 * /proc/dg/dgdm/<board>/tty statistics
	 */
	#define		RxSTAT	0	/* Bytes Rx'ed */
	#define		TxSTAT	1	/* Bytes Tx'ed */
	#define		NSTAT	2	/* Total number of counters */
	MACOUNTER	stat[NSTAT];	/* the counters thenselves */

	/*
	 * queues and process information
	 */
	int		close_delay;
	int		custom_rate;

	WAITQ_T		close_wait;
	WAITQ_T		open_wait;

	int		error_code;
	WAITQ_T		openack_wait;
	WAITQ_T		closeack_wait;

	long		pgrp;		/* Process Group */
	long		session;

	/*
	 * Current view of board state
	 */
	uchar		mstat;

	uchar		sendcmds; /* commands needed to send */
			#define	SEND_DTRDOWN	1
			#define SEND_TXFLUSH	2
	
    ushort		syncseq;
	WAITQ_T		sync_wait;

	long		txaccm;
	long		rxaccm;

	/* for parity scanning, when we must */
	long		pscan_state;
	uchar		pscan_savechar;
	int		needwake;	/* Flag indicating TTY wants wake */
	int		dtrlow;		/* time when dtr went low */

	/* Cook buffer and controls */
	uchar		*CookBuf;
	int		CookCount;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
	spinlock_t	CookLock;
#endif

} DRVCHAN;



/*
 * Per minor number [0..NISDN] DXB STREAMS channel structure.
 */
typedef struct _dxb
{
	ulong		state;		/* Channel state for device. */
			#define	DXBSTRM_IDLE	0x00
			#define	DXBSTRM_OPENING	0x01
			#define	DXBSTRM_OPEN	0x02
			#define	DXBSTRM_CLOSE	0x04

	int		dxbchan;	/* DXB channel number */

	int		error_code;

	/*
	 * DxB channel bufferage.
	 */
	DXB_STRM_MESSAGE rx_msg;	/* User read request. */
	char		*user_buffer;	/* User read buffer. */
	uchar		*rx_buffer;	/* Read buffer. */
	int		rx_buffer_size;	/* Length of read buffer. */
	int		rx_in;		/* Offset to place new data. */
	int		rx_out;		/* Offset to retrieve data. */
	int		rx_buffer_bytes;/* Number of bytes in buffer. */
	int		rx_dropped;	/* Packets dropped due to no space. */

	uchar		*tx_buffer;	/* Write buffer. */
	int		tx_buffer_size;	/* Length of write buffer. */

	/*
	 * queues and process information
	 */
	WAITQ_T		openack_wait;	/* Pended dgdm_isdn_openack_cb. */
	WAITQ_T		closeack_wait;	/* Pended dgdm_isdn_closeack_cb. */
	WAITQ_T		flushack_wait; /* Pended dgdm_isdn_flushack_cb. */
	WAITQ_T		pushack_wait;	/* Pended dgdm_isdn_pushack_cb. */
	WAITQ_T		popack_wait;	/* Pended dgdm_isdn_popack_cb. */
	WAITQ_T		linkack_wait;	/* Pended dgdm_isdn_linkack_cb. */
	WAITQ_T		unlinkack_wait;/* Pended dgdm_isdn_unlinkack_cb. */
	WAITQ_T		ioctl_wait;	/* Pended dgdm_isdn_ioctlack_cb. */
	int		mux_id;		/* link/unlink mux id. */

	int		ioctl_credits_needed;
	int		ioctl_errno;
	int		ioctl_data_len;

	WAITQ_T		read_wait;	/* Pended user read. */
	int		read_sleep_count;
	int		read_wake_count;
	WAITQ_T		write_wait;	/* Pended dgdm_isdb_write_data. */
	int		write_sleep_count;
	int		write_wake_count;
	int		write_credits_needed;

	/*
	 * /proc/dg/dgdm/<board>/dxbstrm statistics
	 */
	#define		STAT_StrRxBytes		0
	#define		STAT_StrRxPackets	1
	#define		STAT_StrTxBytes		2
	#define		STAT_StrTxPackets	3
	#define		NSTAT_Str		4/* Total number of counters */

	MACOUNTER	stat[NSTAT_Str];

#ifdef RAWDXBDRV
	int		very_raw;	/* very raw DXB, no assumptions */
#endif
} DXBSTRM;



/*
 *	Per-board information
 */
typedef struct _board
{
	int		boardnum;	/* Board number: 0-3 */
	int		firstminor;	/* First minor, e.g. 0, 30, 60 */

	char		*name;		/* Product Name */
	uint16		vendor;		/* PCI vendor ID */
	uint16		device;		/* PCI device ID */
	uint16		subvendor;	/* PCI subsystem vendor ID */
	uint16		subdevice;	/* PCI subsystem device ID */
	uchar		rev;		/* PCI revision ID */

	ulong		ram;		/* Physical RAM address */
	uint32		ramlen;		/* RAM length */
	void		*vram;		/* Virtual RAM address */

	ulong		reg;		/* Physical 64011 registers address */
	ulong		reglen;		/* Registers length */
	void		*vreg;		/* Virtual 64011 registers address */

	ulong		flash;		/* Physical flash address */
	ulong		flashlen;	/* Flash length in bytes */
	ulong		*vflash;	/* Virtual Flash address */
	int		flash_refcnt;	/* Usage count - mapped on open */
	FLASHAREA	*flasharea;	/* Flash information */
	int		flash_ided;	/* We have printed the flash ID */

					/* VPD for Houston only... */
	ulong		vpd;		/* Physical VPD or eprom address */
	ulong		vpdlen;		/* VPD or eprom length in bytes */
	uchar		*vvpd;		/* Virtual VPD or eprom address */
	int		vpd_refcnt;	/* Usage count - mapped on open */
	FLASHAREA	*vpdarea;	/* VPD Flash information */

	ulong		dspgpp1;	/* Physical DSPGPP1 address */
	DSPGPP		*vdspgpp1;	/* Virtual DSPGPP1 address */

	int		irq;		/* Interrupt request number */
	int		intr_count;	/* Count of interrupts */
	int		intr_used;	/* Non-zero if using interrupts */

	CACHE		rwcache;	/* Cache for RW requests */

	/*
	 *	Global common area in shared memory
	 */
	uint32		gco;		/* Offset of global common on board */
	GLOBAL_COMMON	*gcp;		/* Ptr global common data */
	int		bigendian_board;/* Board is running big-endian */

	/*
	 *	Global debug data in shared memory
	 */
	GLOBAL_DEBUG_DATA *gdd;		/* Ptr to area in shared memory */
	ulong		dma_buf[2];	/* Phyaddr of two 4k DMA buffers */

	/*
	 *	tty data
	 */
	int		board_type;	/* Type of board returned by POST */
	int		memsize;	/* size of memory returned by POST */
	int		ndsps;		/* # of DSPs on board */
	int		nttychans;	/* # of tty channels (24,30,48,60) */
	int		nxpchans;	/* # of transparent print channels */
	int		nspecchans;	/* # of special channels on board */
	int		nchans;		/* # of ports on board, e.g. 24, 30 */
	int		geoaddr;	/* geographic address on cPCI */ 
	DRVCHAN		*dc;		/* array of per-channel info */

	/*
	 *	DXB data
	 */
	DXB		*dxb;		/* DXB instance */
	void		*dxb_real;	/* actual malloc'ed pointer */
	WAITQ_T		dxb_lock_wait;	/* Used to serialize DXB access */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
	spinlock_t	dxb_spinlock;	/* Used to protect dxb_locked */
#endif
	int		dxb_locked;	/* Flag indicating DXB is locked */
	int		poller_locked;  /* indicates poller is running */
	int		dxb_needpoll;	/* Flag indicating DXB wants poll */
	int		dxb_needwkup;
	int		poll_processor;	/* processor in DXB poller routine */
	int		deferred_polls;	/* Count of deferred polls */

	GLOBAL_HOST_DATA *ghd;		/* Global host data */
	GLOBAL_FEP_DATA	*gfd;		/* Board global struct */

	/*
	 *	DxB ISDN user channel access.
	 */
	DXBSTRM		*dxbisdn_chans;	/* Array of user channels. */

	/*
	 *	Audiotap
	 */
	AUDIOTAP	audio;

	/*
	 *	Eye quality, PCM data monitor from PUMP
	 */
	EQM		eqm;

	/*
	 *	Mgmt.
	 */
	MGMT		mgmt;


	/*
	 *	Triboot
	 */
	TRIBL_BOARD	tribl;
	BOARD		*triboard;
	TRIBL_POST	*post;
        char		*msgbuf_head;
        char		*msgbuf;

} BOARD_T;


typedef struct {
    PADDRS addrs[NPCIADDRS];
} PADDRS_T;



/*
 *  Debugger character queue.
 *
 *  May point to a buffer anywhere in host accessible memory,  will
 *  never cross a page boundary, and may have any size (eg 1234 bytes).
 */
#define CQ_CMD	    0xf40		/* Command Buffer origin */
#define CQ_RESP	    0xf50		/* Response Buffer origin */

typedef struct cq
{
    ulong	cq_base;		/* Host relative buffer origin */
    ulong	cq_in;			/* Buffer relative in pointer */
    ulong	cq_out;			/* Buffer relative out pointer */
    ulong	cq_size;		/* Buffer size (NOT power-of-2) */
} cq_t;







/*************************************************************************
 *
 * Prototypes for non-static functions used in more than one module
 *
 *************************************************************************/

GLOBAL int		dxb_lock_noblock(BOARD_T *brd);
GLOBAL void		dxb_lock(BOARD_T *brd);
GLOBAL void		dxb_unlock(BOARD_T *brd);
GLOBAL int		dxb_lock_poller(BOARD_T *brd);
GLOBAL void		dxb_unlock_poller(BOARD_T *brd);
GLOBAL void		dgdm_mbuf(BOARD_T *brd, const char *fmt, ...);
GLOBAL int              dgdm_sleep(int ticks);

GLOBAL int		dgdm_mgmt_open(BOARD_T *brd);
GLOBAL void		dgdm_mgmt_close(BOARD_T *brd);
GLOBAL int		dgdm_mgmt_read(BOARD_T *brd, READ_PROTO);
GLOBAL int		dgdm_mgmt_ioctl(unsigned int minor, FILE *file,
                                        unsigned int cmd, unsigned long arg);
GLOBAL int		dgdm_eqm_open(BOARD_T *brd);
GLOBAL void		dgdm_eqm_close(BOARD_T *brd);
GLOBAL int		dgdm_eqm_read(BOARD_T *brd, READ_PROTO);
GLOBAL int		dgdm_eqm_write(BOARD_T *brd, WRITE_PROTO);

GLOBAL int		dgdm_audiotap_open(BOARD_T *brd);
GLOBAL void		dgdm_audiotap_close(BOARD_T *brd);
GLOBAL int		dgdm_audiotap_read(BOARD_T *brd, READ_PROTO);
GLOBAL int		dgdm_audiotap_write(BOARD_T *brd, WRITE_PROTO);

GLOBAL int		dgdm_dxbread(DXBQ *q, READ_PROTO);

GLOBAL void		dgdm_tty_poll_board(BOARD_T *brd, int polltx,
                                            int pollrx);
GLOBAL int		dgdm_tty_preinit(BOARD_T *brd);
GLOBAL int		dgdm_tty_brdinit(BOARD_T *brd);
GLOBAL void		dgdm_tty_brduninit(BOARD_T *brd);
GLOBAL int		dgdm_tty_init(void);
GLOBAL void		dgdm_tty_uninit(void);

#if defined(DGDM_TRACER)
GLOBAL void		dgdm_tracef(const char *fmt, ...);
GLOBAL void		dgdm_tracer_free(void);
#endif

GLOBAL void		dgdm_proc_unregister_all(void);
GLOBAL void		dgdm_proc_register_basic(void);
GLOBAL void		dgdm_proc_register_fep(void);

GLOBAL GLOBAL_COMMON	*dgdm_get_gc_ptr(BOARD_T *brd);

GLOBAL struct file 	*dgdm_file_open(struct file *filep, char *filename,
                                        int flags, int mode);
GLOBAL int		dgdm_file_read(struct file *filep, char *buf,
                                       int count);
GLOBAL void		dgdm_file_close(struct file *filep);
