/* -*- linux-c -*- (Autoset linux indent style when using emacs)        */

/*
 * Copyright 1999 by Digi International (www.digi.com)
 *    Jeff Randall <Jeff_Randall@digi.com>
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the 
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/************************************************************************
 *
 * This file implements the tty driver functionality for the
 * DXB2 based product lines.
 *
 ************************************************************************
 * $Id: dgdm_tty.c,v 1.75 2002/04/23 12:37:07 sethb Exp $
 */

#define __NO_VERSION__
#include "dgdm_driver.h"

#include <linux/tty_flip.h>
#if defined(POSXDIR)
# include <bits/posix_opt.h>	/* for _POSIX_VDISABLE */
#else
# include <posix_opt.h>		/* for _POSIX_VDISABLE */
#endif


/*
 * external variables
 */
extern BOARD_T          *dgdm_Board[MAXBOARDS];
extern uchar            dgdm_NumBoards;
extern uchar            dgdm_Major_Serial_Registered;
extern uchar            dgdm_Major_Callout_Registered;
extern uchar            dgdm_Major_TransparentPrint_Registered;
extern uchar            dgdm_Major_ISDN_Registered;
extern uchar            dgdm_Major_Control_Registered;
extern uchar            dgdm_Major_Serial2_Registered;
extern uchar            dgdm_Major_Callout2_Registered;
extern uchar            dgdm_Major_TransparentPrint2_Registered;
extern uchar            dgdm_Major_ISDN2_Registered;
extern int		dgdm_tty;
extern int		dgdm_print;
extern int		dgdm_debug;
extern int		dgdm_major;
extern int		dgdm_latency;
extern int		dgdm_rxbufsize;
extern int		dgdm_loopback;
extern int		dgdm_cook_size;
extern int		dgdm_rawreadok;
extern int		dgdm_dxb_align;
extern int		dgdm_intr_period;
extern int		dgdm_erase_nvm;
extern char		*dgdm_dxbttyname;

#if ALLOW_ALT_FAIL_OPEN
extern int      dgdm_alt_fail_open;
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
extern spinlock_t	dgdm_global_lock;
#endif



/*
 * forward declarations for functions used only in this file.
 */
STATIC int	dgdm_tty_open(TTY *tty, FILE *file);
STATIC int	block_til_ready(TTY *tty, FILE *file, DRVCHAN *dc);
STATIC int	openack_cb(DXB *dxb, int channel, int error_code, DXB_ARG arg);
 
STATIC void	dgdm_tty_close(TTY *tty, FILE *file);
STATIC int	dgdm_tty_shutdown(DRVCHAN *dc);
STATIC void	dgdm_tty_hangup(TTY *tty);
STATIC int	closeack_cb(DXB *dxb, int channel, DXB_ARG arg);

STATIC int	dgdm_tty_write(TTY *tty, int from_user,
                               const unsigned char *buf, int count);
STATIC void	dgdm_tty_flush_chars(TTY *tty);
STATIC void	dgdm_tty_put_char(TTY *tty, unsigned char ch);
STATIC void	dgdm_tty_flush_buffer(TTY *tty);
STATIC int	dgdm_tty_write_room(TTY *tty);
STATIC int	dgdm_tty_chars_in_buffer(TTY *tty);

STATIC int	dgdm_tty_ioctl(TTY *tty, FILE *file, unsigned int cmd,
                               unsigned long arg);
STATIC void	dgdm_tty_settermios(TTY *tty, TERMIOS *old);
STATIC void	dgdm_tty_param(TTY *tty, DRVCHAN *dc, TERMIOS *old_termios);
STATIC uint32	dgdm_tty_getbaud(TTY *tty, DRVCHAN *dc);

STATIC int	dgdm_tty_getserial(TTY *tty, struct serial_struct *retinfo);
STATIC int	dgdm_tty_setserial(TTY *tty, struct serial_struct *new_info);
STATIC int	dgdm_tty_digigeta(TTY *tty, struct digi_struct *retinfo);
STATIC int	dgdm_tty_digiseta(TTY *tty, struct digi_struct *new_info);
STATIC void	dgdm_tty_throttle(TTY *tty);
STATIC void	dgdm_tty_unthrottle(TTY *tty);
STATIC void	dgdm_tty_stop(TTY *tty);
STATIC void	dgdm_tty_start(TTY *tty);
STATIC void	dgdm_tty_input_stop(TTY *tty);
STATIC void	dgdm_tty_input_start(TTY *tty);

STATIC int	get_modem_info(DRVCHAN *dc, unsigned int *value);
STATIC int	set_modem_info(DRVCHAN *dc, unsigned int command,
                               unsigned int *value);

STATIC void	dgdm_tty_poll(unsigned long ignored);
GLOBAL void	dgdm_tty_poll_board(BOARD_T *brd, int polltx, int pollrx);

STATIC int	data_cb(DXB *dxb, int channel, int data_len, DXB_ARG arg);
STATIC int	cdata_cb(DXB *dxb, int channel, int pri, int ctrl_len,
                         int data_len, DXB_ARG arg);
STATIC inline int cdata_cb_mstat(DRVCHAN *dc, char *cp, int len);
STATIC inline int cdata_cb_sync(DRVCHAN *dc, char *cp, int len);
STATIC inline int cdata_cb_seq(DRVCHAN *dc, char *cp, int len);
STATIC inline int cdata_cb_screcv(DRVCHAN *dc, char *cp, int len);
STATIC inline int cdata_cb_event(DRVCHAN *dc, char *cp, int len);
STATIC inline int cdata_cb_txprint(DRVCHAN *dc, char *cp, int len);

STATIC int	credit_cb(DXB *dxb, int channel, int credits, DXB_ARG arg);

STATIC void	parity_scan(DRVCHAN *dc, unsigned char *cbuf,
                            unsigned char *fbuf, int *len);

STATIC void	dxb_rxbuf(BOARD_T *brd, DRVCHAN *dc);
STATIC void	moving_average(MACOUNTER *map, uint randnum);
STATIC void	dxb_start_timer(void);
STATIC void	*kzmalloc(size_t size, int priority);
GLOBAL int	dgdm_dxbread(DXBQ *q, READ_PROTO);
 
STATIC int	sdxbsyncwait(DRVCHAN *dc, int hipri, int drain);
STATIC int	sdxbcmd(DRVCHAN *dc, int wait, int hipri, void *cmd, int sz);
STATIC void	send_deferred_commands(DRVCHAN *dc, int lock);



/*
 * tty globals/statics
 */
STATIC char		*SerialName = "ttyG";
STATIC char		*CalloutName = "cug";
STATIC char		*PrintName = "lpG";

STATIC int		TtyRefCnt = 0;
STATIC int		TimerOn = 0;
STATIC int		poller_running = 0;

STATIC TTYDRIVER	SerialDriver;
STATIC TTYDRIVER	CalloutDriver;
STATIC TTYDRIVER	PrintDriver;

#if MAXBOARDS > 4
STATIC TTYDRIVER	Serial2Driver;
STATIC TTYDRIVER	Callout2Driver;
STATIC TTYDRIVER	Print2Driver;
#endif

STATIC uchar		*TmpWriteBuf = NULL;

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
STATIC DECLARE_MUTEX(TmpWriteSem);
#else
STATIC struct semaphore	TmpWriteSem = MUTEX;
#endif


#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
STATIC struct timer_list Timer = { function: dgdm_tty_poll };
#else
STATIC struct timer_list Timer = { NULL, NULL, 0, 0, dgdm_tty_poll };
#endif

GLOBAL unsigned char	dgdm_bitbucket[PAGE_SIZE*2];


/*
 * Define a local default termios struct.  See dgdm_driver.h for the defaults
 * used.
 *
 * Note: the structure is filled out in dgdm_tty_preinit() because the order
 *       of the structure can apparently change between kernel versions and
 *       architectures - therefore we can't just define it here.
 */
STATIC TERMIOS DefaultTermios;


/*
 * CBAUDS And CBAUDEXS define the baudrate table - this table matches the
 * one used by serial.c as much as possible (1228800 is a new rate).
 */
STATIC uint32 CBAUDS[] =
{
        0, 50, 75, 110, 134, 150, 200, 300, 600,
        1200, 1800, 2400, 4800, 9600, 19200, 38400
};
STATIC uint32 CBAUDEXS[] =
{
	0, 57600, 115200, 230400, 460800, 500000, 576000, 921600, 1228800,
	1200, 1800, 2400, 4800, 9600, 19200, 38400
};
#define CBAUD2NUM(x)  (((x) & CBAUDEX) ? \
			CBAUDEXS[(x)&(CBAUD & ~CBAUDEX)] : CBAUDS[(x)&CBAUD])

#define CIBAUD_SHIFT	16








/************************************************************************
 *
 * TTY Initialization/Cleanup Functions
 *
 ************************************************************************/

/*
 * dgdm_tty_preinit()
 *
 * Initialize any tty and dxb related data before we download the board
 */
GLOBAL int dgdm_tty_preinit(BOARD_T *brd)
{
	int rc;
	unsigned long flags;

        if(!brd)
                return(-ENODEV);

	DPR_BASIC(("dgdm_tty_preinit(%d)\n", brd->boardnum));

	/*
	 * Allocate space for the DXB structure. The DXB structure
	 * must be malloc'ed in order for it to be DMAable. Static
	 * data (like dgdm_Board[]) doesn't work, and I don't know why yet.
	 *
	 * NOTE: There is a potential problem with the 64011 DMA controller.
	 *       We believe that we need to align the DXB structure on a
	 *       cacheline boundary to avoid this problem.  When the dxb_align
	 *       flag is set, the code will guarantee this.
	 *
	 *       The DXB structure itself has been crafted all along
	 *       to preserve alignment of the in/out pointer variables
	 *       on multiples of 32 bytes.
	 */
	if(dgdm_dxb_align) {
		ulong addr;

		brd->dxb_real = kzmalloc(sizeof(DXB) + 32, GFP_KERNEL);
		if(!brd->dxb_real)
			return(-ENOMEM);
		addr = (ulong) brd->dxb_real;
		addr = (addr + 31) & ~31;
		brd->dxb = (DXB *) addr;
		dgdm_mbuf(brd, DRVSTR":  dxb structure (%d bytes) is "
                          "aligned at %lx\n", sizeof(DXB), addr);
	} else
		brd->dxb_real = brd->dxb =
                        kzmalloc(sizeof(DXB), GFP_KERNEL);

	if(!brd->dxb)
		return(-ENOMEM);

	/*
	 * Init DXB library, allocating circular buffers as
	 * several virtually discontinuous pieces totalling 128KB.
	 */
	DGDM_LOCK(dgdm_global_lock, flags);
	rc = dxb_init(brd->dxb, 32, 4096, DXB_OPENSIDE);
	if(rc < 0) {
                dgdm_tty_brduninit(brd);
                DGDM_UNLOCK(dgdm_global_lock, flags);
		dgdm_mbuf(brd, DRVSTR":  dxb_init failure %d\n", rc);
		return(-ENOMEM);
	}

	/*
	 * Allocate a page for the GLOBAL_HOST_DATA (GHD), which
	 * must be contiguous in phyiscal memory
	 */
	brd->ghd = (GLOBAL_HOST_DATA *) get_free_page(GFP_KERNEL);
	if(brd->ghd == NULL) {
                dgdm_tty_brduninit(brd);
                DGDM_UNLOCK(dgdm_global_lock, flags);
		dgdm_mbuf(brd, DRVSTR":  Can't get a page for global "
                          "host data\n");
                return(-ENOMEM);
	}

	/*
	 * Mark the structure with a magic number so the
	 * board knows it is seeing the right thing.  Set
	 * the interrupt period and NVM erase flags as desired.
	 */
	brd->ghd->magic = MAGIC_HOST;
	brd->ghd->intr_period = brd->irq ? dgdm_intr_period : 0;
	brd->ghd->erase_nvm = dgdm_erase_nvm;

	/*
	 * Compute physical (bus) addresses of buffer pieces and
	 * in/out pointers.
	 */
	dxb_set_phys(brd->dxb, &brd->ghd->dxbphys);

	/* Leave a pointer to the GHD in shared memory */
	{
		uint32 *ghdo = F2H(GLOBAL_HOST_OFFSET);

		*ghdo = LE32(virt_to_bus(brd->ghd));
	}

	/*
	 * We need a semaphore to serialize dxb access.  Init it.
         * And if we have them, we need a spinlock as well.
	 */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
	brd->dxb_spinlock = SPIN_LOCK_UNLOCKED;
#endif

	/* Prepare the Default Termios structure. */
	memset(&DefaultTermios, 0, sizeof(struct termios));
	memcpy(DefaultTermios.c_cc, INIT_C_CC, NCCS);
	DefaultTermios.c_iflag = DEFAULT_IFLAGS;
	DefaultTermios.c_oflag = DEFAULT_OFLAGS;
	DefaultTermios.c_cflag = DEFAULT_CFLAGS;
	DefaultTermios.c_lflag = DEFAULT_LFLAGS;

	DGDM_UNLOCK(dgdm_global_lock, flags);
	return(0);

}



/*
 * dgdm_tty_init()
 *
 * Init the tty subsystem.  Called after all boards have been
 * downloaded and initted.
 */
GLOBAL int dgdm_tty_init(void)
{
	int rc;
	int maxminor;

	DPR_BASIC(("dgdm_tty_init()\n"));

	/*
	 * Allocate a buffer for doing the copy from user space to
	 * kernel space in dxb_write().  We only use one buffer and
	 * control access to it with a semaphore.  If we are paging, we
	 * are already in trouble so one buffer won't hurt much anyway.
	 */
	TmpWriteBuf = kmalloc(WRITEBUFLEN, GFP_KERNEL);
	if(!TmpWriteBuf)
		return(-ENOMEM);

	maxminor = dgdm_NumBoards * 64;

	memset(&SerialDriver, 0, sizeof(TTYDRIVER));
	memset(&CalloutDriver, 0, sizeof(TTYDRIVER));
	memset(&PrintDriver, 0, sizeof(TTYDRIVER));

        #if MAXBOARDS > 4
	memset(&Serial2Driver, 0, sizeof(TTYDRIVER));
	memset(&Callout2Driver, 0, sizeof(TTYDRIVER));
	memset(&Print2Driver, 0, sizeof(TTYDRIVER));
        #endif

	SerialDriver.magic = TTY_DRIVER_MAGIC;
	SerialDriver.name = SerialName;
	SerialDriver.name_base = 0;
	SerialDriver.major = dgdm_major + DGDM_SERIAL_MAJOR;
	SerialDriver.minor_start = 0;
	SerialDriver.num = MIN(maxminor,256);
	SerialDriver.type = TTY_DRIVER_TYPE_SERIAL;
	SerialDriver.subtype = SERIAL_TYPE_NORMAL;
	SerialDriver.init_termios = DefaultTermios;
	SerialDriver.flags = TTY_DRIVER_REAL_RAW;
	SerialDriver.refcount = &TtyRefCnt;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
	SerialDriver.driver_name = DRVSTR "(dgdm)";;
#endif

	/*
	 * The kernel wants space to store pointers to
	 * tty_struct's and termios's.
	 */
	SerialDriver.table = kzmalloc(MIN(maxminor,256) * sizeof(TTY *),
                                      GFP_KERNEL);
	if(!SerialDriver.table)
		return(-ENOMEM);

	SerialDriver.termios = kzmalloc(MIN(maxminor,256) * sizeof(TERMIOS *),
                                        GFP_KERNEL);
	if(!SerialDriver.termios)
		return(-ENOMEM);

	SerialDriver.termios_locked =
                kzmalloc(MIN(maxminor,256) * sizeof(TERMIOS *), GFP_KERNEL);
	if(!SerialDriver.termios_locked)
		return(-ENOMEM);

	/*
	 * Entry points for driver.  Called by the kernel from
	 * tty_io.c and n_tty.c.
	 */
	SerialDriver.open = dgdm_tty_open;
	SerialDriver.close = dgdm_tty_close;
	SerialDriver.write = dgdm_tty_write;
	SerialDriver.put_char = dgdm_tty_put_char;
	SerialDriver.flush_chars = dgdm_tty_flush_chars;
	SerialDriver.write_room = dgdm_tty_write_room;
	SerialDriver.flush_buffer = dgdm_tty_flush_buffer;
	SerialDriver.chars_in_buffer = dgdm_tty_chars_in_buffer;
	SerialDriver.ioctl = dgdm_tty_ioctl;
	SerialDriver.set_termios = dgdm_tty_settermios;
	SerialDriver.throttle = dgdm_tty_throttle;
	SerialDriver.unthrottle = dgdm_tty_unthrottle;
	SerialDriver.stop = dgdm_tty_stop;
	SerialDriver.start = dgdm_tty_start;
	SerialDriver.hangup = dgdm_tty_hangup;

	CalloutDriver = SerialDriver;
	CalloutDriver.name = CalloutName;
	CalloutDriver.major = dgdm_major + DGDM_CALLOUT_MAJOR;
	CalloutDriver.minor_start = 0;
	CalloutDriver.subtype = SERIAL_TYPE_CALLOUT;


	#if MAXBOARDS > 4
        if(dgdm_NumBoards > 4) {
                Serial2Driver.magic = TTY_DRIVER_MAGIC;
                Serial2Driver.name = SerialName;
                Serial2Driver.name_base = 0;
                Serial2Driver.major = dgdm_major + DGDM_SERIAL2_MAJOR;
                Serial2Driver.minor_start = 0;
                Serial2Driver.num = (maxminor - 256);
                Serial2Driver.type = TTY_DRIVER_TYPE_SERIAL;
                Serial2Driver.subtype = SERIAL_TYPE_NORMAL;
                Serial2Driver.init_termios = DefaultTermios;
                Serial2Driver.flags = TTY_DRIVER_REAL_RAW;
                Serial2Driver.refcount = &TtyRefCnt;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
                Serial2Driver.driver_name = DRVSTR "(dgdm)";;
#endif

                /*
                 * The kernel wants space to store pointers to
                 * tty_struct's and termios's.
                 */
                Serial2Driver.table = kzmalloc((maxminor - 256) *
                                               sizeof(TTY *),
                                               GFP_KERNEL);
                if(!Serial2Driver.table)
                        return(-ENOMEM);

                Serial2Driver.termios = kzmalloc((maxminor - 256) *
                                                 sizeof(TERMIOS *),
                                                 GFP_KERNEL);
                if(!Serial2Driver.termios)
                        return(-ENOMEM);

                Serial2Driver.termios_locked =
                        kzmalloc((maxminor - 256) * sizeof(TERMIOS *),
                                 GFP_KERNEL);
                if(!Serial2Driver.termios_locked)
                        return(-ENOMEM);

                /*
                 * Entry points for driver.  Called by the kernel from
                 * tty_io.c and n_tty.c.
                 */
                Serial2Driver.open = dgdm_tty_open;
                Serial2Driver.close = dgdm_tty_close;
                Serial2Driver.write = dgdm_tty_write;
                Serial2Driver.put_char = dgdm_tty_put_char;
                Serial2Driver.flush_chars = dgdm_tty_flush_chars;
                Serial2Driver.write_room = dgdm_tty_write_room;
                Serial2Driver.flush_buffer = dgdm_tty_flush_buffer;
                Serial2Driver.chars_in_buffer = dgdm_tty_chars_in_buffer;
                Serial2Driver.ioctl = dgdm_tty_ioctl;
                Serial2Driver.set_termios = dgdm_tty_settermios;
                Serial2Driver.throttle = dgdm_tty_throttle;
                Serial2Driver.unthrottle = dgdm_tty_unthrottle;
                Serial2Driver.stop = dgdm_tty_stop;
                Serial2Driver.start = dgdm_tty_start;
                Serial2Driver.hangup = dgdm_tty_hangup;

                Callout2Driver = SerialDriver;
                Callout2Driver.name = CalloutName;
                Callout2Driver.major = dgdm_major + DGDM_CALLOUT2_MAJOR;
                Callout2Driver.minor_start = 0;
                Callout2Driver.subtype = SERIAL_TYPE_CALLOUT;
        }
	#endif

	/*
	 * If we're doing transparent print, we have to do all of the above
	 * again, seperately so we don't get the LD confused about what major
	 * we are when we get into the dgdm_tty_open() routine.
	 */
	if(dgdm_print) {
		PrintDriver.magic = TTY_DRIVER_MAGIC;
		PrintDriver.name = PrintName;
		PrintDriver.name_base = 0;
		PrintDriver.major = dgdm_major + DGDM_PRINT_MAJOR;
		PrintDriver.minor_start = 0;
		PrintDriver.num = MIN(maxminor, 256);
		PrintDriver.type = TTY_DRIVER_TYPE_SERIAL;
		PrintDriver.subtype = SERIAL_TYPE_NORMAL;
		PrintDriver.init_termios = DefaultTermios;
		PrintDriver.flags = TTY_DRIVER_REAL_RAW;
		PrintDriver.refcount = &TtyRefCnt;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
		PrintDriver.driver_name = DRVSTR "(dgdm)";;
#endif

		/*
		 * The kernel wants space to store pointers to
		 * tty_struct's and termios's.  Must be seperate from
		 * the Serial Driver so we don't get confused 
		 */
		PrintDriver.table = kzmalloc(MIN(maxminor,256) * sizeof(TTY *),
						  GFP_KERNEL);
		if(!PrintDriver.table)
			return(-ENOMEM);

		PrintDriver.termios =
                        kzmalloc(MIN(maxminor,256) * sizeof(TERMIOS *),
                                 GFP_KERNEL);
		if(!PrintDriver.termios)
			return(-ENOMEM);

		PrintDriver.termios_locked =
                        kzmalloc(MIN(maxminor,256) * sizeof(TERMIOS *),
                                 GFP_KERNEL);
		if(!PrintDriver.termios_locked)
			return(-ENOMEM);

		/*
		 * Entry points for driver.  Called by the kernel from
		 * tty_io.c and n_tty.c.
		 */
		PrintDriver.open = dgdm_tty_open;
		PrintDriver.close = dgdm_tty_close;
		PrintDriver.write = dgdm_tty_write;
		PrintDriver.put_char = dgdm_tty_put_char;
		PrintDriver.flush_chars = dgdm_tty_flush_chars;
		PrintDriver.write_room = dgdm_tty_write_room;
		PrintDriver.flush_buffer = dgdm_tty_flush_buffer;
		PrintDriver.chars_in_buffer = dgdm_tty_chars_in_buffer;
		PrintDriver.ioctl = dgdm_tty_ioctl;
		PrintDriver.set_termios = dgdm_tty_settermios;
		PrintDriver.throttle = dgdm_tty_throttle;
		PrintDriver.unthrottle = dgdm_tty_unthrottle;
		PrintDriver.stop = dgdm_tty_stop;
		PrintDriver.start = dgdm_tty_start;
		PrintDriver.hangup = dgdm_tty_hangup;

                #if MAXBOARDS > 4
                if(dgdm_NumBoards > 4) {
                        Print2Driver.magic = TTY_DRIVER_MAGIC;
                        Print2Driver.name = PrintName;
                        Print2Driver.name_base = 0;
                        Print2Driver.major = dgdm_major + DGDM_PRINT2_MAJOR;
                        Print2Driver.minor_start = 0;
                        Print2Driver.num = maxminor - 256;
                        Print2Driver.type = TTY_DRIVER_TYPE_SERIAL;
                        Print2Driver.subtype = SERIAL_TYPE_NORMAL;
                        Print2Driver.init_termios = DefaultTermios;
                        Print2Driver.flags = TTY_DRIVER_REAL_RAW;
                        Print2Driver.refcount = &TtyRefCnt;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
                        Print2Driver.driver_name = DRVSTR "(dgdm)";;
#endif

                        /*
                         * The kernel wants space to store pointers to
                         * tty_struct's and termios's.  Must be seperate from
                         * the Serial Driver so we don't get confused 
                         */
                        Print2Driver.table = kzmalloc((maxminor - 256) *
                                                      sizeof(TTY *),
                                                      GFP_KERNEL);
                        if(!Print2Driver.table)
                                return(-ENOMEM);

                        Print2Driver.termios =
                                kzmalloc((maxminor - 256) * sizeof(TERMIOS *),
                                         GFP_KERNEL);
                        if(!Print2Driver.termios)
                                return(-ENOMEM);

                        Print2Driver.termios_locked =
                                kzmalloc((maxminor - 256) * sizeof(TERMIOS *),
                                         GFP_KERNEL);
                        if(!Print2Driver.termios_locked)
                                return(-ENOMEM);

                        /*
                         * Entry points for driver.  Called by the kernel from
                         * tty_io.c and n_tty.c.
                         */
                        Print2Driver.open = dgdm_tty_open;
                        Print2Driver.close = dgdm_tty_close;
                        Print2Driver.write = dgdm_tty_write;
                        Print2Driver.put_char = dgdm_tty_put_char;
                        Print2Driver.flush_chars = dgdm_tty_flush_chars;
                        Print2Driver.write_room = dgdm_tty_write_room;
                        Print2Driver.flush_buffer = dgdm_tty_flush_buffer;
                        Print2Driver.chars_in_buffer =dgdm_tty_chars_in_buffer;
                        Print2Driver.ioctl = dgdm_tty_ioctl;
                        Print2Driver.set_termios = dgdm_tty_settermios;
                        Print2Driver.throttle = dgdm_tty_throttle;
                        Print2Driver.unthrottle = dgdm_tty_unthrottle;
                        Print2Driver.stop = dgdm_tty_stop;
                        Print2Driver.start = dgdm_tty_start;
                        Print2Driver.hangup = dgdm_tty_hangup;
                }
                #endif
	}

	if(dgdm_tty) {
                if (!dgdm_Major_Serial_Registered) {
                        /* Register tty devices */
                        rc = tty_register_driver(&SerialDriver);
                        if(rc < 0) {
                                APR(("Can't register tty device (%d)\n", rc));
                                return(rc);
                        }
                        dgdm_Major_Serial_Registered = TRUE;
                }

                #if MAXBOARDS > 4
                if(dgdm_NumBoards > 4) {
                        if (!dgdm_Major_Serial2_Registered) {
                                rc = tty_register_driver(&Serial2Driver);
                                if(rc < 0) {
                                        APR(("Can't register high tty device "
                                             "(%d)\n", rc));
                                        return(rc);
                                }
                                dgdm_Major_Serial2_Registered = TRUE;
                        }
                }
		#endif
	}
	if(dgdm_tty) {
                if (!dgdm_Major_Callout_Registered) {
                        /* Register cu (callout) devices */
                        rc = tty_register_driver(&CalloutDriver);
                        if(rc < 0) {
                                APR(("Can't register cu device (%d)\n", rc));
                                return(rc);
                        }
                        dgdm_Major_Callout_Registered = TRUE;
                }

                #if MAXBOARDS > 4
                if(dgdm_NumBoards > 4) {
                        if (!dgdm_Major_Callout2_Registered) {
                                rc = tty_register_driver(&Callout2Driver);
                                if(rc < 0) {
                                        APR(("Can't register high cu device "
                                             "(%d)\n", rc));
                                        return(rc);
                                }
                                dgdm_Major_Callout2_Registered = TRUE;
                        }
                }
                #endif
	}
	if(dgdm_tty && dgdm_print) {
                if (!dgdm_Major_TransparentPrint_Registered) {
                        /* Register Transparent Print devices */
                        rc = tty_register_driver(&PrintDriver);
                        if(rc < 0) {
                                APR(("Can't register Transparent Print "
                                     "device (%d)\n", rc));
                                return(rc);
                        }
                        dgdm_Major_TransparentPrint_Registered = TRUE;
                }

                #if MAXBOARDS > 4
                if(dgdm_NumBoards > 4) {
                        if (!dgdm_Major_TransparentPrint2_Registered) {
                                rc = tty_register_driver(&Print2Driver);
                                if(rc < 0) {
                                        APR(("Can't register high Transparent "
                                             "Print device (%d)\n", rc));
                                        return(rc);
                                }
                                dgdm_Major_TransparentPrint2_Registered = TRUE;
                        }
                }
                #endif
	}

	/* Start the polling routine */
	dxb_start_timer();

	return(0);
}



/*
 * dgdm_tty_uninit()
 *
 * Uninitialize the TTY portion of this driver.  Free all memory and
 * resources.  Do it in reverse allocation order: this might possibly
 * result in less fragmentation of memory, though I don't know this
 * for sure.
 */
GLOBAL void dgdm_tty_uninit(void)
{
	if(TimerOn) {
		TimerOn = 0;
		del_timer(&Timer);
	}
	if(dgdm_Major_Serial_Registered) {
		tty_unregister_driver(&SerialDriver);
		dgdm_Major_Serial_Registered = FALSE;
	}
	if(dgdm_Major_Callout_Registered) {
		tty_unregister_driver(&CalloutDriver);
		dgdm_Major_Callout_Registered = FALSE;
        }
	if(dgdm_Major_TransparentPrint_Registered) {
		tty_unregister_driver(&PrintDriver);
		dgdm_Major_TransparentPrint_Registered = FALSE;
	}

        #if MAXBOARDS > 4
	if(dgdm_Major_Serial2_Registered) {
		tty_unregister_driver(&Serial2Driver);
		dgdm_Major_Serial2_Registered = FALSE;
	}
	if(dgdm_Major_Callout2_Registered) {
		tty_unregister_driver(&Callout2Driver);
		dgdm_Major_Callout2_Registered = FALSE;
	}
	if(dgdm_Major_TransparentPrint2_Registered) {
		tty_unregister_driver(&Print2Driver);
		dgdm_Major_TransparentPrint2_Registered = FALSE;
	}
        #endif

#if LINUX_VERSION_CODE <= KERNEL_VERSION(2,2,12)
	/* As of 2.2.13, this is done for us by tty_unregister_driver */
	if(SerialDriver.termios_locked) {
		kfree(SerialDriver.termios_locked);
		SerialDriver.termios_locked = NULL;
	}
	if(SerialDriver.termios) {
		kfree(SerialDriver.termios);
		SerialDriver.termios = NULL;
	}

	#if MAXBOARDS > 4
	if(Serial2Driver.termios_locked) {
		kfree(Serial2Driver.termios_locked);
		Serial2Driver.termios_locked = NULL;
	}
	if(Serial2Driver.termios) {
		kfree(Serial2Driver.termios);
		Serial2Driver.termios = NULL;
	}
	#endif


	/*
	 * the callout structure is the same as the Serial, but print is
	 * seperate and must be released as well
	 */
	if(dgdm_print) {
		if(PrintDriver.termios_locked) {
			kfree(PrintDriver.termios_locked);
			PrintDriver.termios_locked = NULL;
                }
		if(PrintDriver.termios) {
			kfree(PrintDriver.termios);
			PrintDriver.termios = NULL;
		}

                #if MAXBOARDS > 4
		if(Print2Driver.termios_locked) {
			kfree(Print2Driver.termios_locked);
			Print2Driver.termios_locked = NULL;
		}
		if(Print2Driver.termios) {
			kfree(Print2Driver.termios);
			Print2Driver.termios = NULL;
		}
                #endif
	}
#endif

	if(SerialDriver.table) {
		kfree(SerialDriver.table);
		SerialDriver.table = NULL;
	}
	if(dgdm_print) {
		if(PrintDriver.table) {
			kfree(PrintDriver.table);
			PrintDriver.table = NULL;
		}
	}

        #if MAXBOARDS > 4
	if(Serial2Driver.table) {
		kfree(Serial2Driver.table);
		Serial2Driver.table = NULL;
	}
	if(dgdm_print) {
		if(Print2Driver.table) {
			kfree(Print2Driver.table);
			Print2Driver.table = NULL;
		}
	}
        #endif

	if(TmpWriteBuf) {
		kfree(TmpWriteBuf);
		TmpWriteBuf = NULL;
	}
}



/*
 * dgdm_tty_brdinit()
 *
 * Init a single board for TTY operations.  Called once per board
 * just after the FEP is downloaded and fired up.
 */
GLOBAL int dgdm_tty_brdinit(BOARD_T *brd)
{
	GLOBAL_FEP_DATA *gfd;
	DRVCHAN *dc;
	int i;

        if(!brd)
                return(-ENODEV);

	DPR_BASIC(("dgdm_tty_brdinit(%d)\n", brd->boardnum));

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)

        init_waitqueue_head(&brd->dxb_lock_wait);

        if(brd->dxbisdn_chans) {
                DPR_BASIC(("dgdm_tty_brdinit(%d) - initing dxbisdn_chans "
                           "waitqueues\n", brd->boardnum));
                init_waitqueue_head(&brd->dxbisdn_chans->openack_wait);
                init_waitqueue_head(&brd->dxbisdn_chans->closeack_wait);
                init_waitqueue_head(&brd->dxbisdn_chans->flushack_wait);
                init_waitqueue_head(&brd->dxbisdn_chans->popack_wait);
                init_waitqueue_head(&brd->dxbisdn_chans->linkack_wait);
                init_waitqueue_head(&brd->dxbisdn_chans->unlinkack_wait);
                init_waitqueue_head(&brd->dxbisdn_chans->ioctl_wait);
                init_waitqueue_head(&brd->dxbisdn_chans->read_wait);
                init_waitqueue_head(&brd->dxbisdn_chans->write_wait);
        } else {
                DPR_BASIC(("dgdm_tty_brdinit(%d) - could not init "
                           "dxbisdn_chans waitqueues\n", brd->boardnum));
        }

        init_waitqueue_head(&brd->audio.openack_wait);
        init_waitqueue_head(&brd->audio.closeack_wait);
        init_waitqueue_head(&brd->audio.read_wait);

        init_waitqueue_head(&brd->eqm.openack_wait);
        init_waitqueue_head(&brd->eqm.closeack_wait);
        init_waitqueue_head(&brd->eqm.read_wait);

        init_waitqueue_head(&brd->mgmt.openack_wait);
        init_waitqueue_head(&brd->mgmt.closeack_wait);
        init_waitqueue_head(&brd->mgmt.ioctlack_wait);
        init_waitqueue_head(&brd->mgmt.read_wait);
#endif


	/* 
	 * The global pointer is already guaranteed to be pointing
	 * at a structure that is in STATUS_RUN when we get here.
	 * Insure that it really is the FEP.
	 */
	gfd = brd->gfd = (GLOBAL_FEP_DATA *) brd->gcp;
	if(gfd->common.magic != LE32(MAGIC_FEP)) {
		APR(("Bad FEP magic number %x\n", LE32(gfd->common.magic)));
		return(-ENXIO);
	}

	/*
	 * Allocate space for the drivers channel (dc) structures
	 * and initialize it.
	 */
	brd->dc = kzmalloc(brd->nchans * sizeof(DRVCHAN), GFP_KERNEL);
	if(!brd->dc) {
                DPR_BASIC(("dgdm_tty_brdinit(%d) - ENOMEM\n", brd->boardnum));
		return(-ENOMEM);
        }

	for (dc = brd->dc, i = 0; i < brd->nchans; ++i, ++dc) {
                DPR_BASIC(("dgdm_tty_brdinit(%d_%02d)\n", brd->boardnum, i));
		dc->channum = i;
		dc->brd = brd;
		dc->dxbchan = -1;

		dc->m_rts = SDXB_RTS;
		dc->m_dcd = SDXB_DCD;
		dc->m_dsr = SDXB_DSR;
		dc->m_cts = SDXB_CTS;
		dc->m_ri = SDXB_RI;
		dc->m_dtr = SDXB_DTR;
		dc->xxon = 0;
		dc->xxoff = 0;

		dc->xp_maxcps = 100;	/* default from ditty spec */
		dc->xp_maxchar = 50;	/* default from ditty spec */
		dc->xp_bufsize = 100;	/* assume no buffer on printer */
		dc->xp_onlen = 4;	/* length of default on string */
		dc->xp_offlen = 4;	/* length of default off string */

		/* default to a vt100 - most common terminal on the planet */
		(void) strncpy(dc->xp_onstr, "\033[5i", MIN(4, SDXB_TXP_PLEN));
		(void) strncpy(dc->xp_offstr,"\033[4i", MIN(4, SDXB_TXP_PLEN));
		(void) strncpy(dc->xp_term, "vt100", MIN(5, SDXB_TXP_TLEN));

		dc->close_delay = HZ / 2;	/* half second */
                
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
                init_MUTEX(&dc->ch_lock);	/* initialize the semaphore */

                /* init the waitqueue's in 2.4 */
                init_waitqueue_head(&dc->close_wait);
                init_waitqueue_head(&dc->open_wait);
                init_waitqueue_head(&dc->openack_wait);
                init_waitqueue_head(&dc->closeack_wait);
                init_waitqueue_head(&dc->sync_wait);
#else
		dc->ch_lock = MUTEX;		/* initialize the semaphore */
#endif

		dc->callout_termios = DefaultTermios;
		dc->normal_termios = DefaultTermios;

		/* make sure that we have the proper initial oflag settings */
		dc->ld_oflag = (DEFAULT_OFLAGS & !(OPOST));
		dc->user_oflag = DEFAULT_OFLAGS;

		/*
		 * Allocate Cook buffer and initialize
		 */
		dc->CookBuf = kmalloc(dgdm_cook_size, GFP_KERNEL);

		if(!dc->CookBuf) {
                        DPR_BASIC(("dgdm_tty_brdinit(%d_%02d) - ENOMEM\n",
                                   brd->boardnum, i));
			return(-ENOMEM);
                }

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
		dc->CookLock = SPIN_LOCK_UNLOCKED;
#endif
	}

	DPR_BASIC(("dgdm_tty_brdinit(%d) - done\n", brd->boardnum));
	return(0);
}


/*
 * dgdm_tty_brduninit()
 *
 * Uninitialize the per-board TTY portion of this driver.
 */
GLOBAL void dgdm_tty_brduninit(BOARD_T * brd)
{
	DRVCHAN *tmp_dc;
	int i;

        if(!brd)
                return;

        DPR_BASIC(("dgdm_tty_brduninit(%d)\n", brd->boardnum));

	/* Uninit DXB */
	if(brd->dxb) {
		dxb_uninit(brd->dxb);

		/* Free the allocated space, not the aligned space */
		kfree(brd->dxb_real);

		brd->dxb_real = NULL;
		brd->dxb = NULL;
	}

        DPR_BASIC(("dgdm_tty_brduninit(%d) #1\n", brd->boardnum));

	/* Free the global host data structure */
	if(brd->ghd) {
		free_page((int) brd->ghd);
		brd->ghd = NULL;
	}

        DPR_BASIC(("dgdm_tty_brduninit(%d) #2\n", brd->boardnum));

	/* Free the driver channel array and cook buffer */
	if(brd->dc) {
		for (tmp_dc = brd->dc, i = 0; i < brd->nchans; ++i, ++tmp_dc) {
			if(tmp_dc->CookBuf) {
				kfree(tmp_dc->CookBuf);
				tmp_dc->CookBuf = NULL;
			}
		}

		kfree(brd->dc);
		brd->dc = NULL;
	}

        DPR_BASIC(("dgdm_tty_brduninit(%d) - done\n", brd->boardnum));
}








/************************************************************************
 *
 * TTY Entry points and helper functions
 *
 ************************************************************************/
/*
 * dgdm_tty_open()
 *
 */
STATIC int dgdm_tty_open(TTY *tty, FILE *file)
{
	int majornum;
	int channum;
	int boardnum;
	BOARD_T *brd;
	DRVCHAN *dc;
	int rc;
	unsigned long flags;
    DECLARE_WAITQUEUE(wait, current);

    MOD_INC_USE_COUNT;

    /* determine the board number */
	majornum = MAJOR(tty->device) - dgdm_major;
        switch (majornum) {
        case DGDM_SERIAL_MAJOR:
        case DGDM_CALLOUT_MAJOR:
        case DGDM_PRINT_MAJOR:
                boardnum = MINOR(tty->device) >> 6;
                break;
        case DGDM_SERIAL2_MAJOR:
        case DGDM_CALLOUT2_MAJOR:
        case DGDM_PRINT2_MAJOR:
                boardnum = (MINOR(tty->device) >> 6) + 4;
                break;
        default:
                APR(("dgdm_tty_open() - invalid major %d\n", majornum));
                MOD_DEC_USE_COUNT;
                return(-ENODEV);
        }

	brd = dgdm_Board[boardnum];

        if(!brd) {
                DPR_BASIC(("dgdm_tty_open(%d) - NULL board\n",
                           boardnum));
                MOD_DEC_USE_COUNT;
                return(-ENODEV);
        }

	if((majornum == DGDM_PRINT_MAJOR) || (majornum == DGDM_PRINT2_MAJOR)) {
		channum = ((MINOR(tty->device) & 63) +
			   (brd->nttychans + brd->nspecchans));
	} else {
		channum = (MINOR(tty->device) & 63);
	}

    DPR_BASIC(("dgdm_tty_open(%d_%02d)\n", boardnum, channum));

	if(channum >= brd->nchans) {
		DPR_BASIC(("dgdm_tty_open(%d_%02d) - Port out range, max=%x\n",
			   boardnum, channum, brd->nchans));
		MOD_DEC_USE_COUNT;
		return(-ENODEV);
	}
	dc = &brd->dc[channum];

        /* Lock the channel before we muck with the pointers */
        down(&dc->ch_lock);

	/*
	 * save all the pointers to the related structures
	 */
	dc->tty = tty;
	dc->brd = brd;
	tty->driver_data = dc;

	/*
	 * If the port is the middle of closing, bail out now
	 */
	if(tty_hung_up_p(file) || (dc->flags & RASCON_CLOSING)) {
		if(dc->flags & RASCON_CLOSING) {
			DPR_BASIC(("dgdm_tty_open(%d_%02d_%x) - RASCON_CLOSING"
				   " sleeping\n", boardnum, channum,
				   dc->dxbchan));
                        up(&dc->ch_lock);
			interruptible_sleep_on(&dc->close_wait);
                        down(&dc->ch_lock);
			DPR_BASIC(("dgdm_tty_open(%d_%02d_%x) - RASCON_CLOSING"
				   " done sleeping\n", boardnum, channum,
				   dc->dxbchan));
		}
		/* MOD_DEC_USE_COUNT; "info->tty" will cause this? */
                up(&dc->ch_lock);
		return((dc->flags & RASCON_HUP_NOTIFY) ?
			-EAGAIN : -ERESTARTSYS);
	}
   

	/*
	 * If this is the first open of the channel,
	 * open the DXB channel
	 */
	DGDM_LOCK(dgdm_global_lock, flags);

#if defined(DGDM_PARANOIA_CHECK)
	/*
	 * TEMP DEBUG - JAR 991027
	 */
	if(dc->opencnt < 0) {
		APR(("dgdm_tty_open(%d_%02d) - open count was negative - "
		     "reset to 0, tty->count was %d\n", boardnum,
		     channum, tty->count));
		dc->opencnt = 0;
	}
	if((dc->dxbchan < 0) && (dc->opencnt != 0)) {
		APR(("dgdm_tty_open(%d_%02d) - dxbchan was negative - "
		     "reset opencnt from %d to 0\n", boardnum, channum,
		     dc->opencnt));
		dc->opencnt = 0;
	}
	if((dc->dxbchan >= 0) && (dc->opencnt == 0)) {
		APR(("dgdm_tty_open(%d_%02d) - dxbchan non-negative - "
		     "opencnt is 0, tty->count is %d\n", boardnum,
		     channum, tty->count));
		/* FIXME what should we do here? */
		/* Once the dxbopen is done with a specific major/minor,
		 * doing again while still open will result in a
		 * perpetually hung port since the dxbopen will fail
		 * and dxbchan will be set to -1
		 */
	}
#endif


	if(dc->opencnt++ == 0) {
		uint32 dxbminor;
		char *dxbmajor;

		/* Allow command line override during insmod */
#define	RXBUF_SIZE	(dgdm_rxbufsize)

		/* Sanity check rxbufsize, since user can tinker with it. */
		if(dgdm_rxbufsize & (dgdm_rxbufsize - 1)) {
			--dc->opencnt;
			DPR_BASIC(("dgdm_tty_open(%d_%02d) - bad "
                                   "dgdm_rxbufsize %d, returning ENODEV\n",
				   boardnum, channum, dgdm_rxbufsize));
			DGDM_UNLOCK(dgdm_global_lock, flags);
			up(&dc->ch_lock);
			rc = -ENODEV;
            goto error;
		}
		if(dgdm_loopback) {
			/* all channels are boozers with a drinking problem */
			dxbmajor = "LIQUOR";
			dxbminor = dgdm_loopback;	/* i.e. 3 == vomit */
		} else if(channum >= (brd->nttychans + brd->nspecchans)) {
			/* Transparent Print channels */
			dxbmajor = "TXPRINT";
			dxbminor = channum - (brd->nttychans +
					      brd->nspecchans);
		} else if(channum >= brd->nttychans) {
			/* boozer channels */
			dxbmajor = "LIQUOR";
			dxbminor = channum - brd->nttychans;
		} else {
			/* Normal tty channels - serial and callout */
			dxbmajor = dgdm_dxbttyname;
			dxbminor = channum;
		}

		dc->poll_ok = 0;
		DGDM_UNLOCK(dgdm_global_lock, flags);

		for (;;) {
			dxb_lock(brd);

			dc->dxbchan = dxb_rxbuf_open(brd->dxb,
			      dxbmajor, 0 /*fixed cost */ , &dxbminor, 4,
						     NULL, RXBUF_SIZE);

			if(dc->dxbchan >= 0)
				break;

			dxb_unlock(brd);

			if(dgdm_sleep(1)) {
				DGDM_LOCK(dgdm_global_lock, flags);
				--dc->opencnt;
				DGDM_UNLOCK(dgdm_global_lock, flags);
				up(&dc->ch_lock);
				rc = -EINTR;
                goto error;
			}
		}

		DGDM_LOCK(dgdm_global_lock, flags);
		DPR_BASIC(("dgdm_tty_open(%d_%02d_%x)\n",
			   boardnum, channum, dc->dxbchan));

		dc->maxcred = 0;

		/*
		 * We're gonna send it now.
		 * First make sure we're ready to receive the wakeup.
		 * The reply will set current->state back to TASK_RUNNING
		 */
        {
        add_wait_queue(&dc->openack_wait, &wait);
		current->state = TASK_INTERRUPTIBLE;

		/*
		 * Give the other side its allotment of credits
		 * and set up callbacks.
		 *
		 * NOTE: we are using a currently undocumented
		 *       feature of dxb_space_avail (called by dxb_rxbuf_open
		 *       above) which guarantees that there will be room for
		 *       the return credits message.  This is sleazy, and will
		 *       be fixed by documenting the behavior.
		 */
		dxb_return_credits(brd->dxb, dc->dxbchan, RXBUF_SIZE - 1);

		dxb_set_openack_cb(brd->dxb, dc->dxbchan,
				   openack_cb, (DXB_ARG) dc);

		dxb_set_data_cb(brd->dxb, dc->dxbchan,
				data_cb, (DXB_ARG) dc);

		dxb_set_cdata_cb(brd->dxb, dc->dxbchan,
				 cdata_cb, (DXB_ARG) dc);

		dxb_set_credit_cb(brd->dxb, dc->dxbchan,
				  credit_cb, (DXB_ARG) dc);

		DGDM_UNLOCK(dgdm_global_lock, flags);
		dxb_unlock(brd);

		/*
		 * Wait for open acknowledgement to come back,
		 * then check its error code.
		 */
		schedule();
		remove_wait_queue(&dc->openack_wait, &wait);
        }
		if(signal_pending(current)) {
			up(&dc->ch_lock);
            rc = -EINTR;
            DPR_BASIC(("dgdm_tty_open(%d_%02d_%x) - failed, "
				   "Interrupted.\n", boardnum, channum,
				   dc->dxbchan));
            goto error;
		}
		if(dc->error_code) {
			DPR_BASIC(("dgdm_tty_open(%d_%02d_%x) - failed, "
				   "error_code=%d\n", boardnum, channum,
				   dc->dxbchan, dc->error_code));
			up(&dc->ch_lock);
            rc = -ENODEV;
            goto error;
		} else {
			DPR_BASIC(("dgdm_tty_open(%d_%02d_%x) DXB open "
			     "acked\n", boardnum, channum, dc->dxbchan));
		}

		/*
		 *    Initialize stuff that needs initting on first open
		 */
		dc->state = 0;
		{
			/* raise DTR & RTS, let us know about DCD changes */
			struct sdxb_mstat_command cmd;

			/*
                         * If we haven't held DTR low long enough, see
                         * if DCD is low.  If it is, then assume DTR can
                         * be raised.  Otherwise, we will sleep some more
                         * in sdxbcmd.
			 */
			if((jiffies - dc->dtrlow) < DEFAULT_DTRLOW) {
				int old_low = dc->dtrlow;

				cmd.ctype = SDXB_MSTAT;
				cmd.flags = 0;
				cmd.mint = 0xff;
				cmd.mout = 0;

				sdxbcmd(dc, CMD_WAIT, CMD_HIPRI, &cmd,
					SDXB_MSTAT_CLEN);

				if(dc->mstat & SDXB_DCD) {
					/* dtrlow was reset in MSTAT */
					dc->dtrlow = old_low;
				} else {
					dc->dtrlow =
					    jiffies - DEFAULT_DTRLOW;
				}
			}
			dc->mstat = SDXB_DTR | SDXB_RTS;

			cmd.ctype = SDXB_MSTAT;
			cmd.flags = SDXB_IMMED | SDXB_HIPRI;
			cmd.mint = 0xff;	/* send state of all lines */
			cmd.mout = dc->mstat & (SDXB_DTR | SDXB_RTS);

			sdxbcmd(dc, CMD_NOWAIT, CMD_HIPRI, &cmd,
				SDXB_MSTAT_CLEN);
		}
		{
			/* let us know about breaks and if output flows off */
			struct sdxb_event_command cmd;

			cmd.ctype = SDXB_EVENT;
			cmd.flags = SDXB_HIPRI;
			cmd.eint = SDXB_RXB | SDXB_OPS;

			sdxbcmd(dc, CMD_NOWAIT, CMD_HIPRI, &cmd,
				SDXB_EVENT_CLEN);
		}
		dgdm_tty_param(tty, dc, &DefaultTermios);

		/* wait for the above commands to take effect */
		rc = sdxbsyncwait(dc, FALSE, FALSE);
		if(rc) {
			DPR_BASIC(("dgdm_tty_open(%d_%02d_%x) - bad return "
                                   "from sdxbsyncwait %d\n", dc->brd->boardnum,
				   dc->channum, dc->dxbchan, rc));
		}
		DGDM_LOCK(dgdm_global_lock, flags);
	}
	dc->flags |= RASCON_INITIALIZED;
	DGDM_UNLOCK(dgdm_global_lock, flags);
	up(&dc->ch_lock);

	/*
	 * follow protocol for opening port
	 */
	rc = block_til_ready(tty, file, dc);
	if(rc) {
		DPR_BASIC(("dgdm_tty_open returning after block_til_ready "
			   "with %d\n", rc));
		goto error;
	}
	if((dc->opencnt == 1) && (dc->flags & RASCON_SPLIT_TERMIOS)) {
		if(tty->driver.subtype == SERIAL_TYPE_NORMAL)
			*tty->termios = dc->normal_termios;
		else
			*tty->termios = dc->callout_termios;
		dgdm_tty_param(tty, dc, &DefaultTermios);
	}
	dc->session = current->session;
	dc->pgrp = current->pgrp;

	DPR_BASIC(("dgdm_tty_open(%d_%02d_%x) - open complete\n",
		   boardnum, channum, dc->dxbchan));

error:
#if ALLOW_ALT_FAIL_OPEN
    if(rc){
        if(dgdm_alt_fail_open)
        {
            DPR_BASIC(("dgdm_tty_open(%d_%02d_%x) - failed, "
                       "calling close from open for failed open.", boardnum, channum,
           dc->dxbchan));
            dgdm_tty_close(tty, file);
        }
    }
#endif

    return rc;
 }




/*
 * block_til_ready()
 *
 * Enforce the normal/callout open priority scheme.
 *
 * Then, if needed, call wait_for_carrier() for DCD to come alive.
 */
STATIC int block_til_ready(TTY *tty, FILE *file, DRVCHAN *dc)
{
        DECLARE_WAITQUEUE(wait, current);
	int retval;
	int do_clocal = 0;
	unsigned long flags;
#if 0
	int extra_count = 0;
#endif

	/*
	 * If the device is in the middle of being closed, then block
	 * until it's done, and then try again.
	 */
	if(tty_hung_up_p(file)) {
		DPR_BASIC(("block_til_ready(%d_%02d_%x) - tty_hung_up_p\n",
			   dc->brd->boardnum, dc->channum, dc->dxbchan));
		return((dc->flags & RASCON_HUP_NOTIFY) ?
                       -EAGAIN : -ERESTARTSYS);
	}
	if(dc->flags & RASCON_CLOSING) {
		DPR_BASIC(("block_til_ready(%d_%02d_%x) - RASCON_CLOSING "
			   "sleeping\n", dc->brd->boardnum, dc->channum,
			   dc->dxbchan));
		interruptible_sleep_on(&dc->close_wait);
		DPR_BASIC(("block_til_ready(%d_%02d_%x) - RASCON_CLOSING "
                           "done sleeping\n", dc->brd->boardnum, dc->channum,
			   dc->dxbchan));
		return((dc->flags & RASCON_HUP_NOTIFY) ?
                       -EAGAIN : -ERESTARTSYS);
	}
	/*
	 * If this is a callout device, then just make sure the normal
	 * device isn't being used.
	 */
	if(tty->driver.subtype == SERIAL_TYPE_CALLOUT) {
		if(dc->flags & RASCON_NORMAL_ACTIVE)
			return(-EBUSY);
		if((dc->flags & RASCON_CALLOUT_ACTIVE) &&
                   (dc->flags & RASCON_SESSION_LOCKOUT) &&
                   (dc->session != current->session))
			return(-EBUSY);
		if((dc->flags & RASCON_CALLOUT_ACTIVE) &&
                   (dc->flags & RASCON_PGRP_LOCKOUT) &&
                   (dc->pgrp != current->pgrp))
			return(-EBUSY);
		dc->flags |= RASCON_CALLOUT_ACTIVE;
		DPR_BASIC(("block_til_ready(%d_%02d_%x) - CALLOUT_ACTIVE\n",
			   dc->brd->boardnum, dc->channum, dc->dxbchan));
		return(0);
	}
	/*
	 * If non-blocking mode is set, or the port is not enabled,
	 * then make the check up front and then exit.
	 */
	if((file->f_flags & O_NONBLOCK) ||
           (tty->flags & (1 << TTY_IO_ERROR))) {
		if(dc->flags & RASCON_CALLOUT_ACTIVE)
			return(-EBUSY);
		dc->flags |= RASCON_NORMAL_ACTIVE;
		DPR_BASIC(("block_til_ready(%d_%02d_%x) - non-blocking "
                           "NORMAL_ACTIVE\n", dc->brd->boardnum, dc->channum,
			   dc->dxbchan));
		return(0);
	}
	if(dc->flags & RASCON_CALLOUT_ACTIVE) {
		if((dc->normal_termios.c_cflag & CLOCAL) ||
                   (dc->flags & RASCON_SOFTCAR))
			do_clocal = 1;
	} else {
		if((tty->termios->c_cflag & CLOCAL) ||
                   (dc->flags & RASCON_SOFTCAR))
			do_clocal = 1;
	}


	/*
	 * Block waiting for the carrier detect and the line to become
	 * free (i.e., not in use by the callout).  While we are in
	 * this loop, dc->opencnt is dropped by one, so that
	 * dgdm_tty_close() knows when to free things.  We restore it upon
	 * exit, either normal or abnormal.
	 */
	DGDM_LOCK(dgdm_global_lock, flags);
	retval = 0;
	add_wait_queue(&dc->open_wait, &wait);
	DPR_BASIC(("block_til_ready(%d_%02d_%x) - before block: count %d\n",
                   dc->brd->boardnum, dc->channum, dc->dxbchan, dc->opencnt));
#if 0
	if(!tty_hung_up_p(file)) {
		extra_count = 1;
                
		if(dc->opencnt == 0) {
			APR(("block_til_ready(%d_%02d_%x): open count 0 "
			     "when decrementing\n", dc->brd->boardnum,
                             dc->channum, dc->dxbchan));
		}
		dc->opencnt--;
	}
#endif
	dc->blocked_open++;
	DGDM_UNLOCK(dgdm_global_lock, flags);
	while (1) {
		current->state = TASK_INTERRUPTIBLE;
		if(tty_hung_up_p(file) || !(dc->flags & RASCON_INITIALIZED)) {
			if(dc->flags & RASCON_HUP_NOTIFY)
				retval = -EAGAIN;
			else
				retval = -ERESTARTSYS;
			break;
		}
		if(!(dc->flags & RASCON_CALLOUT_ACTIVE) &&
                   !(dc->flags & RASCON_CLOSING) &&
                   (do_clocal || (dc->mstat & dc->m_dcd)))
			break;
		if(signal_pending(current)) {
			retval = -ERESTARTSYS;
			break;
		}
		DPR_BASIC(("block_til_ready(%d_%02d_%x) - blocking: "
			   "count %d\n", dc->brd->boardnum, dc->channum,
			   dc->dxbchan, dc->opencnt));
		schedule();
	}
	DGDM_LOCK(dgdm_global_lock, flags);
	current->state = TASK_RUNNING;
	remove_wait_queue(&dc->open_wait, &wait);
#if 0
	if(extra_count)
		dc->opencnt++;
#endif
	dc->blocked_open--;
	DGDM_UNLOCK(dgdm_global_lock, flags);
	DPR_BASIC(("block_til_ready(%d_%02d_%x) - after blocking: count %d\n",
                   dc->brd->boardnum, dc->channum, dc->dxbchan, dc->opencnt));

	if(retval)
		return(retval);
	dc->flags |= RASCON_NORMAL_ACTIVE;
	DPR_BASIC(("block_til_ready(%d_%02d_%x) - done NORMAL_ACTIVE\n",
		   dc->brd->boardnum, dc->channum, dc->dxbchan));
    
    return(0);
}



/*
 * openack_cb()
 *
 * callback we get on the open acknowledgement
 */
STATIC int openack_cb(DXB *dxb, int channel, int error_code, DXB_ARG arg)
{
	DRVCHAN *dc = (DRVCHAN *) arg;

	dc->error_code = error_code;

	if(!error_code)
		dc->poll_ok = 1;

	DPR_BASIC(("openack_cb(%d_%02d_%x)\n",
		   dc->brd->boardnum, dc->channum, dc->dxbchan));

	wake_up_interruptible(&dc->openack_wait);
	return(0);
}



/*
 * dgdm_tty_close()
 *
 */
STATIC void dgdm_tty_close(TTY *tty, FILE *file)
{
	DRVCHAN *dc;
	BOARD_T *brd;
	int rc;
	TERMIOS *ts;
	unsigned long flags;

	if(!tty)
		return;
        
	ts = tty->termios;

	dc = tty->driver_data;
	if(!dc)
		return;

	brd = dc->brd;

	DPR_BASIC(("dgdm_tty_close(%d_%02d_%x) - ttycnt %d, opencnt %d\n",
		   brd->boardnum, dc->channum, dc->dxbchan, tty->count,
		   dc->opencnt));

	down(&dc->ch_lock);
	DGDM_LOCK(dgdm_global_lock, flags);

	/* If the DXB open never completed, we are done */
	if(dc->dxbchan < 0) {
		DPR_BASIC(("dgdm_tty_close(%d_%02d_%x) - negative dxbchan\n",
			   brd->boardnum, dc->channum, dc->dxbchan));

		if(dc->opencnt != 0) {
			APR(("dgdm_tty_close(%d_%02d_%x): "
			     "negative dxbchan with opencnt = %d\n",
			     brd->boardnum, dc->channum, dc->dxbchan,
			     dc->opencnt));
			dc->opencnt = 0;	/* TEMP DEBUG - JAR 991027 */
		}
		MOD_DEC_USE_COUNT;
		DGDM_UNLOCK(dgdm_global_lock, flags);
                up(&dc->ch_lock);
		return;
	}


	/*
         * Determine if this is the last close or not - and if we agree about
         * which type of close it is with the Line Discipline
         */
	if((tty->count == 1) && (dc->opencnt != 1)) {
		/*
		 * Uh, oh.  tty->count is 1, which means that the tty
		 * structure will be freed.  dc->opencnt should always
		 * be one in these conditions.  If it's greater than
		 * one, we've got real problems, since it means the
		 * serial port won't be shutdown.
		 */
		APR(("dgdm_tty_close(%d_%02d_%x): tty->count is 1, "
		     "opencnt is %d\n", brd->boardnum, dc->channum,
		     dc->dxbchan, dc->opencnt));
		dc->opencnt = 1;
	}
	if(--dc->opencnt < 0) {
		APR(("dgdm_tty_close(%d_%02d_%x): bad serial port open count "
		     "of %d\n", brd->boardnum, dc->channum, dc->dxbchan,
		     dc->opencnt));
		dc->opencnt = 0;
	}
	if(dc->opencnt) {
		DPR_BASIC(("dgdm_tty_close(%d_%02d_%x): not last close %d\n",
			   brd->boardnum, dc->channum, dc->dxbchan,
			   dc->opencnt));
		MOD_DEC_USE_COUNT;
		DGDM_UNLOCK(dgdm_global_lock, flags);
                up(&dc->ch_lock);
		return;
	}


	/* OK, its the last close on the port */
	DPR_BASIC(("dgdm_tty_close(%d_%02d_%x) - last close procedures\n",
		   dc->brd->boardnum, dc->channum, dc->dxbchan));

	dc->flags |= RASCON_CLOSING;

	/*
	 * Save the termios structure, since this port may have
	 * separate termios for callout and dialin.
	 */
	if(dc->flags & RASCON_NORMAL_ACTIVE)
		dc->normal_termios = *tty->termios;
	if(dc->flags & RASCON_CALLOUT_ACTIVE)
		dc->callout_termios = *tty->termios;

	/*
	 * Now we wait for the transmit buffer to clear; and we notify
	 * the line discipline to only process XON/XOFF characters.
	 */
	tty->closing = 1;
	DGDM_UNLOCK(dgdm_global_lock, flags);

	/* wait for output to drain */
	/* This will also return if we take an interrupt */
	rc = sdxbsyncwait(dc, FALSE, TRUE);
	if(rc) {
		DPR_BASIC(("dgdm_tty_close(%d_%02d_%x) - bad return from "
		     "sdxbsyncwait %d\n", dc->brd->boardnum, dc->channum,
			   dc->dxbchan, rc));
	}
	if(tty->driver.flush_buffer)
		tty->driver.flush_buffer(tty);
	if(tty->ldisc.flush_buffer)
		tty->ldisc.flush_buffer(tty);


        /*
         * Tell the FEP to stop notifying us of the line status changes
         * and if we have HUPCL set, lower DTR and RTS
         */
        {
                struct sdxb_mstat_command cmd;

                if(!ts || ts->c_cflag & HUPCL) {
                        dc->mstat = 0;
                }
                        
                cmd.ctype = SDXB_MSTAT;
                cmd.flags = 0;
                cmd.mint = 0;		/* send state of no lines */
                cmd.mout = dc->mstat & (SDXB_DTR | SDXB_RTS);

                sdxbcmd(dc, CMD_WAIT, CMD_HIPRI, &cmd, SDXB_MSTAT_CLEN);
	}

	/* ensure no deferred messages are sent after close is sent */
	dc->sendcmds = 0;

	(void) dgdm_tty_shutdown(dc);
	dc->tty = 0;
	tty->closing = 0;
	if(dc->blocked_open) {
		if(dc->close_delay) {
			current->state = TASK_INTERRUPTIBLE;
			(void) dgdm_sleep(dc->close_delay);
		}
		wake_up_interruptible(&dc->open_wait);
	}
	DGDM_LOCK(dgdm_global_lock, flags);
	dc->flags &= ~(RASCON_NORMAL_ACTIVE | RASCON_CALLOUT_ACTIVE |
		       RASCON_CLOSING);
    wake_up_interruptible(&dc->close_wait);
	MOD_DEC_USE_COUNT;
	DGDM_UNLOCK(dgdm_global_lock, flags);
	up(&dc->ch_lock);

	DPR_BASIC(("dgdm_tty_close(%d_%02d_%x) - complete\n",
		   dc->brd->boardnum, dc->channum, dc->dxbchan));
}



/*
 * dgdm_tty_shutdown()
 *
 * This does the close at the board if it is able to.
 *
 */
void wake_up_shutdown_timer(unsigned long ptr)
{
    DRVCHAN* dc = (DRVCHAN*)ptr;
    DPR_BASIC(("dgdm_tty_shutdown(%d_%02d_%x) - Shutdown timer waking uninterruptible task.\n",
                   dc->brd->boardnum, dc->channum, dc->dxbchan));
    wake_up(&dc->closeack_wait);
}

STATIC int dgdm_tty_shutdown(DRVCHAN *dc)
{
	int rc = 0;
	int i;
    struct timer_list shutdown_wakeup_timer; 
        DECLARE_WAITQUEUE(wait, current);


        DPR_BASIC(("dgdm_tty_shutdown(%d_%02d_%x)\n",
                   dc->brd->boardnum, dc->channum, dc->dxbchan));

	/*
	 * Attempt to queue close message up to 100 times
	 * If we can't get it thru by then, just give up
	 * because we don't want to hang the user.
	 */
	dxb_set_closeack_cb(dc->brd->dxb, dc->dxbchan, closeack_cb,
			    (DXB_ARG) dc);

	dc->poll_ok = 0;

	/* check for a valid channel */
	if(dc->dxbchan < 0) {
		DPR_BASIC(("dgdm_tty_shutdown(%d_%02d_%x) - INVALID CHANNEL\n",
			   dc->brd->boardnum, dc->channum, dc->dxbchan));
		return(-EINVAL);
	}
	for (i = 0; i < 100; ++i) {
		dxb_lock(dc->brd);
		/*
		 * We're gonna send it now.
		 * First make sure we're ready to receive the wakeup.
		 * The reply will set current->state back to TASK_RUNNING
		 */
        
        add_wait_queue(&dc->closeack_wait, &wait);
		current->state = TASK_UNINTERRUPTIBLE;
        rc = dxb_close(dc->brd->dxb, dc->dxbchan);
        dxb_unlock(dc->brd);

		/*
		 * If close message was queued, wait for ACK and
		 * then blow out of here.
		 */
		if(rc == 0) {
			DPR_BASIC(("dgdm_tty_shutdown(%d_%02d_%x) - queued\n",
				   dc->brd->boardnum, dc->channum,
				   dc->dxbchan));
			dc->dxbchan = -1;
            
            init_timer(&shutdown_wakeup_timer);
            shutdown_wakeup_timer.function = wake_up_shutdown_timer;
            shutdown_wakeup_timer.data = (unsigned long)dc;
            shutdown_wakeup_timer.expires = jiffies + (2 * HZ);
            add_timer(&shutdown_wakeup_timer);
			
            schedule();
            
            del_timer(&shutdown_wakeup_timer);
            if(signal_pending(current)){
                DPR_BASIC(("dgdm_tty_shutdown(%d_%02d_%x) - Interrupted.\n",
				   dc->brd->boardnum, dc->channum,
				   dc->dxbchan));
            }
			remove_wait_queue(&dc->closeack_wait, &wait);
			DPR_BASIC(("dgdm_tty_close(%d_%02d_%x) - done "
				   "waiting\n", dc->brd->boardnum,
				   dc->channum, dc->dxbchan));
			break;
		}

		/*
		 * Wait 1 tick and try again.  If user mashes
		 * interrupt key, blow out of here
		 */
		if(dgdm_sleep(1))
			break;
	}

	if(i >= 99)
		DPR_BASIC(("dgdm_tty_shutdown(%d_%02d_%x) - could not close, "
			   "gave up\n", dc->brd->boardnum, dc->channum,
			   dc->dxbchan));

	dc->flags &= ~ASYNC_INITIALIZED;

        DPR_BASIC(("dgdm_tty_shutdown(%d_%02d_%x) - complete\n",
                   dc->brd->boardnum, dc->channum, dc->dxbchan));
	return(0);
}



/*
 * dgdm_tty_hangup()
 *
 * Hangup the port.  Like a close, but don't wait for output to drain.
 */
STATIC void dgdm_tty_hangup(TTY *tty)
{
	DRVCHAN *dc;
	TERMIOS *ts;
	unsigned long flags;

	if(!tty)
		return;		/* paranoia */

	dc = tty->driver_data;
	ts = tty->termios;

	if(!dc)
		return;		/* more paranoia */

	DPR_BASIC(("dgdm_tty_hangup(%d_%02d_%x)\n",
		   dc->brd->boardnum, dc->channum, dc->dxbchan));

	/*
	 * if the dxbchan is negative, we have no means to communicate with
	 * the fep - reset everything so we can open a new line next time we
	 * try.
	 */
	if(dc->dxbchan < 0) {
		APR(("dgdm_tty_hangup(%d_%02d_%x): dxbchan = -1, opencnt "
		     "is %d, tty->count is %d\n", dc->brd->boardnum,
		     dc->channum, dc->dxbchan, dc->opencnt, tty->count));

		DGDM_LOCK(dgdm_global_lock, flags);
		dc->opencnt = 0;
		DGDM_UNLOCK(dgdm_global_lock, flags);

		return;
	}
	/* flush the transmit queues */
	dgdm_tty_flush_buffer(tty);


	DGDM_LOCK(dgdm_global_lock, flags);

#if defined(DGDM_PARANOIA_CHECK)
	/*
	 * If the opencnt is negative or 0, we have problems because the last
	 * close won't happen unless opencount is 1.  We need to go thru
	 * dgdm_tty_shutdown() in order to free up the rest of the resources
	 * associated with this line.. be sure tty->count is 1 and reset 
	 * dc->opencnt to 1.
	 */
	if(dc->opencnt <= 0) {
		APR(("dgdm_tty_hangup(%d_%02d_%x): opencnt is %d, tty->count "
		     " is %d\n", dc->brd->boardnum, dc->channum,
		     dc->dxbchan, dc->opencnt, tty->count));
		if(tty->count != 1) {
			APR(("dgdm_tty_hangup(%d_%02d_%x): setting tty->count "
			     "to 1.\n", dc->brd->boardnum, dc->channum,
			     dc->dxbchan));
			tty->count = 1;
		}
	}
	/* should this never happen if the LD is behaving properly?? */
	if(tty->count != 1) {
		APR(("dgdm_tty_hangup(%d_%02d_%x): tty->count wierdness %d\n",
                     dc->brd->boardnum, dc->channum, dc->dxbchan, tty->count));
	}
#endif

	dc->opencnt = 1;	/* ensure that the last close happens */
	DGDM_UNLOCK(dgdm_global_lock, flags);
}



/*
 * cloaseack_cb()
 *
 * callback we get on the close acknowledgement
 */
STATIC int closeack_cb(DXB *dxb, int channel, DXB_ARG arg)
{
	DRVCHAN *dc = (DRVCHAN *) arg;

	//wake_up_interruptible(&dc->closeack_wait);
    wake_up(&dc->closeack_wait);
	return(0);
}



/*
 * dgdm_tty_write()
 *
 */
STATIC int dgdm_tty_write(TTY *tty, int from_user, const unsigned char *buf,
                          int count)
{
	DRVCHAN *dc;
	DXB *dxb;
	int rc;
	int space;

	DPR_WRITE(("dgdm_tty_write: tty=%x user=%d len=%d\n",
		   (int) tty, from_user, count));

	if(tty == NULL || TmpWriteBuf == NULL)
		return(0);

	dc = tty->driver_data;
	if(!dc)
		return(0);

	DPR_WRITE(("dgdm_tty_write(%d_%02d_%x) - user=%d len=%d "
		   "dc->brd->dxb_locked=%d\n",
		   dc->brd->boardnum, dc->channum, dc->dxbchan,
		   from_user, count, dc->brd->dxb_locked));

	dxb = dc->brd->dxb;

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0) && defined(DGDM_PARANOIA_CHECK)
        {
                static int interrupt_count = 0;

                if(in_interrupt()) {

                        interrupt_count++;
                        if(from_user)
                                DPR_WRITE(("dgdm_tty_write called from "
                                           "interrupt context && getting "
                                           "TmpWriteSem - BAD!\n"));
                
                        if(!dc->brd->dxb_locked) {
                                DPR_WRITE(("dgdm_tty_write called from "
                                           "interrupt context with poller "
                                           "not locked %d - INFO\n",
                                           dc->brd->dxb_locked));
                        } else {
                                DPR_WRITE(("dgdm_tty_write called from "
                                           "interrupt context with poller "
                                           "locked %d - INFO\n",
                                           dc->brd->dxb_locked));
                        }
                }
                DPR_WRITE(("dgdm_tty_write(%d_%02d_%x) - interrupt_count=%d\n",
                           dc->brd->boardnum, dc->channum, dc->dxbchan,
                           interrupt_count));
        }
#endif

#if defined(DGDM_PARANOIA_CHECK)
	if(dc->CookCount)
		APR(("dgdm_tty_write(%d_%02d_%x) - CookCount=%d, should be "
                     "zero\n", dc->brd->boardnum, dc->channum, dc->dxbchan,
                     dc->CookCount));
#endif

	/*
	 * If data is coming from user space, copy it into a temporary
	 * buffer so we don't get swapped out while doing the copy to
	 * the board.
	 */
	if(from_user) {
		down(&TmpWriteSem);

		if(count) {
			count = MIN(count, WRITEBUFLEN);
			COPY_FROM_USER(TmpWriteBuf, buf, count);
		}
		buf = TmpWriteBuf;

                /* we're allowed to block if it's from_user */
                dxb_lock(dc->brd);
        } else {
                /* if we get a non-zero return, we couldn't lock, bail. */
                if(dxb_lock_noblock(dc->brd))
                        return(-EAGAIN);
        }


	/*
	 * Limit amount we will write to the amount of space
	 * available in the channel buffer.
	 */
	space = dxb_data_payload_space(dxb, dc->dxbchan);

	/*
	 * If payload space is 0 and its because the DXB queue
	 * is full, then request a credit callback when queue
	 * space becomes available.  This insures that we won't
	 * end up sleeping on the write queue with nobody to
	 * wake us up.
	 */
	if(space == 0)
		dxb_set_credit_cb_on_space(dxb, dc->dxbchan, 1);

	count = MIN(count, space);

	if(count) {
		rc = dxb_data(dxb, dc->dxbchan, count);
		rc = dxb_write_data_block(dxb, buf, count);
		rc = dxb_write_done(dxb);
		set_bit(STATE_TXBUSY, &dc->state);
	}

	dxb_unlock(dc->brd);

	if(from_user) {
		up(&TmpWriteSem);
	}

	dc->stat[TxSTAT].cnt += count;
	atomic_add(count, (atomic_t *) & dc->stat[TxSTAT].accum);

	DPR_WRITE(("dgdm_tty_write(%d_%02d_%x) - wrote %d\n",
		   dc->brd->boardnum, dc->channum, dc->dxbchan, count));
	return(count);
}



/*
 * dgdm_tty_flush_chars()
 *
 * Flush the cook buffer
 *
 * Note to self, and any other poor souls who venture here:
 *
 * flush in this case DOES NOT mean dispose of the data.
 * instead, it means "stop buffering and send it if you
 * haven't already."  Just guess how I figured that out...   SRW 2-Jun-98
 *
 * It is also always called in interrupt context - JAR 8-Sept-99
 */
STATIC void dgdm_tty_flush_chars(TTY *tty)
{
	DRVCHAN *dc;
	DXB *dxb;
	int rc;
	unsigned long flags;

	if(tty == NULL)
		return;

	dc = tty->driver_data;
	if(!dc)
		return;

	DPR_WRITE(("dgdm_tty_flush_chars(%d_%02d_%x)\n",
		   dc->brd->boardnum, dc->channum, dc->dxbchan));

	dxb = dc->brd->dxb;

	DGDM_LOCK(dc->CookLock, flags);
	if(!dc->CookCount) {
		DGDM_UNLOCK(dc->CookLock, flags);
		DPR_WRITE(("dgdm_tty_flush_chars(%d_%02d_%x) - already "
			   "flushed this line\n", dc->brd->boardnum,
			   dc->channum, dc->dxbchan));
		return;
	}
	if(dxb_data_payload_space(dxb, dc->dxbchan) < dc->CookCount) {
		DPR_WRITE(("dgdm_tty_flush_chars(%d_%02d_%x) - "
			   "no payload space\n", dc->brd->boardnum,
			   dc->channum, dc->dxbchan));
		DGDM_UNLOCK(dc->CookLock, flags);
		return;
	}
	rc = dxb_data(dxb, dc->dxbchan, dc->CookCount);
	rc = dxb_write_data_block(dxb, dc->CookBuf, dc->CookCount);
	rc = dxb_write_done(dxb);
	set_bit(STATE_TXBUSY, &dc->state);

	/* Reset the count */
	dc->CookCount = 0;

	DGDM_UNLOCK(dc->CookLock, flags);
	DPR_WRITE(("dgdm_tty_flush_chars(%d_%02d_%x) - flushed\n",
		   dc->brd->boardnum, dc->channum, dc->dxbchan));
	return;
}



/*
 * dgdm_tty_put_char()
 *
 * Put a character into the cook buffer to be written.
 *
 */
STATIC void dgdm_tty_put_char(TTY *tty, unsigned char ch)
{
	DRVCHAN *dc;
	unsigned long flags;

	if(tty == NULL)
		return;

	dc = tty->driver_data;
	if(!dc)
		return;

	DPR_WRITE(("dgdm_tty_put_char(%d_%02d_%x)\n",
		   dc->brd->boardnum, dc->channum, dc->dxbchan));

	DGDM_LOCK(dc->CookLock, flags);

	if(dc->CookCount == dgdm_cook_size - 1) {
		DGDM_UNLOCK(dc->CookLock, flags);
		DPR_WRITE(("dgdm_tty_put_char(%d_%02d_%x) - Cook buffer "
                           "overflow\n", dc->brd->boardnum, dc->channum,
                           dc->dxbchan));
		return;
	}
	dc->CookBuf[dc->CookCount++] = ch;

	DGDM_UNLOCK(dc->CookLock, flags);
	return;
}



/*
 * dgdm_tty_flush_buffer()
 *
 * Flush Tx buffer (make in == out)
 */
STATIC void dgdm_tty_flush_buffer(TTY * tty)
{
	DRVCHAN *dc;

	if(!tty)
		return;

	dc = tty->driver_data;
	if(!dc)
		return;

	DPR_BASIC(("dgdm_tty_flush_buffer(%d_%02d_%x)\n",
		   dc->brd->boardnum, dc->channum, dc->dxbchan));

	/* Clear the LD cook buffer count */
	dc->CookCount = 0;

	dc->sendcmds |= SEND_TXFLUSH;
	send_deferred_commands(dc, TRUE);

	wake_up_interruptible(&tty->write_wait);
#if defined(SERIAL_HAVE_POLL_WAIT)
        wake_up_interruptible (&tty->poll_wait);
#endif

	if((tty->flags & (1 << TTY_DO_WRITE_WAKEUP)) &&
           (tty->ldisc.write_wakeup))
		(tty->ldisc.write_wakeup) (tty);
}



/*
 * dgdm_tty_write_room()
 *
 * Return space available in Tx buffer
 */
STATIC int dgdm_tty_write_room(TTY * tty)
{
	DRVCHAN *dc;
	DXB *dxb;
	int ret;

	dc = tty->driver_data;
	if(!dc)
		return(0);

	dxb = dc->brd->dxb;
	ret = dxb_data_payload_space(dxb, dc->dxbchan);

	DPR_WRITE(("dgdm_tty_write_room(%d_%02d_%x) - %d\n",
		   dc->brd->boardnum, dc->channum, dc->dxbchan, ret));
	return(ret);
}



/*
 * dgdm_tty_chars_in_buffer()
 *
 * Return number of characters that have not been transmitted yet.
 *
 * This routine is used by the line discipline to determine if there
 * is data waiting to be transmitted/drained/flushed or not.
 *
 * As of 2.2.12 the Line Discipline assumed that the number of
 * characters left in xmit buffer before select says that we
 * have room (WAKEUP_CHARS) is defined to be 256.  This is
 * obviously not going to be enough for faster baudrates so we
 * lie about the number of characters in the buffer at higher
 * baudrates.
 *
 */
STATIC int dgdm_tty_chars_in_buffer(TTY * tty)
{
	DRVCHAN *dc;
	DXB *dxb;
	int chars;
	int baud;
	int ret;

	dc = tty->driver_data;
	if(!dc)
		return(0);

	dxb = dc->brd->dxb;

	/* This requires carnal knowledge of libdxb. */
	chars = dc->maxcred - dxb->chan[dc->dxbchan].credits + dc->CookCount;

	/* If we have no characters, return no characters. */
	if(!chars) {
		DPR_WRITE(("dgdm_tty_chars_in_buffer(%d_%02d_%x) - chars=%d, "
			   "ret=0\n", dc->brd->boardnum, dc->channum,
			   dc->dxbchan, chars));
		return(0);
	}

	/*
	 * Magic numbers determined by experimentation to give maximum
	 * thruput with minimal CPU usage.
	 */
	baud = dgdm_tty_getbaud(tty, dc);
	if(baud >= 921600) {
		ret = (chars >= 14336) ? 1024 : 1;
	} else if(baud >= 460800) {
		ret = (chars >= 8192) ? 1024 : 1;
	} else if(baud >= 230400) {
		ret = (chars >= 4096) ? 1024 : 1;
	} else if(baud >= 115200) {
		ret = (chars >= 2048) ? 1024 : 1;
	} else {
		ret = chars;
	}

	DPR_WRITE(("dgdm_tty_chars_in_buffer(%d_%02d_%x) - chars=%d, "
                   "ret=%d\n", dc->brd->boardnum, dc->channum, dc->dxbchan,
		   chars, ret));
	return(ret);
}






/*****************************************************************************
 *
 * The IOCTL function and all of its helpers
 *
 *****************************************************************************/

/*
 * dgdm_tty_ioctl()
 *
 * The usual assortment of ioctl's
 */
STATIC int dgdm_tty_ioctl(TTY *tty, FILE *file, unsigned int cmd,
                          unsigned long arg)
{
	DRVCHAN *dc;
	int rc;
	struct digiflow_struct dflow;

	if(!tty)
		return(-ENODEV);

	dc = tty->driver_data;
	if(!dc)
		return(-ENODEV);

	DPR_IOCTL(("dgdm_tty_ioctl(%d_%02d_%x) - cmd %x, arg %lx\n",
		 dc->brd->boardnum, dc->channum, dc->dxbchan, cmd, arg));

	if(dc->dxbchan < 0) {
		DPR_BASIC(("dgdm_tty_ioctl(%d_%02d_%x) - negative dxbchan\n",
			   dc->brd->boardnum, dc->channum, dc->dxbchan));
		return(-EIO);
	}

	switch (cmd) {

	/* Here are all the standard ioctl's that we MUST implement */

	case TCSBRK:
		/* 
		 * TCSBRK is SVID version: non-zero arg --> no break 
		 * this behaviour is exploited by tcdrain().
		 *
		 * According to POSIX.1 spec (7.2.2.1.2) breaks should be
		 * between 0.25 and 0.5 seconds so we'll ask for something
		 * in the middle: 0.375 seconds.
		 */
		rc = tty_check_change(tty);
		if(rc)
			return(rc);
		rc = sdxbsyncwait(dc, FALSE, TRUE);
		if(rc) {
			DPR_IOCTL(("dgdm_tty_ioctl(%d_%02d_%x) - bad return "
			     "from sdxbsyncwait %d\n", dc->brd->boardnum,
				   dc->channum, dc->dxbchan, rc));
			return(-EINTR);
		}
		if(((cmd == TCSBRK) && (!arg)) || (cmd == TCSBRKP)) {
			struct sdxb_break_command brkcmd;

			brkcmd.ctype = SDXB_BREAK;
			brkcmd.fill = 0;
			brkcmd.btime = 375;

			sdxbcmd(dc, CMD_NOWAIT, CMD_NORMAL,
				&brkcmd, SDXB_BREAK_CLEN);
		}
		return(0);

	case TCSBRKP:
		/* support for POSIX tcsendbreak()

		 * According to POSIX.1 spec (7.2.2.1.2) breaks should be
		 * between 0.25 and 0.5 seconds so we'll ask for something
		 * in the middle: 0.375 seconds.
		 */
		rc = tty_check_change(tty);
		if(rc)
			return(rc);
		else {
			struct sdxb_break_command brkcmd;

			rc = sdxbsyncwait(dc, FALSE, TRUE);
			if(rc) {
				DPR_IOCTL(("dgdm_tty_ioctl(%d_%02d_%x) - "
				     "bad return from sdxbsyncwait %d\n",
					   dc->brd->boardnum, dc->channum,
					   dc->dxbchan, rc));
				return(-EINTR);
			}
			brkcmd.ctype = SDXB_BREAK;
			brkcmd.fill = 0;
			brkcmd.btime = 375;

			sdxbcmd(dc, CMD_NOWAIT, CMD_NORMAL,
				&brkcmd, SDXB_BREAK_CLEN);

			return(0);
		}

	case TIOCGSOFTCAR:
		rc = verify_area(VERIFY_WRITE, (void *) arg, sizeof(long));
		if(rc)
			return(rc);
		PUT_USER(C_CLOCAL(tty) ? 1 : 0, (unsigned long *) arg);
		return(0);

	case TIOCSSOFTCAR:
		GET_USER(arg, (unsigned long *) arg);
		tty->termios->c_cflag =
		    ((tty->termios->c_cflag & ~CLOCAL) |
		     (arg ? CLOCAL : 0));
		return(0);

	case TIOCMGET:
		rc = verify_area(VERIFY_WRITE, (void *) arg,
				 sizeof(unsigned int));
		if(rc)
			return(rc);
		return(get_modem_info(dc, (unsigned int *) arg));

	case TIOCMBIS:
	case TIOCMBIC:
	case TIOCMSET:
		return(set_modem_info(dc, cmd, (unsigned int *) arg));

		/*
		 * Here are any additional ioctl's that we want to implement
		 */

	case TCFLSH:
		/*
		 * The linux tty driver doesn't have a flush
		 * input routine for the driver, assuming all backed
		 * up data is in the line disc. buffers.  However,
		 * we all know that's not the case.  Here, we
		 * act on the ioctl, but then lie and say we didn't
		 * so the line discipline will process the flush
		 * also.
		 */
		rc = tty_check_change(tty);
		if(rc)
			return(rc);

		switch (arg) {
		case TCIFLUSH:
		case TCIOFLUSH:

			DPR_IOCTL(("dxb_ioctl(%d_%02d_%x) - TCFLSH flushing "
			       "input\n", dc->brd->boardnum, dc->channum,
				   dc->dxbchan));
			{
				struct sdxb_flush_command fcmd;

				fcmd.ctype = SDXB_FLUSH;
				fcmd.flags = SDXB_FLUSH_INPUT;
				fcmd.fill1 = 0;
				fcmd.fill2 = 0;

				sdxbcmd(dc, CMD_NOWAIT, CMD_HIPRI, &fcmd,
					SDXB_FLUSH_CLEN);
			}
			break;

		case TCOFLUSH:
			break;

		default:
			/* POSIX.1 says return EINVAL if we got a bad arg */
			return(-EINVAL);
		}
		/* pretend we didn't recognize this IOCTL */
		return(-ENOIOCTLCMD);


#ifdef TIOCGETP
	case TIOCGETP:
#endif
	case TCGETS:
	case TCGETA:
		if(tty->ldisc.ioctl) {
			int retval;

			tty->termios->c_oflag = dc->user_oflag;
			retval = (tty->ldisc.ioctl) (tty, file, cmd, arg);
			tty->termios->c_oflag = dc->ld_oflag;
			return(retval);
		}
		return(-ENOIOCTLCMD);


	case TCSETSF:
	case TCSETSW:
		/*
		 * The linux tty driver doesn't have a flush
		 * input routine for the driver, assuming all backed
		 * up data is in the line disc. buffers.  However,
		 * we all know that's not the case.  Here, we
		 * act on the ioctl, but then lie and say we didn't
		 * so the line discipline will process the flush
		 * also.
		 */
		if(cmd == TCSETSF) {
			struct sdxb_flush_command fcmd;

			DPR_IOCTL(("dxb_ioctl(%d_%02d_%x) - TCSETS[WF] "
				   "flushing input\n", dc->brd->boardnum,
				   dc->channum, dc->dxbchan));

			fcmd.ctype = SDXB_FLUSH;
			fcmd.flags = SDXB_FLUSH_INPUT;
			fcmd.fill1 = 0;
			fcmd.fill2 = 0;

			sdxbcmd(dc, CMD_NOWAIT, CMD_HIPRI, &fcmd,
				SDXB_FLUSH_CLEN);
		}
		/* now wait for all the output to drain */
		rc = sdxbsyncwait(dc, FALSE, TRUE);
		if(rc) {
			DPR_IOCTL(("dxb_ioctl(%d_%02d_%x) - TCSETS[WF] bad "
				   "return from sdxbsyncwait %d\n",
				   dc->brd->boardnum, dc->channum,
				   dc->dxbchan, rc));
			return(-EINTR);
		}
		DPR_IOCTL(("dxb_ioctl(%d_%02d_%x) - in TCSETS[WF] after "
			"sdxbsyncwait\n", dc->brd->boardnum, dc->channum,
			   dc->dxbchan));
		/* pretend we didn't recognize this */
		return(-ENOIOCTLCMD);

	case TCSETAW:
		DPR_IOCTL(("dxb_ioctl(%d_%02d_%x) - caught TCSETAW\n",
			   dc->brd->boardnum, dc->channum, dc->dxbchan));
		sdxbsyncwait(dc, FALSE, TRUE);
		DPR_IOCTL(("dxb_ioctl(%d_%02d_%x) - in TCSETAW after "
			"sdxbsyncwait\n", dc->brd->boardnum, dc->channum,
			   dc->dxbchan));
		/* pretend we didn't recognize this */
		return(-ENOIOCTLCMD);

	case TCXONC:
		/*
		 * The Linux Line Discipline (LD) would do this for us if we
		 * let it, but we have the special firmware options to do this 
		 * the "right way" regardless of hardware or software flow
		 * control so we'll do it outselves instead of letting the LD
		 * do it.
		 */
		rc = tty_check_change(tty);
		if(rc)
			return(rc);

		DPR_IOCTL(("dxb_ioctl(%d_%02d_%x) - in TCXONC - %d\n",
		      dc->brd->boardnum, dc->channum, dc->dxbchan, cmd));
		switch (arg) {
		case TCOON:
			dgdm_tty_start(tty);
			return(0);
		case TCOOFF:
			dgdm_tty_stop(tty);
			return(0);
		case TCION:
			dgdm_tty_input_start(tty);
			return(0);
		case TCIOFF:
			dgdm_tty_input_stop(tty);
			return(0);
		default:
			return(-EINVAL);
		}

	case TIOCGSERIAL:
		/* get information for setserial */
		return(dgdm_tty_getserial(tty, (struct serial_struct *) arg));
	case TIOCSSERIAL:
		/* set information for setserial */
		return(dgdm_tty_setserial(tty, (struct serial_struct *) arg));

	case DIGI_GETA:
		/* get information for ditty */
		return(dgdm_tty_digigeta(tty, (struct digi_struct *) arg));

	case DIGI_SETAW:
	case DIGI_SETAF:
		/* set information for ditty */
		if(cmd == (DIGI_SETAW)) {
			DPR_IOCTL(("dxb_ioctl(%d_%02d_%x) - DIGI_SETAW\n",
				   dc->brd->boardnum, dc->channum,
				   dc->dxbchan));
			sdxbsyncwait(dc, FALSE, TRUE);
			DPR_IOCTL(("dxb_ioctl(%d_%02d_%x) - in DIGI_SETAW "
			       "after sdxbsyncwait\n", dc->brd->boardnum,
				   dc->channum, dc->dxbchan));
		} else {
			if(tty->ldisc.flush_buffer)
				tty->ldisc.flush_buffer(tty);
		}
		/* fall thru */
	case DIGI_SETA:
		return(dgdm_tty_digiseta(tty, (struct digi_struct *) arg));

	case DIGI_GETFLOW:
	case DIGI_GETAFLOW:
		if(cmd == (DIGI_GETFLOW)) {
			DPR_IOCTL(("dgdm_tty_ioctl(%d_%02d_%x) - in "
				   "DIGI_GETFLOW\n", dc->brd->boardnum,
				   dc->channum, dc->dxbchan));
			dflow.startc = dc->tty->termios->c_cc[VSTART];
			dflow.stopc = dc->tty->termios->c_cc[VSTOP];
		} else {
			DPR_IOCTL(("dgdm_tty_ioctl(%d_%02d_%x) - in "
				   "DIGI_GETAFLOW\n", dc->brd->boardnum,
				   dc->channum, dc->dxbchan));
			dflow.startc = dc->xxon;
			dflow.stopc = dc->xxoff;
		}

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
		if(copy_to_user((char *) arg, &dflow, sizeof(dflow))) {
			DPR_IOCTL(("dgdm_tty_ioctl(%d_%02d_%x) - DIGI_GETFLOW "
			      "EFAULT\n", dc->brd->boardnum, dc->channum,
				   dc->dxbchan));
			return(-EFAULT);
		}
#else
		memcpy_tofs((char *) arg, &dflow, sizeof(dflow));
#endif
		break;


	case DIGI_SETFLOW:
	case DIGI_SETAFLOW:
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
		if(copy_from_user(&dflow, (char *) arg, sizeof(dflow))) {
			DPR_IOCTL(("dgdm_tty_ioctl(%d_%02d_%x) - DIGI_SETFLOW "
			      "EFAULT\n", dc->brd->boardnum, dc->channum,
				   dc->dxbchan));
			return(-EFAULT);
		}
#else
		memcpy_fromfs(&dflow, (char *) arg, sizeof(dflow));
#endif
		if(cmd == (DIGI_SETFLOW)) {
			DPR_IOCTL(("dgdm_tty_ioctl(%d_%02d_%x) - in "
				   "DIGI_SETFLOW\n", dc->brd->boardnum,
				   dc->channum, dc->dxbchan));
			dc->tty->termios->c_cc[VSTART] = dflow.startc;
			dc->tty->termios->c_cc[VSTOP] = dflow.stopc;
		} else {
			DPR_IOCTL(("dgdm_tty_ioctl(%d_%02d_%x) - in "
				   "DIGI_SETAFLOW\n", dc->brd->boardnum,
				   dc->channum, dc->dxbchan));
			dc->xxon = dflow.startc;
			dc->xxoff = dflow.stopc;
		}
		break;

	default:
		DPR_IOCTL(("dgdm_tty_ioctl(%d_%02d_%x) - in default\n",
			   dc->brd->boardnum, dc->channum, dc->dxbchan));
		return(-ENOIOCTLCMD);
	}

	return(0);
}



/*
 * dgdm_tty_settermios()
 *
 * Set the termios structures as appropriate.
 */
STATIC void dgdm_tty_settermios(TTY *tty, TERMIOS *old)
{
	DRVCHAN *dc;
	TERMIOS *ts;

	if(!tty)
		return;
	dc = tty->driver_data;
	if(!dc)
		return;
	ts = tty->termios;
	if(!ts)
		return;

	DPR_IOCTL(("dgdm_tty_settermios(%d_%02d_%x)\n",
		   dc->brd->boardnum, dc->channum, dc->dxbchan));

	dc->user_oflag = ts->c_oflag;
	if((ts->c_oflag & OPOST) &&
           !(ts->c_oflag & ~(OPOST | OLCUC | ONLCR | OCRNL |
                             ONOCR | ONLRET | TABDLY))) {
		dc->ld_oflag = ts->c_oflag = dc->user_oflag & ~OPOST;
	} else {
		dc->ld_oflag = dc->user_oflag;
	}

	dgdm_tty_param(tty, dc, old);

	if(!(old->c_cflag & CLOCAL) && (tty->termios->c_cflag & CLOCAL))
		wake_up_interruptible(&dc->open_wait);
}



/*
 * dgdm_tty_param()
 *
 * Send termios settings to FEP
 */
STATIC void dgdm_tty_param(TTY *tty, DRVCHAN *dc, TERMIOS *old_termios)
{
	TERMIOS *ts = tty->termios;
	struct sdxb_param_command cmd;
	struct sdxb_flush_command fcmd;
	struct sdxb_mstat_command mcmd;
	struct sdxb_txprint_command pcmd;

        DPR_BASIC(("dgdm_tty_param(%d_%02d_%x)\n", 
		   dc->brd->boardnum, dc->channum, dc->dxbchan));

#if 0	/* this is still broken as of 990528 JAR */
	/*
	 * do some sanity checking before we get going to make sure that
	 * all of the params we're given are acceptable to the driver
	 */

	/*
	 * From the POSIX.1 spec: "If the input baud rate is equal
	 * to the numeral zero in the termios structure when
	 * tcsetattr is called, the input baud rate will be
	 * changed by tcsetattr() to the same value as that
	 * specified by the value of the output baud rate, exactly
	 * as if the input rate had been set to the output rate by
	 * cfsetispeed().  This usage of zero is obsolescent.
	 */
	if((ts->c_cflag & CIBAUD) == 0) {
		ts->c_cflag |= ((ts->c_cflag & CBAUD) << CIBAUD_SHIFT);
	}
	/*
	 * From the POSIX.1 spec: "If the input and output baud
	 * rates differ and are a combination that is not
	 * supported, neither baud rate is changed.  A subsequent
	 * call to tcgetattr() shall return the actual state of
	 * the terminal device [reflecting both the changes made
	 * and not made in the previous tcsetattr call]."
	 */
	if((ts->c_cflag & CBAUD) != ((ts->c_cflag & CIBAUD) >> CIBAUD_SHIFT)){
		/* The baudrates differ.  Change neither. */
		ts->c_cflag &= ~(CBAUD | CIBAUD);
		ts->c_cflag |= (old_termios->c_cflag & (CBAUD | CIBAUD));
	}
#endif

	/*
	 * Compute new parameter values
	 */
	if((ts->c_cflag & CBAUD) == B0) {
		/* discard pending Rx data, flush Tx queue */

		DPR_IOCTL(("dgdm_tty_param(%d_%02d_%x) - output is B0\n",
			   dc->brd->boardnum, dc->channum, dc->dxbchan));

		fcmd.ctype = SDXB_FLUSH;
		fcmd.flags = SDXB_FLUSH_INPUT | SDXB_FLUSH_OUTPUT;
		fcmd.fill1 = 0;
		fcmd.fill2 = 0;
		sdxbcmd(dc, CMD_NOWAIT, CMD_HIPRI, &fcmd, SDXB_FLUSH_CLEN);

		/* lower RTS & DTR */
		mcmd.ctype = SDXB_MSTAT;
		mcmd.flags = SDXB_IMMED | SDXB_HIPRI;
		mcmd.mint = 0xff;
		mcmd.mout = 0;
		sdxbcmd(dc, CMD_NOWAIT, CMD_HIPRI, &mcmd, SDXB_MSTAT_CLEN);

		dc->new.cflag = dc->old.cflag;
	} else {
		dc->new.cflag = ts->c_cflag;
	}

	dc->new.iflag = ts->c_iflag & (IGNBRK | BRKINT | IGNPAR | PARMRK |
				       INPCK | ISTRIP | INLCR | IGNCR |
                                       ICRNL | IUCLC | IXON | IXANY | IXOFF);

	dc->new.hflow = 0;	/* TODO */

	cmd.ctype = SDXB_PARAM;
	cmd.fill1 = 0;


	/*
	 * The S/DXB spec says that we should not be setting HUPCL or CLOCAL
	 * to the firmware as they are "not supported".  I have not checked 
	 * if not setting it breaks things or not, but the firware guys set
	 * 'em so we're gonna set them too.
	 * 
	 * It also says that CBAUD is "not supported" and since we set the
	 * baudrates elsewhere, we won't put CBAUD in the cflags field.
	 */
	cmd.cflags = ts->c_cflag & (CSIZE | CSTOPB | CREAD | PARENB | PARODD |
				    HUPCL | CLOCAL);

	/*
	 * The S/DXB spec says that we should not be setting INLCR, IGNCR, 
	 * ICRNL, or IUCLC to the firmware as they are "not supported".  
	 * I have not checked if not setting them breaks things or not, 
	 * but the firmware guys set 'em so we're going to set them too.
	 */
	cmd.iflags = ts->c_iflag & (IGNBRK | BRKINT | IGNPAR | PARMRK | INPCK |
                                    ISTRIP | INLCR | IGNCR | ICRNL | IUCLC |
				    IXON | IXANY | IXOFF);

	/* 
	 * The S/DXB spec says that we should not be setting OPOST to the
	 * firmware as it is "not supported" but that breaks things and the
	 * firmware guys set it so we're going to set it anyway 
	 *
	 * We're also not going to set anything if OPOST isn't set.
	 */
	if(dc->user_oflag & OPOST) {
		cmd.oflags = dc->user_oflag & (OPOST | OLCUC | ONLCR | OCRNL |
					       ONOCR | ONLRET | TABDLY);
	} else {
		cmd.oflags = 0;
	}

	/*
	 * Linux termios only uses XLNXT and XCASE.  However, the S/DXB spec
	 * says that we shouldn't set XCASE as it is "not supported".
	 *
	 * ALTPIN and XIXON are also set in the xflags field, so we going
	 * to play with the xflags if we need any of them.
	 */
	cmd.xflags = 0;
	if(L_ICANON(tty) && L_IEXTEN(tty)) {
		cmd.xflags |= SDXB_XLNXT;	/* enable lnext */
		cmd.lnext = LNEXT_CHAR(tty);
	}
	if(dc->flags & RASCON_ALTPIN) {
		cmd.xflags |= SDXB_XALTPIN;	/* enable altpin */
	}
	if(dc->flags & RASCON_AIXON) {
		cmd.xflags |= SDXB_XIXON;	/* Extra flow control chars */
	}
	cmd.fflags = 0;			/* Async should always be set to 0 */
	cmd.txaccm = dc->txaccm;
	cmd.rxaccm = dc->rxaccm;

	/* baudrates get their own integer field */
	cmd.speed = dgdm_tty_getbaud(tty, dc);

	cmd.mflow = ((ts->c_cflag & CRTSCTS) ? (SDXB_RTS | SDXB_CTS) : 0);
	cmd.mctrl = 0;


	/*
	 * From the POSIX.1 spec (7.1.2.6): "If {_POSIX_VDISABLE}
	 * is defined for the terminal device file, and the value
	 * of one of the changable special control characters (see
	 * 7.1.1.9) is {_POSIX_VDISABLE}, that function shall be
	 * disabled, that is, no input data shall be recognized as
	 * the disabled special character."
	 *
	 * OK, so we don't ever assign S/DXB XON or XOFF to _POSIX_VDISABLE.
	 */
	if(ts->c_cc[VSTART] != _POSIX_VDISABLE)
		cmd.xon = ts->c_cc[VSTART];
	if(ts->c_cc[VSTOP] != _POSIX_VDISABLE)
		cmd.xoff = ts->c_cc[VSTOP];


	/*
	 * From the POSIX.1 spec (7.1.2.6): "If {_POSIX_VDISABLE}
	 * is defined for the terminal device file, and the value
	 * of one of the changable special control characters (see
	 * 7.1.1.9) is {_POSIX_VDISABLE}, that function shall be
	 * disabled, that is, no input data shall be recognized as
	 * the disabled special character."
	 *
	 * OK, so we don't ever assign S/DXB XON or XOFF to _POSIX_VDISABLE.
	 */
	if(ts->c_cc[VSTART] != _POSIX_VDISABLE)
		cmd.xon = ts->c_cc[VSTART];
	if(ts->c_cc[VSTOP] != _POSIX_VDISABLE)
		cmd.xoff = ts->c_cc[VSTOP];

	/* Or the alternate XON of XOFF either */
	if(dc->xxon != _POSIX_VDISABLE)
		cmd.xxon = dc->xxon;
	if(dc->xxoff != _POSIX_VDISABLE)
		cmd.xxoff = dc->xxoff;

	/*
	 * Also, if either c_cc[START] or c_cc[STOP] is set to
	 * _POSIX_VDISABLE, we can't really do software flow
	 * control--in either direction--so we turn it off as
	 * far as S/DXB is concerned.  In essence, if you disable
	 * one, you disable the other too.
	 */
	if((ts->c_cc[VSTART] == _POSIX_VDISABLE) ||
           (ts->c_cc[VSTOP] == _POSIX_VDISABLE))
		cmd.iflags &= ~(SDXB_IXOFF | SDXB_IXON);

	/* Same with the alternate XON and XOFF characters */
	if((dc->xxon == _POSIX_VDISABLE) ||
           (dc->xxoff == _POSIX_VDISABLE))
		cmd.xflags &= ~SDXB_XIXON;

	cmd.spcr = 0;
	cmd.mspesc = 0;
	cmd.idel = 0;
	cmd.msbr = 0;
	cmd.mser = 0;
	cmd.maxlen = 0;

	sdxbcmd(dc, CMD_NOWAIT, CMD_HIPRI, &cmd, SDXB_PARAM_CLEN);

	if((ts->c_cflag & CBAUD) != B0) {
		mcmd.ctype = SDXB_MSTAT;
		mcmd.flags = SDXB_IMMED | SDXB_HIPRI;
		mcmd.mint = 0xff;
		mcmd.mout = dc->mstat & (SDXB_DTR | SDXB_RTS);

		sdxbcmd(dc, CMD_NOWAIT, CMD_HIPRI, &mcmd, SDXB_MSTAT_CLEN);
	}
	if(dgdm_print) {
		/* config the transparent print stuff */
		memset(&pcmd, 0, sizeof(pcmd));
		pcmd.ctype = SDXB_TXPRINT;
		pcmd.flags = 0;
		pcmd.xpflags = 0;
		pcmd.maxcps = dc->xp_maxcps;
		pcmd.maxchar = dc->xp_maxchar;
		pcmd.bufsize = dc->xp_bufsize;
		pcmd.onlen = dc->xp_onlen;
		pcmd.offlen = dc->xp_offlen;
		strncpy(pcmd.onstr, dc->xp_onstr,
			MIN(dc->xp_onlen, SDXB_TXP_PLEN));
		strncpy(pcmd.offstr, dc->xp_offstr,
			MIN(dc->xp_offlen, SDXB_TXP_PLEN));
                
		sdxbcmd(dc, CMD_NOWAIT, CMD_HIPRI, &pcmd, SDXB_TXPRINT_CLEN);
	}
	DPR_IOCTL(("dgdm_tty_param(%d_%02d_%x): baudrate %d\n",
                   dc->brd->boardnum, dc->channum, dc->dxbchan, cmd.speed));
	DPR_IOCTL(("    iflags 0x%x, cflags 0x%x, iflags 0x%x, oflags 0x%x\n",
		   dc->new.iflag, cmd.cflags, cmd.iflags, cmd.oflags));
	DPR_IOCTL(("    xflags 0x%x, VSTART 0x%x, VSTOP 0x%x\n",
		   cmd.xflags, ts->c_cc[VSTART], ts->c_cc[VSTOP]));

}



/*
 * dgdm_tty_getbaud()
 *
 * Take all of the factors into account and return the integer baudrate
 * requested.
 */
STATIC uint32 dgdm_tty_getbaud(TTY *tty, DRVCHAN *dc)
{
	TERMIOS *ts = tty->termios;
	unsigned int i;

	i = (tty->termios->c_cflag & CBAUD);

	if(!(i & CBAUDEX) && (i == B38400)) {
		switch (dc->flags & RASCON_SPD_MASK) {
		case RASCON_SPD_CUST:
			return(dc->custom_rate);
		case RASCON_SPD_HI:
			return(57600);
		case RASCON_SPD_SHI:
			return(115200);
		case RASCON_SPD_VHI:
			return(230400);
		case RASCON_SPD_WARP:
			return(460800);
		case RASCON_SPD_921:
			return(921600);
		case RASCON_SPD_1228:
			return(1228800);
		default:
			break;
		}
	}
	return(CBAUD2NUM(ts->c_cflag));
}



/*
 * dgdm_tty_getserial()
 *
 * Support setserial's reading of parameters.
 *
 * NOTE: UART type, Transmit FIFO size, closing wait, and hub6 are not
 *       supported, custom_divisor is hardwired to one so that the
 *       baud_base can be hijacked to directly set any arbitrary baudrate
 *       below MAXBAUD directly..
 */
STATIC int dgdm_tty_getserial(TTY *tty, struct serial_struct *retinfo)
{
	DRVCHAN *dc = tty->driver_data;
	struct serial_struct tmp;

	DPR_IOCTL(("dgdm_tty_getserial(%d_%02d_%x)\n",
		   dc->brd->boardnum, dc->channum, dc->dxbchan));

	if(!retinfo) {
		DPR_IOCTL(("dgdm_tty_getserial(%d_%02d_%x) - EFAULT #1\n",
			   dc->brd->boardnum, dc->channum, dc->dxbchan));
		return(-EFAULT);
	}
	memset(&tmp, 0, sizeof(tmp));
	tmp.line = dc->channum;
	tmp.port = dc->brd->ram;
	tmp.irq = dc->brd->irq;
	tmp.flags = dc->flags;
	tmp.baud_base = dc->custom_rate;
	tmp.close_delay = dc->close_delay;
	tmp.custom_divisor = 1;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
	if(copy_to_user(retinfo, &tmp, sizeof(*retinfo))) {
		DPR_IOCTL(("dgdm_tty_getserial(%d_%02d_%x) - EFAULT #2\n",
			   dc->brd->boardnum, dc->channum, dc->dxbchan));
		return(-EFAULT);
	}
#else
	memcpy_tofs(retinfo, &tmp, sizeof(*retinfo));
#endif

	DPR_IOCTL(("dgdm_tty_getserial(%d_%02d_%x) - done\n",
		   dc->brd->boardnum, dc->channum, dc->dxbchan));
	return(0);
}



/*
 * dgdm_tty_setserial()
 *
 * Support setserial's setting of parameters.  See the note for
 * dgdm_tty_getserial()
 *
 */
STATIC int dgdm_tty_setserial(TTY *tty, struct serial_struct *new_info)
{
	struct serial_struct new_serial;
	DRVCHAN *dc = tty->driver_data;

	DPR_IOCTL(("dgdm_tty_setserial(%d_%02d_%x)\n",
		   dc->brd->boardnum, dc->channum, dc->dxbchan));

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
	if(copy_from_user(&new_serial, new_info, sizeof(new_serial))) {
		DPR_IOCTL(("dgdm_tty_setserial(%d_%02d_%x) - EFAULT\n",
			   dc->brd->boardnum, dc->channum, dc->dxbchan));
		return(-EFAULT);
	}
#else
	memcpy_fromfs(&new_serial, new_info, sizeof(new_serial));
#endif

	if(new_serial.baud_base > MAXBAUD) {
		DPR_IOCTL(("dgdm_tty_setserial(%d_%02d_%x) - invalid baud\n",
			   dc->brd->boardnum, dc->channum, dc->dxbchan));
		return(-EINVAL);
	}
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
	if(!capable(CAP_SYS_ADMIN))
#else
        if(!suser())
#endif
	{
		if((new_serial.close_delay != dc->close_delay) ||
                   ((new_serial.flags & ~RASCON_USR_MASK) !=
                    (dc->flags & ~RASCON_USR_MASK))) {
			DPR_IOCTL(("dgdm_tty_setserial(%d_%02d_%x) - EPERM\n",
				   dc->brd->boardnum, dc->channum,
				   dc->dxbchan));
			return(-EPERM);
		}
		dc->flags = ((dc->flags & ~RASCON_USR_MASK) |
			     (new_serial.flags & RASCON_USR_MASK));
		dc->custom_rate = new_serial.baud_base;
	} else {
		dc->flags = ((dc->flags & ~RASCON_USR_MASK) |
			     (new_serial.flags & RASCON_USR_MASK));
		dc->close_delay = ((new_serial.close_delay * HZ) / 100);
		dc->custom_rate = new_serial.baud_base;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
		dc->tty->low_latency = ((dc->flags & RASCON_LOW_LATENCY) ?
					1 : 0);
#endif
	}

	/* Update the settings to the firmware */
	dgdm_tty_param(tty, dc, NULL);

	DPR_IOCTL(("dgdm_tty_setserial(%d_%02d_%x) - done\n",
		   dc->brd->boardnum, dc->channum, dc->dxbchan));
	return(0);
}



/* 
 * dgdm_tty_digigeta()
 *
 * Ioctl to get the information for ditty.
 *
 * NOTE: DIGI_FAST, DIGI_IXON, DSRPACE, DCDPACE, and DTRPACE are unsupported.
 *
 */
STATIC int dgdm_tty_digigeta(TTY * tty, struct digi_struct *retinfo)
{
	DRVCHAN *dc = tty->driver_data;
	struct digi_struct tmp;

	DPR_IOCTL(("dgdm_tty_digigeta(%d_%02d_%x)\n",
		   dc->brd->boardnum, dc->channum, dc->dxbchan));

	if(!retinfo) {
		DPR_IOCTL(("dgdm_tty_digigeta(%d_%02d_%x) - EFAULT #1\n",
			   dc->brd->boardnum, dc->channum, dc->dxbchan));
		return(-EFAULT);
	}
	memset(&tmp, 0, sizeof(tmp));
	tmp.digi_flags = DIGI_IXON;	/* IXON is always done in the FEP */

	if(dc->tty->termios->c_cflag & CRTSCTS) {
		/* using RTS and CTS flow control */
		tmp.digi_flags |= (RTSPACE | CTSPACE);
	}
	if(dc->flags & RASCON_SOFTCAR) {
		tmp.digi_flags |= DIGI_FORCEDCD;
	}
	if(dc->flags & RASCON_ALTPIN) {
		tmp.digi_flags |= DIGI_ALTPIN;
	}
	if(dc->flags & RASCON_AIXON) {
		tmp.digi_flags |= DIGI_AIXON;
	}
	tmp.digi_maxcps = dc->xp_maxcps;
	tmp.digi_maxchar = dc->xp_maxchar;
	tmp.digi_bufsize = dc->xp_bufsize;
	tmp.digi_onlen = dc->xp_onlen;
	tmp.digi_offlen = dc->xp_offlen;

	(void) strncpy(tmp.digi_onstr, dc->xp_onstr,
		       MIN(dc->xp_onlen, SDXB_TXP_PLEN));
	(void) strncpy(tmp.digi_offstr, dc->xp_offstr,
		       MIN(dc->xp_offlen, SDXB_TXP_PLEN));
	(void) strncpy(tmp.digi_term, dc->xp_term,
		       MIN(strlen(dc->xp_term), SDXB_TXP_TLEN));

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
	if(copy_to_user(retinfo, &tmp, sizeof(*retinfo))) {
		DPR_IOCTL(("dgdm_tty_digigeta(%d_%02d_%x) - EFAULT #2\n",
			   dc->brd->boardnum, dc->channum, dc->dxbchan));
		return(-EFAULT);
	}
#else
	memcpy_tofs(retinfo, &tmp, sizeof(*retinfo));
#endif

	DPR_IOCTL(("dgdm_tty_digigeta(%d_%02d_%x) - done\n",
		   dc->brd->boardnum, dc->channum, dc->dxbchan));
	return(0);
}



/* 
 * dgdm_tty_digiseta()
 *
 * Ioctl to set the information from ditty.
 *
 * NOTE: DIGI_FAST, DIGI_IXON, DSRPACE, DCDPACE, and DTRPACE are unsupported.
 *
 */
STATIC int dgdm_tty_digiseta(TTY * tty, struct digi_struct *new_info)
{
	struct digi_struct new_digi;
	DRVCHAN *dc = tty->driver_data;

	DPR_IOCTL(("dgdm_tty_digiseta(%d_%02d_%x)\n",
		   dc->brd->boardnum, dc->channum, dc->dxbchan));

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
	if(copy_from_user(&new_digi, new_info, sizeof(new_digi))) {
		DPR_IOCTL(("dgdm_tty_digiseta(%d_%02d_%x) - EFAULT\n",
			   dc->brd->boardnum, dc->channum, dc->dxbchan));
		return(-EFAULT);
	}
#else
	memcpy_fromfs(&new_digi, new_info, sizeof(new_digi));
#endif

	if((new_digi.digi_flags & RTSPACE) || (new_digi.digi_flags & CTSPACE))
		dc->tty->termios->c_cflag |= CRTSCTS;
	else
		dc->tty->termios->c_cflag &= ~CRTSCTS;

	if(new_digi.digi_flags & DIGI_FORCEDCD)
		dc->flags |= RASCON_SOFTCAR;
	else
		dc->flags &= ~RASCON_SOFTCAR;

	if(new_digi.digi_flags & DIGI_ALTPIN)
		dc->flags |= RASCON_ALTPIN;
	else
		dc->flags &= ~RASCON_ALTPIN;

	if(new_digi.digi_flags & DIGI_AIXON)
		dc->flags |= RASCON_AIXON;
	else
		dc->flags &= ~RASCON_AIXON;

	dc->xp_maxcps = new_digi.digi_maxcps;
	dc->xp_maxchar = new_digi.digi_maxchar;
	dc->xp_bufsize = new_digi.digi_bufsize;
	dc->xp_onlen = new_digi.digi_onlen;
	dc->xp_offlen = new_digi.digi_offlen;

	memset(dc->xp_onstr, 0, SDXB_TXP_PLEN);
	(void) strncpy(dc->xp_onstr, new_digi.digi_onstr,
		       MIN(dc->xp_onlen, SDXB_TXP_PLEN));

	memset(dc->xp_offstr, 0, SDXB_TXP_PLEN);
	(void) strncpy(dc->xp_offstr, new_digi.digi_offstr,
		       MIN(dc->xp_offlen, SDXB_TXP_PLEN));

	memset(dc->xp_term, 0, SDXB_TXP_TLEN);
	(void) strncpy(dc->xp_term, new_digi.digi_term,
		       MIN(strlen(new_digi.digi_term), SDXB_TXP_TLEN));

	/* make the changes now */
	dgdm_tty_param(tty, dc, NULL);

	DPR_IOCTL(("dgdm_tty_digiseta(%d_%02d_%x) - done\n",
		   dc->brd->boardnum, dc->channum, dc->dxbchan));
	return(0);
}



/*
 * dgdm_tty_throttle()
 *
 * Throttle receiving data.  We just set a bit and stop reading
 * data out of the channel buffer.  It will back up and the
 * FEP will do whatever is necessary to stop the far end.
 */
STATIC void dgdm_tty_throttle(TTY *tty)
{
	DRVCHAN *dc;

	if(!tty)
		return;
	dc = tty->driver_data;
	if(!dc)
		return;

	DPR_BASIC(("dgdm_tty_throttle(%d_%02d_%x)\n",
		   dc->brd->boardnum, dc->channum, dc->dxbchan));

	set_bit(STATE_RXSTOP, &dc->state);
}



/*
 * dgdm_tty_unthrottle()
 *
 * Unthrottle receiving data.  Start reading the channel buffer again and
 * the FEP will restart the far end again.
 */
STATIC void dgdm_tty_unthrottle(TTY *tty)
{
	DRVCHAN *dc;

	if(!tty)
		return;
	dc = tty->driver_data;
	if(!dc)
		return;

	DPR_BASIC(("dgdm_tty_unthrottle(%d_%02d_%x)\n",
		   dc->brd->boardnum, dc->channum, dc->dxbchan));

	clear_bit(STATE_RXSTOP, &dc->state);
}



/*
 * dgdm_tty_stop()
 *
 * Stop the transmitter
 */
STATIC void dgdm_tty_stop(TTY *tty)
{
	DRVCHAN *dc;
	struct sdxb_pause_command pcmd;

	if(!tty)
		return;
	dc = tty->driver_data;
	if(!dc)
		return;

	DPR_BASIC(("dgdm_tty_stop(%d_%02d_%x)\n",
		   dc->brd->boardnum, dc->channum, dc->dxbchan));

	pcmd.ctype = SDXB_PAUSE;
	pcmd.pset = SDXB_OPU;
	pcmd.preset = 0;
	pcmd.fill = 0;

	sdxbcmd(dc, CMD_NOWAIT, CMD_HIPRI, &pcmd, SDXB_PAUSE_CLEN);
}



/*
 * dgdm_tty_start()
 *
 * Start the transmitter again.
 */
STATIC void dgdm_tty_start(TTY *tty)
{
	DRVCHAN *dc;
	struct sdxb_pause_command pcmd;

	if(!tty)
		return;
	dc = tty->driver_data;
	if(!dc)
		return;

	DPR_BASIC(("dgdm_tty_start(%d_%02d_%x)\n",
		   dc->brd->boardnum, dc->channum, dc->dxbchan));

	pcmd.ctype = SDXB_PAUSE;
	pcmd.pset = 0;
	pcmd.preset = SDXB_OPS | SDXB_OPU;
	pcmd.fill = 0;

	sdxbcmd(dc, CMD_NOWAIT, CMD_HIPRI, &pcmd, SDXB_PAUSE_CLEN);
}



/*
 * dgdm_tty_input_stop()
 *
 * Stop the reciever
 */
STATIC void dgdm_tty_input_stop(TTY *tty)
{
	DRVCHAN *dc;
	struct sdxb_scimmed_command scmd;

	if(!tty)
		return;
	dc = tty->driver_data;
	if(!dc)
		return;

	if(STOP_CHAR(tty) != _POSIX_VDISABLE) {

		DPR_BASIC(("dgdm_tty_input_stop(%d_%02d_%x)\n",
			   dc->brd->boardnum, dc->channum, dc->dxbchan));

		scmd.ctype = SDXB_SCIMMED;
		scmd.sic = STOP_CHAR(tty);
		scmd.fill = 0;
		scmd.fill2 = 0;

		sdxbcmd(dc, CMD_NOWAIT, CMD_HIPRI, &scmd, SDXB_SCIMMED_CLEN);
	}
}



/*
 * dgdm_tty_input_start()
 *
 * Start the reciever again.
 */
STATIC void dgdm_tty_input_start(TTY *tty)
{
	DRVCHAN *dc;
	struct sdxb_scimmed_command scmd;

	if(!tty)
		return;
	dc = tty->driver_data;
	if(!dc)
		return;

	if(START_CHAR(tty) != _POSIX_VDISABLE) {

		DPR_BASIC(("dgdm_tty_input_start(%d_%02d_%x)\n",
			   dc->brd->boardnum, dc->channum, dc->dxbchan));

		scmd.ctype = SDXB_SCIMMED;
		scmd.sic = START_CHAR(tty);
		scmd.fill = 0;
		scmd.fill2 = 0;

		sdxbcmd(dc, CMD_NOWAIT, CMD_HIPRI, &scmd, SDXB_SCIMMED_CLEN);
	}
}



/*
 * get_modem_info()
 *
 * Get current modem status
 */
STATIC int get_modem_info(DRVCHAN *dc, unsigned int *value)
{
	unsigned int result;

	result = (((dc->mstat & dc->m_rts) ? TIOCM_RTS : 0) |
                  ((dc->mstat & dc->m_dtr) ? TIOCM_DTR : 0) |
                  ((dc->mstat & dc->m_dcd) ? TIOCM_CAR : 0) |
                  ((dc->mstat & dc->m_ri)  ? TIOCM_RNG : 0) |
                  ((dc->mstat & dc->m_dsr) ? TIOCM_DSR : 0) |
                  ((dc->mstat & dc->m_cts) ? TIOCM_CTS : 0));
	put_user(result, value);
	return(0);
}



/*
 * set_modem_info()
 *
 * Set modem lines
 */
STATIC int set_modem_info(DRVCHAN *dc, unsigned int command,
                          unsigned int *value)
{
	int error;
	unsigned int arg;
	struct sdxb_mstat_command cmd;

	error = verify_area(VERIFY_READ, value, sizeof(int));
	if(error)
		return(error);
	GET_USER(arg, value);
	switch (command) {
	case TIOCMBIS:
		if(arg & TIOCM_RTS) {
			dc->mstat |= SDXB_RTS;
		}
		if(arg & TIOCM_DTR) {
			dc->mstat |= SDXB_DTR;
		}
		break;
	case TIOCMBIC:
		if(arg & TIOCM_RTS) {
			dc->mstat &= ~SDXB_RTS;
		}
		if(arg & TIOCM_DTR) {
			dc->mstat &= ~SDXB_DTR;
		}
		break;
	case TIOCMSET:
		dc->mstat = (((dc->mstat & ~(SDXB_RTS | SDXB_DTR)) |
                              ((arg & TIOCM_RTS) ? SDXB_RTS : 0)   |
			      ((arg & TIOCM_DTR) ? SDXB_DTR : 0)));
		break;
	default:
		return(-EINVAL);
	}

	cmd.ctype = SDXB_MSTAT;
	cmd.flags = SDXB_IMMED | SDXB_HIPRI;
	cmd.mint = 0xff;	/* send state of all lines */
	cmd.mout = dc->mstat & (SDXB_DTR | SDXB_RTS);

	sdxbcmd(dc, CMD_NOWAIT, CMD_HIPRI, &cmd, SDXB_MSTAT_CLEN);

	return(0);
}








/************************************************************************
 *
 * Poller and functions called from poller
 *
 ************************************************************************/

/*
 * dgdm_tty_poll()
 *
 * The poller gets called every clock tick (nominally every 10 msecs)
 */
STATIC void dgdm_tty_poll(unsigned long ignored)
{
	BOARD_T *brd;
        int i;
	static int pollcnt = 0;

	TimerOn = 0;

	++pollcnt;

	if (pollcnt > MA_PERIOD)
		pollcnt = 0;
	
	/* Loop, for each board */

	for (i=0; i < dgdm_NumBoards; i++) {
                brd = dgdm_Board[i];

                if(!brd)
                        continue;

		/*
		 * Insure that we didn't interrupt somebody in the
		 * middle of a dxb_lock()...dxb_unlock() region.  If
		 * we did, go on to the next board.  The poller will
		 * be run when dxb_unlock() is called.
		 */
		if(dxb_lock_poller(brd)) {
			/*
			 * Poll a single board for DXB messages as well
			 * as handling any data accumulated in the rxbuf's.
			 */
			if(brd->intr_used) {
				/*
				 * poll only transmit here.  Rx polled
				 * at interrupt time.
				 */
				dgdm_tty_poll_board(brd, TRUE, FALSE);
			} else {
				/* poll both tx & rx */
				dgdm_tty_poll_board(brd, TRUE, TRUE);
			}
			dxb_unlock_poller(brd);
		}
		/*
		 * Every MA_PERIOD (e.g. 100 msec), update the
		 * moving averages.  We split the ISDN and TTY
		 * counters over two ticks.
		 */
		if(pollcnt == 1 && brd->dxbisdn_chans) {
			int i, j;
			DXBSTRM *ds;
			static uint randnum = 152;

			randnum = randnum * 69069 + 1;
			for (ds = brd->dxbisdn_chans, i = 0;
			     i < brd->nttychans; ++i, ++ds) {
				for (j = 0; j < NSTAT_Str; ++j) {
					moving_average(&ds->stat[j], randnum);
				}
			}
		}
		if(pollcnt == MA_PERIOD) {
			int i, j;
			DRVCHAN *dc;
			static uint randnum = 152;

			randnum = randnum * 69069 + 1;
			for (dc = brd->dc, i = 0; i < brd->nchans; ++i, ++dc) {
				for (j = 0; j < NSTAT; ++j) {
					moving_average(&dc->stat[j], randnum);
				}
			}
		}
	}

	/* Reschedule ourselves */
	dxb_start_timer();
}



/*
 * dgdm_tty_poll_board()
 *
 * Poll a single board
 */
GLOBAL void dgdm_tty_poll_board(BOARD_T *brd, int polltx, int pollrx)
{
	DRVCHAN *dc;
	int i;
	int rc;

        if(!brd)
                return;

	poller_running = 1;
	/* Run DXB's polling function */
	rc = 0;

	if(pollrx)
		rc = dxb_rxpoll(brd->dxb);

	if(rc) {
		APR(("DXB poller stopped due to illegal command\n"));
		poller_running = 0;
		return;
	}
	if(polltx)
		dxb_txpoll_all(brd->dxb);

	/*
	 * Loop thru each open channel, processing any data
	 * in its rxbuf receive circular buffer.
	 */
	for (dc = brd->dc, i = 0; i < brd->nchans; ++i, ++dc) {
		if(polltx && dc->sendcmds)
			send_deferred_commands(dc, FALSE);
		if(pollrx && dc->opencnt && dc->poll_ok)
			dxb_rxbuf(brd, dc);
	}
	poller_running = 0;
}



/*
 * data_cb()
 *
 *
 */
STATIC int data_cb(DXB *dxb, int channel, int data_len, DXB_ARG arg)
{
	DRVCHAN *dc = (DRVCHAN *) arg;
	TTY *tty = dc->tty;
	int len;
	int rc;
	int flip_len;
        int boardnum = dc->brd->boardnum;

	static char myflipbuf[MAXBOARDS][MYFLIPLEN];

	DPR_DXB(("data_cb(%d_%02d_%x) - len=%d\n",
		 boardnum, dc->channum, dc->dxbchan, data_len));

	if(!tty || (tty->magic != TTY_MAGIC)) {
                /* We have stuff for us in the DXB queue.  Suck it out. */
                if(data_len > 0) {
                        dxb_return_fixed_cost(dxb, channel, 1);
                        dxb_read_data_block(dxb, dgdm_bitbucket,
                                            data_len, data_len);
                        dxb_return_credits(dxb, channel, data_len);
                        dxb_read_done(dxb);
                }
                DPR_DXB(("data_cb(%d_%02d_%x) - Stale or NULL tty=0x%p\n",
                         boardnum, dc->channum, dc->dxbchan, tty));
		return(0);
        }

	if(!dc->opencnt)
		return(0);

	/*
	 * If the rxbuf is empty and we are not throttled, put as much
	 * as we can directly into the linux TTY flip buffer.
	 * The dgdm_rawreadok case takes advantage of carnal knowldge that 
	 * the char_buf and the flag_buf are next to each other and 
	 * are each of (2 * TTY_FLIPBUF_SIZE) size.
	 *
	 * NOTE: if(!tty->real_raw), the call to ldisc.receive_buf
	 *       actually still uses the flag buffer, so you can't
	 *       use it for input data
	 */
	if(dgdm_rawreadok) {
		if(tty->real_raw)
			flip_len = MYFLIPLEN;
		else
			flip_len = 2 * TTY_FLIPBUF_SIZE;
	} else
		flip_len = TTY_FLIPBUF_SIZE - tty->flip.count;

	len = MIN(data_len, flip_len);

	len = MIN(len, (N_TTY_BUF_SIZE - 1) - tty->read_cnt);

	dxb_return_fixed_cost(dxb, channel, 1);

	if(len && (test_bit(STATE_RXSTOP, &dc->state) == 0) &&
           (dxb_rxbuf_count(dxb, channel) == 0)) {
		/*
                 * If we're bypassing flip buffers on rx, we can blast it
		 * right into the beginning of the buffer.
		 */
		if(dgdm_rawreadok)
			if(tty->real_raw)
				dxb_read_data_block(dxb, myflipbuf[boardnum],
                                                    len, len);
			else
				dxb_read_data_block(dxb, tty->flip.char_buf,
						    len, len);
		else
			dxb_read_data_block(dxb, tty->flip.char_buf_ptr,
					    len, len);

		dxb_return_credits(dxb, channel, len);

		data_len -= len;

		/*
                 * In high performance mode, we don't have to update
		 * flag_buf or any of the counts or pointers into flip buf.
		 */
		if(!dgdm_rawreadok) {
			if(I_PARMRK(tty)) {
				parity_scan(dc, tty->flip.char_buf_ptr,
					    tty->flip.flag_buf_ptr,
					    &len);
			} else {
				memset(tty->flip.flag_buf_ptr, 0, len);
			}
			tty->flip.char_buf_ptr += len;
			tty->flip.flag_buf_ptr += len;
			tty->flip.count += len;
		} else if(!tty->real_raw) {
			if(I_PARMRK(tty)) {
				parity_scan(dc, tty->flip.char_buf,
					    tty->flip.flag_buf, &len);
			} else {
				memset(tty->flip.flag_buf, 0, len);
			}
		}
		DPR_READ(("dxb_data_cb(%d_%02d_%x) - len=%d\n",
                          boardnum, dc->channum, dc->dxbchan, len));

		dc->stat[RxSTAT].cnt += len;
		atomic_add(len, (atomic_t *) & dc->stat[RxSTAT].accum);

		/*
                 * If we're doing raw reads, jam it right into the 
		 * line disc bypassing the flip buffers.   
		 * OPT OPP: do this only once per channel instead of 
		 * once per dxb message.
		 */
		if(dgdm_rawreadok)
			if(tty->real_raw)
				tty->ldisc.receive_buf(tty,
                                                       myflipbuf[boardnum],
						       NULL, len);
			else
				tty->ldisc.receive_buf(tty, tty->flip.char_buf,
                                                       tty->flip.flag_buf,
                                                       len);
		else
			tty_schedule_flip(tty);

	}
	/*
	 * Copy the rest of the data into the rxbuf circular buffer
	 * to be processed on the next poll.
	 */
	if(data_len)
		rc = dxb_rxbuf_copyin(dxb, channel, data_len);

	dxb_read_done(dxb);

	return(0);
}



/*
 * cdata_cb()
 *
 *
 */
STATIC int cdata_cb(DXB *dxb, int channel, int pri, int ctrl_len,
                    int data_len, DXB_ARG arg)
{
	DRVCHAN *dc = (DRVCHAN *) arg;
	TTY *tty = dc->tty;
	int len;
	int rc;
	int flip_len;
	int cmdbuf[(SDXB_MAX_RLEN + 1) / 4];
	char *cp;

	DPR_DXB(("cdata_cb(%d_%02d_%x) - clen=%d dlen=%d\n",
		 dc->brd->boardnum, dc->channum, channel,
		 ctrl_len, data_len));

	if(!tty || (tty->magic != TTY_MAGIC)) {
                /* We have stuff for us in the DXB queue.  Suck it out. */
                if(ctrl_len == 0 && pri == 0 && data_len > 0) {
                        dxb_return_fixed_cost(dxb, channel, 1);
                        dxb_read_data_block(dxb, dgdm_bitbucket,
                                            data_len, data_len);
                        dxb_return_credits(dxb, channel, data_len);
                        dxb_read_done(dxb);
                } else if(ctrl_len > 0) {
                        len = ctrl_len;
                        if(len > SDXB_MAX_RLEN)
                                len = SDXB_MAX_RLEN;
                        dxb_read_ctrl_block(dxb, dgdm_bitbucket,
                                            SDXB_MAX_RLEN, len);
                        dxb_read_done(dxb);
                        if(!pri) {
                                dxb_return_fixed_cost(dxb, channel, 1);
                                dxb_return_credits(dxb,channel, len);
                        }
                }
                DPR_DXB(("cdata_cb(%d_%02d_%x) - stale or null tty=0x%p\n",
			 dc->brd->boardnum, dc->channum, dc->dxbchan, tty));
		return(0);
	}

	/*
	 * The check for dc->opencnt == 0 removed here, and moved
	 * to each particular message processing section.  This is because
	 * some CDATA messages need to be processed while in the close
	 * routine (specifically, the SYNC response).
	 *
	 * And some (like MSTAT) are needed in open when blocking for carrier.
         *
	 */

	if(ctrl_len == 0 && pri == 0) {
		/* normal pri, zero ctrl len is same as data */

		if(!dc->opencnt)
			return(0);

		/*
		 * If the rxbuf is empty and we are not throttled, put
		 * as much as we can directly into the linux TTY flip
		 * buffer.
		 */
		flip_len = TTY_FLIPBUF_SIZE - tty->flip.count;
		len = MIN(data_len, flip_len);

		dxb_return_fixed_cost(dxb, channel, 1);

		if(len && (test_bit(STATE_RXSTOP, &dc->state) == 0) &&
                   (dxb_rxbuf_count(dxb, channel) == 0)) {
			dxb_read_data_block(dxb, tty->flip.char_buf_ptr,
					    len, len);

			dxb_return_credits(dxb, channel, len);

			data_len -= len;

			memset(tty->flip.flag_buf_ptr, 0, len);
			tty->flip.char_buf_ptr += len;
			tty->flip.flag_buf_ptr += len;
			tty->flip.count += len;

			dc->stat[RxSTAT].cnt += len;

			atomic_add(len, (atomic_t *) & dc->stat[RxSTAT].accum);
			APR(("Does this ever happen?\n"));
			tty_schedule_flip(tty);
		}
		/*
		 * Copy the rest of the data into the rxbuf circular
		 * buffer to be processed on the next poll.
		 */
		if(data_len)
			rc = dxb_rxbuf_copyin(dxb, channel, data_len);

		dxb_read_done(dxb);

		return(0);
	}

	/* high priority data?  we dump it :-) */
	if(ctrl_len == 0)
		return(0);

	/* everything left must be a command; either high or low priority */
	cp = (char *) &cmdbuf[0];
	len = ctrl_len;
	if(len > SDXB_MAX_RLEN)
		len = SDXB_MAX_RLEN;

	dxb_read_ctrl_block(dxb, cp, SDXB_MAX_RLEN, len);
	dxb_read_done(dxb);
	if(!pri) {
		dxb_return_fixed_cost(dxb, channel, 1);
		dxb_return_credits(dxb, channel, len);
	}
	switch (*cp) {
	case SDXB_MSTAT:
                return(cdata_cb_mstat(dc, cp, len));
	case SDXB_SYNC:
                return(cdata_cb_sync(dc, cp, len));
	case SDXB_SEQ:
                return(cdata_cb_seq(dc, cp, len));
        case SDXB_SCRECV:
                return(cdata_cb_screcv(dc, cp, len));
        case SDXB_EVENT:
                return(cdata_cb_event(dc, cp, len));
	case SDXB_TXPRINT:
                return(cdata_cb_txprint(dc, cp, len));
	case SDXB_PARAM:
		break;
	default:
		{
			int x, y;
			char buf[81];
			static char hx[] = "0123456789ABCDEF";
			DPR(("cdata_cb(%d_%02d_%x) - unrecognized sdxb "
                             "message: \n", dc->brd->boardnum, dc->channum,
			     dc->dxbchan));
			y = 0;
			for (x = 0; x < len && x < 128; x++) {
				buf[y * 3 + 0] = hx[(cp[x] >> 4) & 0xf];
				buf[y * 3 + 1] = hx[cp[x] & 0xf];
				buf[y * 3 + 2] = ' ';
				y++;
				if(y == 16) {
					buf[y * 3 + 0] = '\n';
					buf[y * 3 + 1] = '\0';
					DPR(("%s", buf));
					y = 0;
				}
			}
			if(y != 0) {
				buf[y * 3 + 0] = '\n';
				buf[y * 3 + 1] = '\0';
				DPR(("%s", buf));
			}
		}
	}
	return(0);
}



/*
 * cdata_cb_mstat
 *
 *
 */
STATIC inline int cdata_cb_mstat(DRVCHAN *dc, char *cp, int len)

{
	TTY *tty = dc->tty;
        struct sdxb_mstat_response *rsp;
        uchar omstat = dc->mstat;

        if(dc->dxbchan < 0) {
                APR(("cdata_cb_mstat(%d_%02d_%x) - after close.\n",
                     dc->brd->boardnum, dc->channum, dc->dxbchan));
                return(-ENXIO);
        }

        rsp = (struct sdxb_mstat_response *) cp;

        if(len != SDXB_MSTAT_RLEN) {
                DPR(("cdata_cb_mstat(%d_%02d_%x) - bad length %x\n",
                     dc->brd->boardnum, dc->channum, dc->dxbchan, len));
                return(-EIO);
        }
        DPR_MCTRL(("cdata_cb_mstat(%d_%02d_%x) -  %x, %x, %x\n",
                   dc->brd->boardnum, dc->channum, dc->dxbchan,
                   rsp->mstat, omstat, tty->termios->c_cflag & CLOCAL));
        dc->mstat &= (SDXB_DTR | SDXB_RTS);
        dc->mstat |= rsp->mstat & ~(SDXB_DTR | SDXB_RTS);

        if((omstat & dc->m_dcd) && (dc->mstat & dc->m_dcd) == 0) {
                DPR_MCTRL(("cdata_cb_mstat(%d_%02d_%x) - DCD DROP\n",
                           dc->brd->boardnum, dc->channum, dc->dxbchan));
		/* DCD dropped */
                if(!(tty->termios->c_cflag & CLOCAL))
                        tty_hangup(tty);

                dc->dtrlow = jiffies - DEFAULT_DTRLOW;
        }
        if(((omstat & dc->m_dcd) == 0) && (dc->mstat & dc->m_dcd)) {
                DPR_MCTRL(("cdata_cb_mstat(%d_%02d_%x) - DCD UP\n",
                           dc->brd->boardnum, dc->channum, dc->dxbchan));
		/* DCD up */
                wake_up_interruptible(&dc->open_wait);
        }
        return(0);
}



/*
 * cdata_cb_sync()
 *
 *
 */
STATIC inline int cdata_cb_sync(DRVCHAN *dc, char *cp, int len)
{
        struct sdxb_sync_response *rsp;
        rsp = (struct sdxb_sync_response *) cp;

        if(len != SDXB_SYNC_RLEN) {
                DPR(("cdata_cb_sync(%d_%02d_%x) - bad length %x\n",
                     dc->brd->boardnum, dc->channum, dc->dxbchan, len));
                return(-EIO);
        }
        DPR_MCTRL(("cdata_cb_sync(%d_%02d_%x) - %x -- %x\n",
                   dc->brd->boardnum, dc->channum, dc->dxbchan,
                   rsp->sseq, dc->syncseq));
        /*
         * Is it valid to only send a response back if these
         * are equal?  Should we not be sending it back up
         * regardless since we can have multiple sync's 
         * pending?   JAR 990709
         */
        if(rsp->sseq == dc->syncseq){
            //wake_up_interruptible(&dc->sync_wait);
            wake_up(&dc->sync_wait);
        }

        return(0);
}



/*
 * cdata_cb_seq()
 *
 *
 */
STATIC inline int cdata_cb_seq(DRVCHAN *dc, char *cp, int len)
{
        struct sdxb_seq_response *rsp;
        rsp = (struct sdxb_seq_response *) cp;

        if(len != SDXB_SEQ_RLEN) {
                DPR(("cdata_cb_seq(%d_%02d_%x) - bad length %x\n",
                     dc->brd->boardnum, dc->channum, dc->dxbchan, len));
                return(-EIO);
        }
        DPR_MCTRL(("cdata_cb_seq(%d_%02d_%x) - TOUT %x\n",
                   dc->brd->boardnum, dc->channum, dc->dxbchan,
                   (unsigned int) rsp->tout));
        return(0);
}



/*
 * cdata_cb_screcv()
 *
 *
 */
STATIC inline int cdata_cb_screcv(DRVCHAN *dc, char *cp, int len)
{
        if(len != SDXB_SCRECV_RLEN) {
                DPR(("cdata_cb_screcv(%d_%02d_%x) - bad length %x\n",
                     dc->brd->boardnum, dc->channum, dc->dxbchan, len));
                return(-EIO);
        }

#if defined(DGDM_TRACER)
        {
                static int xxx;
                DPR(("cdata_cb_screcv(%d_%02d_%x) - %d\n",
                     dc->brd->boardnum, dc->channum, dc->dxbchan, xxx++));
        }
#endif

        return(0);
}



/*
 * cdata_cb_event()
 *
 *
 */
STATIC inline int cdata_cb_event(DRVCHAN *dc, char *cp, int len)
{
	TTY *tty = dc->tty;
        struct sdxb_event_response *rsp;
        rsp = (struct sdxb_event_response *) cp;

        if(len != SDXB_EVENT_RLEN) {
                DPR(("cdata_cb_event(%d_%02d_%x) - bad length %x\n",
                     dc->brd->boardnum, dc->channum, dc->dxbchan, len));
                return(-EIO);
        }
        if(rsp->estat & SDXB_RXB) {
                if(dc->dxbchan < 0) {
                        APR(("cdata_cb_event(%d_%02d_%x) - RXBreak after "
                             "close.\n", dc->brd->boardnum, dc->channum, 
                             dc->dxbchan));
                        return(-ENXIO);
                }

                DPR(("cdata_cb_event(%d_%02d_%x) - RXBreak\n",
                     dc->brd->boardnum, dc->channum, dc->dxbchan));

                tty->flip.count++;
                *tty->flip.flag_buf_ptr++ = TTY_BREAK;
                *tty->flip.char_buf_ptr++ = 0;
                tty_schedule_flip(tty);
        } else {
                DPR(("cdata_cb_event(%d_%02d_%x) - %d\n",
                     dc->brd->boardnum, dc->channum, dc->dxbchan, rsp->estat));
        }
        return(0);
}



/*
 * cdata_cb_txprint()
 *
 *
 */
STATIC inline int cdata_cb_txprint(DRVCHAN *dc, char *cp, int len)
{
        struct sdxb_txprint_response *rsp;
        rsp = (struct sdxb_txprint_response *) cp;

        if(len != SDXB_TXPRINT_RLEN) {
                DPR(("cdata_cb_txprint(%d_%02d_%x) - bad length %x\n",
                     dc->brd->boardnum, dc->channum, dc->dxbchan, len));
                return(-EIO);
        }
        DPR(("cdata_cb_txprint(%d_%02d_%x) - got response\n",
             dc->brd->boardnum, dc->channum, dc->dxbchan));
        return(0);
}



/*
 * credit_cb()
 */
STATIC int credit_cb(DXB *dxb, int channel, int credits, DXB_ARG arg)
{
	DRVCHAN *dc = (DRVCHAN *) arg;
	TTY *tty = dc->tty;

	if(!tty)
		return(0);
	if(!dc->opencnt)
		return(0);

	DPR_DXB(("credit_cb(%d_%02d_%x) - total_credits=%d\n",
		 dc->brd->boardnum, dc->channum, dc->dxbchan, credits));

	if(dc->maxcred < credits) {
		dc->maxcred = credits;
	}
	dxb_set_credit_cb_on_space(dxb, dc->dxbchan, 0);

	/*
	 * If we got some credits, wakeup the tty driver so it can
	 * complete any pending writes.
	 */
	if((tty->flags & (1 << TTY_DO_WRITE_WAKEUP)) &&
            (tty->ldisc.write_wakeup)) {
		(tty->ldisc.write_wakeup) (tty);
	}
	wake_up_interruptible(&tty->write_wait);
#if defined(SERIAL_HAVE_POLL_WAIT)
        wake_up_interruptible (&tty->poll_wait);
#endif

	return(0);
}








/************************************************************************
 *
 * DXB abstraction functions
 *
 * These are the abstracted memory routines needed for the libdxb
 * portable implementation, as implemented for Linux.
 *
 *************************************************************************/

/*
 * dxb_printf()
 *
 * Print debugging information
 */
GLOBAL void dxb_printf(const char *fmt,...)
{
	va_list args;
	int i;
	char buf[256];

	if(!DBG_DXB)
		return;

	va_start(args, fmt);
	i = vsprintf(buf, fmt, args);
	va_end(args);

#if defined(DGDM_TRACER)
	dgdm_tracef(buf);
#else
	printk(DRVSTR ": %s", buf);
#endif
}



/*
 * dxb_malloc()
 *
 * Allocate memory of the size indicated, with no particular
 * alignment gauranteed other than wordsize alignment.  Return
 * its virtual address.
 *
 * Return NULL on error.
 */
GLOBAL void *dxb_malloc(unsigned int size)
{
	return(kmalloc(size, GFP_KERNEL));
}



/*
 * dxb_free()
 *
 * Free memory allocated by dxb_malloc()
 */
GLOBAL void dxb_free(void *buf)
{
	kfree(buf);
}



/*
 * dxb_phys_page_size()
 *
 * Tell DXB the size of a physical page
 */
GLOBAL int dxb_phys_page_size(void)
{
	return(PAGE_SIZE);	/* from linux kernel: 4096 on i386 */
}



/*
 * dxb_alloc_page()
 *
 * Allocate a single page of virtual kernel memory, aligned on a
 * physical page boundary. and return its virtual address.
 * Return NULL on error.
 *
 * NOTE: this routine may be a stub if the DXB buffers can be
 *       allocated in one single virtually contiguous chunk.  Remember,
 *       as a worst case assumption, the two buffers might need to be
 *       up to 128kb each.
 *
 * For linux, we don't really need this, but we include it here
 * for DXB library testing purposes.
 */
GLOBAL void *dxb_alloc_page(void)
{
	return(void *) get_free_page(GFP_KERNEL);
}



/*
 * dxb_free_page()
 *
 * Free a page of virtual kernel memory allocated by dxb_alloc_page().
 */
GLOBAL void dxb_free_page(void *page)
{
	free_page((int) page);
}



/*
 * dxb_virt2bus()
 *
 * Convert a kernel virtual address to a bus physical address
 *
 * NOTE: a physical address and a bus address are typically the same
 *       thing on i386, but they are not the same thing on some other
 *       architectures.
 *
 * NOTE: our current hardware limits bus addresses to 32 bits.
 */
GLOBAL uint32 dxb_virt2bus(void *virt)
{
	return(virt_to_bus(virt));
}





/************************************************************************
 *
 * DXB locking functions
 *
 * We need to lock the DXB library against getting entered by
 * either 1) another kernel thread or 2) by the poller.
 *
 * BUT, we do allow callback routines from the poller to call
 * back into libdxb for non-poller API calls.
 *
 * Keeping the poller out while someone else has dxb locked is a bit
 * hard, since the poller comes off the timer tick and can interrupt
 * us at any time.  We don't want to disable interrupts around all the
 * DXB calls (especially the read/write data calls), so what we do is
 * set a flag when we call dxb_lock().  If the poller comes along and
 * sees that the flag is set, then it immediately returns; otherwise
 * it is safe for it to do its thing.  When we call dxb_unlock(), we
 * check to see if the flag was set.  If it was set, we run the poller
 * directly from dxb_unlock().  This eliminates having to shut off
 * interrupts.
 *
 * Since dgdm_tty_write() can be called in interrupt context by other parts
 * of the kernel, we have added dxb_lock_noblock() which has the same locking
 * requirements as dxb_lock() but will return a failure code of non-zero if
 * it cannot get the lock.
 *
 ************************************************************************/
#define LOCK_NOBLOCK_SUCCESS 0
#define LOCK_NOBLOCK_FAILURE 1

GLOBAL int dxb_lock_noblock(BOARD_T *brd)
{
    unsigned long flags;
    int lock_status = LOCK_NOBLOCK_SUCCESS;
	
    DGDM_LOCK(brd->dxb_spinlock, flags);
    
    if(brd->poller_locked && (brd->poll_processor == DGDM_PROCID())) {
        /*
         * This is the processor in the poller.  He is allowed
         * to obtain the lock while the poller is running.
         */
        if(brd->dxb_locked) {
            DPR(("dxb_lock_noblock called from poller context "
                 "with lock already held\n"));
        }

        brd->dxb_locked = 1;
        goto finished;
    }

	/* can't proceed while either poller lock or dxb lock is held */
	if(brd->poller_locked || brd->dxb_locked) {
        DPR(("dxb_lock_noblock: could not lock, lock already held.  Poller locked = %d, Board locked = %d.\n", brd->poller_locked, brd->dxb_locked));
        lock_status = LOCK_NOBLOCK_FAILURE;
        goto finished;
    }

	brd->dxb_locked = 1;

finished:
    DGDM_UNLOCK(brd->dxb_spinlock, flags);
    
    return lock_status;
}

GLOBAL void dxb_lock(BOARD_T *brd)
{
	unsigned long flags;
    
    DECLARE_WAITQUEUE(wait, current);
	
    DGDM_LOCK(brd->dxb_spinlock, flags);
	if ( brd->poller_locked && (brd->poll_processor == DGDM_PROCID())) {
		/*
		 * This is the processor in the poller.  He is allowed
		 * to obtain the lock while the poller is running.
		 */
		if (brd->dxb_locked) {
			DPR(("dxb_lock called from poller context with "
			     "lock already held\n"));
		}

		brd->dxb_locked = 1;
		goto finished;
	}

	/* can't proceed while either poller lock or dxb lock is held */
	
	while ( brd->poller_locked || brd->dxb_locked )	{
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
        /* if we're in interrupt context, don't schedule! */
        if(in_interrupt()) {
            APR(("dxb_lock: called in interrupt context!  "
                 "Cowardly refusing to block.\n"));
            goto finished;
        }
#endif

        add_wait_queue(&brd->dxb_lock_wait, &wait);
        current->state = TASK_INTERRUPTIBLE;

        brd->dxb_needwkup = 1;
        DGDM_UNLOCK(brd->dxb_spinlock, flags);

        schedule();

        current->state = TASK_RUNNING;
        remove_wait_queue(&brd->dxb_lock_wait, &wait);

        DGDM_LOCK(brd->dxb_spinlock, flags);
    }

    brd->dxb_locked = 1;

finished:
    DGDM_UNLOCK(brd->dxb_spinlock, flags);
}

GLOBAL int dxb_lock_poller(BOARD_T *brd)
{
	unsigned long flags;
	int rc;
	
	DGDM_LOCK(brd->dxb_spinlock, flags);

	if (brd->poller_locked || brd->dxb_locked) {
		brd->dxb_needpoll = 1;
		rc = 0;
	} else {
		brd->poller_locked = 1;
		brd->poll_processor = DGDM_PROCID();
		rc = 1;
	}

	DGDM_UNLOCK(brd->dxb_spinlock, flags);

	return(rc);
}

GLOBAL void dxb_unlock(BOARD_T *brd)
{
	unsigned long flags;

	DGDM_LOCK(brd->dxb_spinlock, flags);

	brd->dxb_locked = 0;

	/*
         * don't call poller recursively, in case dxb_lock was obtained
         * within a dxb callback routine.  In that case, we'll also
         * wake up other tasks when poller_unlock is called, so no
         * point in waking up others now.
         */

	if (!brd->poller_locked) {
		while (brd->dxb_needpoll) {
			brd->poller_locked = 1;
			brd->dxb_needpoll = 0;

			DGDM_UNLOCK(brd->dxb_spinlock, flags);

			/*
			 * we haven't kept track if it was a tx or rx poll
			 * that was deferred.  Do them both.
			 */
			dgdm_tty_poll_board(brd, TRUE, TRUE);

			DGDM_LOCK(brd->dxb_spinlock, flags);

			brd->poller_locked = 0;
		}

		if (brd->dxb_needwkup) {
			brd->dxb_needwkup = 0;
			wake_up_interruptible(&brd->dxb_lock_wait);
		}
	}

	DGDM_UNLOCK(brd->dxb_spinlock, flags);
}

GLOBAL void dxb_unlock_poller(BOARD_T *brd)
{
	unsigned long flags;

	DGDM_LOCK(brd->dxb_spinlock, flags);

	while (brd->dxb_needpoll) {
		brd->dxb_needpoll = 0;

		DGDM_UNLOCK(brd->dxb_spinlock, flags);

		dgdm_tty_poll_board(brd, TRUE, TRUE);

		DGDM_LOCK(brd->dxb_spinlock, flags);
	}
	brd->poller_locked = 0;

	if(brd->dxb_needwkup) {
		brd->dxb_needwkup = 0;
		wake_up_interruptible(&brd->dxb_lock_wait);
	}

	DGDM_UNLOCK(brd->dxb_spinlock, flags);
}


/************************************************************************
 *
 * Utility and supporting functions
 *
 ************************************************************************/

/*
 * dxb_start_timer()
 *
 * Schedule the poller.
 */
STATIC void dxb_start_timer(void)
{
	TimerOn = 1;
	Timer.expires = DXB_TIMEOUT;
	add_timer(&Timer);
}



/*
 * kzmalloc()
 *
 * Malloc and clear memory,
 */
STATIC void *kzmalloc(size_t size, int priority)
{
	void *p;

	p = kmalloc(size, priority);
	if(p)
		memset(p, 0, size);
	return(p);
}



/*
 * dgdm_dxbread()
 *
 * For debugging, dump the entire contents of a DXB circular buffer to the user
 */
GLOBAL int dgdm_dxbread(DXBQ * q, READ_PROTO)
{
	int offset = GET_POS();
	int len;

	if(offset < 0)
		offset = 0;
	if(offset >= q->buf.size)
		offset = q->buf.size;

	count = len = MIN(count, q->buf.size - offset);
	while (len) {
		int amt;
		int page;
		int pageoffset;

		page = offset >> q->buf.pageshift;
		pageoffset = offset & q->buf.pagemask;
		amt = MIN(len, q->buf.pagesize - pageoffset);

		COPY_TO_USER(buf, q->buf.page[page].virt + pageoffset, amt);

		offset += amt;
		len -= amt;
		buf += amt;
	}

	SET_POS(offset);

	return(count);
}



/*
 * sdxbsyncwait()
 *
 * Send a sync message, and wait for it to return
 *
 * Used to wait for data to drain, commands to execute, etc. 
 * For syncronization.
 *
 * Subtle race condition to be avoided here.  If the timer tick comes in while
 * we have the board locked, dxb polling will be done in our call to
 * dxb_unlock().  If the fep somehow manages to process the sync command
 * and respond fast enough that the reply is processed in that poll from
 * dxb_ulnock(), the wakeup will come in before we actually go to sleep.
 * Therefore, I've used the "split up" version of interruptible_sleep_on,
 * per example given in /usr/src/linux/drivers/char/n_tty.c.
 */
void wake_up_sync_cb_timer(unsigned long ptr)
{
    DRVCHAN* dc = (DRVCHAN*)ptr;
    DPR_BASIC(("sdxbsyncwait(%d_%02d_%x) - Sync timer waking uninterruptible task, scmd.sseq %x\n",
		                dc->brd->boardnum, dc->channum, dc->dxbchan, dc->syncseq));
    wake_up(&dc->sync_wait);
}

STATIC int sdxbsyncwait(DRVCHAN * dc, int hipri, int drain)
{
	DXB *dxb;
    struct timer_list sync_wakeup_timer;
	struct sdxb_sync_command scmd;
        DECLARE_WAITQUEUE(wait, current);

#if defined(DGDM_PARANOIA_CHECK)
	if(poller_running) {
		/* This is not good, because this routine can sleep */
		DPR(("sdxbsyncwait called while in poller - Bad Idea.\n"));
	}
# if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
	if(in_interrupt()) {
		/* This is not good, because this routine can sleep */
		DPR(("sdxbsyncwait called while in interrupt contex - "
		     "Bad Idea.\n"));
	}
# endif
#endif

	DPR_BASIC(("sdxbsyncwait(%d_%02d_%x) - hipri %x, drain %x\n",
                   dc->brd->boardnum, dc->channum, dc->dxbchan, hipri, drain));

	if(dc->dxbchan < 0) {
		DPR_BASIC(("sdxbsyncwait(%d_%02d_%x) - negative dxbchan\n",
			   dc->brd->boardnum, dc->channum, dc->dxbchan));
		return(-EIO);
	}
	dxb = dc->brd->dxb;

	dc->syncseq++;

	scmd.ctype = SDXB_SYNC;
	scmd.flags = 0;
	if(drain)
		scmd.flags |= DRAIN;

	for (;;) {

		dxb_lock(dc->brd);

		if(dxb_space_avail(dxb, dc->dxbchan, SDXB_SYNC_CLEN,
                                    0, hipri) == DXB_OK)
        {
            DPR_BASIC(("sdxbsyncwait(%d_%02d_%x) - We have space available in the dxbchannel., scmd.sseq %x\n",
		                dc->brd->boardnum, dc->channum, dc->dxbchan, dc->syncseq));
            break;
        }
		else{
            DPR_BASIC(("sdxbsyncwait(%d_%02d_%x) - No space available in the dxb channel., scmd.sseq %x\n",
		                dc->brd->boardnum, dc->channum, dc->dxbchan, dc->syncseq));
        }

		dxb_unlock(dc->brd);

		if(dgdm_sleep(1)){
            DPR_BASIC(("sdxbsyncwait(%d_%02d_%x) - dgdm_sleep interrupted, scmd.sseq %x\n",
		                dc->brd->boardnum, dc->channum, dc->dxbchan, dc->syncseq));    
			return(-EINTR);
        }
	}

	/*
	 * We're gonna send it now.
	 * First make sure we're ready to receive the wakeup.
	 * The reply will set current->state back to TASK_RUNNING
	 */
	add_wait_queue(&dc->sync_wait, &wait);
	current->state = TASK_UNINTERRUPTIBLE;

	/*
	 * assign scmd.sseq after possible sleep to ensure that
	 * we are using the latest value of dc->syncseq,
	 * even if some other process came in and bumped
	 * dc->syncseq while we slept waiting for space
	 */
	scmd.sseq = dc->syncseq;

	dxb_cdata(dxb, dc->dxbchan, hipri, SDXB_SYNC_CLEN, 0);
	dxb_write_ctrl_block(dxb, &scmd, SDXB_SYNC_CLEN);
	dxb_write_done(dxb);

	dxb_unlock(dc->brd);

	DPR_BASIC(("sdxbsyncwait(%d_%02d_%x) - sleeping, scmd.sseq %x\n",
		dc->brd->boardnum, dc->channum, dc->dxbchan, scmd.sseq));

    /* 
        Since we are uninterruptible, set a timer to allow an interrupt 
        if the board fails.  We'll set the timeout for 2 seconds. 
    */
    init_timer(&sync_wakeup_timer);
    sync_wakeup_timer.function = wake_up_sync_cb_timer;
    sync_wakeup_timer.data = (unsigned long)dc;
    sync_wakeup_timer.expires = jiffies + (2 * HZ);
    add_timer(&sync_wakeup_timer);
	
    /* now wait for the reply */
	schedule();
    
    del_timer(&sync_wakeup_timer);
    if(signal_pending(current)){
        DPR_BASIC(("sdxbsyncwait(%d_%02d_%x) - interrupted, scmd.sseq %x\n",
		dc->brd->boardnum, dc->channum, dc->dxbchan, scmd.sseq));
    }
	remove_wait_queue(&dc->sync_wait, &wait);

	DPR_BASIC(("sdxbsyncwait(%d_%02d_%x) - done, scmd.sseq %x\n",
                   dc->brd->boardnum, dc->channum, dc->dxbchan, scmd.sseq));

	if(dc->dxbchan < 0)
		return(-EIO);

	return(signal_pending(current));
}



/*
 * sdxbcmd_nowait()
 *
 * Send a S/DXB command without waiting if lock is false and with waiting
 * if lock is true.
 */
static int sdxbcmd_nowait(DRVCHAN * dc, int hipri, void *cmd, int sz, int lock)
{
	DXB *dxb;

	DPR_BASIC(("sdxbcmd_nowait(%d_%02d_%x)\n",
		   dc->brd->boardnum, dc->channum, dc->dxbchan));

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0) && defined(DGDM_PARANOIA_CHECK)
	if(in_interrupt() && lock) {
		/* This is bad, because this routine can sleep if locking */
		DPR(("sdxbcmd_nowait called while in interrupt contex and "
		     "locking - Bad Idea.\n"));
	}
#endif

	dxb = dc->brd->dxb;

	if(lock) {
		dxb_lock(dc->brd);
	}
	if(dxb_space_avail(dxb, dc->dxbchan, sz, 0, hipri) != DXB_OK) {
		dxb_unlock(dc->brd);
		return(-1);
	}
	dxb_cdata(dxb, dc->dxbchan, hipri, sz, 0);
	dxb_write_ctrl_block(dxb, cmd, sz);
	dxb_write_done(dxb);

	if(lock)
		dxb_unlock(dc->brd);

	return(0);
}



/*
 * sdxbcmd()
 *
 * Send a S/DXB command.
 */
STATIC int sdxbcmd(DRVCHAN * dc, int wait, int hipri, void *cmd, int sz)
{

#if defined(DGDM_PARANOIA_CHECK)
	if(poller_running) {
		/* This is not good, because this routine can sleep */
		DPR(("sdxbcmd called while in poller - Bad Idea.\n"));
	}
# if LINUX_VERSION_CODE >= KERNEL_VERSION(2,1,0)
	if(in_interrupt()) {
		/* This is not good, because this routine can sleep */
		DPR(("sdxbcmd called while in interrupt contex - "
		     "Bad Idea.\n"));
	}
# endif
#endif

	if(sz) {
		struct sdxb_mstat_command *mcmd =
                        (struct sdxb_mstat_command *) cmd;

		for(;;) {
			long held;

			if(mcmd->ctype == SDXB_MSTAT) {
				if(mcmd->mout & SDXB_DTR) {

					held = (jiffies - dc->dtrlow);

					while (held < DEFAULT_DTRLOW) {

						/* break if carrier is low */
						if(!(dc->mstat & dc->m_dcd)) {
							dc->dtrlow = jiffies -
                                                                DEFAULT_DTRLOW;
							break;
						}
						/* 
						 * sleep with small granularity
						 * expecting DCD to drop
                                                 */
						if(dgdm_sleep(1))
							return(-EINTR);

						held = (jiffies - dc->dtrlow);
					}
				} else {
					held = (jiffies - dc->dtrlow);

					/* if DTR not recently low */
					if(held >= DEFAULT_DTRLOW) {
						/*
						 * If we are clearing DTR,
						 * make a note of when we did.
						 */
						dc->dtrlow = jiffies;
					}
				}
			}
			if(sdxbcmd_nowait(dc, hipri, cmd, sz, TRUE) == 0)
				break;

			if(dgdm_sleep(1))
				return(-EINTR);
		}
	}
	if(wait) {
		return(sdxbsyncwait(dc, hipri, FALSE));
	}
	return(0);
}



/*
 * send_deferred_commands()
 *
 * Try to send any messages we couldn't send from interrupt level
 */
STATIC void send_deferred_commands(DRVCHAN *dc, int lock)
{
	int rc;

	if(dc->sendcmds & SEND_TXFLUSH) {
		struct sdxb_flush_command fcmd;

		fcmd.ctype = SDXB_FLUSH;
		fcmd.flags = SDXB_FLUSH_OUTPUT;
		fcmd.fill1 = 0;
		fcmd.fill2 = 0;

		rc = sdxbcmd_nowait(dc, CMD_HIPRI, &fcmd, SDXB_FLUSH_CLEN,
				    lock);
		if(rc != 0)
			return;
		dc->sendcmds &= ~SEND_TXFLUSH;
	}
	if(dc->sendcmds & SEND_DTRDOWN) {
		struct sdxb_mstat_command cmd;

		cmd.ctype = SDXB_MSTAT;
		cmd.flags = SDXB_IMMED | SDXB_HIPRI;
		cmd.mint = 0xff;	/* send state of all lines */
		cmd.mout = dc->mstat & (SDXB_RTS);

		rc = sdxbcmd_nowait(dc, CMD_HIPRI, &cmd, SDXB_MSTAT_CLEN,
				    lock);
		if(rc != 0)
			return;

		dc->sendcmds &= ~SEND_DTRDOWN;
	}
}



/*
 * parity_scan()
 *
 * I don't see a way to bypass the "FF doubling" in parmrk mode in the
 * linux driver, short of lying to the line discipline about what the
 * flags are set to.
 *
 * We lie to it about the opost flag, and should probably do so concerning
 * parmrk as well, but this function was here when I inherited the driver,
 * so I haven't done it yet -- TODO.
 *
 */
STATIC void parity_scan(DRVCHAN *dc, unsigned char *cbuf, unsigned char *fbuf,
                        int *len)
{
	int l = *len;
	int count = 0;
	unsigned char *in, *cout, *fout;
	unsigned char c;

	in = cbuf;
	cout = cbuf;
	fout = fbuf;

	while (l--) {
		c = *in++;
		switch (dc->pscan_state) {
		default:
			/* reset to sanity and fall through */
			dc->pscan_state = 0;

		case 0:
			/* No FF seen yet */
			if(c == (unsigned char) '\377') {
				/* delete this character from stream */
				dc->pscan_state = 1;
			} else {
				*cout++ = c;
				*fout++ = TTY_NORMAL;
				count += 1;
			}
			break;

		case 1:
			/* first FF seen */
			if(c == (unsigned char) '\377') {
				/* doubled ff, transform to single ff */
				*cout++ = c;
				*fout++ = TTY_NORMAL;
				count += 1;
				dc->pscan_state = 0;
			} else {
				/* save value examination in next state */
				dc->pscan_savechar = c;
				dc->pscan_state = 2;
			}
			break;

		case 2:
			/* third character of ff sequence */
			*cout++ = c;
			if(dc->pscan_savechar & 0x10) {
				*fout++ = TTY_BREAK;
			} else if(dc->pscan_savechar & 0x08) {
				*fout++ = TTY_FRAME;
			} else {
				/* either marked as a parity error, */
				/* indeterminate, or not in DOSMODE */
				/* call it a parity error */
				*fout++ = TTY_PARITY;
			}
			count += 1;
			dc->pscan_state = 0;
		}
	}
	*len = count;
}



/*
 * dxb_rxbuf()
 *
 * Process deferred rxbuf data received on a single channel
 */
STATIC void dxb_rxbuf(BOARD_T *brd, DRVCHAN *dc)
{
	TTY *tty;
	int len = 0;
	int flip_len;
        int boardnum = brd->boardnum;
	static char myflipbuf[MAXBOARDS][MYFLIPLEN];

	/*
	 * Get the usual pointers
	 */
	tty = dc->tty;
	if(!tty)
		return;

        if(!brd)
                return;

	/*
	 * If we are throttled, just return
	 */
	if(test_bit(STATE_RXSTOP, &dc->state))
		return;

	/*
	 * Just go ahead and try to read some data into the flip buffer.
	 * If there isn't any data available, dxb_rxbuf_read will return
	 * 0 back to us right away.
         *
	 * The dgdm_rawreadok case takes advantage of carnal knowldge that 
	 * the char_buf and the flag_buf are next to each other and 
	 * are each of (2 * TTY_FLIPBUF_SIZE) size. 
	 */
	if(dgdm_rawreadok) {
		if(tty->real_raw) {
			flip_len = MYFLIPLEN;
		} else {
			flip_len = 2 * TTY_FLIPBUF_SIZE;
		}
		flip_len = MIN(flip_len, (N_TTY_BUF_SIZE - 1) - tty->read_cnt);

		if(tty->real_raw)
			len = dxb_rxbuf_read(brd->dxb, dc->dxbchan,
					     myflipbuf[boardnum], flip_len);
		else
			len = dxb_rxbuf_read(brd->dxb, dc->dxbchan,
					   tty->flip.char_buf, flip_len);
	} else {
		flip_len = TTY_FLIPBUF_SIZE - tty->flip.count;
		len = dxb_rxbuf_read(brd->dxb, dc->dxbchan,
				     tty->flip.char_buf_ptr, flip_len);
	}

	if(len == 0)
		return;

	DPR_READ(("dxb_rxbuf(%d_%02d_%x) - len=%d\n",
		  dc->brd->boardnum, dc->channum, dc->dxbchan, len));

	dxb_return_credits(brd->dxb, dc->dxbchan, len);

	if(!dgdm_rawreadok) {
		if(I_PARMRK(tty)) {
			parity_scan(dc, tty->flip.char_buf_ptr,
				    tty->flip.flag_buf_ptr, &len);
		} else {
			memset(tty->flip.flag_buf_ptr, 0, len);
		}
		tty->flip.char_buf_ptr += len;
		tty->flip.flag_buf_ptr += len;
		tty->flip.count += len;
	} else if(!tty->real_raw) {
		if(I_PARMRK(tty)) {
			parity_scan(dc, tty->flip.char_buf,
				    tty->flip.flag_buf, &len);
		} else {
			memset(tty->flip.flag_buf, 0, len);
		}
	}
	dc->stat[RxSTAT].cnt += len;

	atomic_add(len, (atomic_t *) & dc->stat[RxSTAT].accum);

	if(dgdm_rawreadok) {
		if(tty->real_raw)
			tty->ldisc.receive_buf(tty, myflipbuf[boardnum],
					       NULL, len);
		else
			tty->ldisc.receive_buf(tty, tty->flip.char_buf,
					       tty->flip.flag_buf, len);
	} else {
		tty_schedule_flip(tty);
	}
}



/*
 * moving_average()
 *
 * Compute moving average(s)
 */
STATIC void moving_average(MACOUNTER *map, uint randnum)
{
	ulong round;

	round = randnum % SMA_NPERIODS;
	map->sma -= (map->sma + round) / SMA_NPERIODS;
	map->sma += map->accum;

#if 0
	/* EMA TODO */
#endif

	map->accum = 0;
}

