/*****************************************************************************
 * Copyright (c) 1988-95 Digi International Inc., All Rights Reserved
 * 
 * This  software  contains  proprietary  and  confidential   information   of
 * Digi International  Inc.   By  accepting  transfer  of this copy, Recipient
 * agrees to retain this software in  confidence,  to  prevent  disclosure  to
 * others,  and  to make no  use of this software other than that for which it
 * was delivered.  This is an unpublished copyright work  Digi   International
 * Inc.    Except   as   permitted   by  federal  law,  17 USC 117, copying is
 * strictly prohibited.
 * 
 * Digi  International  Inc.  CONFIDENTIAL  -   (Digi    International    Inc.
 * Confidential  when   combined   with   the   aggregated  modules  for  this
 * product) OBJECT CODE ONLY SOURCE MATERIALS
 * 
 * (C) COPYRIGHT Digi International Inc. 1995
 * 
 * All Rights Reserved
 * 
 * US   Government   Users   Restricted   Rights   -   Use,   duplication   or
 * disclosure  restricted by GSA ADP Schedule Contract with Digi International
 * Inc.
 * 
 *****************************************************************************/

/*
 * $Id: bchan_proto.h,v 1.7 2000/07/14 14:50:41 garyp Exp $
 *
 * This file contains the protocol structures for the B-channel driver
 */

#ifndef _BCHAN_PROTO_H_
#define _BCHAN_PROTO_H_

#ifndef OLD_BCHAN_API

/*
    The following API conforms to:

             Data Interface Specification for HLC, Architecture 2
*/

/*
 * Command values
 *
 * The commands are divided into four categories:
 *
 *      Reqeusts:  unsolicited command from the user to the firmware
 * Confirmations:  firmware acknowledgement of an earlier Request
 *   Indications:  unsolicited event from the firmware to the user
 *     Responses:  user acknowledgement of an earlier Indication
 */

#define	DG_B_CONFIG_REQ		0x01	/* Configuration Request */
#define	DG_B_CONFIG_CONF	0x02	/* Configuration Confirmation */
#define	DG_B_CONN_REQ		0x03	/* Connect Request */
#define	DG_B_CONN_CONF		0x04	/* Connect Confirmation */
#define	DG_B_CONN_ACT_IND	0x05	/* Connect active Indication */
#define	DG_B_CONN_IND		0x06	/* Connect Indication */
#define	DG_B_CONN_RESP		0x07	/* Connect Response */
#define	DG_B_DATA_REQ		0x08	/* Data Request */
#define	DG_B_DATA_IND		0x09	/* Data Indication */
#define	DG_B_SYNC_REQ		0x0A	/* Sync Request */
#define	DG_B_SYNC_CONF		0x0B	/* Sync Confirmation */
#define	DG_B_MARK_REQ		0x0C	/* Mark Request */
#define	DG_B_MARK_IND		0x0D	/* Mark Indication */
#define	DG_B_FLUSH_REQ		0x0E	/* Flush Request */
#define	DG_B_DISCONN_REQ	0x0F	/* Disconnect Request */
#define	DG_B_DISCONN_CONF	0x10	/* Disconnect Confirmation */
#define	DG_B_DISCONN_IND	0x11	/* Disconnect Indication */
#define	DG_B_RESET_REQ		0x12	/* Reset Request */
#define	DG_B_RESET_IND		0x13	/* Reset Indication */
#define	DG_B_DTMF_REQ		0x14	/* DTMF Request */
#define	DG_B_DTMF_IND		0x15	/* DTMF Indication */
#define	DG_B_BAD_MESSAGE_IND	0xFF	/* Bad message Indication */

/* error codes ------------------------------------------------------------ */
#define	ERR_WRONG_STATE		0x01	/* msg. not valid for current state */
#define	ERR_ILLEGAL_CALL_ID	0x02	/* illegal call ID */
#define	ERR_ILLEGAL_PARAMETER	0x03	/* illegal message parameter coding */
#define	ERR_WRONG_B1_PROT	0x04	/* B1 protocol not supported */
#define	ERR_WRONG_B2_PROT	0x05	/* B2 protocol not supported */
#define	ERR_WRONG_B3_PROT	0x06	/* B3 protocol not supported */
#define	ERR_WRONG_B1_PROT_PARAMETER	0x07  /* B1 prot. parm not supported */
#define	ERR_WRONG_B2_PROT_PARAMETER	0x08  /* B2 prot. parm not supported */
#define	ERR_WRONG_B3_PROT_PARAMETER	0x09  /* B3 prot. parm not supported */
#define	ERR_WRONG_B_PROT_COMBINATION	0x0A  /* protocol combo not supported */
#define	ERR_IGNORE_NCPI		0x0B	/* NCPI ignored */
#define	ERR_ILLEGAL_CALL_ID_2	0x0C	/* Illegal call ID 2 */
#define	ERR_NO_CALL_ID_2_AVAIL	0x0D	/* No Call ID 2 available */
#define	ERR_WRONG_NCPI		0x0E	/* NCPI not supported */
#define	ERR_IGNORE_FLAGS	0x0F	/* Flags ignored */
#define	ERR_WRONG_FLAGS		0x10	/* Wrong flags used */
#define	ERR_DATA_LENGTH_NOT_SUPPORTED	0x11	/* Frame too long */
#define	ERR_WRONG_RESET_PROCEDURE	0x12	/* Reset unsupported */

/* reasons for disconnect ------------------------------------------------- */

/* Protocol independent disconnect reasons */
#define	DRI_PROTOCOL_ERR_L1	0x3301	/* Protocol error - layer 1 */
#define	DRI_PROTOCOL_ERR_L2	0x3302	/* Protocol error - layer 2 */
#define	DRI_PROTOCOL_ERR_L3	0x3303	/* Protocol error - layer 3 */

/* Disconnect reasons for B3 protocols 4 & 5 (T.30 / T.30 extended) */
#define	DRT_CONN_FAIL_NOT_G3	0x3311	/* Remote station not G3 fax device */
#define	DRT_CONN_FAIL_TRAINING	0x3312	/* Training error */
#define	DRT_DISCONN_PRETRANS	0x3313	/* Disconnected before transfer */
#define	DRT_REMOTE_ABORT	0x3314	/* Remote abort */
#define	DRT_REMOTE_PROC_ERR	0x3315	/* Remote procedure error */
#define	DRT_LOCAL_TX_DATA_UF	0x3316	/* Local Tx data underflow */
#define	DRT_LOCAL_RX_DATA_OF	0x3317	/* Local Rx data overflow */
#define	DRT_LOCAL_ABORT		0x3318	/* Local abort */
#define	DRT_ILLEGAL_PARM_CODING	0x3319	/* Illegal parameter coding */

/* Disconnect reasons for B3 protocol 7 (modem) */
#define	DRM_NORMAL_DISCONNECT	0x3500	/* Normal end of connection */
#define	DRM_CARRIER_LOST	0x3501	/* Carrier lost */
#define	DRM_ERR_NEGOTIATION	0x3502	/* Unresolvable issue in negotiation */
#define	DRM_NO_ANSWER_PROT_REQ	0x3503	/* No answer to protocol request */
#define	DRM_REMOTE_ASYNC_ONLY	0x3504	/* Remote modem asynchronous only */
#define	DRM_FRAMING_FAILURE	0x3505	/* Framing fails */
#define	DRM_PROTOCOL_NEGOT_FAIL	0x3506	/* Protocol negotiation fails */
#define	DRM_WRONG_PROTO_REQ	0x3507	/* Remote modem sent wrong proto req */
#define	DRM_SYNC_INFO_MISSING	0x3508	/* Sync information missing */
#define	DRM_NORMAL_REMOTE_DISC	0x3509	/* Normal end of connection (remote) */
#define	DRM_NO_ANSWER		0x350A	/* No answer from remote end */
#define	DRM_PROTOCOL_ERR	0x350B	/* Protocol error */
#define	DRM_COMPRESSION_ERR	0x350C	/* Error in compression */
#define	DRM_NO_CONNECT		0x350D	/* Timeout or wrong modulation */
#define	DRM_NO_PROTO_FALLBACK	0x350E	/* No protocol fallback allowed */
#define	DRM_NO_MODEM_OR_FAX	0x350F	/* No modem/fax at requested number */
#define	DRM_HANDSHAKE_ERROR	0x3510	/* Handshake error */

/* miscellaneous sub-structures ------------------------------------------- */

typedef struct b1_config
{
    dxb_uint32	max_receive_size ;	/* Largest frame allowed */
    dxb_uint32	max_bit_rate ;		/* 0: adaptive */
    dxb_uint32	bits_per_character ;

#define	B1_CONFIG_PARITY_NONE	0
#define	B1_CONFIG_PARITY_ODD	1
#define	B1_CONFIG_PARITY_EVEN	2
    dxb_uint32	parity ;

#define	B1_CONFIG_1_STOP_BIT	0
#define	B1_CONFIG_2_STOP_BITS	1
    dxb_uint32	stop_bits ;

    dxb_uint32	transmit_level ;	/* signed integer value */
    dxb_uint32	options ;		/* See API specification */

#define	B1_CONFIG_SPEED_NEG_NONE	0	/* none */
#define	B1_CONFIG_SPEED_NEG_MOD_CLASS	1	/* within modulation class */
#define	B1_CONFIG_SPEED_NEG_V100	2	/* V.100 */
#define	B1_CONFIG_SPEED_NEG_V8		3	/* I could've had a V.8 */
    dxb_uint32	speed_negotiation ;
}
b1_config_t ;


typedef struct b2_config
{
    dxb_uint32	address_a ;		/* Link address A */
    dxb_uint32	address_b ;		/* Link address B */

#define	B2_CONFIG_MODULO_NORMAL		8
#define	B2_CONFIG_MODULO_EXTENDED	128
    dxb_uint32	modulo_mode ;

    dxb_uint32	window_size ;
    dxb_uint32	options ;		/* See API specification */
}
b2_config_t ;


typedef struct b3_config
{
    dxb_uint32	lic ;			/* lowest incoming channel */
    dxb_uint32	hic ;			/* highest incoming channel */
    dxb_uint32	ltc ;			/* lowest two-way channel */
    dxb_uint32	htc ;			/* highest two-way channel */
    dxb_uint32	loc ;			/* lowest outgoing channel */
    dxb_uint32	hoc ;			/* highest outgoing channel */

#define	B3_CONFIG_MODULO_NORMAL		8
#define	B3_CONFIG_MODULO_EXTENDED	128
    dxb_uint32	modulo_mode ;

    dxb_uint32	window_size ;
    dxb_uint32	options ;		/* See API specification */

#define	B3_CONFIG_FORMAT_SFF	0
#define	B3_CONFIG_FORMAT_PLAIN	1
#define	B3_CONFIG_FORMAT_PCX	2
#define	B3_CONFIG_FORMAT_DCX	3
#define	B3_CONFIG_FORMAT_TIFF	4
#define	B3_CONFIG_FORMAT_ASCII	5
#define	B3_CONFIG_FORMAT_ANSI	6
#define	B3_CONFIG_FORMAT_BINARY	7
    dxb_uint32	format ;

    dxb_uint32	station_id_len ;

#define	B3_CONFIG_STATION_ID_MAX_LEN	20
    dxb_uint8	station_id[B3_CONFIG_STATION_ID_MAX_LEN] ;

    dxb_uint32	head_line_length ;

#define	B3_CONFIG_HEAD_LINE_MAX_LEN	100
    dxb_uint8	head_line[B3_CONFIG_HEAD_LINE_MAX_LEN] ;

} b3_config_t ;


typedef struct ncpi {
    dxb_uint32	used ;
    dxb_uint32	flags ;
    dxb_uint32	group ;
    dxb_uint32	channel ;
    dxb_uint32	plp_info_len ;

#define	NCPI_PLP_INFO_MAX_LEN	128
    dxb_uint8	plp_info[NCPI_PLP_INFO_MAX_LEN] ;

    dxb_uint32	rate ;
    dxb_uint32	options ;
    dxb_uint32	format ;
    dxb_uint32	pages ;
    dxb_uint32	receive_id_len ;

#define	NCPI_RECEIVE_ID_MAX_LEN	20
    dxb_uint8	receive_id[NCPI_RECEIVE_ID_MAX_LEN] ;

    dxb_uint32	protocol ;
} ncpi_t ;

/* API messages ----------------------------------------------------------- */

/* DG_B_MSG --------------------------------------------------------------- */

/* Generic message header */
typedef struct dg_b_msg {
    dxb_uint32	command ;		/* Any valid command */
    dxb_uint32	call_id ;		/* Physical connection identifier */
} dg_b_msg_t ;

/* DG_B_CONFIG_REQ -------------------------------------------------------- */
typedef struct dg_b_config_req {
    dxb_uint32	command ;		/* Always DG_B_CONFIG_REQ */
    dxb_uint32	call_id ;		/* Physical connection identifier */

#define	DG_B_CONFIG_REQ_CHECK_PARM_AND_CONFIG	0	/* check & configure */
#define	DG_B_CONFIG_REQ_CHECK_PARM_ONLY		1	/* check only */
    dxb_uint32	check_parm_only ;	/* check parameters only */

#define	DG_B_CONFIG_REQ_OP_MODE_IMMEDIATE	0	/* Immediate mode */
#define	DG_B_CONFIG_REQ_OP_MODE_DTE		1	/* DTE mode (active) */
#define	DG_B_CONFIG_REQ_OP_MODE_DCE		2	/* DCE mode (passive) */
    dxb_uint32	operation_mode ;	/* operation mode */

#define B1_HDLC64       0      /* 64 kbit/s w/HDLC framing (default) */
#define B1_RAW64        1      /* 64 kbit/s bit-transparent */
#define B1_V110_ASYNC	2      /* V.110 async w/start-stop framing */
/* V110 Sync not supported. Use HDLC 56K instead. */
#define B1_T30		4      /* T.30 Modem for Group 3 FAX */
#define B1_HDLC64_INV	5      /* 64 kbit/s inverted w/HDLC framing */
#define B1_RAW56        6      /* 56 kbit/s bit-transparent */
#define B1_MODEM_FN	7      /* Modem w/full negotiation */
#define B1_MODEM_SSBF	8      /* Modem async w/start-stop framing */
#define B1_MODEM_HDLC	9      /* Modem sync w/ HDLC framing */

#define B1_HDLC56       257
#define B1_ULAW         258    /* Not Supported */
#define B1_ALAW         259    /* Not Supported */
#define B1_NONE         260
    dxb_uint32	b1_protocol ;		/* Layer 1 protocol */

#define B2_X75SLP       0      /* ISO 7776 (X75 SLP) (default) */
#define B2_RAW		1      /* Transparent */
#define B2_SDLC		2      /* SDLC */
#define B2_LAPD		3      /* LAPD (Q.921) for D-channel X.25 on SAPI 16 */
#define B2_T30		4      /* T.30 for Group 3 FAX */
#define B2_PPP		5      /* Point-to-Point Protocol (PPP) */
#define B2_RAW_E	6      /* Transparent (ignore framing errors of B1 ) */
#define B2_MODEM_FN	7      /* Modem w/full neg. (e.g. V.42 bis, MNP 5) */
#define B2_X75SLP_V42	8      /* ISO 7776 w/V.42 bis compression */
#define B2_V120		9      /* V.120 asynchronous mode */
#define B2_V120_V42	10     /* V.120 asynchronous mode w/V.42 bis comp. */
#define B2_V120_RAW	11     /* V.120 bit-transparent mode */
#define B2_LAPD_SAPI	12     /* LAPD (Q.921) for DCH X.25 on SAPI chosen */
    dxb_uint32	b2_protocol ;		/* Layer 2 protocol */

#define B3_RAW		0      /* Transparent (default) */
#define B3_T90NL	1      /* T.90NL w/T.70NL */
#define B3_X25_DTE	2      /* ISO 8208 (X25 DTE-DTE) */
#define B3_X25_DCE	3      /* X25 DCE */
#define B3_T30		4      /* T.30 for Group 3 FAX */
#define B3_T30_EXT	5      /* T.30 for Group 3 FAX Extended */
#define B3_RESERVED	6      /* Reserved */
#define B3_MODEM	7      /* Modem */
    dxb_uint32	b3_protocol ;		/* Layer 3 protocol */

    b1_config_t	b1_config ;		/* Layer 1 additional information */
    b2_config_t	b2_config ;		/* Layer 2 additional information */
    b3_config_t	b3_config ;		/* Layer 3 additional information */

} dg_b_config_req_t ;


/* DG_B_CONFIG_CONF ------------------------------------------------------- */
typedef struct dg_b_config_conf {
    dxb_uint32	command ;		/* Always DG_B_CONFIG_CONF */
    dxb_uint32	call_id ;		/* Physical connection identifier */
    dxb_uint32	error ;			/* error return value */
} dg_b_config_conf_t ;


/* DG_B_CONN_REQ ---------------------------------------------------------- */
typedef struct dg_b_conn_req {
    dxb_uint32	command ;		/* Always DG_B_CONN_REQ */
    dxb_uint32	call_id ;		/* Physical connection identifier */
    ncpi_t	ncpi ;			/* Network control protocol info */
} dg_b_conn_req_t ;


/* DG_B_CONN_CONF --------------------------------------------------------- */
typedef struct dg_b_conn_conf {
    dxb_uint32	command ;		/* Always DG_B_CONN_CONF */
    dxb_uint32	call_id ;		/* Physical connection identifier */
    dxb_uint32	call_id2 ;		/* Logical connection identifier */
    dxb_uint32	error ;			/* error return value */
} dg_b_conn_conf_t ;


/* DG_B_CONN_ACT_IND ------------------------------------------------------ */
typedef struct dg_b_conn_act_ind {
    dxb_uint32	command ;		/* Always DG_B_CONN_ACT_IND */
    dxb_uint32	call_id ;		/* Physical connection identifier */
    dxb_uint32	call_id2 ;		/* Logical connection identifier */
    ncpi_t	ncpi ;			/* Network control protocol info */
} dg_b_conn_act_ind_t ;


/* DG_B_CONN_IND ---------------------------------------------------------- */
typedef struct dg_b_conn_ind {
    dxb_uint32	command ;		/* Always DG_B_CONN_IND */
    dxb_uint32	call_id ;		/* Physical connection identifier */
    dxb_uint32	call_id2 ;		/* Logical connection identifier */
    ncpi_t	ncpi ;			/* Network control protocol info */
} dg_b_conn_ind_t ;


/* DG_B_CONN_RESP --------------------------------------------------------- */
typedef struct dg_b_conn_resp {
    dxb_uint32	command ;		/* Always DG_B_CONN_RESP */
    dxb_uint32	call_id ;		/* Physical connection identifier */
    dxb_uint32	call_id2 ;		/* Logical connection identifier */
    ncpi_t	ncpi ;			/* Network control protocol info */
} dg_b_conn_resp_t ;


/* DG_B_DATA_REQ ---------------------------------------------------------- */
typedef struct dg_b_data_req {
    dxb_uint32	command ;		/* Always DG_B_DATA_REQ */
    dxb_uint32	call_id ;		/* Physical connection identifier */
    dxb_uint32	call_id2 ;		/* Logical connection identifier */

#define	DG_B_DATA_REQ_Q_BIT		0x0001L
#define	DG_B_DATA_REQ_MORE_BIT		0x0002L
#define	DG_B_DATA_REQ_POLL_FINAL_BIT	0x0004L
#define	DG_B_DATA_REQ_EXPEDITE_BIT	0x0008L
    dxb_uint32	flags ;

} dg_b_data_req_t ;


/* DG_B_DATA_IND ---------------------------------------------------------- */
typedef struct dg_b_data_ind {
    dxb_uint32	command ;		/* Always DG_B_DATA_IND */
    dxb_uint32	call_id ;		/* Physical connection identifier */
    dxb_uint32	call_id2 ;		/* Logical connection identifier */

#define	DG_B_DATA_IND_Q_BIT		0x0001L
#define	DG_B_DATA_IND_MORE_BIT		0x0002L
#define	DG_B_DATA_IND_POLL_FINAL_BIT	0x0004L
#define	DG_B_DATA_IND_EXPEDITE_BIT	0x0008L
#define	DG_B_DATA_IND_BREAK_BIT		0x0010L
#define	DG_B_DATA_IND_FRAMING_ERROR_BIT	0x8000L
    dxb_uint32	flags ;

} dg_b_data_ind_t ;


/* DG_B_SYNC_REQ ---------------------------------------------------------- */
typedef struct dg_b_sync_req {
    dxb_uint32	command ;		/* Always DG_B_SYNC_REQ */
    dxb_uint32	call_id ;		/* Physical connection identifier */
    dxb_uint32	call_id2 ;		/* Logical connection identifier */
    dxb_uint32	sync_id ;		/* Unique identification tag */

#define	DG_B_SYNC_REQ_HIPRI_RESPONSE	0x0001L
    dxb_uint32	mode ;

} dg_b_sync_req_t ;


/* DG_B_SYNC_CONF --------------------------------------------------------- */
typedef struct dg_b_sync_conf {
    dxb_uint32	command ;		/* Always DG_B_SYNC_CONF */
    dxb_uint32	call_id ;		/* Physical connection identifier */
    dxb_uint32	call_id2 ;		/* Logical connection identifier */
    dxb_uint32	sync_id ;		/* Unique identification tag */
} dg_b_sync_conf_t ;


/* DG_B_MARK_REQ ---------------------------------------------------------- */
typedef struct dg_b_mark_req {
    dxb_uint32	command ;		/* Always DG_B_MARK_REQ */
    dxb_uint32	call_id ;		/* Physical connection identifier */
    dxb_uint32	call_id2 ;		/* Logical connection identifier */

#define	DG_B_MARK_REQ_INTERFRAME_7E	1	/* interframe fill 0x7E */
#define	DG_B_MARK_REQ_INTERFRAME_FF	2	/* interframe fill 0xFF */
#define	DG_B_MARK_REQ_FF_NEXT_FRAME	3	/* 0xFF after next frame only */
#define	DG_B_MARK_REQ_MARK_DETECT_ON	4	/* listen for mark idle */
#define	DG_B_MARK_REQ_MARK_DETECT_OFF	5	/* don't listen for mark idle */
    dxb_uint32	function ;

} dg_b_mark_req_t ;


/* DG_B_MARK_IND ---------------------------------------------------------- */
typedef struct dg_b_mark_ind {
    dxb_uint32	command ;		/* Always DG_B_MARK_IND */
    dxb_uint32	call_id ;		/* Physical connection identifier */
    dxb_uint32	call_id2 ;		/* Logical connection identifier */
} dg_b_mark_ind_t ;


/* DG_B_FLUSH_REQ --------------------------------------------------------- */
typedef struct dg_b_flush_req {
    dxb_uint32	command ;		/* Always DG_B_FLUSH_REQ */
    dxb_uint32	call_id ;		/* Physical connection identifier */
    dxb_uint32	call_id2 ;		/* Logical connection identifier */

#define	DG_B_FLUSH_REQ_TX_FLUSH		0
#define	DG_B_FLUSH_REQ_RX_FLUSH		1
#define	DG_B_FLUSH_REQ_TX_ABORT		2
#define	DG_B_FLUSH_REQ_RX_ABORT		3
    dxb_uint32	mode ;

} dg_b_flush_req_t ;


/* DG_B_DISCONN_REQ ------------------------------------------------------- */
typedef struct dg_b_disconn_req {
    dxb_uint32	command ;		/* Always DG_B_DISCONN_REQ */
    dxb_uint32	call_id ;		/* Physical connection identifier */
    dxb_uint32	call_id2 ;		/* Logical connection identifier */
    ncpi_t	ncpi ;			/* Network control protocol info */
} dg_b_disconn_req_t ;


/* DG_B_DISCONN_CONF ------------------------------------------------------ */
typedef struct dg_b_disconn_conf {
    dxb_uint32	command ;		/* Always DG_B_DISCONN_CONF */
    dxb_uint32	call_id ;		/* Physical connection identifier */
    dxb_uint32	call_id2 ;		/* Logical connection identifier */
    dxb_uint32	error ;			/* error return value */
} dg_b_disconn_conf_t ;


/* DG_B_DISCONN_IND ------------------------------------------------------- */
typedef struct dg_b_disconn_ind {
    dxb_uint32	command ;		/* Always DG_B_DISCONN_IND */
    dxb_uint32	call_id ;		/* Physical connection identifier */
    dxb_uint32	call_id2 ;		/* Logical connection identifier */
    dxb_uint32	reason ;		/* Reason for disconnect */
    ncpi_t	ncpi ;			/* Network control protocol info */
} dg_b_disconn_ind_t ;


/* DG_B_RESET_REQ --------------------------------------------------------- */
typedef struct dg_b_reset_req {
    dxb_uint32	command ;		/* Always DG_B_RESET_REQ */
    dxb_uint32	call_id ;		/* Physical connection identifier */
    dxb_uint32	call_id2 ;		/* Logical connection identifier */
    ncpi_t	ncpi ;			/* Network control protocol info */
} dg_b_reset_req_t ;


/* DG_B_RESET_IND --------------------------------------------------------- */
typedef struct dg_b_reset_ind {
    dxb_uint32	command ;		/* Always DG_B_RESET_IND */
    dxb_uint32	call_id ;		/* Physical connection identifier */
    dxb_uint32	call_id2 ;		/* Logical connection identifier */
} dg_b_reset_ind_t ;


/* DG_B_DTMF_REQ ---------------------------------------------------------- */
typedef struct dg_b_dtmf_req {
    dxb_uint32	command ;		/* Always DG_B_DTMF_REQ */
    dxb_uint32	call_id ;		/* Physical connection identifier */
    dxb_uint32	call_id2 ;		/* Logical connection identifier */

#define	DG_B_DTMF_REQ_START_INBAND	1
#define	DG_B_DTMF_REQ_START_NO_INBAND	2
#define	DG_B_DTMF_REQ_START_NO_DATA	3
#define	DG_B_DTMF_REQ_STOP_DETECTION	4
#define	DG_B_DTMF_REQ_SEND_DIGITS	5
    dxb_uint32	function ;

#define	DG_B_DTMF_REQ_NORMAL_BANDWIDTH	0
#define	DG_B_DTMF_REQ_SMALL_BANDWIDTH	1
#define	DG_B_DTMF_REQ_WIDE_BANDWIDTH	2
    dxb_uint32	selectivity ;

#define	DG_B_DTMF_SENSITIVITY_LOW	0
#define	DG_B_DTMF_SENSITIVITY_MEDIUM	1
#define	DG_B_DTMF_SENSITIVITY_HIGH	2
    dxb_uint32	sensitivity ;

    dxb_uint32	duration ;		/* Tone duration in milliseconds */
    dxb_uint32	gap ;			/* Pause duration in milliseconds */
    dxb_uint32	level ;			/* Attenuation (in negative dB's) */

    /*
	Data portion of message shall contain DTMF digits when applicable.
	The length of the DTMF sequence is implied by the data length
	parameter in the DXB control-data header.
    */
} dg_b_dtmf_req_t ;


/* DG_B_DTMF_IND ---------------------------------------------------------- */
typedef struct dg_b_dtmf_ind {
    dxb_uint32	command ;		/* Always DG_B_DTMF_IND */
    dxb_uint32	call_id ;		/* Physical connection identifier */
    dxb_uint32	call_id2 ;		/* Logical connection identifier */

    /*
	Data portion of message shall contain DTMF digits.
	The length of the DTMF sequence is implied by the data length
	parameter in the DXB control-data header.
    */
} dg_b_dtmf_ind_t ;


/* DG_B_BAD_MESSAGE_IND --------------------------------------------------- */
typedef struct dg_b_bad_message_ind {
    dxb_uint32	command ;		/* Always DG_B_BAD_MESSAGE_IND */
    dxb_uint32	call_id ;		/* Physical connection identifier */
    dxb_uint32	call_id2 ;		/* Logical connection identifier */
    dxb_uint32	error_command ;		/* Faulty message identifier */
    dxb_uint32	error_type ;		/* Error return value */
} dg_b_bad_message_ind_t ;

#ifndef __cplusplus
typedef union bchan_api_union
{
    dxb_uint32 command;	      
    dg_b_msg_t msg;

    dg_b_bad_message_ind_t bad_msg;
    dg_b_config_req_t config_req ;
    dg_b_config_conf_t config_conf ;
    dg_b_conn_req_t conn_req ;
    dg_b_conn_conf_t conn_conf ;
    dg_b_conn_act_ind_t conn_act_ind ;
    dg_b_conn_ind_t conn_ind ;
    dg_b_conn_resp_t conn_resp ;
    dg_b_data_req_t data_req ;
    dg_b_data_ind_t data_ind ;
    dg_b_sync_req_t sync_req ;
    dg_b_sync_conf_t sync_conf ;
    dg_b_mark_req_t mark_req ;
    dg_b_mark_ind_t mark_ind ;
    dg_b_flush_req_t flush_req ;
    dg_b_disconn_req_t disconn_req ;
    dg_b_disconn_conf_t disconn_conf ;
    dg_b_disconn_ind_t disconn_ind ;
    dg_b_reset_req_t reset_req ;
    dg_b_reset_ind_t reset_ind ;
    dg_b_dtmf_req_t dtmf_req ;
    dg_b_dtmf_ind_t dtmf_ind ;
} bchan_api_t ;

#define	BCHAN_API_MAX_MSG_SIZE	(sizeof(union bchan_api_union))

#else
typedef struct bchan_api_union {
private:
	/* pointer is stored here */
	volatile void *location ;

public:
	/* methods to present casts of that pointer */
	inline dg_b_bad_message_ind_t *badmsg() { return ((dg_b_bad_message_ind_t *) location) ; }
	inline dg_b_config_conf_t *config_conf() { return ((dg_b_config_conf_t *) location) ; }
	inline dg_b_conn_conf_t *conn_conf() { return ((dg_b_conn_conf_t *) location) ; }
	inline uint32 command() { return (uint32) *((dxb_uint32 *) location) ; }
	inline uint32 call_id() { return (uint32) ((dg_b_msg_t *) location)->call_id ; }

	/* Constructor #1 */
	bchan_api_union(void *ptr) {
		location = ptr ;
	}

	/* Constructor #2 */
	bchan_api_union(uint8 *ptr) {
		location = (void *) ptr ;
	}

	/* Constructor #3 */
	bchan_api_union(char *ptr) {
		location = (void *) ptr ;
	}

	/* Dummy null constructor */
	bchan_api_union() {}

	/* Eventually, we should come up with a way of quickly giving them */
	/* the right size.  Right now, the configuration request structure */
	/* seems to be the largest...                                      */
	inline uint32 max_size() {
		return sizeof(dg_b_config_req_t) ;
	}
} bchan_api_t ;
#endif

/* ----------------------------- New API defined above -------------------- */
#else	/* --------------------- New API defined above -------------------- */
/* ----------------------------- New API defined above -------------------- */

#define XN_CONNECT_B	0x01      /* Uses message type `xn_b_call_t' */
#define XN_CONNACK_B	0x02      /* Uses message type `xn_b_call_t' */
#define XN_DISCONNECT_B	0x03      /* Uses message type `xn_b_call_t' */
#define XN_DISCONNACK_B	0x04      /* Uses message type `xn_b_call_t' */
#define XN_CACTIVE_B3	0x05      /* Uses message type `xn_b_call_t' */
#define XN_B_PROTOCOL	0x06      /* Uses message type `xn_sel_b_proto_t' */
#define	XN_B_DATA	0x07      /* Uses message type `xn_b_data_t' */

#define	XN_BAD_B_MSG	0xFF   /* Uses message type `xn_bad_b_msg_ind_t' */

/*
 * Bad message (XN_BAD_B_MSG) reason codes.
 */
#define	BCHAN_REASON_MESSAGE_SIZE    0x01  /* Message size wrong for command */
#define	BCHAN_REASON_BAD_COMMAND     0x02  /* Command not recognized */
#define	BCHAN_REASON_BAD_STATE       0x03  /* Inappropriate message */
#define	BCHAN_REASON_BAD_CHAN_TYPE   0x04  /* Channel type not supported */
#define	BCHAN_REASON_INVALID_CALL_ID 0x05  /* Call ID cannot possibly exist */
#define	BCHAN_REASON_UNUSED_CALL_ID  0x06  /* Call ID not in use */

/*
 * Error (XN_B_DATA) codes.
 */
#define ERROR_NONE		0x00000000	/* Good data */
#define ERROR_HDLC_UNDERRUN	0x00000001
#define ERROR_HDLC_SHORT	0x00000020
#define ERROR_HDLC_FORMAT	0x00000040
#define ERROR_HDLC_OVERRUN	0x00000080	/* Includes long frame */
#define ERROR_HDLC_ABORTED	0x00000100
#define ERROR_HDLC_CRC		0x00000200
#define ERROR_HDLC_LENGTH	0x00000400	/* Good long frame */
#define ERROR_HDLC_MARK		0x00000800	/* Mark spacing follows */

/******************************************************************************
 * Redefinitions of ISDN message primitives that are easier to understand
 *****************************************************************************/

/* b-channel control */
#define XN_SELECT_B_PROTO_REQ	XN_B_PROTOCOL
#define XN_SELECT_B_PROTO_CONF	XN_B_PROTOCOL
#define XN_CONN_B_REQ		XN_CONNECT_B
#define XN_CONN_B_CONF		XN_CONNACK_B
#define XN_CONN_B_IND		XN_CONNECT_B
#define XN_CONN_B_RESP		XN_CONNACK_B
#define XN_CONN_B3_ACT_IND	XN_CACTIVE_B3
#define XN_CONN_B3_ACT_RESP	XN_CACTIVE_B3
#define XN_DISCONN_B_REQ	XN_DISCONNECT_B
#define XN_DISCONN_B_CONF	XN_DISCONNACK_B
#define XN_DISCONN_B_IND	XN_DISCONNECT_B
#define XN_DISCONN_B_RESP	XN_DISCONNACK_B
#define XN_BAD_B_MSG_IND	XN_BAD_B_MSG

/* bchannel hotpath control primitives */
#define XN_FLUSH_REQ		XN_FLUSH
#define XN_MARK_REQ		XN_MARK
#define XN_SYNC_REQ		XN_SYNC
#define XN_SYNC_CONF		XN_SYNC
#define	XN_DATA_REQ		XN_B_DATA
#define	XN_DATA_IND		XN_B_DATA

/******************************************************************************
 * B protocol definitions and structures 
 *****************************************************************************/

/*
 * B1 protocol definitions
 */
#define B1_HDLC64       0      /* 64 kbit/s w/HDLC framing (default) */
#define B1_RAW64        1      /* 64 kbit/s bit-transparent */
#define B1_V110_ASYNC	2      /* V.110 async w/start-stop framing */
#define B1_V110_SYNC	3      /* V.110 sync w/HDLC framing */
#define B1_T30		4      /* T.30 Modem for Group 3 FAX */
#define B1_HDLC64_INV	5      /* 64 kbit/s inverted w/HDLC framing */
#define B1_RAW56        6      /* 56 kbit/s bit-transparent */
#define B1_MODEM_FN	7      /* Modem w/full negotiation */
#define B1_MODEM_SSBF	8      /* Modem async w/start-stop framing */
#define B1_MODEM_HDLC	9      /* Modem sync w/ HDLC framing */

#define B1_CIP_DEFAULT  256
#define B1_HDLC56       257
#define B1_ULAW         258    /* Not Supported */
#define B1_ALAW         259    /* Not Supported */
#define B1_NONE         260
#define B1_HDLC_ID      261    /* Layer 1 auto frame detection mode */

/* B1 protocol/layer 1 config map 

 * B1_HDLC64    L1_HDLC64
 * B1_RAW64     L1_RAW64
 * B1_V110_SYNC L1_V110         
 * B1_RAW56     L1_RAW56
 * ?            B1_CIP_DEFAULT 
 * ?            B1_HDLC56
 * ?            B1_ULAW
 * ?            B1_ALAW
 * ?            B1_V120 
 * ?            B1_NONE
 * ?            B1_HDLC_ID
 */

/*
 * B2 protocol definitions
 */
#define B2_X75SLP       0      /* ISO 7776 (X75 SLP) (default) */
#define B2_RAW		1      /* Transparent */
#define B2_SDLC		2      /* SDLC */
#define B2_LAPD		3      /* LAPD (Q.921) for D-channel X.25 on SAPI 16 */
#define B2_T30		4      /* T.30 for Group 3 FAX */
#define B2_PPP		5      /* Point-to-Point Protocol (PPP) */
#define B2_RAW_E	6      /* Transparent (ignore framing errors of B1 ) */
#define B2_MODEM_FN	7      /* Modem w/full neg. (e.g. V.42 bis, MNP 5) */
#define B2_X75SLP_V42	8      /* ISO 7776 w/V.42 bis compression */
#define B2_V120		9      /* V.120 asynchronous mode */
#define B2_V120_V42	10     /* V.120 asynchronous mode w/V.42 bis comp. */
#define B2_V120_RAW	11     /* V.120 bit-transparent mode */
#define B2_LAPD_SAPI	12     /* LAPD (Q.921) for DCH X.25 on SAPI chosen */

/*
 * B3 protocol definitions
 */
#define B3_RAW		0      /* Transparent (default) */
#define B3_T90NL	1      /* T.90NL w/T.70NL */
#define B3_X25_DTE	2      /* ISO 8208 (X25 DTE-DTE) */
#define B3_X25_DCE	3      /* X25 DCE */
#define B3_T30		4      /* T.30 for Group 3 FAX */
#define B3_T30_EXT	5      /* T.30 for Group 3 FAX Extended */
#define B3_RESERVED	6      /* Reserved */
#define B3_MODEM	7      /* Modem */

/*
 * Mode definitions
 */
#define	BMODE_STATIC	0      /* Connect req. -- no disc. indications */
#define	BMODE_ACTIVE	1      /* Connect req. -- allow disc. indications */
#define	BMODE_PASSIVE	2      /* Connect ind. -- allow disc. indications */

typedef struct
{
    dxb_uint16 max_bit_rate;
    dxb_uint16 bits_per_character;
    dxb_uint16 parity;
    dxb_uint16 stop_bits;
    dxb_uint16 options;
    dxb_uint16 speed_negotiation;
    dxb_uint32 max_frame_size; /* maximum frame size for HDLC  */
}
b1_config_t;

typedef struct
{
    dxb_uint8 address_a;
    dxb_uint8 address_b;
    dxb_uint8 modulo_mode;
    dxb_uint8 window_size;
    dxb_uint16 direction;
    dxb_uint16 code_words;
    dxb_uint16 max_string;
}
b2_config_t;

typedef struct
{
    dxb_uint16 options;
    dxb_uint16 resolution;
    dxb_uint16 format;
    char station_id[20];
    char headline[80];
}
b3_t30_config_t;

typedef struct
{
    dxb_uint16 lic;
    dxb_uint16 hic;
    dxb_uint16 ltc;
    dxb_uint16 htc;
    dxb_uint16 loc;
    dxb_uint16 hoc;
    dxb_uint16 modulo_mode;
    dxb_uint16 window_size;
}
b3_config_t;

typedef struct
{
    dxb_uint16 mode ;
    dxb_uint16 b1_protocol;
    dxb_uint16 b2_protocol;
    dxb_uint16 b3_protocol;
    b1_config_t b1_config;
    b2_config_t b2_config;
    b3_config_t b3_config;
}
b_proto_t;

/* typedef struct ncpi_x25 */
typedef struct
{
    dxb_uint8 options;
    dxb_uint8 group;
    dxb_uint8 channel;
    dxb_uint8 contents[1];
}
ncpi_x25_t;

/* typedef struct ncpi_t30 */
typedef struct 
{
    dxb_uint16 rate;
    dxb_uint16 options;
    dxb_uint16 format;
    dxb_uint16 pages;
    dxb_uint32 rx_id;
}
ncpi_t30_t;

/* typedef struct ncpi_modem */
typedef struct 
{
    dxb_uint16 rate;
    dxb_uint16 protocol;
}
ncpi_modem_t;

/*
 * XN_BAD_B_MSG_IND
 *
 * Firmware indication of the receipt of a bad message from the user
 */
typedef struct
{
    dxb_uint32 command;	       /* XN_BAD_B_MSG_IND             */
    dxb_uint32 id;	       /* call or listen id            */
    dxb_uint32 message;	       /* errored message              */
    dxb_uint32 reason;	       /* Cause for error message      */
}
xn_bad_b_msg_ind_t;

/*
 * XN_SELECT_B_PROTO_REQ / XN_SELECT_B_PROTO_CONF
 *
 * User request to change B protocol
 */
typedef struct
{
    dxb_uint32 command;	       /* XN_SELECT_B_PROTO_REQ / CONF */
    dxb_uint32 call_id;	       /* call id                      */
    b_proto_t b_protocol;      /* B-channel protocol config    */
}
xn_sel_b_proto_t;

/******************************************************************************
 * Data API primitive definitions and structures 
 *****************************************************************************/

/*
 * XN_DATA_REQ
 *
 * User request to transmit B channel data
 *
 * XN_DATA_IND
 *
 * Firmware indication of received B channel data 
 *
 * Note: These are control_data messages. The structure below is in the control
 * portion and the data is in the data portion.
 *
 * Note: Many protocols will just use plain data messages.
 * Note: The secondary call id is for protocols supporting virtual calls (X.25)
 */
typedef struct
{
    dxb_uint32 command;	       /* XN_DATA                           */
    dxb_uint32 call_id;	       /* call id                           */
    dxb_uint32 call_id_2;      /* secondary call id                 */
    dxb_uint32 error;	       /* error message type                */
    dxb_uint32 flags;	       /* CAPI flags for X.25 and V.120     */
}
xn_b_data_t;

/*
 * XN_CONN_B_REQ, XN_CONN_B_CONF, XN_CONN_B_IND, XN_CONN_B_RESP
 * XN_DISCONN_B_REQ, XN_DISCONN_B_CONF, XN_DISCONN_B_IND, XN_DISCONN_B_RESP
 * XN_CONN_B3_ACT_IND, XN_CONN_B3_ACT_RESP
 * XN_CONNECT_B, XN_CONNACK_B, XN_DISCONNECT_B, XN_DISCONNACK_B
 *
 * B protocol connect and disconnect messages.
 */
typedef struct
{
    dxb_uint32 command;	       /* XN_B_*                       */
    dxb_uint32 call_id;	       /* call id                      */
    dxb_uint32 call_id_2;      /* secondary call id            */
    dxb_uint32 info;	       /* info, reason or reject       */
    ncpi_x25_t x25;            /* x25 protocol specific info   */
    ncpi_t30_t t30;            /* t30 protocol specific info   */
    ncpi_modem_t modem;        /* modem protocol specific info */
}
xn_b_call_t;

/*
 * XN_FLUSH
 *
 * Flush inbound and/or outbound data.  This command does NOT affect API
 * commands, including XN_SYNC.  Any of the flags may be used in any
 * combination.  This message may be sent as normal priority or high
 * priority.  No response is generated.
 */
typedef struct
{
    dxb_uint32 command;	       /* XN_FLUSH           */
    dxb_uint32 call_id;	       /* call id            */
    dxb_uint32 call_id_2;      /* secondary call id  */
    dxb_uint32 flags;	       /* See below          */
}
xn_flush_t;

typedef xn_flush_t xn_flsh_t;

#define	FLUSH_INPUT_BUFFER	0x01	/* Flush read-side data         */
#define	FLUSH_OUTPUT_BUFFER	0x02	/* Flush write-side data  */
#define	FLUSH_ABORT_TX_FRAME	0x04	/* Abort current outbound frame */

/*
 * XN_MARK
 *
 * Set mark idle detect mode, or issue mark idle interframe fill orders.
 * This command should be issued during active calls only.
 */
typedef struct
{
    dxb_uint32 command;	       /* XN_MARK            */
    dxb_uint32 call_id;	       /* call id            */
    dxb_uint32 call_id_2;      /* secondary call id  */
    dxb_uint32 mode;	       /* Operational mode -- see below */
}
xn_mark_t;

/* Choices for xn_mark_t mode field */
#define MARK_TX_DEACTIVATE      1
#define MARK_TX_ACTIVATE        2
#define MARK_TX_ONCE            3
#define MARK_RX_DETECT_ON       4
#define MARK_RX_DETECT_OFF      5

/*
 * XN_SYNC
 *
 * Synchronization message.  Message is sent by user, and firmware bounces
 * message back to user when all data before the message is transmitted,
 * if sync message is sent normal priority.
 */
typedef struct
{
    dxb_uint32 command;	       /* XN_SYNC                    */
    dxb_uint32 call_id;	       /* call id                    */
    dxb_uint32 sync_id;	       /* Optional unique identifier */
    dxb_uint32 flags;	       /* See below                  */
}
xn_sync_t;

#define	SYNC_RESHIPRI		0x02	/* Make sync response high priority */

/******************************************************************************
 * Generic B-channel API primitive
 *****************************************************************************/

/*
 * Message header
 */
typedef struct
{
    dxb_uint32 command;
    dxb_uint32 call_id;
}
xn_b_msg_t;

#ifndef __cplusplus
union bchan_primitives
{
    dxb_uint32 command;	      
    xn_b_msg_t msg;

    xn_bad_b_msg_ind_t bad_msg;
    xn_sel_b_proto_t sel_b;
    xn_data_t data;
    xn_b_call_t b_stuff;
    xn_flush_t flush;
    xn_mark_t mark;
    xn_sync_t sync;
};

#define	BCHAN_API_MAX_MSG_SIZE	(sizeof(union bchan_primitives))

#else
typedef struct union_bchan_api {
private:
	/* pointer is stored here */
	volatile void *location ;

public:
	/* methods to present casts of that pointer */
	inline xn_bad_b_msg_ind_t *badmsg() { return ((xn_bad_b_msg_ind_t *) location) ; }
	inline xn_sel_b_proto_t *selb() { return ((xn_sel_b_proto_t *) location) ; }
	inline xn_b_data_t *data() { return ((xn_b_data_t *) location) ; }
	inline xn_b_call_t *bcall() { return ((xn_b_call_t *) location) ; }
	inline xn_flush_t *flush() { return ((xn_flush_t *) location) ; }
	inline xn_mark_t *mark() { return ((xn_mark_t *) location) ; }
	inline xn_sync_t *sync() { return ((xn_sync_t *) location) ; }
	inline uint32 command() { return (uint32) *((dxb_uint32 *) location) ; }
	inline uint32 call_id() { return (uint32) ((xn_b_msg_t *) location)->call_id ; }

	/* Constructor #1 */
	union_bchan_api(void *ptr) {
		location = ptr ;
	}

	/* Constructor #2 */
	union_bchan_api(uint8 *ptr) {
		location = (void *) ptr ;
	}

	/* Constructor #3 */
	union_bchan_api(char *ptr) {
		location = (void *) ptr ;
	}

	/* Dummy null constructor */
	union_bchan_api() {}

	/* Eventually, we should come up with a way of quickly giving them */
	/* the right size. */
	inline uint32 max_size() {
		return 512 ;
	}
} bchan_api_t ;

#define	BCHAN_API_MAX_MSG_SIZE	(((bchan_api_t *) 0)->max_size())

#endif /* __cplusplus */

#endif	/* NEW_BCHAN_API */
#endif	/* _BCHAN_PROTO_H */

