
/*
 Copyright 1999 by Digi International (www.digi.com)

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the 
 implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
 * drivercc.h
 *
 */

/*
 * Mappings from ISO 3166 Country Codes to ASCII Country Name Strings
 *
 * $Id: drivercc.h,v 1.7 1999/10/21 18:30:18 jamesp Exp $
 */
#ifndef _DRIVERCC_H_INCLUDED
#define _DRIVERCC_H_INCLUDED

/*
 *  The country code definitions are required.
 */
#include "mgmtcc.h"

/*
 *  The intent of a "driver" specific header file associated with the
 *  country codes is to provide a location for information not crucial
 *  to the operation of our software, but necessary for the consistency
 *  of the user interface of our products across platforms.
 *
 *  The information currently stored here would useful in "pick lists"
 *  which allow a customer to select the modem nationality for a
 *  particular device.
 *
 *  The contents of this file are considered (by its creator) to be
 *  of questionable value until verified.  Certainly, the format of the
 *  data archived below may change before everyone is comfortable with
 *  the list.
 *
 *  Before creating this list, a number of questions were posed to a
 *  subset of the management team who took the action to examine these
 *  concerns.  None have yet been answered.  The use of the list may
 *  be affected by the lack of "official" answers to these concerns.
 *
 *    1. Do we, as a business, care whether selections made by users
 *       in our software across multiple platforms are consistent?
 *
 *    2. Do we, as a business, care whether listed selections will be
 *       viewed as correct in the target countries?
 *
 *    3. In an effort to preserve both correctness and consistency, how
 *       should we (as a business) handle the following scenarios
 *       (provided, of course, we care that they may affect the
 *       presentation of the country names):
 *
 *       a. lists which may not change case (either all upper, or all lower)
 *
 *       b. lists which may not have spaces
 *
 *       c. lists which may not have punctuation
 *
 *       d. lists with field width restrictions
 *
 *    By way of example... take the cases of "Korea, Democratic People's
 *    Republic of" and "Taiwan, Province of China".  What if the comma is
 *    forbidden?  Can the descriptive phrases (i.e. "Democratic...") be
 *    parenthesized?  If no punctuation is allowed, how can they be
 *    rephrased?  Are simply "Korea" and "Taiwan" correct?
 *
 *
 * 05/25/99: Clarification: According to Greg Helmken, product manager,
 *           it would be in our best interest to drop the "Province of
 *           China" descriptive text for Taiwan and to simply use the
 *           string "Taiwan".  The data below has been changed accordingly.
 *
 * 06/10/99: Clarification: "Korea" is sufficient.
 *
 */

/*
 *  The CC_NAME_PAIR structure describes a country code/country name pairing.
 */
typedef struct cc_name_pair {
	char   *country_name;
	uint32  country_code;
} CC_NAME_PAIR;


/*
 *  The ISO_CC_NAME_PAIR_LIST includes most of the country codes defined by
 *  ISO 3166.  The list includes all countries found in the "mgmtcc.h"
 *  header file, and may be used to map country names to country codes.
 *
 *  The following questions were posed at one point, and no answers
 *  were ever given.  Use of the list may be hindered by one or more
 *  of the concerns below.
 */

/*
 *	Usage:
 *	CC_NAME_PAIR <array_name_here>[] = ISO_CC_NAME_PAIR_INIT ;
 */

#define ISO_CC_NAME_PAIR_INIT {\
	{"Afghanistan",                           COUNTRY_CODE_AFGHANISTAN},\
	{"Albania",                               COUNTRY_CODE_ALBANIA},\
	{"Algeria",                               COUNTRY_CODE_ALGERIA},\
	{"American Samoa",                        COUNTRY_CODE_AMERICAN_SAMOA},\
	{"Andorra",                               COUNTRY_CODE_ANDORRA},\
	{"Angola",                                COUNTRY_CODE_ANGOLA},\
	{"Anguilla",                              COUNTRY_CODE_ANGUILLA},\
	{"Antarctica",                            COUNTRY_CODE_ANTARCTICA},\
	{"Antigua and Barbuda",                   COUNTRY_CODE_ANTIGUA_AND_BARBUDA},\
	{"Argentina",                             COUNTRY_CODE_ARGENTINA},\
	{"Armenia",                               COUNTRY_CODE_ARMENIA},\
	{"Aruba",                                 COUNTRY_CODE_ARUBA},\
	{"Australia",                             COUNTRY_CODE_AUSTRALIA},\
	{"Austria",                               COUNTRY_CODE_AUSTRIA},\
	{"Azerbaijan",                            COUNTRY_CODE_AZERBAIJAN},\
	{"Bahamas",                               COUNTRY_CODE_BAHAMAS},\
	{"Bahrain",                               COUNTRY_CODE_BAHRAIN},\
	{"Bangladesh",                            COUNTRY_CODE_BANGLADESH},\
	{"Barbados",                              COUNTRY_CODE_BARBADOS},\
	{"Belarus",                               COUNTRY_CODE_BELARUS},\
	{"Belgium",                               COUNTRY_CODE_BELGIUM},\
	{"Belize",                                COUNTRY_CODE_BELIZE},\
	{"Benin",                                 COUNTRY_CODE_BENIN},\
	{"Bermuda",                               COUNTRY_CODE_BERMUDA},\
	{"Bhutan",                                COUNTRY_CODE_BHUTAN},\
	{"Bolivia",                               COUNTRY_CODE_BOLIVIA},\
	{"Bosnia and Herzegovina",                COUNTRY_CODE_BOSNIA_AND_HERZEGOWINA},\
	{"Botswana",                              COUNTRY_CODE_BOTSWANA},\
	{"Bouvet Island",                         COUNTRY_CODE_BOUVET_ISLAND},\
	{"Brazil",                                COUNTRY_CODE_BRAZIL},\
	{"British Indian Ocean Territory",        COUNTRY_CODE_BRITISH_INDIAN_OCEAN_TERRITORY},\
	{"Brunei",                                COUNTRY_CODE_BRUNEI_DARUSSALAM},\
	{"Bulgaria",                              COUNTRY_CODE_BULGARIA},\
	{"Burkina Faso",                          COUNTRY_CODE_BURKINA_FASO},\
	{"Burundi",                               COUNTRY_CODE_BURUNDI},\
	{"Cambodia",                              COUNTRY_CODE_CAMBODIA},\
	{"Cameroon",                              COUNTRY_CODE_CAMEROON},\
	{"Canada",                                COUNTRY_CODE_CANADA},\
	{"Cape Verde",                            COUNTRY_CODE_CAPE_VERDE},\
	{"Cayman Islands",                        COUNTRY_CODE_CAYMAN_ISLANDS},\
	{"Central African Republic",              COUNTRY_CODE_CENTRAL_AFRICAN_REPUBLIC},\
	{"Chad",                                  COUNTRY_CODE_CHAD},\
	{"Chile",                                 COUNTRY_CODE_CHILE},\
	{"China",                                 COUNTRY_CODE_CHINA},\
	{"Christmas Island",                      COUNTRY_CODE_CHRISTMAS_ISLAND},\
	{"Cocos Islands",                         COUNTRY_CODE_COCOS_ISLANDS},\
	{"Colombia",                              COUNTRY_CODE_COLOMBIA},\
	{"Comoros",                               COUNTRY_CODE_COMOROS},\
	{"Congo",                                 COUNTRY_CODE_CONGO},\
	{"Congo, Democratic Republic of",         COUNTRY_CODE_CONGO_DEMOCRATIC_REPUBLIC_OF},\
	{"Cook Islands",                          COUNTRY_CODE_COOK_ISLANDS},\
	{"Costa Rica",                            COUNTRY_CODE_COSTA_RICA},\
	{"Cote d'Ivoire",                         COUNTRY_CODE_COTE_D_IVOIRE},\
	{"Croatia",                               COUNTRY_CODE_CROATIA},\
	{"Cuba",                                  COUNTRY_CODE_CUBA},\
	{"Cyprus",                                COUNTRY_CODE_CYPRUS},\
	{"Czech Republic",                        COUNTRY_CODE_CZECH_REPUBLIC},\
	{"Denmark",                               COUNTRY_CODE_DENMARK},\
	{"Djibouti",                              COUNTRY_CODE_DJIBOUTI},\
	{"Dominica",                              COUNTRY_CODE_DOMINICA},\
	{"Dominican Republic",                    COUNTRY_CODE_DOMINICAN_REPUBLIC},\
	{"East Timor",                            COUNTRY_CODE_EAST_TIMOR},\
	{"Ecuador",                               COUNTRY_CODE_ECUADOR},\
	{"Egypt",                                 COUNTRY_CODE_EGYPT},\
	{"El Salvador",                           COUNTRY_CODE_EL_SALVADOR},\
	{"Equatorial Guinea",                     COUNTRY_CODE_EQUATORIAL_GUINEA},\
	{"Eritrea",                               COUNTRY_CODE_ERITREA},\
	{"Estonia",                               COUNTRY_CODE_ESTONIA},\
	{"Ethiopia",                              COUNTRY_CODE_ETHIOPIA},\
	{"Falkland Islands (Islas Malvinas)",     COUNTRY_CODE_FALKLAND_ISLANDS},\
	{"Faroe Islands",                         COUNTRY_CODE_FAROE_ISLANDS},\
	{"Fiji Islands",                          COUNTRY_CODE_FIJI},\
	{"Finland",                               COUNTRY_CODE_FINLAND},\
	{"France",                                COUNTRY_CODE_FRANCE},\
	{"French Guiana",                         COUNTRY_CODE_FRENCH_GUIANA},\
	{"French Polynesia",                      COUNTRY_CODE_FRENCH_POLYNESIA},\
	{"French Southern Territories",           COUNTRY_CODE_FRENCH_SOUTHERN_TERRITORIES},\
	{"Gabon",                                 COUNTRY_CODE_GABON},\
	{"Gambia",                                COUNTRY_CODE_GAMBIA},\
	{"Georgia",                               COUNTRY_CODE_GEORGIA},\
	{"Germany",                               COUNTRY_CODE_GERMANY},\
	{"Ghana",                                 COUNTRY_CODE_GHANA},\
	{"Gibraltar",                             COUNTRY_CODE_GIBRALTAR},\
	{"Greece",                                COUNTRY_CODE_GREECE},\
	{"Greenland",                             COUNTRY_CODE_GREENLAND},\
	{"Grenada",                               COUNTRY_CODE_GRENADA},\
	{"Guadeloupe",                            COUNTRY_CODE_GUADELOUPE},\
	{"Guam",                                  COUNTRY_CODE_GUAM},\
	{"Guatemala",                             COUNTRY_CODE_GUATEMALA},\
	{"Guinea",                                COUNTRY_CODE_GUINEA},\
	{"Guinea-Bissau",                         COUNTRY_CODE_GUINEA_BISSAU},\
	{"Guyana",                                COUNTRY_CODE_GUYANA},\
	{"Haiti",                                 COUNTRY_CODE_HAITI},\
	{"Heard and McDonald Islands",            COUNTRY_CODE_HEARD_AND_MCDONALD_ISLANDS},\
	{"Vatican City",                          COUNTRY_CODE_HOLY_SEE},\
	{"Honduras",                              COUNTRY_CODE_HONDURAS},\
	{"Hong Kong S.A.R.",                      COUNTRY_CODE_HONG_KONG},\
	{"Hungary",                               COUNTRY_CODE_HUNGARY},\
	{"Iceland",                               COUNTRY_CODE_ICELAND},\
	{"India",                                 COUNTRY_CODE_INDIA},\
	{"Indonesia",                             COUNTRY_CODE_INDONESIA},\
	{"Iran",                                  COUNTRY_CODE_IRAN},\
	{"Iraq",                                  COUNTRY_CODE_IRAQ},\
	{"Ireland",                               COUNTRY_CODE_IRELAND},\
	{"Israel",                                COUNTRY_CODE_ISRAEL},\
	{"Italy",                                 COUNTRY_CODE_ITALY},\
	{"Jamaica",                               COUNTRY_CODE_JAMAICA},\
	{"Japan",                                 COUNTRY_CODE_JAPAN},\
	{"Jordan",                                COUNTRY_CODE_JORDAN},\
	{"Kazakhstan",                            COUNTRY_CODE_KAZAKHSTAN},\
	{"Kenya",                                 COUNTRY_CODE_KENYA},\
	{"Kiribati",                              COUNTRY_CODE_KIRIBATI},\
	{"Korea, North",                          COUNTRY_CODE_KOREA_DEMOCRATIC_PEOPLES_REPUBLIC_OF},\
	{"Korea (South)",                         COUNTRY_CODE_KOREA_REPUBLIC_OF},\
	{"Kuwait",                                COUNTRY_CODE_KUWAIT},\
	{"Kyrgyzstan",                            COUNTRY_CODE_KYRGYZSTAN},\
	{"Laos",                                  COUNTRY_CODE_LAO_PEOPLES_DEMOCRATIC_REPUBLIC},\
	{"Latvia",                                COUNTRY_CODE_LATVIA},\
	{"Lebanon",                               COUNTRY_CODE_LEBANON},\
	{"Lesotho",                               COUNTRY_CODE_LESOTHO},\
	{"Liberia",                               COUNTRY_CODE_LIBERIA},\
	{"Libya",                                 COUNTRY_CODE_LIBYAN_ARAB_JAMAHIRIYA},\
	{"Liechtenstein",                         COUNTRY_CODE_LIECHTENSTEIN},\
	{"Lithuania",                             COUNTRY_CODE_LITHUANIA},\
	{"Luxembourg",                            COUNTRY_CODE_LUXEMBOURG},\
	{"Macau S.A.R.",                          COUNTRY_CODE_MACAU},\
	{"Macedonia, Former Yugoslav Republic of",COUNTRY_CODE_MACEDONIA_THE_FORMER_YUGOSLAV_REPUBLIC_OF},\
	{"Madagascar",                            COUNTRY_CODE_MADAGASCAR},\
	{"Malawi",                                COUNTRY_CODE_MALAWI},\
	{"Malaysia",                              COUNTRY_CODE_MALAYSIA},\
	{"Maldives",                              COUNTRY_CODE_MALDIVES},\
	{"Mali",                                  COUNTRY_CODE_MALI},\
	{"Malta",                                 COUNTRY_CODE_MALTA},\
	{"Marshall Islands",                      COUNTRY_CODE_MARSHALL_ISLANDS},\
	{"Martinique",                            COUNTRY_CODE_MARTINIQUE},\
	{"Mauritania",                            COUNTRY_CODE_MAURITANIA},\
	{"Mauritius",                             COUNTRY_CODE_MAURITIUS},\
	{"Mayotte",                               COUNTRY_CODE_MAYOTTE},\
	{"Mexico",                                COUNTRY_CODE_MEXICO},\
	{"Micronesia, Federated States of",       COUNTRY_CODE_MICRONESIA_FEDERATED_STATES_OF},\
	{"Moldova",                               COUNTRY_CODE_MOLDOVA_REPUBLIC_OF},\
	{"Monaco",                                COUNTRY_CODE_MONACO},\
	{"Mongolia",                              COUNTRY_CODE_MONGOLIA},\
	{"Montserrat",                            COUNTRY_CODE_MONTSERRAT},\
	{"Morocco",                               COUNTRY_CODE_MOROCCO},\
	{"Mozambique",                            COUNTRY_CODE_MOZAMBIQUE},\
	{"Myanmar",                               COUNTRY_CODE_MYANMAR},\
	{"Namibia",                               COUNTRY_CODE_NAMIBIA},\
	{"Nauru",                                 COUNTRY_CODE_NAURU},\
	{"Nepal",                                 COUNTRY_CODE_NEPAL},\
	{"Netherlands",                           COUNTRY_CODE_NETHERLANDS},\
	{"Netherlands Antilles",                  COUNTRY_CODE_NETHERLANDS_ANTILLES},\
	{"New Caledonia",                         COUNTRY_CODE_NEW_CALEDONIA},\
	{"New Zealand",                           COUNTRY_CODE_NEW_ZEALAND},\
	{"Nicaragua",                             COUNTRY_CODE_NICARAGUA},\
	{"Niger",                                 COUNTRY_CODE_NIGER},\
	{"Nigeria",                               COUNTRY_CODE_NIGERIA},\
	{"Niue",                                  COUNTRY_CODE_NIUE},\
	{"Norfolk Island",                        COUNTRY_CODE_NORFOLK_ISLAND},\
	{"Northern Mariana Islands",              COUNTRY_CODE_NORTHERN_MARIANA_ISLANDS},\
	{"Norway",                                COUNTRY_CODE_NORWAY},\
	{"Oman",                                  COUNTRY_CODE_OMAN},\
	{"Pakistan",                              COUNTRY_CODE_PAKISTAN},\
	{"Palau",                                 COUNTRY_CODE_PALAU},\
	{"Panama",                                COUNTRY_CODE_PANAMA},\
	{"Papua New Guinea",                      COUNTRY_CODE_PAPUA_NEW_GUINEA},\
	{"Paraguay",                              COUNTRY_CODE_PARAGUAY},\
	{"Peru",                                  COUNTRY_CODE_PERU},\
	{"Philippines",                           COUNTRY_CODE_PHILIPPINES},\
	{"Pitcairn",                              COUNTRY_CODE_PITCAIRN},\
	{"Poland",                                COUNTRY_CODE_POLAND},\
	{"Portugal",                              COUNTRY_CODE_PORTUGAL},\
	{"Puerto Rico",                           COUNTRY_CODE_PUERTO_RICO},\
	{"Qatar",                                 COUNTRY_CODE_QATAR},\
	{"Reunion",                               COUNTRY_CODE_REUNION},\
	{"Romania",                               COUNTRY_CODE_ROMANIA},\
	{"Russia",                                COUNTRY_CODE_RUSSIAN_FEDERATION},\
	{"Rwanda",                                COUNTRY_CODE_RWANDA},\
	{"Saint Helena",                          COUNTRY_CODE_SAINT_HELENA},\
	{"Saint Kitts and Nevis",                 COUNTRY_CODE_SAINT_KITTS_AND_NEVIS},\
	{"Saint Lucia",                           COUNTRY_CODE_SAINT_LUCIA},\
	{"Saint Pierre and Miquelon",             COUNTRY_CODE_SAINT_PIERRE_AND_MIQUELON},\
	{"Saint Vincent and the Grenadines",      COUNTRY_CODE_SAINT_VINCENT_AND_THE_GRENADINES},\
	{"Samoa",                                 COUNTRY_CODE_SAMOA},\
	{"San Marino",                            COUNTRY_CODE_SAN_MARINO},\
	{"Sao Tome and Principe",                 COUNTRY_CODE_SAO_TOME_AND_PRINCIPE},\
	{"Saudi Arabia",                          COUNTRY_CODE_SAUDI_ARABIA},\
	{"Senegal",                               COUNTRY_CODE_SENEGAL},\
	{"Seychelles",                            COUNTRY_CODE_SEYCHELLES},\
	{"Sierra Leone",                          COUNTRY_CODE_SIERRA_LEONE},\
	{"Singapore",                             COUNTRY_CODE_SINGAPORE},\
	{"Slovakia",                              COUNTRY_CODE_SLOVAKIA},\
	{"Slovenia",                              COUNTRY_CODE_SLOVENIA},\
	{"Solomon Islands",                       COUNTRY_CODE_SOLOMON_ISLANDS},\
	{"Somalia",                               COUNTRY_CODE_SOMALIA},\
	{"South Africa",                          COUNTRY_CODE_SOUTH_AFRICA},\
	{"South Georgia and the South Sandwich Islands",COUNTRY_CODE_SOUTH_GEORGIA_AND_THE_SOUTH_SANDWICH_ISLANDS},\
	{"Spain",                                 COUNTRY_CODE_SPAIN},\
	{"Sri Lanka",                             COUNTRY_CODE_SRI_LANKA},\
	{"Sudan",                                 COUNTRY_CODE_SUDAN},\
	{"Suriname",                              COUNTRY_CODE_SURINAME},\
	{"Svalbard and Jan Mayen Islands",        COUNTRY_CODE_SVALBARD_AND_JAN_MAYEN_ISLANDS},\
	{"Swaziland",                             COUNTRY_CODE_SWAZILAND},\
	{"Sweden",                                COUNTRY_CODE_SWEDEN},\
	{"Switzerland",                           COUNTRY_CODE_SWITZERLAND},\
	{"Syria",                                 COUNTRY_CODE_SYRIAN_ARAB_REPUBLIC},\
	{"Taiwan",                                COUNTRY_CODE_TAIWAN_PROVINCE_OF_CHINA},\
	{"Tajikistan",                            COUNTRY_CODE_TAJIKISTAN},\
	{"Tanzania",                              COUNTRY_CODE_TANZANIA_UNITED_REPUBLIC_OF},\
	{"Thailand",                              COUNTRY_CODE_THAILAND},\
	{"Togo",                                  COUNTRY_CODE_TOGO},\
	{"Tokelau",                               COUNTRY_CODE_TOKELAU},\
	{"Tonga",                                 COUNTRY_CODE_TONGA},\
	{"Trinidad and Tobago",                   COUNTRY_CODE_TRINIDAD_AND_TOBAGO},\
	{"Tunisia",                               COUNTRY_CODE_TUNISIA},\
	{"Turkey",                                COUNTRY_CODE_TURKEY},\
	{"Turkmenistan",                          COUNTRY_CODE_TURKMENISTAN},\
	{"Turks and Caicos Islands",              COUNTRY_CODE_TURKS_AND_CAICOS_ISLANDS},\
	{"Tuvalu",                                COUNTRY_CODE_TUVALU},\
	{"Uganda",                                COUNTRY_CODE_UGANDA},\
	{"Ukraine",                               COUNTRY_CODE_UKRAINE},\
	{"United Arab Emirates",                  COUNTRY_CODE_UNITED_ARAB_EMIRATES},\
	{"United Kingdom",                        COUNTRY_CODE_UNITED_KINGDOM},\
	{"United States",                         COUNTRY_CODE_UNITED_STATES},\
	{"United States Minor Outlying Islands",  COUNTRY_CODE_UNITED_STATES_MINOR_OUTLYING_ISLANDS},\
	{"Uruguay",                               COUNTRY_CODE_URUGUAY},\
	{"Uzbekistan",                            COUNTRY_CODE_UZBEKISTAN},\
	{"Vanuatu",                               COUNTRY_CODE_VANUATU},\
	{"Venezuela",                             COUNTRY_CODE_VENEZUELA},\
	{"Vietnam",                               COUNTRY_CODE_VIET_NAM},\
	{"Virgin Islands, British",               COUNTRY_CODE_VIRGIN_ISLANDS_BRITISH},\
	{"Virgin Islands, U.S.",                  COUNTRY_CODE_VIRGIN_ISLANDS_US},\
	{"Wallis and Futuna Islands",             COUNTRY_CODE_WALLIS_AND_FUTUNA_ISLANDS},\
	{"Western Sahara",                        COUNTRY_CODE_WESTERN_SAHARA},\
	{"Yemen",                                 COUNTRY_CODE_YEMEN},\
	{"Yugoslavia",                            COUNTRY_CODE_YUGOSLAVIA},\
	{"Zambia",                                COUNTRY_CODE_ZAMBIA},\
	{"Zimbabwe",                              COUNTRY_CODE_ZIMBABWE},\
}

#define CC_NAME_PAIR_LEN(your_variable_name) \
        (sizeof(your_variable_name) / sizeof(CC_NAME_PAIR))


#endif /* _DRIVERCC_H_INCLUDED */
