/*****************************************************************************
 * Copyright (c) 1988-2000 Digi International Inc., All Rights Reserved
 *
 * This  software  contains  proprietary  and  confidential   information   of
 * Digi International  Inc.   By  accepting  transfer  of this copy, Recipient
 * agrees to retain this software in  confidence,  to  prevent  disclosure  to
 * others,  and  to make no  use of this software other than that for which it
 * was delivered.  This is an unpublished copyright work  Digi   International
 * Inc.    Except   as   permitted   by  federal  law,  17 USC 117, copying is
 * strictly prohibited.
 *
 * Digi  International  Inc.  CONFIDENTIAL  -   (Digi    International    Inc.
 * Confidential  when   combined   with   the   aggregated  modules  for  this
 * product) OBJECT CODE ONLY SOURCE MATERIALS
 *
 * (C) COPYRIGHT Digi International Inc. 2000
 *
 * All Rights Reserved
 *
 * US   Government   Users   Restricted   Rights   -   Use,   duplication   or
 * disclosure  restricted by GSA ADP Schedule Contract with Digi International
 * Inc.
 *
 *****************************************************************************/

/*
 * signal_proto.h
 *
 * $Id: signal_proto.h,v 1.20 2000/08/30 05:27:21 joergs Exp $
 *
 * This file contains the protocol structures for the arbiter and signalling
 * stacks that live below it.
 */

#ifndef _SIGNAL_PROTO_
#define _SIGNAL_PROTO_

/*****************************************************************************
 * Definitions of signalling message primitives
 ****************************************************************************/

/* signalling */
#define DG_LISTEN_REQ		0x01
#define DG_LISTEN_CONF		0x02

#define DG_CONN_REQ		0x03
#define DG_CONN_CONF		0x04
#define DG_CONN_IND		0x05
#define DG_CONN_RESP		0x06

#define DG_CONN_ABORT_IND	0x07
#define DG_CONN_ABORT_RESP	0x08

#define DG_DISCONN_REQ		0x09
#define DG_DISCONN_CONF		0x0A
#define DG_DISCONN_IND		0x0B
#define DG_DISCONN_RESP		0x0C

/* informative */
#define DG_ALERT_REQ		0x11
#define DG_ALERT_IND		0x12
#define DG_CALL_PROC_REQ	0x13
#define DG_CALL_PROC_IND	0x14
#define DG_CONN_ACK_IND		0x15
#define DG_DATA_ACTIVE_IND	0x16
#define DG_FACILITY_REQ		0x17
#define DG_FACILITY_IND		0x18
#define DG_INFO_REQ		0x19
#define DG_INFO_IND		0x1A
#define DG_PROGRESS_REQ		0x1B
#define DG_PROGRESS_IND		0x1C
#define DG_SETUP_ACK_IND	0x1D
#define DG_DL_ESTABLISH_IND	0x1E
#define DG_DL_UNESTABLISH_IND	0x1F

/* supplementary */
#define DG_GET_SUPP_SERV_REQ	0x21
#define DG_GET_SUPP_SERV_CONF	0x22
#define DG_HOLD_REQ		0x23
#define DG_HOLD_CONF		0x24
#define DG_HOLD_IND		0x25
#define DG_HOLD_RESP		0x26
/*
#define DG_SUSPEND_REQ		0x27
#define DG_SUSPEND_CONF		0x28
#define DG_SUSPEND_IND		0x29
#define DG_SUSPEND_RESP		0x2A
*/
#define DG_ECT_REQ		0x2B
#define DG_ECT_CONF		0x2C
#define DG_ECT_ALERT_IND	0x2D
#define DG_ECT_ACTIVE_IND	0x2E
/*
#define DG_3PTY_BEGIN_REQ	0x2F
#define DG_3PTY_BEGIN_CONF	0x30
#define DG_3PTY_END_REQ		0x31
#define DG_3PTY_END_CONF	0x32
*/
#define DG_CF_ACTIVE_REQ	0x34
#define DG_CF_ACTIVE_CONF	0x35
#define DG_CF_DEACTIVE_REQ	0x36
#define DG_CF_DEACTIVE_CONF	0x37
#define DG_CF_PARAM_REQ		0x38
#define DG_CF_PARAM_CONF	0x39
#define DG_CF_NUMBER_REQ	0x3A
#define DG_CF_NUMBER_CONF	0x3B
#define DG_CF_DIVERT_IND	0x3C
#define DG_CF_DIVERT_ACTIVE_IND	0x3D
#define DG_CF_ACTIVE_IND	0x3E
#define DG_CF_DEACTIVE_IND	0x3F
#define DG_CF_DIVERT_INFO_IND	0x40
#define DG_CD_REQ		0x41
#define DG_CD_CONF		0x42

/* bad message indication */
#define	DG_BAD_MESSAGE_IND		0xFF

/*
 * Bad message (DG_BAD_MSG_IND): Cause for Error Message
 */
#define	DG_REASON_MESSAGE_SIZE  	0x01	/* Message size wrong for command */
#define	DG_REASON_BAD_DIRECTORY 	0x02	/* Unable to make directory match */
#define	DG_REASON_BAD_SUBADDR   	0x03	/* Unable to make subaddress match */
#define	DG_REASON_INVALID_SNID  	0x04	/* Line not recognized */
#define	DG_REASON_BAD_COMMAND   	0x05	/* Command not recognized */
#define	DG_REASON_BAD_STATE     	0x06	/* Inappropriate message */
#define	DG_REASON_INVALID_APPID 	0x07	/* Appid not recognized */
#define	DG_REASON_INVALID_INDEX 	0x08	/* Channel index not recognized */
#define	DG_REASON_BAD_INFO_MASK 	0x09	/* Request not supported */
#define	DG_REASON_BAD_CHAN_TYPE 	0x10	/* Channel type not supported */

#define	DG_REASON_INVALID_CALL_ID 	0x11	/* Invalid call ID */
#define	DG_REASON_USED_CALL_ID 		0x12	/* Call ID already in use */
#define	DG_REASON_UNUSED_CALL_ID 	0x12	/* Unused call ID */
#define DG_REASON_CIP_VAL_UNKNOWN	0x14	/* CIP value unknown */
#define DG_REASON_FAC_NOT_SUPPORTED	0x15	/* Facility not Supported */

/**********************************************************************
 * start of misc temporary stuff that eventually will move somewhere else
 */

/*
 * Redefinitions of ISDN message primitives to ease change-over
 */

/*
 * Bad message (XN_BAD_MSG) reason codes.
 */
#define	IL3REASON_MESSAGE_SIZE  DG_REASON_MESSAGE_SIZE
#define	IL3REASON_BAD_DIRECTORY DG_REASON_BAD_DIRECTORY
#define	IL3REASON_BAD_SUBADDR   DG_REASON_BAD_SUBADDR
#define	IL3REASON_INVALID_SNID  DG_REASON_INVALID_SNID
#define	IL3REASON_BAD_COMMAND   DG_REASON_BAD_COMMAND
#define	IL3REASON_BAD_STATE     DG_REASON_BAD_STATE
#define	IL3REASON_INVALID_APPID DG_REASON_INVALID_APPID
#define	IL3REASON_INVALID_INDEX DG_REASON_INVALID_INDEX
#define	IL3REASON_BAD_INFO_MASK DG_REASON_BAD_INFOMASK
#define	IL3REASON_BAD_CHAN_TYPE DG_REASON_BAD_CHAN_TYPE

#define	IL3REASON_INVALID_CALL_ID DG_REASON_INVALID_CALL_ID
#define	IL3REASON_USED_CALL_ID DG_REASON_USED_CALL_ID
#define	IL3REASON_UNUSED_CALL_ID DG_REASON_UNUSED_CALL_ID




#define ISDN_NUMBER_MAXSZ	32
#define ISDN_SUBADR_MAXSZ	32 /* changed because of HICOM300 needs more, was 20, schj */

/*
 * IE Sizes
 */
#define SMALL_IE                2	/* BODGE for IEDARG_MIN */
#define NORMAL_IE               36	/* any IE > 36 => il3cglob */
#define MAX_BC_IE		12
#define MAX_CRV_IE		4
#define MAX_CAUSE_IE		32
#define MAX_CI_IE		6
#define MAX_CURRENCY		10
#define MAX_DT_IE		6
#define MAX_DISPLAY_IE		34
#define MAX_TEXT_IE		128	/* Display Text */
#define MAX_HLC_IE		5
#define MAX_KEYPAD_IE		ISDN_NUMBER_MAXSZ
#define MAX_LLC_IE		24
#define MAX_PROG_IE		6
#define MAX_UU_IE		131
#define MAX_IE                  255	/* BODGE for IEDARG_MAX */

#define NORMAL_SIZE_IE          NORMAL_IE
#define MAX_BC_LEN		MAX_BC_IE

/* Generic IE struct */
typedef struct
{
    dxb_uint32 length;
    dxb_uint8 contents[MAX_IE];
}
xn_ie_t;

/*
 * Layer 1 configuration of HDLC controller
 * If adding another layer_1_cfg remember to increase size of FILL_MASK_SZ and
 * and another entry to fill_mask[] in il3_to_wan() in il3_wan.c
 */
#define L1_CIP_DEFAULT  0
#define L1_RAW64        1
#define L1_RAW56        2
#define L1_HDLC64       3
#define L1_HDLC56       4
#define L1_ULAW         5      /* Not Supported */
#define L1_ALAW         6      /* Not Supported */
#define L1_V110         7      /* 56K sync*/
#define L1_V120         8
#define L1_NONE         9
#define L1_HDLC_ID      10
#define	L1_PIAFS1X	11     /* 32Kbs bearer cap */
#define	L1_PIAFS2X	12     /* Uses calling party subaddress */
#define	L1_X75		13

/*
 * Specific V.110 async rates.
 */
#define	L1_V110_9600	14
#define	L1_V110_19200	15
#define	L1_V110_38400	16

#define MAX_L1_SZ	17	/* one greater than largest L1_ value */

#define XN_DATA         8      /* Uses message type `xn_data_t' */

typedef struct
{
    uint32 command;            /* XN_DATA                           */
    uint32 snid;               /* subnetwork id                     */
    uint32 appid;              /* application id for arbitrator use */
    uint32 error;              /* Error message type                */
}
xn_data_t;

/*
 * Error (XN_DATA) codes.
 */
#define ERROR_NONE              0x00000000      /* Good data */
#define ERROR_HDLC_UNDERRUN     0x00000001
#define ERROR_HDLC_SHORT        0x00000020
#define ERROR_HDLC_FORMAT       0x00000040
#define ERROR_HDLC_OVERRUN      0x00000080      /* Includes long frame */
#define ERROR_HDLC_ABORTED      0x00000100
#define ERROR_HDLC_CRC          0x00000200
#define ERROR_HDLC_LENGTH       0x00000400      /* Good long frame */
#define ERROR_HDLC_MARK         0x00000800      /* Mark spacing follows */

/*
 * used internally
 */
typedef struct
{
    uint32 number_plan;
    uint32 number_type;
    uint32 pres_ind_scr;
    uint32 number_len;
    unsigned char number[ISDN_NUMBER_MAXSZ];
    uint32 subadr_type;
    uint32 subadr_len;
    unsigned char subadr[ISDN_SUBADR_MAXSZ];
}
xisdnf_t;

#define CIP_NONE	DG_CIP_NONE
#define CIP_SPEECH	DG_CIP_SPEECH
#define CIP_UDI		DG_CIP_UDI
#define CIP_RDI		DG_CIP_RDI
#define CIP_AUDIO31K	DG_CIP_AUDIO31K
#define CIP_AUDIO7K	DG_CIP_AUDIO7K
#define CIP_PACKET	DG_CIP_PACKET
#define CIP_RA56K	DG_CIP_RA56K
#define CIP_TOLLSAVER	DG_CIP_TOLLSAVER
#define CIP_TELEPHONY	DG_CIP_TELEPHONY
#define CIP_FAX3	DG_CIP_FAX3
#define	CIP_PIAFS	DG_CIP_PIAFS

#define CIP_TABLE \
	{ \
        {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}, \
        {0x04,0x03,0x80,0x90,0xA2,0x00,0x00,0x00,0x00,0x00,0x00,0x00}, \
        {0x04,0x02,0x88,0x90,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}, \
        {0x04,0x02,0x89,0x90,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}, \
        {0x04,0x03,0x90,0x90,0xA2,0x00,0x00,0x00,0x00,0x00,0x00,0x00}, \
        {0x04,0x02,0x91,0x90,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}, \
        {0x04,0x02,0x98,0x90,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}, \
        {0x04,0x04,0x88,0xC0,0xC6,0xE6,0x00,0x00,0x00,0x00,0x00,0x00}, \
        {0x04,0x04,0x88,0x90,0x21,0x8F,0x00,0x00,0x00,0x00,0x00,0x00}, \
        {0x04,0x03,0x91,0x90,0xA5,0x00,0x00,0x00,0x00,0x00,0x00,0x00}, \
        {0x04,0x02,0x89,0x90,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}, \
        {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}, \
        {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}, \
        {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}, \
        {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}, \
        {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}, \
        {0x04,0x03,0x80,0x90,0xA2,0x7D,0x02,0x91,0x81,0x00,0x00,0x00}, \
        {0x04,0x03,0x90,0x90,0xA2,0x7D,0x02,0x91,0x84,0x00,0x00,0x00}, \
        {0x04,0x02,0x88,0x90,0x7D,0x02,0x91,0xA1,0x00,0x00,0x00,0x00}, \
        {0x04,0x02,0x88,0x90,0x7D,0x02,0x91,0xA4,0x00,0x00,0x00,0x00}, \
        {0x04,0x02,0x88,0x90,0x7D,0x02,0x91,0xA8,0x00,0x00,0x00,0x00}, \
        {0x04,0x02,0x88,0x90,0x7D,0x02,0x91,0xB1,0x00,0x00,0x00,0x00}, \
        {0x04,0x02,0x88,0x90,0x7D,0x02,0x91,0xB2,0x00,0x00,0x00,0x00}, \
        {0x04,0x02,0x88,0x90,0x7D,0x02,0x91,0xB5,0x00,0x00,0x00,0x00}, \
        {0x04,0x02,0x88,0x90,0x7D,0x02,0x91,0xB8,0x00,0x00,0x00,0x00}, \
        {0x04,0x02,0x88,0x90,0x7D,0x02,0x91,0xC1,0x00,0x00,0x00,0x00}, \
        {0x04,0x03,0x91,0x90,0xA5,0x7D,0x02,0x91,0x81,0x00,0x00,0x00}, \
        {0x04,0x03,0x91,0x90,0xA5,0x7D,0x03,0x91,0x60,0x01,0x00,0x00}, \
        {0x04,0x02,0x88,0x90,0x7D,0x03,0x91,0x60,0x02,0x00,0x00,0x00}, \
        {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}, \
        {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}, \
        {0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00}  \
	}

/* usage: uint32 cip_table[][] = CIP_TABLE */

/*
 * Used for size of an array in il3_to_wan()
 */
#define CIPVALSZ 12

/*
 * XN_RING_IND
 *
 * Ring event message.  Generally expected by internal analog device which
 * speak to the arbiter.  Will only appear for inbound Channelized T1 calls.
 */
typedef struct
{
    dxb_uint32 command;	       /* XN_RING_IND			*/
    dxb_uint32 call_id;	       /* call id			*/
    dxb_uint32 ring_event;     /* type of ring event		*/
}
xn_ring_t;

#define	RING_START_EVENT	0x00000001
#define	RING_STOP_EVENT		0x00000002

/*
 * ring type definitions
 */
#define RING_TYPE_GENERIC	0x0000

/* end of temp misc stuff */
/**********************************************************************/

/*
 * ISDN Channel type
 */
#define NO_B_CH		0
#define B_CH		1
#define H0_CH		2
#define H11_CH		3
#define H12_CH		4
#define MULT_CH		5
#define ANY_CH		6	/* for incoming calls only */
#define ANY_B_CH	8
#define B_OR_D_CH	9	/* for incoming calls only */
#define D_CH		10
#define LEASED_CHAN	13

/*
 * ISDN addressing
 */
#define ADDR_EXT_BIT		0x80

#define ADDR_TYPE_MASK		0x70

#define ADDR_TYPE_UNKNOWN	(ADDR_EXT_BIT|0x00)
#define ADDR_TYPE_INTL		(ADDR_EXT_BIT|0x10)
#define ADDR_TYPE_NATIONAL	(ADDR_EXT_BIT|0x20)
#define ADDR_TYPE_NETWORK	(ADDR_EXT_BIT|0x30)
#define ADDR_TYPE_SUBSCR	(ADDR_EXT_BIT|0x40)
#define ADDR_TYPE_ABBREV	(ADDR_EXT_BIT|0x60)

#define ADDR_PLAN_MASK 		0x0F

#define ADDR_PLAN_UNKNOWN	(ADDR_EXT_BIT|0x00)
#define ADDR_PLAN_ISDN		(ADDR_EXT_BIT|0x01)
#define ADDR_PLAN_DATA		(ADDR_EXT_BIT|0x03)
#define ADDR_PLAN_TELEX		(ADDR_EXT_BIT|0x04)
#define ADDR_PLAN_NATIONAL	(ADDR_EXT_BIT|0x08)
#define ADDR_PLAN_PRIVATE	(ADDR_EXT_BIT|0x09)

#define ADDR_PRESENT_MASK	0x60

#define ADDR_PRESENT_ALLOW	(ADDR_EXT_BIT|0x00)
#define ADDR_PRESENT_RESTRICT	(ADDR_EXT_BIT|0x20)
#define ADDR_PRESENT_UNAVAIL	(ADDR_EXT_BIT|0x40)

#define ADDR_SCREEN_MASK	0x03

#define ADDR_SCREEN_USER	(ADDR_EXT_BIT|0x00)
#define ADDR_SCREEN_PASS	(ADDR_EXT_BIT|0x01)
#define ADDR_SCREEN_FAIL	(ADDR_EXT_BIT|0x02)
#define ADDR_SCREEN_NETWORK	(ADDR_EXT_BIT|0x03)

#define ADDR_ALLOW_USER		(ADDR_PRESENT_ALLOW|ADDR_SCREEN_USER)
#define ADDR_ALLOW_PASS		(ADDR_PRESENT_ALLOW|ADDR_SCREEN_PASS)
#define ADDR_ALLOW_FAIL		(ADDR_PRESENT_ALLOW|ADDR_SCREEN_FAIL)
#define ADDR_ALLOW_NETWORK	(ADDR_PRESENT_ALLOW|ADDR_SCREEN_NETWORK)
#define ADDR_RESTRICT_USER	(ADDR_PRESENT_RESTRICT|ADDR_SCREEN_USER)
#define ADDR_RESTRICT_PASS	(ADDR_PRESENT_RESTRICT|ADDR_SCREEN_PASS)
#define ADDR_RESTRICT_FAIL	(ADDR_PRESENT_RESTRICT|ADDR_SCREEN_FAIL)
#define ADDR_RESTRICT_NETWORK	(ADDR_PRESENT_RESTRICT|ADDR_SCREEN_NETWORK)
#define ADDR_UNAVAIL_NETWORK	(ADDR_PRESENT_RESTRICT|ADDR_SCREEN_UNAVAIL)

#define SUBADDR_EXT_BIT		0x80

#define SUBADDR_TYPE_MASK	0x70

#define SUBADDR_TYPE_NSAP	(ADDR_EXT_BIT|0x00)
#define SUBADDR_TYPE_USER	(ADDR_EXT_BIT|0x20)

#define SUBADDR_ODDEVEN_MASK	0x08

#define SUBADDR_EVEN_IND	(ADDR_EXT_BIT|0x00)
#define SUBADDR_ODD_IND		(ADDR_EXT_BIT|0x08)

/*
 * ISDN progress
 */
#define PROG_EXT_BIT		0x80

#define PROG_CODING_MASK 	0x70

#define PROG_CODING_CCITT	(PROG_EXT_BIT|0x00)
#define PROG_CODING_ISO		(PROG_EXT_BIT|0x20)
#define PROG_CODING_NATIONAL	(PROG_EXT_BIT|0x40)
#define PROG_CODING_NETWORK	(PROG_EXT_BIT|0x60)

#define PROG_LOCATION_MASK 	0x0F

#define PROG_LOCATION_USER	(PROG_EXT_BIT|0x00)
#define PROG_LOCATION_PRIV_LOC	(PROG_EXT_BIT|0x01)
#define PROG_LOCATION_PUB_LOC	(PROG_EXT_BIT|0x02)
#define PROG_LOCATION_TRANSIT	(PROG_EXT_BIT|0x03)
#define PROG_LOCATION_PUB_REM	(PROG_EXT_BIT|0x04)
#define PROG_LOCATION_PRIV_REM	(PROG_EXT_BIT|0x05)
#define PROG_LOCATION_INTL	(PROG_EXT_BIT|0x07)
#define PROG_LOCATION_BEYOND	(PROG_EXT_BIT|0x0A)

#define PROG_DESCRIPTION_MASK 	0x0F

#define PROG_ALL_NOT_ISDN	(PROG_EXT_BIT|0x01)
#define PROG_DEST_NOT_ISDN	(PROG_EXT_BIT|0x02)
#define PROG_ORG_NOT_ISDN	(PROG_EXT_BIT|0x03)
#define PROG_RETURN_TO_ISDN	(PROG_EXT_BIT|0x04)
#define PROG_IN_BAND_NOW	(PROG_EXT_BIT|0x08)
#define PROG_DELAY_AT_DEST	(PROG_EXT_BIT|0x0A)

/*
 * Generic (protocol independent) cause codes for DG_DISCONN_REQ
 */
#define GENERIC_CAUSE_IGNORE	0x0101
#define GENERIC_CAUSE_NORMAL	0x0102
#define GENERIC_CAUSE_BUSY	0x0103
#define GENERIC_CAUSE_NO_CHAN	0x0104
#define GENERIC_CAUSE_FAC_REJ	0x0105
#define GENERIC_CAUSE_BAD_CHAN	0x0106
#define GENERIC_CAUSE_BAD_DEST	0x0107
#define GENERIC_CAUSE_DEST_OOO	0x0108

/*
 * Local cause codes for DG_DISCONN_IND, DG_DISCONN_CONF, DG_CONN_ABORT_IND
 */
#define LOCAL_CAUSE_NO			0x0000
#define LOCAL_CAUSE_L1			0x0100
#define LOCAL_CAUSE_L2			0x0200
#define LOCAL_CAUSE_L2_NO_TEI		0x0201
#define LOCAL_CAUSE_L3			0x0300
#define LOCAL_CAUSE_L3_RESET		0x0301
#define LOCAL_CAUSE_L3_NO_RESP		0x0302
#define LOCAL_CAUSE_ARB 		0x0400
#define LOCAL_CAUSE_ARB_ANOTHER_APPL 	0x0401


/*
 * Sending Complete codes for DG_DCONN_IND, DG_CONN_REQ, DG_INFO_IND, DG_INFO_REQ
 */
#define SENDINGCOMP_FALSE	0x00
#define SENDINGCOMP_TRUE	0x01

/*****************************************************************************
  Information element definitions and structures
*****************************************************************************/

/*
 * IE Sizes
 */
#define MAX_CPN_IE		(ISDN_NUMBER_MAXSZ)
#define MAX_SA_IE		(ISDN_SUBADR_MAXSZ)

/* Called Party Number */
typedef struct
{
    dxb_uint32 plan;
    dxb_uint32 type;
    dxb_uint32 length;
    dxb_uint8 contents[MAX_CPN_IE];
}
dg_cdpn_ie_t;

/* Calling Party Number */
typedef struct
{
    dxb_uint32 plan;
    dxb_uint32 type;
    dxb_uint32 present_screen;
    dxb_uint32 length;
    dxb_uint8 contents[MAX_CPN_IE];
}
dg_cgpn_ie_t;

/* Called and Calling Subaddress */
typedef struct
{
    dxb_uint32 type;
    dxb_uint32 length;
    dxb_uint8 contents[MAX_SA_IE];
}
dg_sa_ie_t;

/* Progress */
typedef struct
{
    dxb_uint32 standard;
    dxb_uint32 location;
    dxb_uint32 description;
}
dg_prog_ie_t;

/* Redirecting Number */
typedef struct
{
    dxb_uint32 plan;
    dxb_uint32 type;
    dxb_uint32 present;
    dxb_uint32 reason;
    dxb_uint32 length;
    dxb_uint8 contents[MAX_CPN_IE];
}
dg_rdgn_ie_t;

/* Redirection Number */
typedef struct
{
    dxb_uint32 plan;
    dxb_uint32 type;
    dxb_uint32 present;
    dxb_uint32 length;
    dxb_uint8 contents[MAX_CPN_IE];
}
dg_rdnn_ie_t;

/*****************************************************************************
 * Call control API primitive definitions and structures
 ****************************************************************************/

/*
 * Normalized channels provide a simple, consistent channel numbering scheme
 * across varying media types.
 * Channel numbering is problematic due to the various ways timeslots are
 * assigned to channels on a given media.
 * Channelized T1 supports 24 channels in 24 timeslots numbered 1 to 24.
 * Primary Rate ISDN T1 supports 23 channels in 24 timelsots numbered 1 to 24.
 * Timeslot 24 is used for D-channel signalling.
 * Primary Rate ISDN E1 supports 30 channels in 32 timelsots numbered 0 to 31.
 * Timeslot 16 is used for D-channel signalling. Timeslot 0 is reserved.
 * Normalized channels hide this complexity from the user. Channels are
 * presented in simple linear order. Timeslot assignment is handled
 * transparently by the firmware based on the media based on the mapps below.
 *
 * T1 Channelized - Normalized/Physical Channel Map
 *
 * Normalized 0 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1 1 2 2 2 2 2
 * Channel    1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4
 *
 * Physical   0 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1 1 2 2 2 2 2
 * Channel    1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4
 * (Timeslot)
 *
 * T1 ISDN - Normalized/Physical Channel Map
 *
 * Normalized 0 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1 1 2 2 2 2
 * Channel    1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3
 *
 * Physical   0 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1 1 2 2 2 2 2
 * Channel    1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4
 * (Timeslot)
 *
 * E1 ISDN and E1 R2 - Normalized/Physical Channel Map
 *
 * Normalized   0 0 0 0 0 0 0 0 0 1 1 1 1 1 1   1 1 1 1 2 2 2 2 2 2 2 2 2 2 3
 * Channel      1 2 3 4 5 6 7 8 9 0 1 2 3 4 5   6 7 8 9 0 1 2 3 4 5 6 7 8 9 0
 *
 * Physical   0 0 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1 1 2 2 2 2 2 2 2 2 2 2 3 3
 * Channel    0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
 * (Timeslot)
 */

/*
 * CIP is defined as in CAPI document Ver 2.0.
 *
 * Note: CIP_TOLLSAVER is defined by Digi, not CAPI.
 *
 * The following values are currently supported
 */
/* Default CIP */
#define DG_CIP_NONE        0
/* CIP 0x00-0xff defined by BC ie only, no HLC ie existing */
#define DG_CIP_SPEECH      1	/* BC :0x04,0x03,0x80,0x90,0xA2|0xA3 */
#define DG_CIP_UDI         2	/* BC :0x04,0x02,0x88,0x90           */
#define DG_CIP_RDI         3	/* BC :0x04,0x02,0x89,0x90           */
#define DG_CIP_AUDIO31K    4	/* BC :0x04,0x03,0x90,0x90,0xA2|0xA3 */
#define DG_CIP_AUDIO7K     5	/* BC :0x04,0x02,0x91,0x90           */
#define DG_CIP_VIDEO       6	/* BC :0x04,0x02,0x98,0x90           */
#define DG_CIP_PACKET      7	/* BC :0x04,0x04,0x88,0xC0,0xC6,0xE6 */
#define DG_CIP_RA56K       8	/* BC :0x04,0x04,0x88,0x90,0x21,0x8F */
#define DG_CIP_UDITA       9	/* BC :0x04,0x03,0x91,0x90,0xA5      */
#define DG_CIP_TOLLSAVER   10	/* BC :0x04,0x02,0x80,0x90           */
/* CIP 0x10-0x1f defined by BC and HLC ie */
#define DG_CIP_TELEPHONY   16	/* BC :0x04,0x03,0x80,0x90,0xA2|0xA3 */
				/* HLC:0x7D,0x02,0x91,0x81           */
#define DG_CIP_FAX3        17	/* BC :0x04,0x03,0x90,0x90,0xA2|0xA3 */
				/* HLC:0x7D,0x02,0x91,0x84           */
#define DG_CIP_FAX4        18	/* BC :0x04,0x02,0x88,0x90           */
				/* HLC:0x7D,0x02,0x91,0xA1           */
#define DG_CIP_TELEXMIXED  19	/* BC :0x04,0x02,0x88,0x90           */
				/* HLC:0x7D,0x02,0x91,0xA4           */
#define DG_CIP_TELEXPROC   20	/* BC :0x04,0x02,0x88,0x90           */
				/* HLC:0x7D,0x02,0x91,0xA8           */
#define DG_CIP_TELEXBASIC  21	/* BC :0x04,0x02,0x88,0x90           */
				/* HLC:0x7D,0x02,0x91,0xB1           */
#define DG_CIP_VIDEOTEX    22	/* BC :0x04,0x02,0x88,0x90           */
				/* HLC:0x7D,0x02,0x91,0xB2           */
#define DG_CIP_TELEX       23	/* BC :0x04,0x02,0x88,0x90           */
				/* HLC:0x7D,0x02,0x91,0xB5           */
#define DG_CIP_X400        24	/* BC :0x04,0x02,0x88,0x90           */
				/* HLC:0x7D,0x02,0x91,0xB8           */
#define DG_CIP_X200        25	/* BC :0x04,0x02,0x88,0x90           */
				/* HLC:0x7D,0x02,0x91,0xC1           */
#define DG_CIP_TELEPH7K    26	/* BC :0x04,0x03,0x91,0x90,0xA5      */
				/* HLC:0x7D,0x02,0x91,0x81           */
#define DG_CIP_TELEPHF7211 27	/* BC :0x04,0x03,0x91,0x90,0xA5      */
				/* HLC:0x7D,0x03,0x91,0x60,0x01      */
#define DG_CIP_TELEPHF7212 28	/* BC :0x04,0x02,0x88,0x90           */
				/* HLC:0x7D,0x03,0x91,0x60,0x02      */
#define	DG_CIP_PIAFS	   31	/* BC, HLC unknown */

#define DG_CIP_MAX	32


/*
 * Defines for CIP mask
 */
#define DG_CIP_MASK_ANY		0x00000001
#define DG_CIP_MASK_SPEECH	(1<<DG_CIP_SPEECH)
#define DG_CIP_MASK_UDI		(1<<DG_CIP_UDI)
#define DG_CIP_MASK_RDI		(1<<DG_CIP_RDI)
#define DG_CIP_MASK_AUDIO31K	(1<<DG_CIP_AUDIO31K)
#define DG_CIP_MASK_AUDIO7K     (1<<DG_CIP_AUDIO7K)
#define DG_CIP_MASK_VIDEO       (1<<DG_CIP_VIDEO)
#define DG_CIP_MASK_PACKET	(1<<DG_CIP_PACKET)
#define DG_CIP_MASK_RA56K	(1<<DG_CIP_RA56K)
#define DG_CIP_MASK_UDITA       (1<<DG_CIP_UDITA)
#define DG_CIP_MASK_TOLLSAVER	(1<<DG_CIP_TOLLSAVER)
#define DG_CIP_MASK_TELEPHONY   (1<<DG_CIP_TELEPHONY)
#define DG_CIP_MASK_FAX3        (1<<DG_CIP_FAX3)
#define DG_CIP_MASK_FAX4        (1<<DG_CIP_FAX4)
#define DG_CIP_MASK_TELEXMIXED  (1<<DG_CIP_TELEXMIXED)
#define DG_CIP_MASK_TELEXPROC   (1<<DG_CIP_TELEXPROC)
#define DG_CIP_MASK_TELEXBASIC  (1<<DG_CIP_TELEXBASIC)
#define DG_CIP_MASK_VIDEOTEX    (1<<DG_CIP_VIDEOTEX)
#define DG_CIP_MASK_TELEX       (1<<DG_CIP_TELEX)
#define DG_CIP_MASK_X400        (1<<DG_CIP_X400)
#define DG_CIP_MASK_X200        (1<<DG_CIP_X200)
#define DG_CIP_MASK_TELEPH7K    (1<<DG_CIP_TELEPH7K)
#define DG_CIP_MASK_TELEPHF7211 (1<<DG_CIP_TELEPH7K211)
#define DG_CIP_MASK_TELEPHF7212 (1<<DG_CIP_TELEPH7K212)
#define DG_CIP_MASK_PIAFS       (1<<DG_CIP_PIAFS)


/*
 * DG_ALERT_REQ
 *
 * Generate ALERTING message
 *
 * No additional IEs accepted in data portion of primitive
 */
typedef struct
{
    dxb_uint32 command;	       /* DG_ALERT_REQ                 */
    dxb_uint32 call_id;	       /* call id                      */
}
dg_alert_req_t;

/*
 * DG_ALERT_IND
 *
 * Firmware indication of receipt of ALERT message
 *
 * Any IE's received are contained in data portion of primative
 */
typedef struct
{
    dxb_uint32 command;	       /* DG_ALERT_IND                 */
    dxb_uint32 call_id;	       /* call id                      */
    dxb_uint32 chan_type;      /* channel type                 */
    dxb_uint32 channel;	       /* normalized channel           */
    dxb_uint32 cref;	       /* call reference value         */
}
dg_alert_ind_t;

/*
 * DG_BAD_MESSAGE_IND
 *
 * Firmware indication of the receipt of a bad message from the user
 */
typedef struct
{
    dxb_uint32 command;	       /* DG_BAD_MESSAGE_IND           */
    dxb_uint32 id;	       /* call or listen id            */
    dxb_uint32 message;	       /* errored message              */
    dxb_uint32 reason;	       /* Cause for error message      */
}
dg_bad_msg_ind_t;

/*
 * DG_CALL_PROC_REQ
 *
 * Generate CALL PROCEEDING message
 *
 * No additional IEs accepted in data portion of primitive
 */
typedef struct
{
    dxb_uint32 command;	       /* DG_CALL_PROC_REQ             */
    dxb_uint32 call_id;	       /* call id                      */
    dg_prog_ie_t progress;     /* progress IE                  */
}
dg_call_proc_req_t;

/*
 * DG_CALL_PROC_IND
 *
 * Firmware indication of receipt of CALL PROCEEDING message
 *
 * Any IE's received are contained in data portion of primative
 */
typedef struct
{
    dxb_uint32 command;	       /* DG_CALL_PROC_IND             */
    dxb_uint32 call_id;	       /* call id                      */
    dxb_uint32 chan_type;      /* channel type                 */
    dxb_uint32 channel;	       /* normalized channel           */
    dxb_uint32 cref;	       /* call reference value         */
    dg_prog_ie_t progress;     /* progress indicator           */
}
dg_call_proc_ind_t;

/*
 * DG_CONN_ACK_IND
 *
 * Firmware indication of receipt of CONNECT ACKNOWLEDGE
 *
 * Any IE's received are contained in data portion of primative
 */
typedef struct
{
    dxb_uint32 command;	       /* DG_CONN_ACK_IND           */
    dxb_uint32 call_id;	       /* call id                      */
    dxb_uint32 chan_type;      /* channel type: B, H0, etc     */
    dxb_uint32 channel;	       /* selected normalized channel  */
    dxb_uint32 cref;	       /* call reference value         */
}
dg_conn_ack_ind_t;

/*
 * DG_CONN_REQ
 *
 * User request to generate SETUP message
 *
 * Additional IEs accepted in data portion of primitive:
 *	bc, llc, hlc, facility and user-user
 */
typedef struct
{
    dxb_uint32 command;	       /* DG_CONN_REQ                  */
    dxb_uint32 call_id;	       /* call id                      */
    dxb_uint32 snid;	       /* subnetwork id                */
    dxb_uint32 chan_type;      /* channel type: B, H0, etc     */
    dxb_uint32 channel;	       /* selected normalized channel  */
    dxb_uint32 cip_value;      /* CAPI style CIP value         */
    dxb_uint32 send_comp;      /* sending complete             */
    dg_cdpn_ie_t dest_addr;    /* called address               */
    dg_sa_ie_t dest_subaddr;   /* called subaddress            */
    dg_cgpn_ie_t org_addr;     /* calling address              */
    dg_sa_ie_t org_subaddr;    /* calling subaddress           */
}
dg_conn_req_t;

/*
 * DG_CONN_CONF
 *
 * Firmware confirmation of receipt of CONNECT message
 *
 * Any IE's received are contained in data portion of primative
 */
typedef struct
{
    dxb_uint32 command;	       /* DG_CONN_CONF                 */
    dxb_uint32 call_id;	       /* call id                      */
    dxb_uint32 chan_type;      /* channel type: B, H0, etc     */
    dxb_uint32 channel;	       /* actual normalized channel    */
    dxb_uint32 cref;	       /* call reference value         */
    dg_cgpn_ie_t conn_addr;    /* connected address (primary)  */
    dg_sa_ie_t conn_subaddr;   /* connected subaddress         */
}
dg_conn_conf_t;

/*
 * DG_CONN_IND
 *
 * Firmware indication of receipt of SETUP message
 *
 * Any IE's received are contained in data portion of primative
 */
typedef struct
{
    dxb_uint32 command;	       /* DG_CONN_IND                  */
    dxb_uint32 call_id;	       /* call id                      */
    dxb_uint32 snid;	       /* subnetwork id                */
    dxb_uint32 chan_type;      /* channel type: B, H0, etc     */
    dxb_uint32 channel;	       /* selected normalized channel  */
    dxb_uint32 cip_value;      /* CAPI style CIP value         */
    dxb_uint32 send_cmplt;     /* sending complete             */
    dxb_uint32 cref;	       /* call reference value         */
    dg_cdpn_ie_t dest_addr;    /* called address (primary)     */
    dg_sa_ie_t dest_subaddr;   /* called subaddress            */
    dg_cgpn_ie_t org_addr;     /* calling address              */
    dg_sa_ie_t org_subaddr;    /* calling subaddress           */
}
dg_conn_ind_t;

/*
 * DG_CONN_RESP
 *
 * User response to generate CONNECT message
 *
 * Additional IEs accepted in data portion of primitive:
 *	llc, keypad facility, facility and user-user
 */
typedef struct
{
    dxb_uint32 command;	       /* DG_CONN_RESP                 */
    dxb_uint32 call_id;	       /* call id                      */
    dxb_uint32 chan_type;      /* channel type: B, H0, etc     */
    dxb_uint32 channel;	       /* selected normalized channel  */
    dg_cgpn_ie_t conn_addr;    /* connected address            */
    dg_sa_ie_t conn_subaddr;   /* connected subaddress         */
}
dg_conn_resp_t;

/*
 * DG_CONN_ABORT_IND
 *
 * Firmware indication of a connection abort condition.
 * E.G. Unexpected RELEASE or L1 or L2 failure.
 *
 * Any IE's received are contained in data portion of primative
 */
typedef struct
{
    dxb_uint32 command;	       /* DG_CONN_ABORT_IND            */
    dxb_uint32 call_id;	       /* call id                      */
    dxb_uint32 net_cause;      /* cause (if any) else zero     */
    dxb_uint32 local_cause;    /* cause (if any) else zero     */
    dxb_uint32 cref;	       /* call reference value         */
}
dg_conn_abort_ind_t;

/*
 * DG_CONN_ABORT_RESP
 *
 * User response to a firmware connection abort indication.
 *
 * No additional IEs accepted in data portion of primitive
 */
typedef struct
{
    dxb_uint32 command;	       /* DG_CONN_ABORT_RESP           */
    dxb_uint32 call_id;	       /* call id                      */
}
dg_conn_abort_resp_t;

/*
 * DG_DATA_ACTIVE_IND
 *
 * Firmware indication of active B or D channel data path
 *  - generated by receipt of:
 *  - CONNECT ACKNOWLEDGE for an inbound call
 *  - SETUP ACK, CALL PROC, ALERT or CONNECT for an outbound call
 *  - availablity of channel for leased B channel call
 *  - availablity of channel for D channel packet mode call
 */
typedef struct
{
    dxb_uint32 command;	       /* DG_DATA_ACTIVE_IND           */
    dxb_uint32 call_id;	       /* call id                      */
    dxb_uint32 chan_type;      /* channel type: B, H0, etc     */
    dxb_uint32 channel;	       /* selected normalized channel  */
}
dg_data_act_ind_t;

/*
 * DG_DISCONN_REQ
 *
 * User request to reject a call (generates RELEASE COMPLETE message) or
 * terminate a call (generates a DISCONNECT message).
 *
 * Additional IEs accepted in data portion of primitive:
 *	keypad facility?, facility and user-user
 */
typedef struct
{
    dxb_uint32 command;	       /* DG_DISCONN_REQ               */
    dxb_uint32 call_id;	       /* call id                      */
    dxb_uint32 cause;	       /* cause (if any) else zero     */
}
dg_disconn_req_t;

/*
 * DG_DISCONN_CONF
 *
 * Firmware indication of final call clearing.
 *
 * Any IE's received are contained in data portion of primative
 */
typedef struct
{
    dxb_uint32 command;	       /* DG_DISCONN_CONF              */
    dxb_uint32 call_id;	       /* call id                      */
    dxb_uint32 net_cause;      /* cause (if any) else zero     */
    dxb_uint32 local_cause;    /* cause (if any) else zero     */
    dxb_uint32 cref;	       /* call reference value         */
}
dg_disconn_conf_t;

/*
 * DG_DISCONN_IND
 *
 * Firmware indication of DISCONNECT message.
 *
 * Any IE's received are contained in data portion of primative
 */
typedef struct
{
    dxb_uint32 command;	       /* DG_DISCONN_IND               */
    dxb_uint32 call_id;	       /* call id                      */
    dxb_uint32 net_cause;      /* cause (if any) else zero     */
    dxb_uint32 local_cause;    /* cause (if any) else zero     */
    dxb_uint32 cref;	       /* call reference value         */
}
dg_disconn_ind_t;

/*
 * DG_DISCONN_RESP
 *
 * User response to a firmware disconnect indication.
 *
 * No additional IEs accepted in data portion of primitive
 */
typedef struct
{
    dxb_uint32 command;	       /* DG_DISCONN_RESP              */
    dxb_uint32 call_id;	       /* call id                      */
}
dg_disconn_resp_t;

/*
 * DG_FACILITY_REQ
 *
 * Generate FACILITY message
 *
 * Additional IEs accepted in data portion of primitive:
 *	facility
 */
typedef struct
{
    dxb_uint32 command;	       /* DG_FACILITY_REQ              */
    dxb_uint32 call_id;	       /* call id                      */
}
dg_facility_req_t;

/*
 * DG_FACILITY_IND
 *
 * Firmware indication of receipt of FACILITY message
 *
 * Any IE's received are contained in data portion of primative
 */
typedef struct
{
    dxb_uint32 command;	       /* DG_FACILITY_IND              */
    dxb_uint32 call_id;	       /* call id                      */
    dxb_uint32 cref;	       /* call reference value         */
}
dg_facility_ind_t;

/*
 * DG_INFO_REQ
 *
 * User request to generate INFORMATION message
 *
 * Additional IEs accepted in data portion of primitive:
 *	keypad facility, facility and user-user
 */
typedef struct
{
    dxb_uint32 command;	       /* DG_INFO_REQ			*/
    dxb_uint32 call_id;	       /* call id			*/
    dxb_uint32 send_comp;      /* sending complete              */
    dg_cdpn_ie_t cpn;          /* called party number           */
}
dg_info_req_t;

/*
 * DG_INFO_IND
 *
 * Firmware indication of receipt of INFORMATION message.
 *
 * Any IE's received are contained in data portion of primative
 */

typedef struct
{
    dxb_uint32 command;	       /* DG_INFO_IND			*/
    dxb_uint32 call_id;	       /* call id			*/
    dxb_uint32 send_cmplt;     /* sending complete             */
    dxb_uint32 cref;	       /* call reference value         */
    dg_cdpn_ie_t dest_addr;    /* called address               */
}
dg_info_ind_t;

/*
 * DG_LISTEN_REQ
 *
 * User request to listen for incoming calls
 *
 * DG_LISTEN_CONF
 *
 * Firmware confirmation of listen request
 */

typedef struct
{
    dxb_uint32 command;	       /* DG_LISTEN_REQ/DG_LISTEN_CONF */
    dxb_uint32 listen_id;      /* listener id                  */
    dxb_uint32 snid;	       /* subnetwork id                */
    dg_cdpn_ie_t dest_addr;    /* listen called address        */
    dg_sa_ie_t dest_subaddr;   /* listen called subaddress     */
    dxb_uint32 cip_mask;       /* CAPI style CIP mask          */
    dxb_uint32 cip_mask_2;     /* CAPI style CIP mask          */
}
dg_lstn_t;

typedef dg_lstn_t dg_listen_req_t;
typedef dg_lstn_t dg_listen_conf_t;

/*
 * DG_PROGRESS_REQ
 *
 * Generate PROGRESS message
 *
 * No additional IEs accepted in data portion of primitive
 */
typedef struct
{
    dxb_uint32 command;	       /* DG_PROGRESS_REQ              */
    dxb_uint32 call_id;	       /* call id                      */
    dg_prog_ie_t progress;     /* progress indicator IE        */
}
dg_progress_req_t;

/*
 * DG_PROGRESS_IND
 *
 * Firmware indication of receipt of PROGRESS message.
 *
 * Any IE's received are contained in data portion of primative
 */
typedef struct
{
    dxb_uint32 command;	       /* DG_PROGRESS_IND              */
    dxb_uint32 call_id;	       /* call id                      */
    dxb_uint32 cref;	       /* call reference value         */
    dg_prog_ie_t progress;     /* progress indicator IE        */
}
dg_progress_ind_t;

/*
 * DG_SETUP_ACK_IND
 *
 * Firmware indication of receipt of SETUP ACKNOWLEDGE message.
 *
 * Any IE's received are contained in data portion of primative
 */
typedef struct
{
    dxb_uint32 command;	       /* DG_SETUP_ACK_IND             */
    dxb_uint32 call_id;	       /* call id                      */
    dxb_uint32 chan_type;      /* channel type                 */
    dxb_uint32 channel;	       /* normalized channel           */
    dxb_uint32 cref;	       /* call reference value         */
}
dg_setup_ack_ind_t;

/*
 * DG_DL_ESTABLISH_IND
 *
 * Firmware indication of going into DL_ESTABLISH state LAPD.
 */
typedef struct
{
    dxb_uint32 command;	       /* DG_DL_ESTABLISH_IND          */
    dxb_uint32 call_id;	       /* call id                      */
    dxb_uint32 snid;	       /* subnetwork id                */
}
dg_dl_establish_ind_t;

/*
 * DG_DL_UNESTABLISH_IND
 *
 * Firmware indication of going into DL_UNESTABLISH state LAPD.
 */
typedef struct
{
    dxb_uint32 command;	       /* DG_DL_UNESTABLISH_IND        */
    dxb_uint32 call_id;	       /* call id                      */
    dxb_uint32 snid;	       /* subnetwork id                */
}
dg_dl_unestablish_ind_t;

/*****************************************************************************
 * Generic ISDN API primitive
 ****************************************************************************/

/*
 * Message header
 */
typedef struct
{
    dxb_uint32 command;
    dxb_uint32 call_id;
}
dg_msg_t;

#ifndef __cplusplus
union dg_isdn_primitives
{
    dxb_uint32 command;
    dg_msg_t msg;

    dg_alert_ind_t		alertind;
    dg_alert_req_t		alertreq;
    dg_bad_msg_ind_t		badmsg;
    dg_call_proc_ind_t		callprocind;
    dg_call_proc_req_t		callprocreq;
    dg_conn_req_t		connreq;
    dg_conn_conf_t		connconf;
    dg_conn_ind_t		connind;
    dg_conn_resp_t		connresp;
    dg_data_act_ind_t		dataactind;
    dg_disconn_req_t		disconnreq;
    dg_disconn_ind_t		disconnind;
    dg_disconn_conf_t		disconnconf;
    dg_disconn_resp_t		disconnresp;
    dg_conn_abort_ind_t		connabortind;
    dg_conn_abort_resp_t	connabortresp;
    dg_facility_req_t		facilityreq;
    dg_facility_ind_t		facilityind;
    dg_progress_req_t		progressreq;
    dg_progress_ind_t		progressind;
    dg_info_req_t		inforeq;
    dg_info_ind_t		infoind;
    dg_lstn_t			listen;
    dg_setup_ack_ind_t		setupackind;
    dg_conn_ack_ind_t		connackind;
    dg_dl_establish_ind_t	establishind;
    dg_dl_unestablish_ind_t	unestablishind;
};

#define	ISDN_API_MAX_MSG_SIZE	(sizeof(union dg_isdn_primitives))

#else
#define ISDN_MAX_MSG_SIZE       512

typedef struct union_isdn_api {
private:
	/* pointer is stored here */
	volatile void *location ;

public:
	/* methods to present casts of that pointer */
	inline dg_alert_req_t		*alertreq()		{ return ((dg_alert_req_t *) location) ; }
	inline dg_alert_ind_t		*alertind()		{ return ((dg_alert_ind_t *) location) ; }
	inline dg_bad_msg_ind_t		*badmsg()		{ return ((dg_bad_msg_ind_t *) location) ; }
	inline dg_call_proc_req_t	*callprocreq()		{ return ((dg_call_proc_req_t *) location) ; }
	inline dg_call_proc_ind_t	*callprocind()		{ return ((dg_call_proc_ind_t *) location) ; }
	inline dg_conn_req_t		*connreq()		{ return ((dg_conn_req_t *) location) ; }
	inline dg_conn_conf_t		*connconf()		{ return ((dg_conn_conf_t *) location) ; }
	inline dg_conn_ind_t		*connind()		{ return ((dg_conn_ind_t *) location) ; }
	inline dg_conn_resp_t		*connresp()		{ return ((dg_conn_resp_t *) location) ; }
	inline dg_conn_ack_ind_t	*connackind()		{ return ((dg_conn_ack_ind_t *) location) ; }
	inline dg_data_act_ind_t	*dataactind()		{ return ((dg_data_act_ind_t *) location) ; }
	inline dg_disconn_req_t		*discreq()		{ return ((dg_disconn_req_t *) location) ; }
	inline dg_disconn_ind_t		*discind()		{ return ((dg_disconn_ind_t *) location) ; }
	inline dg_disconn_conf_t	*discconf()		{ return ((dg_disconn_conf_t *) location) ; }
	inline dg_disconn_resp_t	*discresp()		{ return ((dg_disconn_resp_t *) location) ; }
	inline dg_conn_abort_ind_t	*connabortind()		{ return ((dg_conn_abort_ind_t *) location) ; }
	inline dg_conn_abort_resp_t	*connabortresp()	{ return ((dg_conn_abort_resp_t *) location) ; }
	inline dg_facility_req_t	*facreq()		{ return ((dg_facility_req_t *) location) ; }
	inline dg_facility_ind_t	*facind()		{ return ((dg_facility_ind_t *) location) ; }
	inline dg_info_req_t		*inforeq()		{ return ((dg_info_req_t *) location) ; }
	inline dg_info_ind_t		*infoind()		{ return ((dg_info_ind_t *) location) ; }
	inline dg_lstn_t		*listen()		{ return ((dg_lstn_t *) location) ; }
	inline dg_progress_req_t	*progreq()		{ return ((dg_progress_req_t *) location) ; }
	inline dg_progress_ind_t	*progind()		{ return ((dg_progress_ind_t *) location) ; }
	inline dg_setup_ack_ind_t	*setupackind()		{ return ((dg_setup_ack_ind_t *) location) ; }
	inline dg_dl_establish_ind_t	*establishind()		{ return ((dg_dl_establish_ind_t *) location) ; }
	inline dg_dl_unestablish_ind_t	*unestablishind()	{ return ((dg_dl_unestablish_ind_t *) location) ; }
	inline xn_ring_t 		*ring()			{ return ((xn_ring_t *) location) ; }
	inline uint32 			command()		{ return (uint32) *((dxb_uint32 *) location) ; }
	inline uint32			call_id()		{ return (uint32) ((dg_msg_t *) location)->call_id ; }
	/* Constructor #1 */
	union_isdn_api(void *ptr) {
		location = ptr ;
	}

	/* Constructor #2 */
	union_isdn_api(uint8 *ptr) {
		location = (void *) ptr ;
	}

	/* Constructor #3 */
	union_isdn_api(char *ptr) {
		location = (void *) ptr ;
	}

	/* Dummy null constructor */
	union_isdn_api() {}

	/* Eventually, we should come up with a way of quickly giving them */
	/* the right size. */
	inline uint32 max_size() {
		return ISDN_MAX_MSG_SIZE ;
	}
} isdn_api_t ;

#define	ISDN_API_MAX_MSG_SIZE	(((isdn_api_t *) 0)->max_size())

#endif /* __cplusplus */

#define ISDN_LSTN_SIZE sizeof(dg_lstn_t)
#define ISDN_CALL_SIZE sizeof(dg_call_t)
#define ISDN_CCCC_SIZE sizeof(dg_cccc_t)

/**********************************************************************
 * start of more misc temporary stuff that eventually will move somewhere else
 */

#endif/*_SIGNAL_PROTO_*/

