#!/usr/bin/wish
#
# Copyright 1999 by Digi International (www.digi.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the 
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

#
# Version 2.43
#

set version 2.43

global	dmlib
set	dmlib "/usr/lib/dg/dgdm"
set	max_num_boards	8

## These help us keep track of what is open: ##
set	DM_GUI_WIN_TOPLEVEL	1
set	DM_GUI_WIN_CONFIG	2
set	DM_GUI_WIN_STATUS	4

## This variable prevents the user from opening config and status: ##
set	dm_gui_open_wins	0

## dm_status globals ##
set	dying	0
set	status_update_delay	1

proc dm_gui_win_open { window_val } {
        global dm_gui_open_wins
	        
	set dm_gui_open_wins [ expr $dm_gui_open_wins | $window_val ]

	return $dm_gui_open_wins
}

proc dm_gui_win_close { window_val } {
	global dm_gui_open_wins
				        
	set dm_gui_open_wins [ expr $dm_gui_open_wins & ~$window_val ]

	return $dm_gui_open_wins
}

proc dm_gui_win_isopen { window_val } {
        global dm_gui_open_wins

	if { [expr $dm_gui_open_wins & $window_val ] } {
		return 1
	} else {
		return 0
	}
}

proc create_main_button_bar {w} {
	global dmlib

	frame $w.title -relief ridge  -borderwidth 5
	label $w.title.label -text "Dense Modem Control Panel" 
	pack $w.title.label -expand 1
	pack $w.title -fill x

	image create photo config_icon \
		-file [file join $dmlib houston-board.gif]
	image create photo status_icon -file [file join $dmlib chart.gif]
	image create photo help_icon -file [file join $dmlib help-book.gif]
	image create photo exit_icon -file [file join $dmlib houston-board.gif]

	frame $w.frame
	tixBalloon $w.frame.balloon

	button $w.frame.config -text "test" -image config_icon \
		-command "create_config_toplevel"
	pack $w.frame.config -side left -fill y
	$w.frame.balloon bind $w.frame.config -balloonmsg "Configure Adapters"

	button $w.frame.status -text "test" -image status_icon \
		-command "create_status_toplevel"
	pack $w.frame.status -side left -fill y
	$w.frame.balloon bind $w.frame.status -balloonmsg "Adapters Status"

	button $w.frame.help -text "Documentation" -image help_icon \
		-command "create_help_toplevel"
	pack $w.frame.help -side left -fill y
	$w.frame.balloon bind $w.frame.help -balloonmsg "Help and Documentation"

	button $w.frame.exit -text "Exit" \
		-command exit
	pack $w.frame.exit -side left -fill y
	$w.frame.balloon bind $w.frame.exit -balloonmsg "Exit Control Panel"

	pack $w.frame -expand 1
}

#
#       The main procedure
#
proc main {w} {

	create_main_button_bar $w

}

#
#	Initialization code
#
if {![info exists tix_demo_running]} {
	wm withdraw .

	set w .dm_control
	toplevel $w
	wm title $w "Digi Dense Modem Control Panel $version"
	wm resizable $w 0 0

	dm_gui_win_open $DM_GUI_WIN_TOPLEVEL

	# Handle the X Destroy event.
	bind $w <Destroy> { 
		exit 0
	}

	lappend auto_path .
	lappend auto_path /usr/lib/tix4.1
	lappend auto_path $dmlib

	package require Tix
	package require dm_gui

	main $w
}
