
#
# Copyright 1999 by Digi International (www.digi.com)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the 
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

#
#	$Id: dm_status.tcl,v 1.16 2001/03/23 22:16:14 jamesp Exp $
#
package provide dm_gui 1.0

#
#	FIXME	FIXME	FIXME	FIXME	FIXME	FIXME	FIXME	FIXME
#
#	WARNING!!!!!  Many globals have to go.  Otherwise,
#	you cannot have both the config and status pages up
#	at the same time.
#
#	FIXME	FIXME	FIXME	FIXME	FIXME	FIXME	FIXME	FIXME
#

proc do_update {w} {
	global nboards
	global ports_n_chans_current_path
	global status_update_done

	for { set bn 0 } { $bn < $nboards } { incr bn } {
		if { [string length $ports_n_chans_current_path($bn)] } {
			update_ports_n_chans $ports_n_chans_current_path($bn)
		}
		update_status_call_monitor $bn
	}

	set status_update_done 1
}

proc status_event_loop {w} {
	global dying
	global status_update_delay
	global status_update_done
	# From dm_osdep.tcl
	global CAN_DO_VWAIT_UPDATE

	update
	set dying 0

	if  { !$CAN_DO_VWAIT_UPDATE } {
		set timer1 0
		while {1} {
			if { [expr $timer1 < 1 ] } {
				set timer1 [ clock seconds ]
			}
			update
			set timer2 [ clock seconds ]
			if { [expr ($timer2 - $timer1) >= $status_update_delay] 
				&& !$dying} {
				    if {[catch {do_update $w} msg]} {
					puts $msg
				    }
				set timer1 0
			}
			after 50
		}
	} else {
		# This update loop is better as it allows the
		# widgets to update more fluidly while tcl is
		# sitting in the vwait; however, not all O/Ss
		# TCL implementation seem to handle vwait
		# correctly.

		while {1} {
			set status_update_done 0
			after [expr $status_update_delay * 1000 ] {
				if { !$dying } {
					catch {do_update $w}
				} else {
					after 50 {
						set status_update_done 1
					}
				}
			}
			vwait status_update_done
		}
	}
}

#
#       Create the notebook page for a single board
#
proc create_status_board {w bn} {
	global  mgmt          #(bn)
	global  nboards
	global	adapter_info
	global	port_count
	global	board_type
	global	board_family

	#
	#	Open the management channel and get the adapter info
	#
	open_mgmt $bn
	if { $mgmt($bn) <= 0 } { return; }

	get_adapter_info $bn
	set port_count($bn) [adapter_info_t_port_count_get $adapter_info($bn) ]

	#
	# Add the board page
	#
	$w add board$bn -label "Adapter $bn" -underline 8

	#
	# Create the Adapter/Ports/Channels notebook
	#
	set f [$w subwidget board$bn]
	tixNoteBook $f.status_nb -ipadx 6 -ipady 6 -height 320
	$f.status_nb subwidget nbframe config -backpagecolor gray

	#
	# Create the pages of the notebook
	#

	create_adapter_info $f.status_nb $bn
	create_status_ports $f.status_nb $bn
	create_status_call_monitor $f.status_nb $bn

	pack $f.status_nb -expand yes -fill both -padx 5 -pady 5 -side top

	incr nboards
}

proc create_status_ports {w bn} {
	global mgmt
	global ports_n_chans_w
	global ports_n_chans_w_frame
	global ports_n_chans_current_path

	#
	# This is the "Ports and Channels" tree and info screens init function
	#

	#
	# Add the stats page to our notebook
	#
	$w add adapter_ports -label "Ports & Channels" -underline 0

	#
	# Create the stats notebook page
	#
	set f [$w subwidget adapter_ports]
	frame $f.tbl

	# Create two panes for the port view

	tixPanedWindow $f.tbl.p -orient horizontal -height 225
	$f.tbl.p add pane1 -size 225 -max 250
	$f.tbl.p add pane2 -size 250 -max 275

	set p1 [$f.tbl.p subwidget pane1]
	set ports_n_chans_w($bn) [set p2 [$f.tbl.p subwidget pane2] ]
	set ports_n_chans_current_path($bn) ""
	set ports_n_chans_w_frame($bn) ""

	# Set up the tree and commands
	create_ports_panes $p1 $p2 $bn

	# Pack out the panes
	pack $f.tbl.p -expand yes -fill both

	#
	# Pack the whole lot out.
	#
	pack $f.tbl -anchor w -expand yes -fill both

}

proc create_mdm_sig_button { w col serial_status sig } {
	# For a modem signal 'sig', a serial_status_t 'serial_status',
	# and a window 'w', it will make a green or red button with a label
	# on column 'col'.

	set SIG [string toupper $sig]
	global SERIAL_SIGNAL_${SIG}
	set sig_id SERIAL_SIGNAL_${SIG}

	label $w.label_${sig} -text $SIG \
	 -font "-*-Courier-bold-r-normal-*-12-*-*-*-*-*-*-*"
	if { [ expr [serial_status_t_mdm_sig_state_var_get \
	    $serial_status [set ${sig_id} ] ] > 1 ]} {
		checkbutton $w.value_${sig} -selectcolor green -state disabled
	} else {
		checkbutton $w.value_${sig} -selectcolor red -state disabled
	}

	grid $w.label_${sig} -column $col -row 0
	grid $w.value_${sig} -column $col -row 1
	$w.value_${sig} select

}

proc update_mdm_sig_button { w col serial_status sig } {
	# For a modem signal 'sig', a serial_status_t 'serial_status',
	# and a window 'w', it will update the button previously defined for
	# that sig.

	set SIG [string toupper $sig]
	global SERIAL_SIGNAL_${SIG}
	set sig_id SERIAL_SIGNAL_${SIG}

	if { [ expr [serial_status_t_mdm_sig_state_var_get \
	    $serial_status [set ${sig_id} ] ] > 1 ]} {
		$w.value_${sig} config -selectcolor green -state disabled
	} else {
		$w.value_${sig} config -selectcolor red -state disabled
	}

}

proc create_ports_n_chans_channel { w bn cn } {
	global mgmt
	global ports_n_chans_w
	global ioc_serial_status
	global serial_status
	global ioc_serial_ext_status
	global serial_ext_status
	global board_family

	# Globals associated with the AccelePort Xp's test mode
	global ioc_serial_channel_test
	global serial_channel_test

	# Ask the mgmt channel for more data
	get_serial_status $bn $cn
	get_serial_ext_status $bn $cn

	#
	# Determine if test_mode is valid (test_mode_valid is 0 or 1),
	# then remember the current test mode (test_mode) if it is valid.
	#
	# Also note: the vertical spacing should be reduced if a test is active.
	#
	set test_mode_valid 0
	set test_mode 0
	set vertical_spacing 10

	if { $board_family($bn) == "ap2k" } {
		get_serial_channel_test $bn $cn
		set test_mode_valid 1
		set test_mode \
		    [serial_channel_test_t_mode_get $serial_channel_test($bn,$cn)]
		set vertical_spacing 1
	}

	tixLabelFrame $w.stats -label "Channel $cn:" -bg lightgray
	set f [$w.stats subwidget frame]

	label $f.label_tx_bytes -text "Tx Bytes:"
	label $f.value_tx_bytes -text \
		[serial_ext_status_t_tx_bytes_get $serial_ext_status($bn,$cn)]
	grid $f.label_tx_bytes $f.value_tx_bytes
	grid $f.label_tx_bytes -sticky e -padx 10
	grid $f.value_tx_bytes -sticky w

	label $f.label_rx_bytes -text "Rx Bytes:"
	label $f.value_rx_bytes -text \
		[serial_ext_status_t_rx_bytes_get $serial_ext_status($bn,$cn)]
	grid $f.label_rx_bytes $f.value_rx_bytes
	grid $f.label_rx_bytes -sticky e -padx 10
	grid $f.value_rx_bytes -sticky w

	grid $w.stats 

	tixLabelFrame $w.status_mdm_sig -label "Modem Signals:" -bg lightgray
	set f [$w.status_mdm_sig subwidget frame]

	## Define lots of buttons colored green for high, red for low.

	set col 0
	foreach sig { rts cts dtr dsr ri dcd } {
		create_mdm_sig_button $f $col $serial_status($bn,$cn) $sig
		incr col	
	}

	grid $w.status_mdm_sig -pady $vertical_spacing 

	if { $board_family($bn) == "dfrp" ||
	     $board_family($bn) == "dfrb" ||
	     $board_family($bn) == "apr" } {
		button $w.reset -text "Reset Modem" \
			-command "reset_modem $bn $cn"
	} else { 
		if { $board_family($bn) == "ap2k" } {
			button $w.reset -text "Reset Port" \
				-command "reset_rs232 $bn $cn"
		}
	}

	grid $w.reset -pady $vertical_spacing

	if { $test_mode_valid == 1 } {
		button $w.loopback -text "--- Loopback Test" \
			-command "manage_loopback_test $bn $cn 2"
		grid $w.loopback

		$w.loopback configure -width 18
		$w.reset configure -width 18

		tixLabelFrame $w.testinfo -label "Test mode unknown" -bg lightgray
		set f [$w.testinfo subwidget frame]

		label $f.label_good -text "Passed:"
		label $f.value_good -text 0
		grid $f.label_good $f.value_good
		grid $f.label_good -sticky e -padx 10
		grid $f.value_good -sticky w

		label $f.label_fail -text "Failed:"
		label $f.value_fail -text 0
		grid $f.label_fail $f.value_fail
		grid $f.label_fail -sticky e -padx 10
		grid $f.value_fail -sticky w

		grid $w.testinfo -pady $vertical_spacing
	}

	delete_get_serial_status_t $ioc_serial_status($bn,$cn)
	delete_get_serial_ext_status_t $ioc_serial_ext_status($bn,$cn)

	if { $test_mode_valid == 1 } {
		delete_getset_serial_channel_test_t $ioc_serial_channel_test($bn,$cn)
	}

	update_ports_n_chans_channel $w $bn $cn
}

proc update_ports_n_chans_channel { w bn cn } {
	global mgmt
	global ioc_serial_status
	global serial_status
	global ioc_serial_ext_status
	global serial_ext_status
	global board_family

	global ioc_serial_channel_test
	global serial_channel_test
	global TEST_MODE
	global EXT_LOOP_AUTO
	global EXT_LOOP_MANUAL
	global ECHO_TEST
	global REPORT_RESULTS
	global STOP_TEST
	global TEST_EXIT

	# Ask the mgmt channel for more data
	get_serial_status $bn $cn
	get_serial_ext_status $bn $cn

	#
	# Determine if test_mode is valid (test_mode_valid is 0 or 1),
	# then remember the current test mode (test_mode) if it is valid.
	#
	set test_mode_valid 0
	set test_mode 0

	if { $board_family($bn) == "ap2k" } {
		get_serial_channel_test $bn $cn
		set test_mode_valid 1
		set test_mode \
		    [serial_channel_test_t_mode_get $serial_channel_test($bn,$cn)]
	}

	set f [$w.status_mdm_sig subwidget frame]

	## Define lots of buttons colored green for high, red for low.

	set col 0
	foreach sig { rts cts dtr dsr ri dcd } {
		update_mdm_sig_button $f $col $serial_status($bn,$cn) $sig
		incr col	
	}

	set f [$w.stats subwidget frame]
	$f.value_tx_bytes config -text \
		[serial_ext_status_t_tx_bytes_get $serial_ext_status($bn,$cn)]
	$f.value_rx_bytes config -text \
		[serial_ext_status_t_rx_bytes_get $serial_ext_status($bn,$cn)]

	if { $test_mode_valid == 1 } {
		#
		# Determine whether we should allow the test to be started or stopped
		#
		if { $test_mode & $TEST_MODE } {
			$w.loopback configure -text "Stop Loopback Test" \
				-command "manage_loopback_test $bn $cn 0"
			$w.reset configure -state disabled
		} else {
			$w.loopback configure -text "Start Loopback Test" \
				-command "manage_loopback_test $bn $cn 1"
			set curr_state [$w.reset cget -state]
			if { $curr_state == "disabled" } {
				$w.reset configure -state normal
			}
		}

		#
		# Determine if testinfo subwidget should be drawn, and if so,
		# make sure it has the valid data in it.
		#
		if { $test_mode & $TEST_MODE } {
			set f [$w.testinfo subwidget frame]

			$w.testinfo configure -label "Test mode enabled"

			if { $test_mode & $EXT_LOOP_AUTO } {
				$w.testinfo configure -label "External loop test progress: "
			}
			if { $test_mode & $EXT_LOOP_MANUAL } {
				$w.testinfo configure -label "Ext. loopack (manual) test..."
			}
			if { $test_mode & $ECHO_TEST } {
				$w.testinfo configure -label "Echo test progress: "
			}

			$f.value_good configure -text \
				[serial_channel_test_t_good_get $serial_channel_test($bn,$cn)]
			$f.value_fail configure -text \
				[serial_channel_test_t_fail_get $serial_channel_test($bn,$cn)]

			grid $w.testinfo
		} else {
			grid remove $w.testinfo
		}
	}

	delete_get_serial_status_t $ioc_serial_status($bn,$cn)
	delete_get_serial_ext_status_t $ioc_serial_ext_status($bn,$cn)
	if { $test_mode_valid == 1 } {
		delete_getset_serial_channel_test_t $ioc_serial_channel_test($bn,$cn)
	}
}

proc create_port_status_dfrp {w bn pn} {
        global ioc_ds1_status
        global ds1_status
        global ioc_ds1_ext_status
        global ds1_ext_status

	# From mgmthlc.h
	global DS1_NO_ALARM
	global DS1_RED_ALARM
	global DS1_BLUE_ALARM
	global DS1_YELLOW_ALARM

	get_ds1_status $bn $pn
	get_ds1_ext_status $bn $pn

	tixLabelFrame $w.status -label "T1/PRI Port $pn:" -bg lightgray
	set f [$w.status subwidget frame]

	label $f.label_alarm -text "Alarm State:"
	switch -exact [ds1_status_t_alarm_get $ds1_status($bn,$pn)] \
		$DS1_NO_ALARM { \
			checkbutton $f.button_alarm -selectcolor green \
				 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
				-state disabled -text "(Ok)" \
				-disabledforeground #000000
				} \
		$DS1_RED_ALARM { \
			checkbutton $f.button_alarm -selectcolor red \
				 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
				-state disabled -text "(Red)" \
				-disabledforeground #000000
				} \
		$DS1_BLUE_ALARM { \
			checkbutton $f.button_alarm -selectcolor blue \
				 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
				-state disabled -text "(Blue)" \
				-disabledforeground #000000
				} \
		$DS1_YELLOW_ALARM { \
			checkbutton $f.button_alarm -selectcolor yellow \
				 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
				-state disabled -text "(Yellow)" \
				-disabledforeground #000000
				} \
		default { \
			checkbutton $f.button_alarm -selectcolor black \
				 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
				-state disabled -text "(Unknown)" \
				-disabledforeground #000000
		}

	grid $f.label_alarm $f.button_alarm 
	grid $f.label_alarm
	grid $f.button_alarm -sticky w -padx 5
		$f.button_alarm select

	label $f.label_linecodevio -text "Line Violations:" \
		-font "-*-Courier-bold-r-normal-*-12-*-*-*-*-*-*-*" 
	label $f.value_linecodevio -text \
		[ds1_ext_status_t_line_code_violations_get $ds1_ext_status($bn,$pn)]

	grid $f.label_linecodevio $f.value_linecodevio
	grid $f.label_linecodevio 
	grid $f.value_linecodevio -sticky w -padx 5

	label $f.label_frameloss -text "Lost Frames:"
	label $f.value_frameloss -text \
		[ds1_ext_status_t_loss_of_frame_count_get $ds1_ext_status($bn,$pn)]

	grid $f.label_frameloss $f.value_frameloss
	grid $f.label_frameloss 
	grid $f.value_frameloss -sticky w -padx 5

	label $f.label_crcerrs -text "CRC Errors:" 
	label $f.value_crcerrs -text \
		[ds1_ext_status_t_crc_error_count_get $ds1_ext_status($bn,$pn)]

	grid $f.label_crcerrs $f.value_crcerrs
	grid $f.label_crcerrs 
	grid $f.value_crcerrs -sticky w -padx 5

	grid $w.status

	delete_get_ds1_status_t $ioc_ds1_status($bn,$pn)
	delete_get_ds1_ext_status_t $ioc_ds1_ext_status($bn,$pn)
}

proc update_port_status_dfrp {w bn pn} {
        global ioc_ds1_status
        global ds1_status
        global ioc_ds1_ext_status
        global ds1_ext_status

	# From mgmthlc.h
	global DS1_NO_ALARM
	global DS1_RED_ALARM
	global DS1_BLUE_ALARM
	global DS1_YELLOW_ALARM

	get_ds1_status $bn $pn
	get_ds1_ext_status $bn $pn

	set f [$w.status subwidget frame]
	switch -exact [ ds1_status_t_alarm_get $ds1_status($bn,$pn)] \
		$DS1_NO_ALARM { \
			$f.button_alarm config -selectcolor green \
				-state disabled -text "(Ok)"
				} \
		$DS1_RED_ALARM { \
			$f.button_alarm config -selectcolor red \
				-state disabled -text "(Red)"
				} \
		$DS1_BLUE_ALARM { \
			$f.button_alarm config -selectcolor blue \
				-state disabled -text "(Blue)"
				} \
		$DS1_YELLOW_ALARM { \
			$f.button_alarm config -selectcolor yellow \
				-state disabled -text "(Yellow)"
				} \
		default { \
			$f.button_alarm config -selectcolor black \
				-state disabled -text "(Unknown)"
		}

	$f.value_linecodevio config -text \
		[ds1_ext_status_t_line_code_violations_get $ds1_ext_status($bn,$pn)]
	$f.value_frameloss config -text \
		[ds1_ext_status_t_loss_of_frame_count_get $ds1_ext_status($bn,$pn)]
	$f.value_crcerrs config -text \
		[ds1_ext_status_t_crc_error_count_get $ds1_ext_status($bn,$pn)]

	delete_get_ds1_status_t $ioc_ds1_status($bn,$pn)
	delete_get_ds1_ext_status_t $ioc_ds1_ext_status($bn,$pn)
}

proc create_port_status_dfrb {w bn pn} {
	global bri_config
	global ioc_bri_config
	global bri_status
	global ioc_bri_status
	global dchan_ext_status
	global ioc_dchan_ext_status

	# From mgmthlc.h
	## Switch Types
	global BRI_NONE
	global BRI_NI1
	global BRI_5ESS
	global BRI_DMS100
	global BRI_ETSI
	global BRI_JAPAN
	## Good activation state
	global BRI_ACT_F7
	## Good TEI state(s)
	global BRI_MAX_TEIS
	global DCHAN_TEI_ASSIGNED
	global DCHAN_MULTIFRAME_EST
	global DCHAN_MULTIFRAME_AWAITING_REL
	## Good SPID state
	global BRI_MAX_SPIDS
	global BRI_SPID_NEGOTIATE_SUCCESS

	get_bri_config $bn $pn
	get_bri_status $bn $pn
	get_dchan_ext_status $bn $pn

	tixLabelFrame $w.status -label "BRI Port Status" -bg lightgray
	set f [$w.status subwidget frame]

	## Layer status varies depending on local, we will use the switch
	## type to determine "where we are."

	set layer1 0
	set layer2 0
	set layer3 0

	set switch_type [bri_config_t_switch_type_get $bri_config($bn,$pn)]
  	label $f.label_switchtype -text "\nSwitch Type:" -justify left \
		-font "-*-Courier-bold-r-normal-*-12-*-*-*-*-*-*-*" 

	label $f.label_layer1 -text "Layer 1: " -justify left
	label $f.label_layer2 -text "Layer 2: " -justify left
	label $f.label_layer3 -text "Layer 3: " -justify left

	   # Layer 1 check
	   if {[bri_status_t_activation_state_get $bri_status($bn,$pn)] == \
	       $BRI_ACT_F7} {
		set layer1 1
	   } else {
		set layer1 0
	   }
	   # Layer 2 check
	   if {$layer1 && [check_all_elements \
	       dchan_ext_status_t_tei_state_var_get $dchan_ext_status($bn,$pn) \
	       $BRI_MAX_TEIS $DCHAN_TEI_ASSIGNED]} {
		set layer2 1
	   } elseif { $layer1 && [check_all_elements \
	       dchan_ext_status_t_tei_state_var_get $dchan_ext_status($bn,$pn) \
	       $BRI_MAX_TEIS $DCHAN_MULTIFRAME_EST]} {
		set layer2 1
	   } elseif { $layer1 && [check_all_elements \
	       dchan_ext_status_t_tei_state_var_get $dchan_ext_status($bn,$pn) \
	       $BRI_MAX_TEIS $DCHAN_MULTIFRAME_AWAITING_REL]} {
		# I am not sure about this one, is this a good state?
		set layer2 1
	   } else {
	   	set layer2 0
           }
	   # Layer 3 check
	   if {$layer2 && [check_all_elements \
	       bri_status_t_spid_state_var_get $bri_status($bn,$pn) \
	       $BRI_MAX_SPIDS $BRI_SPID_NEGOTIATE_SUCCESS]} {
	   	set layer3 1
	   } else {
	   	set layer3 0
	   }

	if {$switch_type == $BRI_ETSI || $switch_type == $BRI_JAPAN} {
		# If layer 1 & 2 are up, and assume 3 is too.
		if {$layer1} {
			checkbutton $f.button_layer1 -selectcolor green \
				-font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
				-state disabled -text "(Up)" \
				-disabledforeground #000000
		} else {
			checkbutton $f.button_layer1 -selectcolor red \
				-font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
				-state disabled -text "(Down)" \
				-disabledforeground #000000
		}
		if {$layer1 && $layer2} {
			checkbutton $f.button_layer2 -selectcolor green \
				-font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
				-state disabled -text "(Up)" \
				-disabledforeground #000000
			checkbutton $f.button_layer3 -selectcolor green \
				-font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
				-state disabled -text "(Up)" \
				-disabledforeground #000000
		} else {
			checkbutton $f.button_layer2 -selectcolor red \
				-font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
				-state disabled -text "(Down)" \
				-disabledforeground #000000
			checkbutton $f.button_layer3 -selectcolor red \
				-font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
				-state disabled -text "(Down)" \
				-disabledforeground #000000
		}
	} elseif { $switch_type == $BRI_NONE } {
		# Show all layers as up.
			checkbutton $f.button_layer1 -selectcolor green \
				-font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
				-state disabled -text "(Down)" \
				-state disabled -text "(Up)" \
				-disabledforeground #000000
			checkbutton $f.button_layer2 -selectcolor green \
				-font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
				-state disabled -text "(Up)" \
				-disabledforeground #000000
			checkbutton $f.button_layer3 -selectcolor green \
				-font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
				-state disabled -text "(Up)" \
				-disabledforeground #000000
	} else {
		# Do "normal" display of Layers 1, 2, & 3 for N.A.
		if {$layer1} {
			checkbutton $f.button_layer1 -selectcolor green \
				-font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
				-state disabled -text "(Up)" \
				-disabledforeground #000000
		} else {
			checkbutton $f.button_layer1 -selectcolor red \
				-font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
				-state disabled -text "(Down)" \
				-disabledforeground #000000
		}
		if {$layer1 && $layer2} {
			checkbutton $f.button_layer2 -selectcolor green \
				-font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
				-state disabled -text "(Up)" \
				-disabledforeground #000000
		} else {
			checkbutton $f.button_layer2 -selectcolor red \
				-font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
				-state disabled -text "(Down)" \
				-disabledforeground #000000
		}
		if {$layer2 && $layer3} {
			checkbutton $f.button_layer3 -selectcolor green \
				-font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
				-state disabled -text "(Up)" \
				-disabledforeground #000000
		} else {
			checkbutton $f.button_layer3 -selectcolor red \
				-font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
				-state disabled -text "(Down)" \
				-disabledforeground #000000
		}
	}

	grid $f.label_layer1 $f.button_layer1
	grid $f.label_layer1
	grid $f.button_layer1 -sticky w -padx 5
		$f.button_layer1 select
	grid $f.label_layer2 $f.button_layer2
	grid $f.label_layer2
	grid $f.button_layer2 -sticky w -padx 5
		$f.button_layer2 select
	grid $f.label_layer3 $f.button_layer3
	grid $f.label_layer3
	grid $f.button_layer3 -sticky w -padx 5
		$f.button_layer3 select

	switch -exact $switch_type \
		$BRI_NONE { \
  		  label $f.value_switchtype \
			-font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
			-text "\nNone"
		} \
		$BRI_NI1 { \
  		  label $f.value_switchtype \
			-font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
			-text "\nNational ISDN 1"
		} \
		$BRI_5ESS { \
  		  label $f.value_switchtype \
			-font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
			-text "\nAT&T 5ESS"
		} \
		$BRI_DMS100 { \
  		  label $f.value_switchtype \
			-font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
			-text "\nDMS100"
		} \
		$BRI_ETSI { \
  		  label $f.value_switchtype \
			-font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
			-text "\nETSI"
		} \
		$BRI_JAPAN { \
  		  label $f.value_switchtype \
			-font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
			-text "\nINS-64"
		} \
		default { \
		  label $f.value_switchtype \
			-font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
			-text "\n(Not Set)"
		}

	grid $f.label_switchtype $f.value_switchtype
	grid $f.label_switchtype
	grid $f.value_switchtype -sticky w -padx 5

	label $f.label_number0 \
		-font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
		-text "Number 0:"
	label $f.value_number0 \
		-font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" -text \
		[bri_config_t_directory_var_get $bri_config($bn,$pn) "number" 0]
	label $f.label_number1 \
		-font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
		-text "Number 1:"
	label $f.value_number1 \
		-font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" -text \
		[bri_config_t_directory_var_get $bri_config($bn,$pn) "number" 1]

	grid $f.label_number0 $f.value_number0
	grid $f.label_number0
	grid $f.value_number0 -sticky w -padx 5
	grid $f.label_number1 $f.value_number1
	grid $f.label_number1
	grid $f.value_number1 -sticky w -padx 5

	if {$switch_type != $BRI_ETSI && $switch_type != $BRI_JAPAN && \
	    $switch_type != $BRI_NONE && $switch_type != 0} {
		label $f.label_spid0 \
			-font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
			-text "SPID 0:"
		label $f.value_spid0 \
			-font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" -text \
			[bri_config_t_directory_var_get $bri_config($bn,$pn) "spid" 0]
		label $f.label_spid1 \
			-font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
			-text "SPID 1:"
		label $f.value_spid1 \
			-font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" -text \
			[bri_config_t_directory_var_get $bri_config($bn,$pn) "spid" 1]
		
		grid $f.label_spid0 $f.value_spid0
		grid $f.label_spid0
		grid $f.value_spid0 -sticky w -padx 5
		grid $f.label_spid1 $f.value_spid1
		grid $f.label_spid1
		grid $f.value_spid1 -sticky w -padx 5
	}

	grid $w.status

	delete_getset_bri_config_t $ioc_bri_config($bn,$pn)
	delete_get_bri_status_t $ioc_bri_status($bn,$pn)
	delete_get_dchan_ext_status_t $ioc_dchan_ext_status($bn,$pn)
}

proc update_port_status_dfrb {w bn pn} {
	global bri_config
	global ioc_bri_config
	global bri_status
	global ioc_bri_status
	global dchan_ext_status
	global ioc_dchan_ext_status

	# From mgmthlc.h
	## Switch Types
	global BRI_NONE
	global BRI_NI1
	global BRI_5ESS
	global BRI_DMS100
	global BRI_ETSI
	global BRI_JAPAN
	## Good activation state
	global BRI_ACT_F7
	## Good TEI state(s)
	global BRI_MAX_TEIS
	global DCHAN_TEI_ASSIGNED
	global DCHAN_MULTIFRAME_EST
	global DCHAN_MULTIFRAME_AWAITING_REL
	## Good SPID state
	global BRI_MAX_SPIDS
	global BRI_SPID_NEGOTIATE_SUCCESS

	get_bri_config $bn $pn
	get_bri_status $bn $pn
	get_dchan_ext_status $bn $pn

	set f [$w.status subwidget frame]

	## Layer status varies depending on local, we will use the switch
	## type to determine "where we are."

	set layer1 0
	set layer2 0
	set layer3 0

	set switch_type [bri_config_t_switch_type_get $bri_config($bn,$pn)]

	   # Layer 1 check
	   if {[bri_status_t_activation_state_get $bri_status($bn,$pn)] == \
	       $BRI_ACT_F7} {
		set layer1 1
	   } else {
		set layer1 0
	   }
	   # Layer 2 check
	   if {$layer1 && [check_all_elements \
	       dchan_ext_status_t_tei_state_var_get $dchan_ext_status($bn,$pn) \
	       $BRI_MAX_TEIS $DCHAN_TEI_ASSIGNED]} {
		set layer2 1
	   } elseif { $layer1 && [check_all_elements \
	       dchan_ext_status_t_tei_state_var_get $dchan_ext_status($bn,$pn) \
	       $BRI_MAX_TEIS $DCHAN_MULTIFRAME_EST]} {
		set layer2 1
	   } elseif { $layer1 && [check_all_elements \
	       dchan_ext_status_t_tei_state_var_get $dchan_ext_status($bn,$pn) \
	       $BRI_MAX_TEIS $DCHAN_MULTIFRAME_AWAITING_REL]} {
		# I am not sure about this one, is this a good state?
		set layer2 1
	   } else {
	   	set layer2 0
           }
	   # Layer 3 check
	   if {$layer2 && [check_all_elements \
	       bri_status_t_spid_state_var_get $bri_status($bn,$pn) \
	       $BRI_MAX_SPIDS $BRI_SPID_NEGOTIATE_SUCCESS]} {
	   	set layer3 1
	   } else {
	   	set layer3 0
	   }

	if {$switch_type == $BRI_ETSI || $switch_type == $BRI_JAPAN} {
		# If layer 1 & 2 are up, and assume 3 is too.
		if {$layer1} {
			$f.button_layer1 config -selectcolor green \
				-state disabled -text "(Up)"
		} else {
			$f.button_layer1 config -selectcolor red \
				-state disabled -text "(Down)"
		}
		if {$layer1 && $layer2} {
			$f.button_layer2 config -selectcolor green \
				-state disabled -text "(Up)"
			$f.button_layer3 config -selectcolor green \
				-state disabled -text "(Up)"
		} else {
			$f.button_layer2 config -selectcolor red \
				-state disabled -text "(Down)"
			$f.button_layer3 config -selectcolor red \
				-state disabled -text "(Down)"
		}
	} elseif { $switch_type == $BRI_NONE } {
		# Show all layers as up.
			$f.button_layer1 config -selectcolor green \
				-state disabled -text "(Up)"
			$f.button_layer2 config -selectcolor green \
				-state disabled -text "(Up)"
			$f.button_layer3 config -selectcolor green \
				-state disabled -text "(Up)"
	} else {
		# Do "normal" display of Layers 1, 2, & 3 for N.A.
		if {$layer1} {
			$f.button_layer1 config -selectcolor green \
				-state disabled -text "(Up)"
		} else {
			$f.button_layer1 config -selectcolor red \
				-state disabled -text "(Down)"
		}
		if {$layer1 && $layer2} {
			$f.button_layer2 config -selectcolor green \
				-state disabled -text "(Up)"
		} else {
			$f.button_layer2 config -selectcolor red \
				-state disabled -text "(Down)"
		}
		if {$layer2 && $layer3} {
			$f.button_layer3 config -selectcolor green \
				-state disabled -text "(Up)"
		} else {
			$f.button_layer3 config -selectcolor red \
				-state disabled -text "(Down)"
		}
	}

	switch -exact $switch_type \
		$BRI_NONE { \
  		  $f.value_switchtype config \
			-text "\nNone"
		} \
		$BRI_NI1 { \
  		  $f.value_switchtype config \
			-text "\nNational ISDN 1"
		} \
		$BRI_5ESS { \
  		  $f.value_switchtype config \
			-text "\nAT&T 5ESS"
		} \
		$BRI_DMS100 { \
  		  $f.value_switchtype config \
			-text "\nDMS100"
		} \
		$BRI_ETSI { \
  		  $f.value_switchtype config \
			 -text "\nETSI"
		} \
		$BRI_JAPAN { \
  		  $f.value_switchtype config \
			-text "\nINS-64"
		} \
		default { \
		  $f.value_switchtype config \
			-text "\n(Not Set)"
		}

	$f.value_number0 config -text \
		[bri_config_t_directory_var_get $bri_config($bn,$pn) "number" 0]
	$f.value_number1 config -text \
		[bri_config_t_directory_var_get $bri_config($bn,$pn) "number" 1]

	if {$switch_type != $BRI_ETSI && $switch_type != $BRI_JAPAN && \
	    $switch_type != $BRI_NONE && $switch_type != 0} {
		$f.value_spid0 config -text \
			[bri_config_t_directory_var_get $bri_config($bn,$pn) "spid" 0]
		$f.value_spid1 config -text \
			[bri_config_t_directory_var_get $bri_config($bn,$pn) "spid" 1]
	}

	delete_getset_bri_config_t $ioc_bri_config($bn,$pn)
	delete_get_bri_status_t $ioc_bri_status($bn,$pn)
	delete_get_dchan_ext_status_t $ioc_dchan_ext_status($bn,$pn)

}

proc create_port_status_apr {w bn pn} {
	global ioc_serial_config
	global serial_config

	global SERIAL_PORTTYPE_RS232
	global SERIAL_PORTTYPE_RS422
	global SERIAL_PORTTYPE_RS423
	global SERIAL_PORTTYPE_V35

	global SERIAL_CLOCKSOURCE_INT
	global SERIAL_CLOCKSOURCE_EXT
	global SERIAL_CLOCKSOURCE_SPLIT

	global SERIAL_ALTPIN_ON
	global SERIAL_ALTPIN_OFF

	global board_type

	tixLabelFrame $w.status -label "Port: $pn" -bg lightgray
	set f [$w.status subwidget frame]

	set pn [expr $pn - 1]
	get_serial_config $bn $pn

	label $f.label_type -text "Port Type:"

	switch -exact [$serial_config($bn,$pn) cget -port_type] \
	$SERIAL_PORTTYPE_RS232 { \
		label $f.value_type -text "RS232" 
	} \
	$SERIAL_PORTTYPE_RS422 { \
		label $f.value_type -text "RS422"
	} \
	$SERIAL_PORTTYPE_RS423 { \
		label $f.value_type -text "RS423"
	} \
	$SERIAL_PORTTYPE_V35 { \
		label $f.value_type -text "V35"
	} \
	default { \
		label $f.value_type -text "Other"
	}

	grid $f.label_type $f.value_type
	grid $f.label_type -sticky ne -padx 10
	grid $f.value_type -sticky w
	
	label $f.label_clock -text "Clock Source:"
	switch -exact [$serial_config($bn,$pn) cget -clock_source] \
	$SERIAL_CLOCKSOURCE_INT { \
		label $f.value_clock -text "Internal"
	} \
	$SERIAL_CLOCKSOURCE_EXT { \
		label $f.value_clock -text "External"
	} \
	$SERIAL_CLOCKSOURCE_SPLIT { \
		label $f.value_clock -text "Split"
	} 

	grid $f.label_clock $f.value_clock
	grid $f.label_clock -sticky e -padx 10
	grid $f.value_clock -sticky w 

	label $f.label_altpin -text "Altpin Status:"
	switch -exact [$serial_config($bn,$pn) cget -alt_pin] \
	$SERIAL_ALTPIN_ON { \
		checkbutton $f.button_altpin -selectcolor green \
			-state disabled -text "(On)" \
			-disabledforeground #000000
	} \
	$SERIAL_ALTPIN_OFF { \
		checkbutton $f.button_altpin -selectcolor red \
			-state disabled -text "(Off)" \
			-disabledforeground #000000
	} \
	default { \
		checkbutton $f.button_altpin -selectcolor red \
			-state disabled -text "(Off)" \
			-disabledforeground #000000
	}

	grid $f.label_altpin $f.button_altpin
	grid $f.button_altpin -sticky w -padx 5
	$f.button_altpin select

	grid $w.status 

	rename $serial_config($bn,$pn) ""
	rename $ioc_serial_config($bn,$pn) ""
}

proc update_port_status_apr {w bn pn} {
	global ioc_serial_config
	global serial_config

	global SERIAL_PORTTYPE_RS232
	global SERIAL_PORTTYPE_RS422
	global SERIAL_PORTTYPE_RS423
	global SERIAL_PORTTYPE_V35

	global SERIAL_CLOCKSOURCE_INT
	global SERIAL_CLOCKSOURCE_EXT
	global SERIAL_CLOCKSOURCE_SPLIT

	global SERIAL_ALTPIN_ON
	global SERIAL_ALTPIN_OFF

	global board_type
	

	set f [$w.status subwidget frame]

	set pn [expr $pn - 1]
	get_serial_config $bn $pn

	switch -exact [$serial_config($bn,$pn) cget -port_type] \
	$SERIAL_PORTTYPE_RS232 { \
		$f.value_type config -text "RS232" 
	} \
	$SERIAL_PORTTYPE_RS422 { \
		$f.value_type config -text "RS422"
	} \
	$SERIAL_PORTTYPE_RS423 { \
		$f.value_type config -text "RS423"
	} \
	$SERIAL_PORTTYPE_V35 { \
		$f.value_type config -text "V35" 
	} \
	default { \
		$f.value_type config -text "Other"
	}

	switch -exact [$serial_config($bn,$pn) cget -clock_source] \
	$SERIAL_CLOCKSOURCE_INT { \
		$f.value_clock config -text "Internal"
	} \
	$SERIAL_CLOCKSOURCE_EXT { \
		$f.value_clock config -text "External"
	} \
	$SERIAL_CLOCKSOURCE_SPLIT { \
		$f.value_clock config -text "Split"
	} 

	switch -exact [$serial_config($bn,$pn) cget -alt_pin] \
	$SERIAL_ALTPIN_ON { \
		$f.button_altpin config -selectcolor green \
			-state disabled -text "(On)" \
			-disabledforeground #000000
	} \
	$SERIAL_ALTPIN_OFF { \
		$f.button_altpin config -selectcolor red \
			-state disabled -text "(Off)" \
			-disabledforeground #000000
	} 

	rename $serial_config($bn,$pn) ""
	rename $ioc_serial_config($bn,$pn) ""
}

proc create_ports_n_chans_port { w bn pn } {
	## This is filled by an earlier call to create_adapter_info
	global  board_family

	switch -exact $board_family($bn) \
		"dfrp" { \
			create_port_status_dfrp $w $bn $pn } \
		"dfrb" { \
			create_port_status_dfrb $w $bn $pn } \
		"apr"  { \
			create_port_status_apr $w $bn $pn } \
		"ap2k" { \
			create_port_status_apr $w $bn $pn }
}

proc update_ports_n_chans_port { w bn pn } {
	## This is filled by an earlier call to create_adapter_info
	global  board_family

	switch -exact $board_family($bn) \
		"dfrp" { \
			update_port_status_dfrp $w $bn $pn } \
		"dfrb" { \
			update_port_status_dfrb $w $bn $pn } \
		"apr"  { \
			update_port_status_apr $w $bn $pn } \
		"ap2k" { \
			update_port_status_apr $w $bn $pn }

}

proc create_ports_n_chans_board {w bn} {
	global adapter_state
	global ioc_adapter_state

	#
	# Given a list of labels and board-level vars, create labels on w
	#
	# Used on "Adapter Statistics" tab
	#

	get_adapter_state $bn

	tixLabelFrame $w.stats -label "Adapter $bn:" -bg lightgray
	set f [$w.stats subwidget frame]

	label $f.label_cpu_usage -text "\nCpu Utilization:" \
	  -font "-*-Courier-bold-r-normal-*-12-*-*-*-*-*-*-*"
	tixMeter $f.value_cpu_usage -fillcolor #800030 -value \
	  [ expr [adapter_state_t_cpu_usage_get $adapter_state($bn)] * .01 ]
	grid $f.label_cpu_usage -sticky w -padx 10
	grid $f.value_cpu_usage -sticky w -padx 20

	label $f.label_memory_usage -text "\nMemory Utilization:" \
	  -font "-*-Courier-bold-r-normal-*-12-*-*-*-*-*-*-*"
	tixMeter $f.value_memory_usage -fillcolor #108000 -value \
	  [ expr [adapter_state_t_memory_usage_get $adapter_state($bn)] * .01 ]
	grid $f.label_memory_usage -sticky w -padx 10
	grid $f.value_memory_usage -sticky w -padx 20

	#
	# Nicely format the adapter uptime
	#
	set a_uptime \
	  [adapter_state_t_uptime_var_get $adapter_state($bn) "seconds" ]
	set uptime_str [ uptimeString $a_uptime ]

	label $f.label_uptime -text "\nAdapter Uptime:" \
	  -font "-*-Courier-bold-r-normal-*-12-*-*-*-*-*-*-*"
	label $f.value_uptime -text $uptime_str \
	  -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	grid $f.label_uptime -sticky w -padx 10
	grid $f.value_uptime -sticky w -padx 20

	grid $w.stats

	delete_get_adapter_state_t $ioc_adapter_state($bn)
}

proc update_ports_n_chans_board {w bn} {
	global adapter_state
	global ioc_adapter_state

	get_adapter_state $bn

	set f [$w.stats subwidget frame]

	$f.value_cpu_usage config -value \
	  [ expr [adapter_state_t_cpu_usage_get $adapter_state($bn)] * .01 ]
	$f.value_memory_usage config -value \
	  [ expr [adapter_state_t_memory_usage_get $adapter_state($bn)] * .01 ]

	# Set adapter uptime
	set a_uptime \
	  [adapter_state_t_uptime_var_get $adapter_state($bn) "seconds" ]
	set uptime_str [ uptimeString $a_uptime ]
	$f.value_uptime config -text $uptime_str

	delete_get_adapter_state_t $ioc_adapter_state($bn)
}

proc create_ports_n_chans { path } {
	global dying
	global ports_n_chans_w
	global ports_n_chans_w_frame
	global ports_n_chans_current_path

	#
	# This function discovers what type of item is selected (board,
	# port, or channel), destroys an open ports_n_chans window
	# (if any), and calls the proper display function to draw on
	# the ports_n_chans_w.
	#

	# Grab the board number from the path
	regexp -- "board(\[0-9]*).*" \
		$path path bn

	# This "de-bounces" our TixTree	
	if { $ports_n_chans_current_path($bn) == $path } { return; }

	if { [string length $ports_n_chans_w_frame($bn)] } {
		# Manipulate the dying flag to stop us from killing ourselves
		set dying 1
		destroy $ports_n_chans_w_frame($bn)
		set dying 0
	}

	set w $ports_n_chans_w($bn)
	set f [frame $w.frame -bg lightgray]
	set ports_n_chans_w_frame($bn) $f

	set ports_n_chans_current_path($bn) $path

	switch -regexp -- $path {
                "board\[0-9]*.port\[0-9]*.channel\[0-9]*" {
                        regexp -- "board(\[0-9]*).port\[0-9]*.channel(\[0-9]*)"\
                                $path path board channel
			create_ports_n_chans_channel $f $board $channel
                }
                "board\[0-9]*.port\[0-9]*" {
                        regexp -- "board(\[0-9]*).port(\[0-9]*)" \
                                $path path board port
			create_ports_n_chans_port $f $board $port
                }
                "board\[0-9]*" {
                        regexp -- "board(\[0-9]*)" \
                                $path path board
			create_ports_n_chans_board $f $board 
                }
        }

	pack $f -expand yes -fill both
}

proc update_ports_n_chans { path } {
	global ports_n_chans_w
	global ports_n_chans_w_frame
	global ports_n_chans_current_path

	#
	# This function discovers what type of item is selected (board,
	# port, or channel), and updates the statistics on an assumed
	# drawn ports_n_chans_w.
	#

	# Grab the board number from the path
	regexp -- "board(\[0-9]*).*" \
		$path path bn

	set f $ports_n_chans_w_frame($bn)

	switch -regexp -- $path {
                "board\[0-9]*.port\[0-9]*.channel\[0-9]*" {
                        regexp -- "board(\[0-9]*).port\[0-9]*.channel(\[0-9]*)"\
                                $path path board channel
			update_ports_n_chans_channel $f $board $channel
                }
                "board\[0-9]*.port\[0-9]*" {
                        regexp -- "board(\[0-9]*).port(\[0-9]*)" \
                                $path path board port
			update_ports_n_chans_port $f $board $port
                }
                "board\[0-9]*" {
                        regexp -- "board(\[0-9]*)" \
                                $path path board
			update_ports_n_chans_board $f $board 
                }
        }
}

proc create_ports_panes {l_pane r_pane bn} {
	#
	# Create the ports & channels tree
	#

	global dmlib
	global mgmt
	global adapter_info
	# adapter_cn from create_adapter_info{}
	global adapter_cn
	global ports_n_chans_current_path
	global board_family

	tixTree $l_pane.tree -browsecmd create_ports_n_chans -options {
		hlist.itemType		imagetext
		hlist.drawBranch 	True
		hlist.indent		14
		hlist.selectmode	single
	}
	pack $l_pane.tree -expand yes -fill both

	# Get info for generating tree:
	set ports [adapter_info_t_port_count_get $adapter_info($bn)]
	set c_p_p [adapter_info_t_channels_per_port_get $adapter_info($bn)]

	# Build icons for the tree
	image create photo adapter_icon -file \
		[file join $dmlib "tree-adapter.gif" ]
	image create photo port_icon -file \
		[file join $dmlib "tree-port.gif" ]
	image create photo channel_icon -file \
		[file join $dmlib "tree-chan.gif" ]

	# Start the root of our tree
	set ports_tree [$l_pane.tree subwidget hlist]
	$ports_tree add board${bn} -image adapter_icon -text $adapter_cn
	# Add all of the ports
	set channel 0
	for { set port 1 } { $port <= $ports } { incr port } {
		$ports_tree add board${bn}.port${port} \
			-image port_icon -text "Port $port"
		for { set c_i 0 } { $c_i < $c_p_p } { incr c_i } {
			$ports_tree add board${bn}.port${port}.channel${channel} \
				-image channel_icon \
				-text "[get_device_name $board_family($bn) $bn $port $channel]"
			incr channel
		}
	}

	$l_pane.tree autosetmode

	#
	# Close all ports nodes to make it look better
	#
	for { set port 1 } { $port <= $ports } { incr port } {
		$l_pane.tree close board${bn}.port${port}
	}

	#
	# Select the adapter-view by default.
	#
	$ports_tree selection set board${bn}
	create_ports_n_chans "board${bn}"
}

#
# Creates a key for what the button colors mean on the first pane
# of the call monitor.
#
proc create_call_monitor_key { w } {
	tixLabelFrame $w.key -label "Call Monitor Key:" -bg lightgray
	set f [$w.key subwidget frame ]

	checkbutton $f.button_idle -selectcolor "lightgray" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
	 -state disabled 
	
	label $f.label_idle -text " = Channel Idle" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	
	grid $f.button_idle $f.label_idle
	grid $f.button_idle -sticky e
	grid $f.label_idle -sticky w

	$f.button_idle select

	checkbutton $f.button_modem -selectcolor "blue" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
	 -state disabled
	
	label $f.label_modem -text " = Modem Call" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" 

	grid $f.button_modem $f.label_modem
	grid $f.button_modem -sticky e
	grid $f.label_modem -sticky w

	$f.button_modem select

	checkbutton $f.button_bchan -selectcolor "green" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*" \
	 -state disabled
	
	label $f.label_bchan -text " = ISDN Call" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"

	grid $f.button_bchan $f.label_bchan
	grid $f.button_bchan -sticky e
	grid $f.label_bchan -sticky w

	$f.button_bchan select

	grid $w.key
}

proc create_call_monitor_details_bchan { w bn pn cn } {
	global adapter_info
	global ioc_adapter_info
	global bchan_status
	global ioc_bchan_status

	# This ioctl requires the total channel number (i.e. 31), rather
	# than the port-level channel number (i.e. 8).
	
	set c_p_p [adapter_info_t_channels_per_port_get $adapter_info($bn)]
	set cn [total_channel_number $pn $cn $c_p_p]

	get_bchan_status $bn $cn

	tixLabelFrame $w.tlf -label "ISDN Call on Channel $cn:" -bg lightgray
	set f [$w.tlf subwidget frame ]

	label $f.label_mode -text "Call Direction:" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	label $f.value_mode -text \
	 [ bchan_mode2text [bchan_status_t_mode_get $bchan_status($bn,$cn)] ] \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	grid $f.label_mode $f.value_mode
	grid $f.label_mode -sticky e -padx 10
	grid $f.value_mode -sticky w

	label $f.label_connect_time -text "Connect Time:" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	label $f.value_connect_time -text \
	 [connectTimeString [bchan_status_t_connect_time_get $bchan_status($bn,$cn)]] \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	grid $f.label_connect_time $f.value_connect_time
	grid $f.label_connect_time -sticky e -padx 10
	grid $f.value_connect_time -sticky w

	label $f.label_channel_rate -text "Channel Rate:" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	label $f.value_channel_rate -text \
	 [bchan_status_t_channel_rate_get $bchan_status($bn,$cn)] \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	grid $f.label_channel_rate $f.value_channel_rate
	grid $f.label_channel_rate -sticky e -padx 10
	grid $f.value_channel_rate -sticky w

	label $f.label_dte_tx_rate -text "Tx Throughput:" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	label $f.value_dte_tx_rate -text \
	 "[bchan_status_t_dte_tx_rate_get $bchan_status($bn,$cn)] cps" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	grid $f.label_dte_tx_rate $f.value_dte_tx_rate
	grid $f.label_dte_tx_rate -sticky e -padx 10
	grid $f.value_dte_tx_rate -sticky w

	label $f.label_dte_rx_rate -text "Rx Throughput:" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	label $f.value_dte_rx_rate -text \
	 "[bchan_status_t_dte_rx_rate_get $bchan_status($bn,$cn)] cps" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	grid $f.label_dte_rx_rate $f.value_dte_rx_rate
	grid $f.label_dte_rx_rate -sticky e -padx 10
	grid $f.value_dte_rx_rate -sticky w

	label $f.label_compression -text "Compression:" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	label $f.value_compression -text \
	 [bchan_compression2text [bchan_status_t_compression_get $bchan_status($bn,$cn)] ] \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	grid $f.label_compression $f.value_compression
	grid $f.label_compression -sticky e -padx 10
	grid $f.value_compression -sticky w

	label $f.label_local_dn -text "Local Number:" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	set text [any2str [bchan_status_t_local_dn_get $bchan_status($bn,$cn)] ]
	if {![string length $text]} {
		set text "Unavailable"
	}
	label $f.value_local_dn -text "$text" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	grid $f.label_local_dn $f.value_local_dn
	grid $f.label_local_dn -sticky e -padx 10
	grid $f.value_local_dn -sticky w

	label $f.label_remote_dn -text "Remote Number:" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	set text [any2str [bchan_status_t_remote_dn_get $bchan_status($bn,$cn)] ]
	if {![string length $text]} {
		set text "Unavailable"
	}
	label $f.value_remote_dn -text "$text" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	grid $f.label_remote_dn $f.value_remote_dn
	grid $f.label_remote_dn -sticky e -padx 10
	grid $f.value_remote_dn -sticky w

	grid $w.tlf

	create_call_monitor_key $w

	delete_get_bchan_status_t $ioc_bchan_status($bn,$cn)
}

proc update_call_monitor_details_bchan { w bn pn cn } { 
	global adapter_info
	global ioc_adapter_info
	global bchan_status
	global ioc_bchan_status

	# This ioctl requires the total channel number (i.e. 31), rather
	# than the port-level channel number (i.e. 8).
	
	set c_p_p [adapter_info_t_channels_per_port_get $adapter_info($bn)]
	set cn [total_channel_number $pn $cn $c_p_p]

	get_bchan_status $bn $cn

	set f "$w.tlf.border.frame"

	$f.value_mode configure -text \
	 [ bchan_mode2text [bchan_status_t_mode_get $bchan_status($bn,$cn)] ] 

	$f.value_connect_time configure -text \
	 [connectTimeString [bchan_status_t_connect_time_get $bchan_status($bn,$cn)]]

	$f.value_channel_rate configure -text \
	 [bchan_status_t_channel_rate_get $bchan_status($bn,$cn)] 

	$f.value_dte_tx_rate configure -text \
	 "[bchan_status_t_dte_tx_rate_get $bchan_status($bn,$cn)] cps" 

	$f.value_dte_rx_rate configure -text \
	 "[bchan_status_t_dte_rx_rate_get $bchan_status($bn,$cn)] cps" 

	$f.value_compression configure -text \
	 [bchan_compression2text [bchan_status_t_compression_get $bchan_status($bn,$cn)] ]

	set text [any2str [bchan_status_t_local_dn_get $bchan_status($bn,$cn)] ]
	if {![string length $text]} {
		set text "Unavailable"
	}
	$f.value_local_dn configure -text "$text"

	set text [any2str [bchan_status_t_remote_dn_get $bchan_status($bn,$cn)] ]
	if {![string length $text]} {
		set text "Unavailable"
	}
	$f.value_remote_dn configure -text "$text" 
}

proc create_call_monitor_details_modem { w bn pn cn } {
	global adapter_info
	global ioc_adapter_info
	global modem_status
	global ioc_modem_status

	# This ioctl requires the total channel number (i.e. 31), rather
	# than the port-level channel number (i.e. 8).
	
	set c_p_p [adapter_info_t_channels_per_port_get $adapter_info($bn)]
	set cn [total_channel_number $pn $cn $c_p_p]
	get_modem_status $bn $cn

	tixLabelFrame $w.tlf -label "Modem Call on Channel $cn:" -bg lightgray
	set f [$w.tlf subwidget frame ]

	label $f.label_mode -text "Call Direction:" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	label $f.value_mode -text \
	 [ modem_mode2text [modem_status_t_mode_get $modem_status($bn,$cn)] ] \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	grid $f.label_mode $f.value_mode
	grid $f.label_mode -sticky e -padx 10
	grid $f.value_mode -sticky w

	label $f.label_connect_time -text "Connect Time:" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	label $f.value_connect_time -text \
	 [connectTimeString [modem_status_t_connect_time_get $modem_status($bn,$cn)]] \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	grid $f.label_connect_time $f.value_connect_time
	grid $f.label_connect_time -sticky e -padx 10
	grid $f.value_connect_time -sticky w

	label $f.label_current_tx_rate -text "Tx Line Rate:" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	label $f.value_current_tx_rate -text \
	 [modem_status_t_current_tx_rate_get $modem_status($bn,$cn)] \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	grid $f.label_current_tx_rate $f.value_current_tx_rate
	grid $f.label_current_tx_rate -sticky e -padx 10
	grid $f.value_current_tx_rate -sticky w

	label $f.label_current_rx_rate -text "Rx Line Rate:" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	label $f.value_current_rx_rate -text \
	 [modem_status_t_current_rx_rate_get $modem_status($bn,$cn)] \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	grid $f.label_current_rx_rate $f.value_current_rx_rate
	grid $f.label_current_rx_rate -sticky e -padx 10
	grid $f.value_current_rx_rate -sticky w

	label $f.label_dte_tx_rate -text "Tx Throughput:" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	label $f.value_dte_tx_rate -text \
	 "[modem_status_t_dte_tx_rate_get $modem_status($bn,$cn)] cps" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	grid $f.label_dte_tx_rate $f.value_dte_tx_rate
	grid $f.label_dte_tx_rate -sticky e -padx 10
	grid $f.value_dte_tx_rate -sticky w

	label $f.label_dte_rx_rate -text "Rx Throughput:" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	label $f.value_dte_rx_rate -text \
	 "[modem_status_t_dte_rx_rate_get $modem_status($bn,$cn)] cps" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	grid $f.label_dte_rx_rate $f.value_dte_rx_rate
	grid $f.label_dte_rx_rate -sticky e -padx 10
	grid $f.value_dte_rx_rate -sticky w

	label $f.label_modulation -text "Modulation:" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	label $f.value_modulation -text \
	 [modem_modulation2text [modem_status_t_modulation_get $modem_status($bn,$cn)] ] \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	grid $f.label_modulation $f.value_modulation
	grid $f.label_modulation -sticky e -padx 10
	grid $f.value_modulation -sticky w

	label $f.label_compression -text "Compression:" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	label $f.value_compression -text \
	 [modem_compression2text [modem_status_t_compression_get $modem_status($bn,$cn)] ] \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	grid $f.label_compression $f.value_compression
	grid $f.label_compression -sticky e -padx 10
	grid $f.value_compression -sticky w

	label $f.label_error_correction -text "Error Correction:" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	label $f.value_error_correction -text \
	 [modem_error_correction2text [modem_status_t_error_correction_get $modem_status($bn,$cn)] ] \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	grid $f.label_error_correction $f.value_error_correction
	grid $f.label_error_correction -sticky e -padx 10
	grid $f.value_error_correction -sticky w

	label $f.label_clid -text "Caller ID:" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	set text [any2str [modem_status_t_clid_get $modem_status($bn,$cn)] ]
	if {![string length $text]} {
		set text "Unavailable"
	}
	label $f.value_clid -text "$text" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	grid $f.label_clid $f.value_clid
	grid $f.label_clid -sticky e -padx 10
	grid $f.value_clid -sticky w

	label $f.label_local_dn -text "Local Number:" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	set text [any2str [modem_status_t_local_dn_get $modem_status($bn,$cn)] ]
	if {![string length $text]} {
		set text "Unavailable"
	}
	label $f.value_local_dn -text "$text" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	grid $f.label_local_dn $f.value_local_dn
	grid $f.label_local_dn -sticky e -padx 10
	grid $f.value_local_dn -sticky w

	label $f.label_remote_dn -text "Remote Number:" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	set text [any2str [modem_status_t_remote_dn_get $modem_status($bn,$cn)] ]
	if {![string length $text]} {
		set text "Unavailable"
	}
	label $f.value_remote_dn -text "$text" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	grid $f.label_remote_dn $f.value_remote_dn
	grid $f.label_remote_dn -sticky e -padx 10
	grid $f.value_remote_dn -sticky w

	grid $w.tlf

	create_call_monitor_key $w

	delete_get_modem_status_t $ioc_modem_status($bn,$cn)
}

proc update_call_monitor_details_modem { w bn pn cn } {
	global adapter_info
	global ioc_adapter_info
	global modem_status
	global ioc_modem_status
	global call_status
	global ioc_call_status

	# This ioctl requires the total channel number (i.e. 31), rather
	# than the port-level channel number (i.e. 8).
	
	set c_p_p [adapter_info_t_channels_per_port_get $adapter_info($bn)]
	set cn [total_channel_number $pn $cn $c_p_p]

	get_modem_status $bn $cn

	set f "$w.tlf.border.frame"

	$f.value_mode configure -text \
	 [ modem_mode2text [modem_status_t_mode_get $modem_status($bn,$cn)] ] 

	$f.value_connect_time configure -text \
	 [connectTimeString [modem_status_t_connect_time_get $modem_status($bn,$cn)]]

	$f.value_current_tx_rate configure -text \
	 [modem_status_t_current_tx_rate_get $modem_status($bn,$cn)]

	$f.value_current_rx_rate configure -text \
	 [modem_status_t_current_rx_rate_get $modem_status($bn,$cn)]

	$f.value_dte_tx_rate configure -text \
	 "[modem_status_t_dte_tx_rate_get $modem_status($bn,$cn)] cps"

	$f.value_dte_rx_rate configure -text \
	 "[modem_status_t_dte_rx_rate_get $modem_status($bn,$cn)] cps"

	$f.value_modulation configure -text \
	 [modem_modulation2text [modem_status_t_modulation_get $modem_status($bn,$cn)] ] 

	$f.value_compression configure -text \
	 [modem_compression2text [modem_status_t_compression_get $modem_status($bn,$cn)] ] 

	$f.value_error_correction configure -text \
	 [modem_error_correction2text [modem_status_t_error_correction_get $modem_status($bn,$cn)] ] 

	set text [any2str [modem_status_t_clid_get $modem_status($bn,$cn)] ]
	if {![string length $text]} {
		set text "Unavailable"
	}
	$f.value_clid configure -text "$text"
	set text [any2str [modem_status_t_local_dn_get $modem_status($bn,$cn)] ]
	if {![string length $text]} {
		set text "Unavailable"
	}

	$f.value_local_dn configure -text "$text" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"
	set text [any2str [modem_status_t_remote_dn_get $modem_status($bn,$cn)] ]

	if {![string length $text]} {
		set text "Unavailable"
	}
	$f.value_remote_dn configure -text "$text" \
	 -font "-*-Courier-bold-r-normal-*-10-*-*-*-*-*-*-*"

	delete_get_modem_status_t $ioc_modem_status($bn,$cn)
}

proc create_call_monitor_details_default { w bn pn cn } {
	global adapter_info

	set c_p_p [adapter_info_t_channels_per_port_get $adapter_info($bn)]
	set cn [total_channel_number $pn $cn $c_p_p]

	label $w.label_mode -text "Channel $cn is Idle.\n"
	grid $w.label_mode

	create_call_monitor_key $w

}

proc update_call_monitor_details_default { w bn pn cn } {
}

proc create_call_monitor_details { bn pn } {
	global adapter_info
	global board_family
	global call_monitor_current_path
	global call_monitor_details_path
	global call_monitor_details_frame
	global call_monitor_call_type
	global dying

	# From mgmthlc.h
	global CALL_BCHAN
	global CALL_MODEM

	if { $board_family($bn) == "dfrp" } {
		# Graphical port number...
		set gpn $pn
		set cn $call_monitor_current_path($bn,$pn)
	} else {
		set gpn 1
		set c_p_p [adapter_info_t_channels_per_port_get $adapter_info($bn)]
		set cn $call_monitor_current_path($bn,$gpn)
		if { $cn } {
			set pn [ expr ($cn / $c_p_p) + 1 ]
		} else {
			set pn 1
		}
		set cn [ expr $cn % $c_p_p ]
	}

	if { [string length $call_monitor_details_frame($bn,$gpn)] } {
		# Manipulate the dying flag to stop updates on non-existant
		# widgets.
		set dying 1
		destroy $call_monitor_details_frame($bn,$gpn)
		set dying 0
	}

	set w $call_monitor_details_path($bn,$gpn)
	set f [frame $w.frame -bg lightgray]
	set call_monitor_details_frame($bn,$gpn) $f

	switch -exact $call_monitor_call_type($bn,$pn,$cn) \
		$CALL_BCHAN { \
			create_call_monitor_details_bchan $f $bn $pn $cn
		} \
		$CALL_MODEM { \
			create_call_monitor_details_modem $f $bn $pn $cn
		} \
		default { \
			create_call_monitor_details_default $f $bn $pn $cn
		}

	pack $f -expand yes -fill both
}

proc update_call_monitor_details { bn pn } {
	global board_family
	global adapter_info
	global call_monitor_current_path
	global call_monitor_details_path
	global call_monitor_details_frame
	global call_monitor_call_type
	global dying

	# From mgmthlc.h
	global CALL_BCHAN
	global CALL_MODEM

	if { $board_family($bn) == "dfrp" } {
		# Graphical port number...
		set gpn $pn
		set cn $call_monitor_current_path($bn,$pn)
	} else {
		set gpn 1
		set c_p_p [adapter_info_t_channels_per_port_get $adapter_info($bn)]
		set cn $call_monitor_current_path($bn,$gpn)
		if { $cn } {
			set pn [ expr ($cn / $c_p_p) + 1 ]
		} else {
			set pn 1
		}
		set cn [ expr $cn % $c_p_p ]
	}

	if { ![string length $call_monitor_details_frame($bn,$gpn)] } {
		return
	}

	set f $call_monitor_details_frame($bn,$gpn)

	if {![string length $call_monitor_current_path($bn,$gpn)] } {
		return
	}

	#set pn [ lindex $call_monitor_current_path($bn) 0 ]
	#set cn [ lindex $call_monitor_current_path($bn) 1 ]

	if {![string length $call_monitor_call_type($bn,$pn,$cn)] } {
		return
	}

	switch -exact $call_monitor_call_type($bn,$pn,$cn) \
		$CALL_BCHAN { \
			update_call_monitor_details_bchan $f $bn $pn $cn
		} \
		$CALL_MODEM { \
			update_call_monitor_details_modem $f $bn $pn $cn
		} \
		default { \
			update_call_monitor_details_default $f $bn $pn $cn
		}
}

proc call_monitor_button_press { button_path_prefix bn pn cn } {
	global call_monitor_current_path

	set new_pn $pn
	set new_cn $cn

	if {[string length $call_monitor_current_path($bn,$pn)]} {
		set cn $call_monitor_current_path($bn,$pn)
		$button_path_prefix.border.frame.channel${cn} configure \
			-relief raised
	}

	set call_monitor_current_path($bn,$pn) $new_cn
	$button_path_prefix.border.frame.channel${new_cn} configure \
		-relief sunken

	create_call_monitor_details $bn $pn
}

proc create_status_call_monitor {w bn} {
	global mgmt
	global adapter_info
	global ioc_adapter_info
	global board_family
	global cm_height
	global cm_pos
	global cm_barerror
	global call_monitor_current_path
	global call_monitor_buttons_path
	global call_monitor_details_path
	global call_monitor_details_frame
	global call_monitor_call_type
	# From dm_osdep.tcl
	global CAN_DO_CALL_MONITOR

	#
	# Some versions of tcl/tk that work with other O/Ss seem to have
	# a buggy canvas widget.  For now, disable this feature.
	#
	if { !$CAN_DO_CALL_MONITOR } {
		return
	}

	#
	# Nothing to do for AccelePort Xp Family
	#
	if { $board_family($bn) == "ap2k" } {
		return
	}

	# Get info for generating the graphics:
	set ports [adapter_info_t_port_count_get $adapter_info($bn)]
	set c_p_p [adapter_info_t_channels_per_port_get $adapter_info($bn)]

	#
	# Add the call monitor page to our notebook
	#
	#
	# In order to preserve screen space, we will break up
	# the call monitor between ports on dfrp adapters.
	#
	if { $board_family($bn) == "dfrp" } {
		for { set pn 1 } { $pn <= $ports } { incr pn } {
			$w add call_monitor${pn} -label "Calls on Port $pn"\
						-underline [expr $pn - 1]
		}
	} else {
		set ports 1
		$w add call_monitor1 -label "Call Monitor" -underline 5
	}

	#
	# Create the stats notebook page(s)
	#
	for { set pn 1 } { $pn <= $ports } { incr pn } {
		set call_monitor_current_path($bn,$pn) ""
		set call_monitor_buttons_path($bn,$pn) ""
		set call_monitor_details_path($bn,$pn) ""
		set call_monitor_details_frame($bn,$pn) ""

		set f($pn) [$w subwidget call_monitor${pn}]
		frame $f($pn).tbl
	}
	
	# 
	# Draw the panes in our notebook's frame(s).
	#
	for { set pn 1 } { $pn <= $ports } { incr pn } {
		tixPanedWindow $f($pn).tbl.p -orient horizontal
		$f($pn).tbl.p add pane1 -size 225 -max 250
		$f($pn).tbl.p add pane2 -size 225 -max 250
		set p1($pn) [$f($pn).tbl.p subwidget pane1]
		set p2($pn) [$f($pn).tbl.p subwidget pane2]

		set call_monitor_details_path($bn,$pn) $p2($pn)
	}

	# Create the frame(s) for our pane(s).
	for { set pn 1 } { $pn <= $ports } { incr pn } {
		set p1f($pn) [frame $p1($pn).frame -bg lightgray]
	}

	for { set pn 1 } { $pn <= $ports } { incr pn } {
		
		if { $board_family($bn) == "dfrp" } {	
			tixLabelFrame $p1f($pn).port${pn} -label "Port $pn:" -bg lightgray
		} else {
			tixLabelFrame $p1f($pn).port${pn} -label "Channels:" -bg lightgray

			# Lie on the the channels per port for the graphical
			# layouts sake.  This will take extra logic in the
			# update to manage, but it will be well worth it.

			set c_p_p [ expr [adapter_info_t_port_count_get $adapter_info($bn)] * $c_p_p ]
		}

		set row 0
		set col 0
		for { set cn 0 } { $cn < $c_p_p } { incr cn } {
			if { $col > 3 } {
				set col 0
				incr row
			}
			set p1lf [$p1f($pn).port${pn} subwidget frame]
			set call_monitor_buttons_path($bn,$pn) $p1f($pn).port${pn}
			button $p1lf.channel${cn} -text "$cn" \
			  -font "-*-Courier-bold-r-normal-*-8-*-*-*-*-*-*-*" \
			  -command "call_monitor_button_press $p1f($pn).port${pn} $bn $pn $cn" \
			  -width 1
			grid $p1lf.channel${cn} -column $col -row $row -padx 0
			incr col
		}
		grid $p1f($pn).port${pn}
	}

	# Pack out the frame that holds our leftmost pane.
	for { set pn 1 } { $pn <= $ports } { incr pn } {
		pack $p1f($pn) -side left -expand yes -fill both
	}

	# Pack out the panes
	for { set pn 1 } { $pn <= $ports } { incr pn } {
		pack $f($pn).tbl.p -expand yes -fill both
	}

	#
	# Pack the whole lot out.
	#
	for { set pn 1 } { $pn <= $ports } { incr pn } {
		pack $f($pn).tbl -anchor w -expand yes -fill both
	}

	#
	# Initialize the call_type array for _all_ ports and channels
	# not just how we arrange them on the screen.
	#
	for { set pn 1 } \
	    { $pn <= [adapter_info_t_port_count_get $adapter_info($bn)] } \
	    { incr pn } {
	    	for { set cn 0 } { $cn < $c_p_p } { incr cn } {
			set call_monitor_call_type($bn,$pn,$cn) 0
		}
	}

	#
	# Select the first port, first channel for the user.
	#
	for { set pn 1 } { $pn <= $ports } { incr pn } {
		call_monitor_button_press $p1f($pn).port${pn} $bn $pn 0
	}
}

proc update_call_monitor_button { w bn pn cn } {
	global mgmt
	global all_call_status
	global ioc_all_call_status
	global adapter_info
	global ioc_adapter_info
	global board_family
	global call_monitor_call_type
	global call_monitor_current_path

	#
	# Nothing to do for AccelePort Xp Family
	#
	if { $board_family($bn) == "ap2k" } {
		return
	}

	# From mgmthlc.h
	global CALL_BCHAN
	global CALL_MODEM

	set call_status \
		[ all_call_status_t_status_var_get \
		   $all_call_status($bn,$pn) $cn ]

	#
	# Preserve the old call type for comparison.
	#
	if {[string length $call_monitor_call_type($bn,$pn,$cn)]} {
		set old_call_type $call_monitor_call_type($bn,$pn,$cn)
	} else {
		set old_call_type 0
	}

	switch -exact [ call_status_t_call_type_get $call_status ] \
		$CALL_BCHAN { \
			set color "green"
			set highlight "lightgreen"
			set call_monitor_call_type($bn,$pn,$cn) $CALL_BCHAN
		} \
		$CALL_MODEM { \
			set color "blue"
			set highlight "lightblue"
			set call_monitor_call_type($bn,$pn,$cn) $CALL_MODEM
		} \
		default { \
			set color "gray"
			set highlight "lightgray"
			set call_monitor_call_type($bn,$pn,$cn) 0
		}

	#
	# Let's see if we need to switch visuals for the user.
	#
	if {$call_monitor_call_type($bn,$pn,$cn) != $old_call_type} {
		if { $board_family($bn) == "dfrp" } {
			set gcn $cn
			set selected_cn $call_monitor_current_path($bn,$pn)
		} else {
			set gcn [expr $pn - 1]
			set selected_cn $call_monitor_current_path($bn,1)
		}

		if {$gcn == $selected_cn} {
			# The call types are different, but the user
			# is viewing the same button.  We need to switch
			# details views.
			create_call_monitor_details $bn $pn
		}
	}

	$w configure -background $color -activebackground $highlight 
}

proc update_status_call_monitor { bn } {
	global mgmt
	global adapter_info
	global ioc_adapter_info
	global all_call_status
	global ioc_all_call_status
	global call_monitor_current_path
	global call_monitor_buttons_path
	global board_family
	# From dm_osdep.tcl
	global CAN_DO_CALL_MONITOR

	#
	# Some versions of tcl/tk that work with other O/Ss seem to have
	# a buggy canvas widget.  For now, disable this feature.
	#
	if { !$CAN_DO_CALL_MONITOR } {
		return
	}

	#
	# Nothing to do for AccelePort Xp Family
	#
	if { $board_family($bn) == "ap2k" } {
		return
	}

	# Get info for generating the graphics:
	set ports [adapter_info_t_port_count_get $adapter_info($bn)]
	set c_p_p [adapter_info_t_channels_per_port_get $adapter_info($bn)]

	for { set pn 1 } { $pn <= $ports } { incr pn } {

		get_all_call_status $bn $pn

		for { set cn 0 } { $cn < $c_p_p } { incr cn } {
			# Unless we are dealing with a dfrp, it is
			# all displayed on the same port pane...
			if { $board_family($bn) == "dfrp" } {
				update_call_monitor_button $call_monitor_buttons_path($bn,$pn).border.frame.channel${cn} $bn $pn $cn
			} else {
				# A little tricky, but for the graphics
				# sake we fake the presentation of 8
				# channels per port rather than 8 ports
				# of a single channel.
			
				# JRH
				set disp_cn [total_channel_number $pn $cn $c_p_p]
				# set disp_cn [ expr $pn - 1 ]

				update_call_monitor_button $call_monitor_buttons_path($bn,1).border.frame.channel${disp_cn} $bn $pn $cn
			}
			update
		}

		delete_get_all_call_status_t $ioc_all_call_status($bn,$pn)
	}

	if { $board_family($bn) == "dfrp" } {
		for { set pn 1 } { $pn <= $ports } { incr pn } {
			update_call_monitor_details $bn $pn
		}
	} else {
		update_call_monitor_details $bn 1
	}
}

proc create_status_toplevel {} {
	global	dmlib
	global  status_update_delay
	global	max_num_boards
	global	DM_GUI_WIN_STATUS
	global	DM_GUI_WIN_CONFIG

	#
	# The "dying" flag controls how events are processed in the event
	# loop.  When a window is destroyed, it would be possible to update
	# a variable that no longer exsists; therefore, a flag is needed to
	# stop this from happening.
	#
	global  version
	global	dying

	set w .dm_status
	toplevel $w
	wm title $w "Digi Dense Modem Status $version"
	wm resizable $w 0 0

	dm_gui_win_open $DM_GUI_WIN_STATUS
	if { [ dm_gui_win_isopen $DM_GUI_WIN_CONFIG ] } {
		frame $w.f
		label $w.f.errorlbl -justify "left" -text "\
You may not have the adapter configurator and status\n\
windows open simultaneously.  Please close the config\n\
window and try again.\n"
		pack $w.f.errorlbl
		button $w.f.dismiss -text "Dismiss" -command {
		                                  	dm_gui_win_close $DM_GUI_WIN_CONFIG
							destroy .dm_status
		                                  }
		pack $w.f.dismiss
		pack $w.f -padx 5 -pady 5

		return
	}

	# Bind the X-event "Destroy" to manage the dying flag as well
	bind $w <Destroy> { 
		if {!$dying} { 
			dismiss_status_toplevel $w 
			exit 0
		}
	}

	global  nboards
	set nboards 0

	#
	#	Load the wrapper functions
	#
	load $dmlib/tcldgdm.so

	#
	# Create the Board notebook
	#
	tixNoteBook $w.boards_nb -ipadx 6 -ipady 6
	$w config -bg gray
	$w.boards_nb subwidget nbframe config -backpagecolor gray

	#
	# Create the Board pages
	#
	# jrh-TODO: What if more than 4 boards?
	for {set i 0} {$i < $max_num_boards} {incr i} {
		create_status_board $w.boards_nb $i
	}
	pack $w.boards_nb -expand yes -fill both -padx 5 -pady 5 -side top

	if {$nboards == 0} {
		label $w.noboards \
			-text "Sorry, Dense Modem GUI unable to access the driver.\n\
Possible reasons are:  No adapters installed;\n\
driver not loaded; driver devices not created;\n\
insufficient permissions on the management device.\n\
See man dm_driver for additional information."
		pack $w.noboards
	}

	#
	# Create the Dismiss button
	#
	frame $w.buttons -bg gray
	pack $w.buttons -side bottom -fill x -pady 2m

	if {$nboards > 0} {
		tixControl $w.buttons.update_delay -label "Update Delay (sec):"\
			-bg gray -integer true -max 600 -min 1 -step 1 \
			-variable status_update_delay

		pack $w.buttons.update_delay -side left -expand yes
		# Set the dying flag to say we have been instantiated
		set dying 0
	}

	button $w.buttons.dismiss -text "Dismiss" \
		-command "dismiss_status_toplevel $w"

	pack $w.buttons.dismiss -side right -expand yes


	# !!!
	#     Enter the Main Event Loop
	# !!!

	if { $nboards > 0 } {
		status_event_loop $w
	}

}

#
#	Dismiss:
#		Destroy the status toplevel and free all resources
#
proc dismiss_status_toplevel {w} {
	global dying
	global nboards
	global mgmt
	global port_count
	global ioc_adapter_info
	global ioc_ds1_status
	global ioc_adapter_state
	global board_type
	global board_family
	global status_update_delay
	global	DM_GUI_WIN_STATUS
	global	DM_GUI_WIN_CONFIG

	set dying 1

	for {set bn 0} {$bn < $nboards} {incr bn} {

		close_mgmt $bn

		delete_get_adapter_info_t $ioc_adapter_info($bn)

	}

	dm_gui_win_close $DM_GUI_WIN_STATUS

	destroy $w
	update
}
