/* -*- mode: C++; c-basic-offset: 4; tab-width: 8 -*- */
/* ex:set ts=8 sw=4: */

/*
 Copyright 1999-2000 by Digi International (www.digi.com)

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the 
 implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/**********************************************************************
 * $Header: /cvs_tick/triboot/hlc_hw.c,v 1.22 2002/06/25 14:04:24 dickl Exp $
 */

#include "types.h"
#include "tribl.h"
#include "triglbl.h"
#include "64011.h"
#include "gal9mmap.h"

extern uint16 swap_le16(uint16, BOARD*);
extern uint32 swap_le32(uint32, BOARD*);
extern uint32 swap32(uint32, BOARD*);

/*
 * get_gc_ptr
 *
 * Get a pointer to the current Global Common Structure.
 * If the pointer is invalid return a null.
 */
GLOBAL_COMMON *
get_gc_ptr(BOARD *brd)
{
    GLOBAL_COMMON *gcp;
    uint32 gco;
    void *vaddr;
    int unmap;

    if(brd->vram == 0)
    {
	unmap = 1;
	vaddr = (void *)brd->ram;
	brd->cb(MAPP2V, (tribl_uintptr_t *)&vaddr, &brd->ramlen, NULL, NULL);
    }
    else
    {
	unmap = 0;
	vaddr = brd->vram;
    }

/*DBUstart: parameterize GLOBAL_DATA_OFFSET 
    gco = TRIBL_PCI_READ32((uint8 *) vaddr + GLOBAL_DATA_OFFSET,
			   brd->id, TRIBL_MEM);
*/
    gco = TRIBL_PCI_READ32((uint8 *) vaddr + brd->gco,
			   brd->id, TRIBL_MEM);
/*DBUend*/
    gco = swap_le32(gco, brd);

    if(gco == 0 || gco > brd->ramlen)
	gcp = 0;
    else
	gcp = (GLOBAL_COMMON *) ((uint8 *) vaddr + gco);

    if(unmap)
	brd->cb(UNMAP, (tribl_uintptr_t *)&vaddr, &brd->ramlen, NULL, NULL);

    return(gcp);
}

uint32
hlc_postmap(BOARD *brd)
{
    uint8 *gpd;
    void  *vaddr;
    uint32 unmap;
    uint32 rc = 0;

    if(brd->vram == 0)
    {
	unmap = 1;
	vaddr = (void *)brd->ram;
	brd->cb(MAPP2V, (tribl_uintptr_t *)&vaddr, &brd->ramlen, NULL, NULL);
    }
    else
    {
	unmap = 0;
	vaddr = brd->vram;
    }

    if(vaddr == 0)
	return(0);

    gpd = (uint8 *) vaddr;
    gpd += swap_le32(TRIBL_PCI_READ32((uint8 *) vaddr + 0x1000,
				      brd->id, TRIBL_MEM), brd);

    if((gpd >= (uint8*) vaddr && gpd < (uint8*) vaddr + brd->ramlen) &&
       (swap_le32(TRIBL_PCI_READ32(&(((GLOBAL_POST_DATA *) gpd)->common.magic),
				   brd->id, TRIBL_MEM), brd)
	   == MAGIC_POST))
    {
	TRIBL_MEMCPY_FROM_PCI(&brd->post->pst, gpd, sizeof(GLOBAL_POST_DATA),
			      brd->id, TRIBL_MEM);

	/* Copy in the error report array */

	{
	    uint8 *rep;
	    int i;
	    int offset = brd->post->pst.summary.error_list_offset;

	    rep = (uint8 *) vaddr + swap_le32(offset, brd);

	    for(i = 0; i < POST_MAX_ENTRIES; i++)
	    {
		TRIBL_MEMCPY_FROM_PCI(&brd->post->err[i], rep,
				      sizeof(POST_ERROR_REPORT),
				      brd->id, TRIBL_MEM);

		if(((POST_ERROR_REPORT *) rep)->identifier == POST_ERROR_EOL)
		    break;

		rep += sizeof(POST_ERROR_REPORT);
	    }
	}

	rc = 1;
    }

    if(unmap)
	brd->cb(UNMAP, (tribl_uintptr_t *)&vaddr, &brd->ramlen, NULL, NULL);

    return(rc);
}

#define DSPGPP1_OFFSET	0x80000
#define GPP4_OFFSET	0x040a
#define GPP4_4640_RESET	(1 << 15)

int
hlc_reset(BOARD *brd, uint32 force)
{
    REG64011 *regptr;
    uint32 decode;
    uint16 *reg;
    uint8 *gpd;
    void *vaddr;
    uint32 unmap;

#   if DEBUG
    myprintf("hlc_reset flash(%x)\n", brd->flash);
#   endif

    if(brd->vreg == NULL)
    {
	unmap = 1;
	vaddr = (void *)brd->reg;
	brd->cb(MAPP2V, (tribl_uintptr_t *)&vaddr, &brd->reglen, NULL, NULL);
    }
    else
    {
	unmap = 0;
	vaddr = brd->vreg;
    }

    if(vaddr == 0)
	return (0);

    /* Get to the 64011's registers  (This is the remapped address.) */
    regptr = (REG64011 *) vaddr;

    /* start initializing the 64011 registers. */

    decode = (brd->ram >> 20) & 0xff;

    if (brd->device == DIGI_CPCIHOUSTON_DID)
    {
	TRIBL_PCI_WRITE32(&(regptr->ras0_low_decode),
			  swap32(decode, brd),
			  brd->id, TRIBL_REGS);
	TRIBL_PCI_WRITE32(&(regptr->ras0_high_decode),
			  swap32(decode + 0x0f, brd),
			  brd->id, TRIBL_REGS);
	TRIBL_PCI_WRITE32(&(regptr->ras1_low_decode),
			  swap32(decode + 0x10, brd),
			  brd->id, TRIBL_REGS);
	TRIBL_PCI_WRITE32(&(regptr->ras1_high_decode),
			  swap32(decode + 0x10, brd),
			  brd->id, TRIBL_REGS);
    }
    else
    {
	TRIBL_PCI_WRITE32(&(regptr->ras0_low_decode), swap32(decode, brd),
			  brd->id, TRIBL_REGS);
	TRIBL_PCI_WRITE32(&(regptr->ras0_high_decode),
			  swap32(decode + 0x03, brd),
			  brd->id, TRIBL_REGS);
	TRIBL_PCI_WRITE32(&(regptr->ras1_low_decode),
			  swap32(decode + 0x04, brd),
			  brd->id, TRIBL_REGS);
	TRIBL_PCI_WRITE32(&(regptr->ras1_high_decode),
			  swap32(decode + 0x07, brd),
			  brd->id, TRIBL_REGS);
    }

    decode = (brd->flash >> 20) & 0xff;

    TRIBL_PCI_WRITE32(&(regptr->bootcs_low_decode), swap32(decode, brd),
		      brd->id, TRIBL_REGS);
    TRIBL_PCI_WRITE32(&(regptr->bootcs_high_decode), swap32(decode + 0x01, brd),
		      brd->id, TRIBL_REGS);

    TRIBL_PCI_WRITE32(&(regptr->cs3_low_decode), swap32(decode + 0x02, brd),
		      brd->id, TRIBL_REGS);
    TRIBL_PCI_WRITE32(&(regptr->cs3_high_decode), swap32(decode + 0x03, brd),
		      brd->id, TRIBL_REGS);

    if(unmap)
	brd->cb(UNMAP, (tribl_uintptr_t *)&vaddr, &brd->reglen, NULL, NULL);

    if(brd->vram == NULL)
    {
	unmap = 1;
	vaddr = (void *)brd->ram;
	brd->cb(MAPP2V, (tribl_uintptr_t *)&vaddr, &brd->ramlen, NULL, NULL);
    }
    else
    {
	unmap = 0;
	vaddr = brd->vram;
    }

    if(vaddr == 0)
	return (0);

    gpd = (uint8 *) vaddr;
    gpd += swap_le32(TRIBL_PCI_READ32((uint8 *) vaddr + 0x1000,
				      brd->id, TRIBL_MEM), brd);

    if(gpd >= (uint8*) vaddr && gpd < (uint8*) vaddr + brd->ramlen)
    {
	/*
	 * If already at "post" no need to reset
	 */
    	if(swap_le32(TRIBL_PCI_READ32(&(((GLOBAL_POST_DATA*) gpd)->common.magic),
				      brd->id, TRIBL_MEM), brd)
    	   == MAGIC_POST)
	{
#	    if DEBUG
	    myprintf("hlc_reset: early return\n");
#	    endif

	    if(unmap)
		brd->cb(UNMAP, (tribl_uintptr_t *)&vaddr, &brd->ramlen, NULL, NULL);
	    return;
	}
    }
    else
    {
#	if DEBUG
	myprintf("hlc_reset: bad GLOBAL_COMMON pointer: 0x%lx\n",
		 (long) gpd);
#	endif

	if(unmap)
	    brd->cb(UNMAP, (tribl_uintptr_t *)&vaddr, &brd->ramlen, NULL, NULL);
	return (-1);
    }

    if(unmap)
	brd->cb(UNMAP, (tribl_uintptr_t *)&vaddr, &brd->ramlen, NULL, NULL);

    if(brd->vflash == NULL)
    {
	unmap = 1;
	vaddr = (void *)brd->flash;
	brd->cb(MAPP2V, (tribl_uintptr_t *)&vaddr, &brd->flashlen, NULL, NULL);
    }
    else
    {
	unmap = 0;
	vaddr = brd->vflash;
    }

    if(vaddr == 0)
	return (0);

    reg = (uint16 *) ((uint8*) vaddr + DSPGPP1_OFFSET + GPP4_OFFSET);

    TRIBL_PCI_WRITE16(reg, swap_le16(GPP4_4640_RESET, brd),
		      brd->id, TRIBL_FLASH);

    if(!(swap_le16(TRIBL_PCI_READ16(reg, brd->id, TRIBL_FLASH),
		   brd) & GPP4_4640_RESET))
	TRIBL_PCI_WRITE16(reg, swap_le16(GPP4_4640_RESET, brd),
			  brd->id, TRIBL_FLASH);

    TRIBL_PCI_WRITE16(reg, swap_le16(0, brd), brd->id, TRIBL_FLASH);

    if(unmap)
	brd->cb(UNMAP, (tribl_uintptr_t *)&vaddr, &brd->flashlen, NULL, NULL);

    return (0);
}

void
hlc_start(BOARD *brd, uint32 address)
{
    uint8 *gpd;
    uint32 unmap;
    void *vaddr;

    if(brd->vram == NULL)
    {
	unmap = 1;
	vaddr = (void *)brd->ram;
	brd->cb(MAPP2V, (tribl_uintptr_t *)&vaddr, &brd->ramlen, NULL, NULL);
    }
    else
    {
	unmap = 0;
	vaddr = brd->vram;
    }

    if(vaddr == 0)
	return;

    gpd = (uint8 *) vaddr;
    gpd += swap_le32(TRIBL_PCI_READ32(((uint8 *) vaddr + 0x1000),
				      brd->id, TRIBL_MEM), brd);

    if((gpd >= (uint8*) vaddr && gpd < (uint8*) vaddr + brd->ramlen)
       && (swap_le32(TRIBL_PCI_READ32(&(((GLOBAL_POST_DATA *) gpd)->common.magic),
				      brd->id, TRIBL_MEM), brd)
	   == MAGIC_POST
	   || swap_le32(TRIBL_PCI_READ32(&(((GLOBAL_POST_DATA *) gpd)->common.magic),
					 brd->id, TRIBL_MEM), brd)
	   == MAGIC_FEP))
    {
	if(unmap)
	    brd->vram = (void*) vaddr;
	brd->gcp = get_gc_ptr(brd);
	if(unmap)
	    brd->vram = 0;

	if(brd->gcp)
	{
	    /*
	     * the 0x80000000 bit tells the world we
	     * want to go to the cached address range.
	     */

	    TRIBL_PCI_WRITE32(&(brd->gcp->rw.rw_addr), 
			      swap_le32(address | 0x80000000, brd),
			      brd->id, TRIBL_MEM);
	    TRIBL_PCI_WRITE32(&(brd->gcp->rw.rw_req), swap_le32(RW_EXEC, brd),
			      brd->id, TRIBL_MEM);
	}
	else
	{
#	    if DEBUG
	    myprintf("triboot:hlc_start:Unable to get gcp for board at 0x%lx\n",
		     (long) brd->ram);
#	    endif
	}
    }
    else
    {
#	if DEBUG
	myprintf("hlcstart: failed to start board at: 0x%lx, 0x%lx\n",
		  (long) brd->ram, (long) gpd);
#	endif
    }

    if(unmap)
	brd->cb(UNMAP, (tribl_uintptr_t *)&vaddr, &brd->ramlen, NULL, NULL);

    return;
}

void
hlc_shutdown_board(BOARD *brd)
{
#   if DEBUG
    myprintf("hlc_shutdown_board");
#   endif

    hlc_reset(brd, 0);

    return;
}

/*
 * This is called to post an interrupt to the board.   Specifically,
 * it's used to get the board's attention while it's running code to
 * make it go service the debugger queues.
 */
void
hlc_post_board_int(BOARD *brd)
{
    uint32 unmap;
    void *vaddr;
    REG64011 *reg;

    if(brd->vreg == NULL)
    {
	unmap = 1;
	vaddr = (void *)brd->reg;
	brd->cb(MAPP2V, (tribl_uintptr_t *)&vaddr, &brd->reglen, NULL, NULL);
    }
    else
    {
	vaddr = brd->vreg;
	unmap = 0;
    }

    if(vaddr == 0)
	return;

    reg = (REG64011 *) vaddr;

    TRIBL_PCI_WRITE32(&(reg->intr_cause), swap32(~INT_PCIINT0, brd), 
		      brd->id, TRIBL_REGS);

    if(unmap)
	brd->cb(UNMAP, (tribl_uintptr_t *)&vaddr, &brd->reglen, NULL, NULL);

    return;
}


/*
 *	Allow us to change the interrupts that are enabled
 *	from the insmod command line.  For testing.
 */
static int	want_ints = INT_CPUINT0 | INT_CPUINT1;


/*
 * Acknowledge the interrupt source from the board.
 *
 * return 0 if no interrupt cleared, return 1 if interrupt cleared.
 */
int
hlc_clear_board_int(BOARD *brd)
{
    uint32 unmap;
    void *vaddr;
    REG64011 *reg;

    if(brd->vreg == NULL)
    {
	unmap = 1;
	vaddr = (void *)brd->reg;
	brd->cb(MAPP2V, (tribl_uintptr_t *)&vaddr, &brd->reglen, NULL, NULL);
    }
    else
    {
	vaddr = brd->vreg;
	unmap = 0;
    }

    if(vaddr == 0)
	return(0);

    reg = (REG64011 *) vaddr;

	if (swap32(want_ints, brd) & TRIBL_PCI_READ32(&(reg->intr_cause), brd->id, TRIBL_REGS))
    {
        TRIBL_PCI_WRITE32(&(reg->intr_cause), swap32(~want_ints, brd), brd->id, TRIBL_REGS);
        brd->intr_count++;
    }

    if(unmap)
	brd->cb(UNMAP, (tribl_uintptr_t *)&vaddr, &brd->reglen, NULL, NULL);

    return(1);
}


/*
 * Arm interrupts (to or from?) the host.
 */
void
hlc_unmask_board_int(BOARD *brd)
{
    uint32 unmap;
    void *vaddr;
    REG64011 *reg;

    if(brd->vreg == NULL)
    {
	unmap = 1;
	vaddr = (void *)brd->reg;
	brd->cb(MAPP2V, (tribl_uintptr_t *)&vaddr, &brd->reglen, NULL, NULL);
    }
    else
    {
	vaddr = brd->vreg;
	unmap = 0;
    }

    if(vaddr == 0)
	return;

    reg = (REG64011 *) vaddr;

    TRIBL_PCI_WRITE32(&(reg->pci_mask),
		      TRIBL_PCI_READ32(&(reg->pci_mask), brd->id, TRIBL_REGS) |
		      swap32(want_ints, brd), brd->id, TRIBL_REGS);

    if(unmap)
	brd->cb(UNMAP, (tribl_uintptr_t *)&vaddr, &brd->reglen, NULL, NULL);

    return;
}

/*
 * blah.
 */
void
hlc_mask_board_int(BOARD *brd)
{
    uint32 unmap;
    void *vaddr;
    REG64011 *reg;

    if(brd->vreg == NULL)
    {
	unmap = 1;
	vaddr = (void *)brd->reg;
	brd->cb(MAPP2V, (tribl_uintptr_t *)&vaddr, &brd->reglen, NULL, NULL);
    }
    else
    {
	vaddr = brd->vreg;
	unmap = 0;
    }

    if(vaddr == 0)
	return;

    reg = (REG64011*) vaddr;

	TRIBL_PCI_WRITE32(&(reg->pci_mask),
			  TRIBL_PCI_READ32(&(reg->pci_mask), 
					   brd->id, TRIBL_REGS) &
			  swap32(~want_ints, brd), brd->id, TRIBL_REGS);
	TRIBL_PCI_WRITE32(&(reg->intr_cause), swap32(~want_ints, brd),
			  brd->id, TRIBL_REGS);

    if(unmap)
	brd->cb(UNMAP, (tribl_uintptr_t *)&vaddr, &brd->reglen, NULL, NULL);

    return;
}
