/* -*- mode: C++; c-basic-offset: 4; tab-width: 8 -*- */
/* ex:set ts=8 sw=4: */

/*
 Copyright 1999-2000 by Digi International (www.digi.com)

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the 
 implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/**********************************************************************
 * $Header: /cvs_tick/triboot/tb_mdone.c,v 1.21 2002/06/25 14:04:25 dickl Exp $
 */

#include "types.h"
#include "tribl.h"
#include "pci_ids.h"

#ifndef S2K_4PP_PCI_NAME
#include "s2k_pci_ids.h"
#endif

#include "rap2mmap.h"
// #include "mipsheader.hh"

extern void return_error(TRIBL_BOARD *, int);
extern int32 ramtest(void *, uint32, uint32, BOARD *);
extern struct d_info drv_info[NDEVS];

int
tb_map_done(TRIBL_BOARD *tb, BOARD *brd, struct d_info *dp)
{
    PADDRS *paddr;
    void *vaddr;
    int unmap;
    int i;
    int rc = 0;

    paddr = (PADDRS *) tb->pdata;

    for(i = 0; i < NBARS; i++, paddr++)
	switch(paddr->type)
	{
	    /*
	     * Use only the first address of each supported
	     * type found.
	     */
	case TRIBL_MEM:
	    if(brd->vram == NULL)
		brd->vram = (void *) paddr->addr;
	    brd->ramlen = paddr->size;
	    break;
	case TRIBL_REGS:
	    if(brd->vreg == NULL)
		brd->vreg = (void *) paddr->addr;
	    brd->reglen = paddr->size;
	    break;
	case TRIBL_FLASH:
	    if(brd->vflash == NULL)
		brd->vflash = (void *) paddr->addr;
	    brd->flashlen = paddr->size;
	    break;
	case TRIBL_ROM:
	    if(brd->vrom == NULL)
		brd->vrom = (void *) paddr->addr;
	    brd->romlen = paddr->size;
	    break;
	default:
	    break;
	}

    /*
     * save the PADDRS values away for the driver's
     * future use if it needs it.
     */

    dp->paddrs = (PADDRS *) tb->pdata;
    tb->pdata = 0;

    /* Ensure the board is in reset */
    if(brd->reset_board)
    {
	tribl_uintptr_t dtime = 20;

	if (brd->reset_board(brd, 0) == -1)
	{
	    return_error(tb, TRIBL_RESET_ERROR);
	    return(-1);
	}

	brd->cb(DELAY, &dtime, NULL, NULL, NULL);
    }

    if (brd->unmask_board_int) {
        brd->unmask_board_int(brd);
    }

    if(brd->vram == NULL)
    {
	unmap = 1;
	vaddr = (void *) brd->ram;
	brd->cb(MAPP2V, (tribl_uintptr_t *)&vaddr, &brd->ramlen, NULL, NULL);
    }
    else
    {
	unmap = 0;
	vaddr = brd->vram;
    }

    if(vaddr == 0)
    {
	return_error(tb, TRIBL_NO_VADDR);
	return(-1);
    }

    /*
     * Now that vram is known, at least do a simple memory test
     * of the area where BOOT and POST will be downloaded, POST
     * will do a more extensive memory test.
     */

    switch(tb->pci_device)
    {
    case PCI_TIGGER8:
	rc = ramtest((uint8 *) vaddr + FEP_CODE_OFFSET, 0, brd->ramtest_len,
		     brd);
	break;

    case DIGI_CHASKA_DID:
    case DIGI_PCIHOUSTON_DID:
    case DIGI_LONDON_DID:
    case DIGI_CPCIHOUSTON_DID:
	/* Ramtest for HLC moved to tb_wdone under DL_POST to avoid 
	   interfering with a possibly running POST */
#if 0	
	/*
	 * Can have 4Meg or 8Meg, if 8 then test the first 4
	 * leaving the running POST area out.
	 * Then test all of the remaining 4 meg.
	 */

	if(brd->ramtest_len <= 1024 * 1024 * 4) {
	    /* Test from 128K to RAMSIZE-512K. */
	    rc = ramtest((uint8 *) vaddr + 0x20000, 0,
			 brd->ramtest_len - 0x20000 - 512 * 1024, brd);
	} else
	{
	    /* Test from 128K to 4M-512K. */
	    rc = ramtest((uint8*) vaddr, 0x20000,
			 4096 * 1024 - 0x20000 - 512 * 1024, brd);
	    /* Test from 4M to RAMSIZE. */
	    rc += ramtest((uint8*) vaddr, 4096*1024,
			  brd->ramtest_len - 4096 * 1024, brd);
	}
#endif
	break;

/*DBUstart*/
    CASE_S2K_DID :
	rc=0;
	break;
/*DBUend*/
    default:
	rc = 1;
	break;
    }

    if(unmap)
	brd->cb(UNMAP, (tribl_uintptr_t *)&vaddr, &brd->ramlen, NULL, NULL);

    if(rc != 0)
    {
	return_error(tb, TRIBL_MEMORY_ERROR);
	return(-1);
    }

    /* save where we are coming from */

    dp->cur_state = tb->state;

    /*
     * This means we will start by downloading FEP, it
     * contains chip initialization code already.
     * This should be all we need to test the mechanism
     * of LibTriBoot, the rest is in the details.
     */

    tb->status = TRIBL_OK;
    tb->sequence = 0;
    tb->count = 0;
    tb->size = 0;

    if(!(dp->flags & (DXBSENT | DXBDONE)))
    {
	dp->flags |= DXBSENT;
	tb->state = TRIBL_DXBINIT;
    }
    else
    {
	dp->code = brd->dl_start;


	switch(dp->code)
	{
	case DL_BOOT:
	    tb->state = TRIBL_FW_BOOT;
	    memcpy(tb->file_name, brd->bin_names.fepname[dp->code], FLEN);
	    break;
	case DL_POST:
	    tb->state = TRIBL_FW_POST;
	    memcpy(tb->file_name, brd->bin_names.fepname[dp->code], FLEN);
	    break;	    
	case DL_POSTWAIT:	/* Don't actually run the POST, */
				/* just wait for it. */
	    dp->code = DL_POST;
	    tb->state = TRIBL_WAIT;
	    
	    tb->count = 1000;
	    memset(tb->file_name, 0, FLEN);
	    tb->pdata = 0;

	    dp->wait_count = 1;
	    break;
	case DL_FEP:
	    tb->state = TRIBL_FW_FEP;
	    memcpy(tb->file_name, brd->bin_names.fepname[dp->code], FLEN);
	    break;
	default:
	    dp->code = DL_DONE;
	    tb->state = TRIBL_DONE;
	    break;
	}
    }

    dp->cur_sequence = 0;
    return(0);
}
