/* -*- mode: C++; c-basic-offset: 4; tab-width: 8 -*- */
/* ex:set ts=8 sw=4: */

/*
 Copyright 1999-2000 by Digi International (www.digi.com)

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the 
 implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/**********************************************************************
 * $Header: /net/cvs/eng/cvs/triboot/tribl.c,v 1.31 2001/01/03 17:13:39 dave Exp $
 */

#include "types.h"
#include "tribl.h"
#include "rap2mmap.h"

extern int tb_register(TRIBL_BOARD *, BOARD *, struct d_info *);
extern int tb_map_done(TRIBL_BOARD *, BOARD *, struct d_info *);
extern int tb_download(TRIBL_BOARD *, BOARD *, struct d_info *);
extern int tb_wait_done(TRIBL_BOARD *, BOARD *, struct d_info *);

void init_crc32(void);
uint32 crc32(uint32, uint8 *, uint32, BOARD *);

uint16 swap16(uint16, BOARD*);
uint32 swap32(uint32, BOARD*);
uint16 swap_le16(uint16, BOARD*);
uint32 swap_le32(uint32, BOARD*);
int32 ramtest(void *, uint32, uint32, BOARD *);

int rap2_start = DL_POST;
int hlc2_start = DL_POSTWAIT;
/*DBUstart*/
int s2k2_start = DL_POSTWAIT;
/*DBUend*/

int nbrds;
BOARD Board[NDEVS];

struct d_info drv_info[NDEVS];

TRIBL_POST Post[NDEVS];

/************************************************************************
 *	Utility functions
 */

/**********************************************************************
 * I really do not want to add this here, use the library versions
 */

#define CRC32_POLY	0x04c11db7	/* AUTODIN II, Ethernet, & FDDI */

static uint32	crc32_table[256];

void
init_crc32(void)
{
    int i, j;
    uint32 c;

    for(i = 0; i < 256; ++i)
    {
	for(c = i << 24, j = 8; j > 0; --j)
	    c = c & 0x80000000 ? (c << 1) ^ CRC32_POLY : (c << 1);
	crc32_table[i] = c;
    }

    return;
}

uint32
crc32(uint32 crc, uint8 *buf, uint32 length, BOARD *brd)
{
    uint8 *p;
    int32 len = length;

    for(p = buf; len > 0; ++p, --len)
	crc = (crc << 8) ^ crc32_table[(crc >> 24) ^ 
				      TRIBL_PCI_READ8(p, brd->id, TRIBL_MEM)];

    return(crc);
}

/*
 * find this drv_info pointer
 */

void
return_error(TRIBL_BOARD *tb, int error)
{
    tb->count = 0;
    tb->sequence = 0;
    tb->status = error;
    tb->state = TRIBL_DONE;

    return;
}

int
get_d_info_index(TRIBL_BOARD *tb)
{
    int i;

    for(i = 0; i < NDEVS; i++)
	if(tb->id == drv_info[i].u_id)
	    break;

    if(i == NDEVS)
    {
	return_error(tb, TRIBL_NOT_REGISTERED);
	i = -1;
    }

    return(i);
}

void
next_state(TRIBL_BOARD *tb, struct d_info *dp, BOARD *brd)
{
    switch(dp->code)
    {
    case DL_START:
	tb->state = TRIBL_FW_BOOT;
	dp->code = DL_BOOT;
	break;
    case DL_BOOT:
	tb->state = TRIBL_FW_POST;
	dp->code  = DL_POST;
	break;
    case DL_POST:
	tb->state = TRIBL_FW_FEP;
	dp->code = DL_FEP;
	break;
    default:
	tb->state = TRIBL_DONE;
	dp->code = DL_DONE;
	break;
    }

    if(dp->code == DL_DONE || dp->code == DL_START)
	memset(tb->file_name, 0, FLEN);
    else
	memcpy(tb->file_name, brd->bin_names.fepname[dp->code], FLEN);

    tb->count = 0;
    tb->sequence = 0;
    tb->status = TRIBL_OK;

    dp->cur_sequence = 0;
    dp->wait_count = 0;
    dp->hp = 0;

    return;
}

void
tri_boot(TRIBL_BOARD *tb)
{
    struct d_info *dp;
    BOARD *brd;
    int i;

    if(tb == NULL)
	return;


    switch(tb->state)
    {
    case TRIBL_REGISTER:
	dump_tb(tb);

	    /* check for dulicate 'id's */

	for(i = 0; i < NDEVS; i++)
	    if(tb->id == drv_info[i].u_id)
	    {
		return_error(tb, TRIBL_DUPLICATE);
		goto rtn;
	    }

	/* Now find an empty slot */

	for(i = 0; i < NDEVS; i++)
	    if(drv_info[i].u_id == 0)
		break;

	if(i == NDEVS)
	{
	    return_error(tb, TRIBL_NO_DEV);
	    goto rtn;
	}

	dp = &drv_info[i];
	brd = &Board[i];

	memset(dp, 0, sizeof(*dp));
	memset(brd, 0, sizeof(*brd));

	brd->post = &Post[i];
	memset(brd->post, 0, sizeof(TRIBL_POST));

	brd->id = tb->id;
	dp->u_id = tb->id;

	/*
	 * can we agree that it is pretty hard to go
	 * thru the shutdown if the register fails.
	 */

	if(tb_register(tb, brd, dp) < 0)
	{
	    memset(&Board[i], 0, sizeof(BOARD));
	    memset(&drv_info[i], 0, sizeof(struct d_info));
	    goto rtn;
	}

	/* Now we can do this */
	nbrds++;
	break;

    case TRIBL_UNREGISTER:
	dump_tb(tb);

	for(i = 0; i < NDEVS; i++)

	    if(tb->id == drv_info[i].u_id)
	    {
		/* Success! */
		drv_info[i].u_id=0;
		nbrds--;
		goto rtn;
	    }

	if(i == NDEVS)
	{
	    return_error(tb, TRIBL_NOT_REGISTERED);
	    goto rtn;
	}

	break;

    case TRIBL_MADDRS_DONE:
	dump_tb(tb);

	if((i = get_d_info_index(tb)) == -1)
	    goto rtn;

	brd = &Board[i];
	dp = &drv_info[i];

	if(tb_map_done(tb, brd, dp) < 0)
	    goto errrtn;

	break;

    case TRIBL_FW_BOOT:
    case TRIBL_FW_POST:
    case TRIBL_FW_FEP:
	/*
	 * Register done??
	 * Ensure processor is in reset, download
	 * FEP, checksum, and release, if all OK
	 * inform the driver we are all done.
	 */

	dump_tb(tb);

	if((i = get_d_info_index(tb)) == -1)
	    goto rtn;

	brd = &Board[i];
	dp = &drv_info[i];

	if(tb->sequence <= dp->cur_sequence)
	{
	    return_error(tb, TRIBL_SEQUENCE_ERROR);
	    goto errrtn;
	}

	if(tb_download(tb, brd, dp) < 0)
	    goto errrtn;

	break;

    case TRIBL_DRIVER_UNLOAD:
	/*
	 * Until we have reason to do more, when the driver unloads,
	 * simply put the board in reset.
	 */

	dump_tb(tb);

	if((i = get_d_info_index(tb)) == -1)
	    goto rtn;

	brd = &Board[i];
	dp = &drv_info[i];

	if(dp->paddrs == NULL)
	{
	    goto errrtn;
	}

	if(brd->shutdown_board)
	    brd->shutdown_board(brd);

	tb->state = TRIBL_DONE;
	tb->status = TRIBL_OK;
	tb->count = 0;
	tb->sequence = 0;
	tb->pdata = dp->paddrs;

	memset(dp, 0, sizeof(*dp));
	memset(brd, 0, sizeof(*brd));
	break;

    case TRIBL_PANIC:
	{
	    uint8 *gd;

	    dump_tb(tb);
	    if((i = get_d_info_index(tb)) == -1)
		goto rtn;

	    brd = &Board[i];

	    gd = ((uint8 *) brd->vram + GLOBAL_DATA_OFFSET);

	    /* Send a pointer to the board's panic structure */

	    if(gd)
		tb->pdata = &((GLOBAL_FEP_DATA *) gd)->panic;
	    else
		tb->pdata = 0;
	}
	break;

    case TRIBL_WAIT_DONE:
	dump_tb(tb);

	if((i = get_d_info_index(tb)) == -1)
	    goto rtn;

	brd = &Board[i];
	dp = &drv_info[i];

	if(tb_wait_done(tb, brd, dp) < 0)
	    goto errrtn;

	break;

    case TRIBL_BOARD_STRUCT:
	dump_tb(tb);
	if((i = get_d_info_index(tb)) == -1)
	    goto rtn;

	if(tb->pdata != 0)
	{
	    return_error(tb, TRIBL_PROTO_ERROR);
	    goto rtn;
	}

	tb->pdata = &Board[i];
	break;

    case TRIBL_DXBINIT:
	{
	    void *vaddr;
	    uint32 unmap;

	    dump_tb(tb);

	    if((i = get_d_info_index(tb)) == -1)
		goto rtn;

	    /*
	     * This simply fills in the GLOBAL_HOST_DATA pointer
	     * for the FEP.  If not done this is obviously not an
	     * S/DXB type FEP.
	     */

	    dp = &drv_info[i];
	    brd = &Board[i];

	    /*
	     * The address passed in 'pdata' better be correct,
	     * who knows what is reasonable, given OS differences,
	     * what is kernel memory, what is user memory, who
	     * accesses what?
	     */

	    if(brd->vram == NULL)
	    {
		unmap = 1;
		vaddr = (void *) brd->ram;
		brd->cb(MAPP2V, (tribl_uintptr_t *)&vaddr, &brd->ramlen, NULL, NULL);
	    }
	    else
	    {
		unmap = 0;
		vaddr = brd->vram;
	    }

	    if(vaddr == 0)
	    {
		return_error(tb, TRIBL_NO_VADDR);
		return;
	    }

	    brd->ghd = (GLOBAL_HOST_DATA *) tb->pdata;
	    tb->pdata = 0;

/*DBUstart: GLOBAL_HOST_OFFSET of [of course!] different for S2K 
	    TRIBL_PCI_WRITE32((uint8 *) vaddr + GLOBAL_HOST_OFFSET,
			      swap_le32((uint32) brd->ghd, brd), 
			      brd->id, TRIBL_MEM);
*/
	    TRIBL_PCI_WRITE32((uint8 *) vaddr + brd->gho,
			      swap_le32((uint32) brd->ghd, brd), 
			      brd->id, TRIBL_MEM);
/*DBUend*/

	    if(unmap)
		brd->cb(UNMAP, (tribl_uintptr_t *)&vaddr, &brd->ramlen, NULL, NULL);

	    dp->flags |= DXBDONE;

	    /* Duplicate code from MADDRS_DONE */
	    dp->code = brd->dl_start;



	    tb->sequence = 0;
	    tb->count = 0;
	    tb->status = TRIBL_OK;
	    tb->size = 0;

	    switch(dp->code)
	    {
	    case DL_BOOT:
		tb->state = TRIBL_FW_BOOT;
		memcpy(tb->file_name, brd->bin_names.fepname[dp->code], FLEN);
		break;
	    case DL_POST:
		tb->state = TRIBL_FW_POST;
		memcpy(tb->file_name, brd->bin_names.fepname[dp->code], FLEN);
		break;
	    case DL_POSTWAIT:	/* Don't actually run the POST, */
				/* just wait for it. */
		dp->code = DL_POST;
		tb->state = TRIBL_WAIT;
		
		tb->count = 1000;
		memset(tb->file_name, 0, FLEN);
		tb->pdata = 0;

		dp->wait_count = 1;
		break;
	    case DL_FEP:
		tb->state = TRIBL_FW_FEP;
		memcpy(tb->file_name, brd->bin_names.fepname[dp->code], FLEN);
		break;
	    default:
		dp->code = DL_DONE;
		tb->state = TRIBL_DONE;
		break;
	    }

	    dp->cur_sequence = 0;
	}
	break;

    case TRIBL_POST_DATA:
	dump_tb(tb);

	if((i = get_d_info_index(tb)) == -1)
	    goto rtn;

	/*
	 * This simply fills in the pointer to the
	 * POST results saved immediately after POST
	 * was done - successful or not.
	 */

	dp = &drv_info[i];
	brd = &Board[i];

	if(dp->flags & POSTMAP)
	    tb->pdata = brd->post;
	else
	    tb->pdata = 0;

	tb->status = TRIBL_OK;

	break;

    default:
	dump_tb(tb);
	break;
    }

 rtn:
#   if DEBUG
    myprintf("return: state(%x) status(%x)\n",
	     tb->state, tb->status);
#   endif
    return;

 errrtn:
#   if DEBUG
    myprintf("error return: state(%x) status(%x)\n",
	     tb->state, tb->status);
#   endif
    /*
     * wipe the board structure and d_info structure associated
     * with this 'id'.
     */

    for(i = 0; i < NDEVS; i++)
	if(tb->id == drv_info[i].u_id)
	    break;

    if(i == NDEVS)
	return;
#if 0
    memset(&Board[i], 0, sizeof(BOARD));
    memset(&drv_info[i], 0, sizeof(struct d_info));
    nbrds--;
#endif
    return;
}

uint32
tribl_intr(uint32 intr)
{
    struct d_info *dp;
    BOARD *brd;
    int i;
    int icnt;
    uint32 ret = 0;

    if(intr == 0)
	return(intr);

    for(i = 0; i < NDEVS; i++)
    {
	dp = &drv_info[i];
	brd = &Board[i];
	if(brd->clear_board_int == 0
	   || dp->intr_index == -1
	   || (dp->code != DL_DONE)
	   || !((1 << dp->intr_index) & intr))
	    continue;

	icnt = brd->intr_count;
	brd->clear_board_int(brd);
	if(icnt != brd->intr_count)
	    ret |= (1 << dp->intr_index);
    }
    return(ret);
}

/*
 * swap16 will swap bytes if the board is an endian issue between
 * the board and the host.
 */

uint16
swap16(uint16 s, BOARD *brd)
{
    if(brd->need_swap)
    {
	uint16 t = s;
	s = ((t & 0xff00) >> 8) | ((t & 0x0ff) << 8);
    }
    return(s);
}

uint32
swap32(uint32 l, BOARD *brd)
{
    if(brd->need_swap)
    {
	uint32 t = l;
	l = ((t & 0xff000000) >> 24)
	    | ((t & 0x00ff0000) >> 8)
	    | ((t & 0x0000ff00) << 8)
	    | ((t & 0x000000ff) << 24);
    }
    return(l);
}

uint16
swap_le16(uint16 s, BOARD *brd)
{
#ifdef BIG_ENDIAN_HOST
    uint16 t = s;

    s = ((t & 0xff00) >> 8) | ((t & 0x0ff) << 8);
#endif
	brd = brd;
    return(s);
}

uint32
swap_le32(uint32 l, BOARD *brd)
{
#ifdef BIG_ENDIAN_HOST
    uint32 t = l;

    l = ((t & 0xff000000) >> 24)
	| ((t & 0x00ff0000) >>  8)
	| ((t & 0x0000ff00) << 8)
	| ((t & 0x000000ff) << 24);
#endif
	brd = brd;
    return(l);
}

/*
 * Test board memory
 *
 * Memory IS mapped before calling this function!!!
 */
int32
ramtest(void *addr, uint32 offset, uint32 length, BOARD *brd)
{
    uint32 *lp = addr;
    uint32 i;
    uint32 val;
    uint32 len;

    len = length / sizeof(uint32);

    lp += offset/sizeof(uint32);

    /* TODO: this is a whimpy test */

    for(i = 0; i < len; ++i)	/* first a constant pattern */
	TRIBL_PCI_WRITE32(&lp[i], 0x11223344, brd->id, TRIBL_MEM);

    for(i = 0; i < len; ++i)	/* then address tags */
	TRIBL_PCI_WRITE32(&lp[i], i, brd->id, TRIBL_MEM);

    for(i = 0; i < len; ++i)
    {
	val = TRIBL_PCI_READ32(&lp[i], brd->id, TRIBL_MEM);
	if(val != i)
	    return(-1);
    }

    return(0);
}

#if defined(HPUX)

/*The d_info structure needs to be cleared before triboot
  is used.  Otherwise a duplicate id error could result.
*/ 
void triboot_init()
{
    uint32 i;

    for (i=0; i < NDEVS; i++)
        memset(&drv_info[i], 0, sizeof(struct d_info));
}

#endif

